#!/bin/sh

#
# BurgnerIfaceMaker
#
# Copyright (c) 2002 Alexandre Archambault
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#

#
# ChangeLog
#
#
# 2001-09-19  Alexandre Archambault <aarchambault@free.fr>
#	* Added ChangeLog
#	* Fixed check/check_with_val
#
# A-Long-Time-Ago  Alexandre Archambault <aarchambault@free.fr>
#	* Creation/Bug fixes


usage() {
    echo "Usage: $0 MyIface my_iface MY IFACE file_prefix "
    exit 1
}

[ $# -ne 1 ] && usage
[ -z "$1" ] && usage

My_Iface="$1"

MAJ="AZERTYUIOPQSDFGHJKLMWXCVBN"
MIN="azertyuiopqsdfghjklmwxcvbn"

MyIface=`echo ${My_Iface} | sed 's/_//g'`
my_iface=`echo ${My_Iface} | sed y/$MAJ/$MIN/`
MY_IFACE=`echo ${My_Iface} | sed y/$MIN/$MAJ/`
MY=`echo ${MY_IFACE} | sed 's/_/ /' | cut -f1 -d ' '`
IFACE=`echo ${MY_IFACE} | sed 's/_/ /' | cut -f2 -d ' '`
file_prefix=`echo ${my_iface} | sed 's/_/-/g'`
header_file="$file_prefix.h"
code_file="$file_prefix.c"

if [ -e $header_file ]; then
echo -e "'$header_file' already exists."
echo -n -e "Do you want to continue (this will erase its actual content) ? [No]:"
read continue
[ "$continue" != "yes" -a "$continue" != "y" -a "$continue" != "Yes" -a "$continue" != "Y" ] && exit 1
continue=""
fi

if [ -e $code_file ]; then
echo -e "'$code_file' already exists."
echo -n -e "Do you want to continue (this will erase its actual content) ? [No]:"
read continue
[ "$continue" != "yes" -a "$continue" != "y" -a "$continue" != "Yes" -a "$continue" != "Y" ] && exit 1
fi


for i in $header_file $code_file; do
cat > $i << EOF
/*
 * $MyIface - $MyIface iface
 *
 * Copyright (c) `date "+%Y"` Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


EOF
done

cat >> $header_file << EOF
#ifndef __${MY_IFACE}_H__
#define __${MY_IFACE}_H__

#include <glib-object.h>
#include 

G_BEGIN_DECLS


#define ${MY}_TYPE_${IFACE}		(${my_iface}_get_type ())
#define ${MY}_IS_${IFACE}(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((obj), ${MY}_TYPE_${IFACE}))
#define ${MY_IFACE}(o)			(G_TYPE_CHECK_INSTANCE_CAST ((obj), ${MY}_TYPE_${IFACE}, ${MyIface}))
#define ${MY_IFACE}_GET_IFACE(o)	(G_TYPE_INSTANCE_GET_INTERFACE ((o), ${MY}_TYPE_${IFACE}, ${MyIface}Iface))


typedef struct _${MyIface} ${MyIface};
typedef struct _${MyIface}Iface ${MyIface}Iface;

struct _${MyIface}Iface {
	GTypeInterface __parent;

	
};


GType ${my_iface}_get_type (void) G_GNUC_CONST;



G_END_DECLS

#endif /* __${MY_IFACE}_H__ */

EOF

cat >> $code_file << EOF
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "${header_file}"


GType
${my_iface}_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		GTypeInfo info = {
			szeof (${MyIface}Iface),
			(GBaseInitFunc) NULL,
			(GBaseFinalizeFunc) NULL,
			
		};

		type = g_type_register_static (G_TYPE_INTERFACE, "${MyIface}", &info, 0);
	}

	return type;
}

EOF
