#!/bin/sh

#
# BurgnerObjectMaker
#
# Copyright (c) 2001-2002 Alexandre Archambault
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#

#
# ChangeLog
#
#
# 2002-08-06  Alexandre Archambault <aarchambault@free.fr>
#	* Rewrote argument line parsing
#
# 2001-09-19  Alexandre Archambault <aarchambault@free.fr>
#	* Added ChangeLog
#	* Fixed check/check_with_val
#
# A-Long-Time-Ago  Alexandre Archambault <aarchambault@free.fr>
#	* Creation/Bug fixes


usage() {
    echo "Usage: $0 My_Object [ Parent_Object ]"
    exit 1
}

[ $# -lt 1 ] && usage

My_Object="$1"
Parent_Object="$2"
Parent_Object=${Parent_Object:-G_Object}

MAJ="AZERTYUIOPQSDFGHJKLMWXCVBN"
MIN="azertyuiopqsdfghjklmwxcvbn"

MyObject=`echo ${My_Object} | sed 's/_//g'`
my_object=`echo ${My_Object} | sed y/$MAJ/$MIN/`
MY_OBJECT=`echo ${My_Object} | sed y/$MIN/$MAJ/`
MY=`echo ${MY_OBJECT} | sed 's/_/ /' | cut -f1 -d ' '`
OBJECT=`echo ${MY_OBJECT} | sed 's/_/ /' | cut -f2 -d ' '`
file_prefix=`echo ${my_object} | sed 's/_/-/g'`
header_file="$file_prefix.h"
code_file="$file_prefix.c"

ParentObject=`echo ${Parent_Object} | sed 's/_//g'`
parent_object=`echo ${Parent_Object} | sed y/$MAJ/$MIN/`
PARENT_OBJECT=`echo ${Parent_Object} | sed y/$MIN/$MAJ/`
PARENT=`echo ${PARENT_OBJECT} | sed 's/_/ /' | cut -f1 -d ' '`
POBJECT=`echo ${PARENT_OBJECT} | sed 's/_/ /' | cut -f2 -d ' '`

if [ -e $header_file ]; then
echo -e "'$header_file' already exists."
echo -n -e "Do you want to continue (this will erase its actual content) ? [No]:"
read continue
[ "$continue" != "yes" -a "$continue" != "y" -a "$continue" != "Yes" -a "$continue" != "Y" ] && exit 1
continue=""
fi

if [ -e $code_file ]; then
echo -e "'$code_file' already exists."
echo -n -e "Do you want to continue (this will erase its actual content) ? [No]:"
read continue
[ "$continue" != "yes" -a "$continue" != "y" -a "$continue" != "Yes" -a "$continue" != "Y" ] && exit 1
fi


for i in $header_file $code_file; do
cat > $i << EOF
/*
 * $MyObject - $MyObject object
 *
 * Copyright (c) `date "+%Y"` Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


EOF
done

if [ "$ParentObject" = "GObject" ]; then
    PINCLUDE="glib-object.h"
else
    PINCLUDE=`echo ${parent_object} | sed 's/_/-/g'`.h
fi

cat >> $header_file << EOF
#ifndef __${MY_OBJECT}_H__
#define __${MY_OBJECT}_H__

#include <glib-object.h>
#include <$PINCLUDE>

G_BEGIN_DECLS


#define ${MY}_TYPE_${OBJECT}		(${my_object}_get_type ())
#define ${MY_OBJECT}(o)			(G_TYPE_CHECK_INSTANCE_CAST ((o), ${MY}_TYPE_${OBJECT}, ${MyObject}))
#define ${MY_OBJECT}_CLASS(k)		(G_TYPE_CHECK_CLASS_CAST ((k), ${MY}_TYPE_${OBJECT}, ${MyObject}Class))
#define ${MY}_IS_${OBJECT}(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), ${MY}_TYPE_${OBJECT}))
#define ${MY}_IS_${OBJECT}_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), ${MY}_TYPE_${OBJECT}))
#define ${MY_OBJECT}_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), ${MY}_TYPE_${OBJECT}, ${MyObject}Class))


typedef struct _${MyObject} ${MyObject};
typedef struct _${MyObject}Class ${MyObject}Class;

struct _${MyObject} {
	${ParentObject} __parent;

};

struct _${MyObject}Class {
	${ParentObject}Class __parent_class;

	/*
	 * Virtual Functions
	 */
	

	/*
	 * Signals
	 */
	
};


GType ${my_object}_get_type (void) G_GNUC_CONST;
GQuark  ${my_object}_error_quark (void) G_GNUC_CONST;

#define ${MY_OBJECT}_ERROR		(${my_object}_error_quark ())


${MyObject} *${my_object}_new (void);



/* Routines for emitting signals on the ${MyObject} */


G_END_DECLS

#endif /* __${MY_OBJECT}_H__ */

EOF

cat >> $code_file << EOF
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib-object.h>
#include "${header_file}"


static ${ParentObject}Class *parent_class = NULL;

enum {
	PROP_0,
	PROP_,
	${MY_OBJECT}_LAST_ARG
};

enum {
	${MY_OBJECT}_LAST_SIGNAL
};

static guint _${my_object}_signals[${MY_OBJECT}_LAST_SIGNAL] = { 0, };

static void
_${my_object}_finalize (GObject *object)
{
	${MyObject} *self = ${MY_OBJECT} (object);

	

	if (G_OBJECT_CLASS (parent_class)->finalize)
		G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
_${my_object}_set_property (GObject *object,
	      guint prop_id,
	      const GValue *value,
	      GParamSpec *pspec)
{
	${MyObject} *self = ${MY_OBJECT} (object);

	switch (prop_id) {
		case PROP_:
			
			g_object_notify (object, "");
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
_${my_object}_get_property (GObject *object,
	      guint prop_id,
	      GValue *value,
	      GParamSpec *pspec)
{
	${MyObject} *self = ${MY_OBJECT} (object);

	switch (prop_id) {
		case PROP_:
			
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
_${my_object}_class_init (${MyObject}Class *klass)
{
	GObjectClass *object_class;

	object_class = (GObjectClass *) klass;
	object_class->set_property = _${my_object}_set_property;
	object_class->get_property = _${my_object}_get_property;
	object_class->finalize = _${my_object}_finalize;

	/* Virtual functions */
	klass-> = ;

	/* Signals */
	_${my_object}_signals[] = g_signal_new ("",
						G_TYPE_FROM_CLASS (klass),
						G_SIGNAL_RUN_FIRST,
						G_STRUCT_OFFSET (${MyObject}Class, ),
						NULL, NULL,
						${my_object}_marshal_VOID__VOID,
						G_TYPE_NONE, 0);

	/* Properties */
	g_object_class_install_property (object_class,
					 PROP_,
					 g_param_spec_ ("",
					                _(""),
							_(""),
							G_PARAM_READABLE | G_PARAM_WRITABLE));


	parent_class = g_type_class_peek_parent (klass);
}

static void
_${my_object}_init (${MyObject} *self)
{
	
}

GType
${my_object}_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		static GTypeInfo info = {
			sizeof (${MyObject}Class),
			NULL,
			NULL,
			(GClassInitFunc) _${my_object}_class_init,
			NULL,
			NULL,
			sizeof (${MyObject}),
			0,
			(GInstanceInitFunc) _${my_object}_init,
		};
		/*static GInterfaceInfo iface_info = {
			(GInterfaceInitFunc) _${my_object}_iface_init,
			NULL,
			NULL
		};*/

		type = g_type_register_static (${PARENT}_TYPE_${POBJECT}, "${MyObject}", &info, 0 /*G_TYPE_FLAG_ABSTRACT*/);
		/*g_type_add_interface_static (type, _TYPE_, &iface_info);*/
	}

	return type;
}

${MyObject} *
${my_object}_new (void)
{
	${MyObject} *self;

	self = g_object_new (${MY}_TYPE_${OBJECT}, NULL);

	

	return self;
}


GQuark
${my_object}_error_quark (void)
{
	return g_quark_from_string ("${MyObject}");
}

EOF

