/*
 * BurgnerIsofsDirectoryDotdotRecord - BurgnerIsofsDirectoryDotdotRecord object
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include <glib-object.h>
#include "burgner-isofs-directory.h"
#include "burgner-isofs-impl.h"
#include "burgner-isofs-rrip-nm.h"
#include "burgner-isofs-rrip-loc.h"
#include "burgner-isofs-susp-ce.h"
#include "burgner-isofs-extent.h"
#include "burgner-isofs-utils.h"
#include "burgner-isofs-directory-dotdot-record.h"


static BurgnerIsofsDirectoryRecordClass *parent_class = NULL;

enum {
	PROP_0,
	PROP_DIR,
	PROP_M_DIR,
	PROP_WITH_RRIP,
	PROP_IS_PARENT_LINK,
	LAST_ARG
};

static void
finalize (GObject *object)
{
	BurgnerIsofsDirectoryDotdotRecord *self = BURGNER_ISOFS_DIRECTORY_DOTDOT_RECORD (object);

	if (self->dir) {
		g_object_unref (self->dir);
		self->dir = NULL;
	}

	if (self->m_dir) {
		g_object_unref (self->m_dir);
		self->m_dir = NULL;
	}

	self->with_rrip = 1;
	self->is_parent_link = 0;

	if (G_OBJECT_CLASS (parent_class)->finalize)
		G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
set_property (GObject *object,
	      guint prop_id,
	      const GValue *value,
	      GParamSpec *pspec)
{
	BurgnerIsofsDirectoryDotdotRecord *self = BURGNER_ISOFS_DIRECTORY_DOTDOT_RECORD (object);

	switch (prop_id) {
		case PROP_DIR:
			if (self->dir)
				g_object_unref (self->dir);
			self->dir = (BurgnerIsofsItem *) g_value_get_object (value);
			if (self->dir)
				self->dir = BURGNER_ISOFS_ITEM (g_object_ref (self->dir));
			g_object_notify (object, "dir");
			break;
		case PROP_M_DIR:
			if (self->m_dir)
				g_object_unref (self->m_dir);
			self->m_dir = (BurgnerIsofsItem *) g_value_get_object (value);
			if (self->m_dir)
				self->m_dir = BURGNER_ISOFS_ITEM (g_object_ref (self->m_dir));
			g_object_notify (object, "m_dir");
			break;
		case PROP_WITH_RRIP:
			self->with_rrip = g_value_get_boolean (value)?1:0;
			g_object_notify (object, "with_rrip");
			break;
		case PROP_IS_PARENT_LINK:
			self->is_parent_link = g_value_get_boolean (value)?1:0;
			g_object_notify (object, "is_parent_link");
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
get_property (GObject *object,
	      guint prop_id,
	      GValue *value,
	      GParamSpec *pspec)
{
	BurgnerIsofsDirectoryDotdotRecord *self = BURGNER_ISOFS_DIRECTORY_DOTDOT_RECORD (object);

	switch (prop_id) {
		case PROP_DIR:
			g_value_set_object (value, self->dir);
			break;
		case PROP_M_DIR:
			g_value_set_object (value, self->m_dir);
			break;
		case PROP_WITH_RRIP:
			g_value_set_boolean (value, self->with_rrip?TRUE:FALSE);
			break;
		case PROP_IS_PARENT_LINK:
			g_value_set_boolean (value, self->is_parent_link?TRUE:FALSE);
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static GSList *
_bidddr_get_susp_ent (BurgnerIsofsDirectoryRecord *r)
{
	BurgnerIsofsDirectoryDotdotRecord *rec = BURGNER_ISOFS_DIRECTORY_DOTDOT_RECORD (r);
	GSList *l = NULL;
	BurgnerIsofsItem *i;

	if (!rec->with_rrip)
		return NULL;

	i = burgner_isofs_rrip_nm_new ();
	g_object_set (i, "parent", TRUE, NULL);
	l = g_slist_prepend (l, g_slist_prepend (NULL, i));

	if (rec->is_parent_link) {
		i = burgner_isofs_rrip_pl_new ();
		g_object_set (i, "item", rec->dir, NULL);
		l = g_slist_prepend (l, g_slist_prepend (NULL, i));
	}

	return l;
}

static BurgnerIsofsItem *
_bidddr_get_ce_ent (BurgnerIsofsDirectoryRecord *r)
{
	BurgnerIsofsDirectoryDotdotRecord *rec = BURGNER_ISOFS_DIRECTORY_DOTDOT_RECORD (r);
	BurgnerIsofsItem *ce = NULL, *ca;

	g_return_val_if_fail (rec->dir != NULL, NULL);

	if (!rec->with_rrip)
		return NULL;

	ca = burgner_isofs_directory_get_continuation_area (BURGNER_ISOFS_DIRECTORY (rec->dir));
	if (ca) {
		ce = burgner_isofs_susp_ce_new ();
		g_object_set (ce, "ca", ca, NULL);
	}

	return ce;
}

static guint
_bidddr_get_base_size (BurgnerIsofsDirectoryRecord *rec)
{
	return 34;
}

static void
_bidddr_write_base (BurgnerIsofsDirectoryRecord *r,
		    BurgnerIsofsMapTable *table,
		    gpointer buf,
		    guint len_dr,
		    guint n_bytes)
{
	__BurgnerIsofsImplDirectoryRecord *impl;
	BurgnerIsofsDirectoryDotdotRecord *rec = BURGNER_ISOFS_DIRECTORY_DOTDOT_RECORD (r);
	BurgnerIsofsItem *dir;

	impl = (__BurgnerIsofsImplDirectoryRecord *) buf;
	memset (impl, 0, sizeof (*impl));
	buf += sizeof (*impl);

	burgner_isofs_set_711 (&impl->len_dr, len_dr);
	burgner_isofs_set_711 (&impl->ear_len, 0);

	dir = rec->is_parent_link ? rec->m_dir : rec->dir;
	if (dir) {
		guint32 addr = 0;
		gboolean ret;

		ret = burgner_isofs_map_table_get_addr (table, dir, &addr, NULL);
		g_return_if_fail (ret != FALSE);
		burgner_isofs_set_733 (impl->location, addr);

		if (BURGNER_IS_ISOFS_EXTENT (dir)) {
			addr = burgner_isofs_extent_get_size (BURGNER_ISOFS_EXTENT (dir));
		} else {
			guint32 n_s = 0, n_b = 0;
			burgner_isofs_item_get_size (dir, &n_s, &n_b);
			addr = (n_s << 11) + n_b;
		}
		g_return_if_fail (addr != 0);
		burgner_isofs_set_733 (impl->data_len, addr);
	} else {
		burgner_isofs_set_733 (impl->location, 0);
		burgner_isofs_set_733 (impl->data_len, 0);
	}

	impl->file_flags = __BURGNER_ISOFS_IMPL_DIRECTORY_RECORD_FLAGS_DIRECTORY;

	burgner_isofs_set_711 (&impl->file_unit_size, 0);
	burgner_isofs_set_711 (&impl->interleave_gap_size, 0);

	burgner_isofs_set_723 (impl->volume_sequence_number, 1);

	burgner_isofs_set_711 (&impl->len_fi, 1);
	*((guchar *) buf) = 1;
}

static void
class_init (BurgnerIsofsDirectoryDotdotRecordClass *klass)
{
	GObjectClass *object_class;
	BurgnerIsofsDirectoryRecordClass *dir_rec_class;

	object_class = (GObjectClass *) klass;
	object_class->set_property = set_property;
	object_class->get_property = get_property;
	object_class->finalize = finalize;

	dir_rec_class = (BurgnerIsofsDirectoryRecordClass *) klass;
	dir_rec_class->get_susp_ent = _bidddr_get_susp_ent;
	dir_rec_class->get_ce_ent = _bidddr_get_ce_ent;
	dir_rec_class->get_base_size = _bidddr_get_base_size;
	dir_rec_class->write_base = _bidddr_write_base;

	/* Properties */
	g_object_class_install_property (object_class,
					 PROP_DIR,
					 g_param_spec_object ("dir",
					            	      "dir",
							      "Directory",
							      BURGNER_TYPE_ISOFS_DIRECTORY,
							      G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_M_DIR,
					 g_param_spec_object ("m_dir",
					                      "m_dir",
							      "Moved-to Directory",
							      BURGNER_TYPE_ISOFS_DIRECTORY,
							      G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_WITH_RRIP,
					 g_param_spec_boolean ("with_rrip",
					            	       "with_rrip",
							       "With RockRidge",
							       TRUE,
							       G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_IS_PARENT_LINK,
					 g_param_spec_boolean ("is_parent_link",
					                       "is_parent_link",
							       "Is Parent Link",
							       FALSE,
							       G_PARAM_READABLE | G_PARAM_WRITABLE));


	parent_class = g_type_class_peek_parent (klass);
}

static void
init (BurgnerIsofsDirectoryDotdotRecord *self)
{
	self->dir = self->m_dir = NULL;
	self->with_rrip = 1;
	self->is_parent_link = 0;
}

GType
burgner_isofs_directory_dotdot_record_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		static GTypeInfo info = {
			sizeof (BurgnerIsofsDirectoryDotdotRecordClass),
			NULL,
			NULL,
			(GClassInitFunc) class_init,
			NULL,
			NULL,
			sizeof (BurgnerIsofsDirectoryDotdotRecord),
			0,
			(GInstanceInitFunc) init,
		};

		type = g_type_register_static (BURGNER_TYPE_ISOFS_DIRECTORY_RECORD, "BurgnerIsofsDirectoryDotdotRecord", &info, 0 /*G_TYPE_FLAG_ABSTRACT*/);
	}

	return type;
}

BurgnerIsofsItem *
burgner_isofs_directory_dotdot_record_new (void)
{
	BurgnerIsofsDirectoryDotdotRecord *self;

	self = g_object_new (BURGNER_TYPE_ISOFS_DIRECTORY_DOTDOT_RECORD, NULL);

	return BURGNER_ISOFS_ITEM (self);
}


GQuark
burgner_isofs_directory_dotdot_record_error_quark (void)
{
	return g_quark_from_string ("BurgnerIsofsDirectoryDotdotRecord");
}

