/*
 * BurgnerIsofsDirectoryRecord - BurgnerIsofsDirectoryRecord object
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifndef __BURGNER_ISOFS_DIRECTORY_RECORD_H__
#define __BURGNER_ISOFS_DIRECTORY_RECORD_H__

#include <glib-object.h>
#include <libburgnerisofs/burgner-isofs-item.h>
#include <libburgnerisofs/burgner-isofs-map-table.h>

G_BEGIN_DECLS


#define BURGNER_TYPE_ISOFS_DIRECTORY_RECORD		(burgner_isofs_directory_record_get_type ())
#define BURGNER_ISOFS_DIRECTORY_RECORD(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), BURGNER_TYPE_ISOFS_DIRECTORY_RECORD, BurgnerIsofsDirectoryRecord))
#define BURGNER_ISOFS_DIRECTORY_RECORD_CLASS(k)		(G_TYPE_CHECK_CLASS_CAST ((k), BURGNER_TYPE_ISOFS_DIRECTORY_RECORD, BurgnerIsofsDirectoryRecordClass))
#define BURGNER_IS_ISOFS_DIRECTORY_RECORD(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), BURGNER_TYPE_ISOFS_DIRECTORY_RECORD))
#define BURGNER_IS_ISOFS_DIRECTORY_RECORD_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), BURGNER_TYPE_ISOFS_DIRECTORY_RECORD))
#define BURGNER_ISOFS_DIRECTORY_RECORD_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), BURGNER_TYPE_ISOFS_DIRECTORY_RECORD, BurgnerIsofsDirectoryRecordClass))


typedef struct _BurgnerIsofsDirectoryRecord BurgnerIsofsDirectoryRecord;
typedef struct _BurgnerIsofsDirectoryRecordClass BurgnerIsofsDirectoryRecordClass;

struct _BurgnerIsofsDirectoryRecord {
	BurgnerIsofsItem __parent;

	/* Base */
	guint base_b;
	GSList *suf_ent;
	GSList *suf_ent_b;
	guint suf_ent_size;

	/* Inter */
	union {
		GSList *inter_ent;
		BurgnerIsofsItem *ce;
	} inter;
	GSList *inter_ent_b;
	guint inter_ent_size;

	/* Bonus */
	GSList *bonus_ent;
	GSList *bonus_ent_b;
	guint bonus_ent_size;

	/* Ca */
	GSList *ca_ent;

	/* Frigo */
	GSList *ent;
	guint ent_size;
	BurgnerIsofsItem *ce;

	guint frigo_fetched : 1;
	guint inter_is_ce : 1;
	guint inter_is_our_ce : 1;
	guint with_susp : 1;
	guint prepared : 1;
};

struct _BurgnerIsofsDirectoryRecordClass {
	BurgnerIsofsItemClass __parent_class;

	/*
	 * Virtual Functions
	 */
	GSList *(* get_susp_suf_ent)(BurgnerIsofsDirectoryRecord *rec);
	GSList *(* get_susp_ent)(BurgnerIsofsDirectoryRecord *rec);
	BurgnerIsofsItem *(* get_ce_ent)(BurgnerIsofsDirectoryRecord *rec);
	guint (* get_base_size)(BurgnerIsofsDirectoryRecord *rec);
	void (* write_base)(BurgnerIsofsDirectoryRecord *rec,
			    BurgnerIsofsMapTable *table,
			    gpointer buf,
		            guint len_dr,
			    guint n_bytes);

	const gchar *(* get_identifier)(BurgnerIsofsDirectoryRecord *rec);

	guint (* bonus_size)(BurgnerIsofsDirectoryRecord *rec,
			     guint bonus_size);
	gboolean (* need_continuation_area)(BurgnerIsofsDirectoryRecord *rec);
	void (* set_continuation_area)(BurgnerIsofsDirectoryRecord *rec,
				       BurgnerIsofsItem *ca);

};


GType burgner_isofs_directory_record_get_type (void) G_GNUC_CONST;
GQuark  burgner_isofs_directory_record_error_quark (void);

#define BURGNER_ISOFS_DIRECTORY_RECORD_ERROR		(burgner_isofs_directory_record_error_quark ())

GSList *burgner_isofs_directory_record_get_susp_suf_ent (BurgnerIsofsDirectoryRecord *rec);
GSList *burgner_isofs_directory_record_get_susp_ent (BurgnerIsofsDirectoryRecord *rec);
BurgnerIsofsItem *burgner_isofs_directory_record_get_ce_ent (BurgnerIsofsDirectoryRecord *rec);
guint burgner_isofs_directory_record_get_base_size (BurgnerIsofsDirectoryRecord *rec);
void burgner_isofs_directory_record_write_base (BurgnerIsofsDirectoryRecord *rec, 
		    				BurgnerIsofsMapTable *table,
						gpointer buf,
		                                guint len_dr,
		    				guint n_bytes);

const gchar *burgner_isofs_directory_record_get_identifier (BurgnerIsofsDirectoryRecord *rec);

guint burgner_isofs_directory_record_bonus_size (BurgnerIsofsDirectoryRecord *rec,
		                                 guint bonus_size);
gboolean burgner_isofs_directory_record_need_continuation_area (BurgnerIsofsDirectoryRecord *rec);
void burgner_isofs_directory_record_set_continuation_area (BurgnerIsofsDirectoryRecord *rec,
			                                   BurgnerIsofsItem *ca);

G_END_DECLS

#endif /* __BURGNER_ISOFS_DIRECTORY_RECORD_H__ */

