/*
 * BurgnerIsofsDirectory - BurgnerIsofsDirectory object
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifndef __BURGNER_ISOFS_DIRECTORY_H__
#define __BURGNER_ISOFS_DIRECTORY_H__

#include <glib-object.h>
#include <libburgnerisofs/burgner-isofs-item.h>
#include <libburgnerisofs/burgner-isofs-directory-record.h>
#include <libburgnerisofs/burgner-isofs-file-info.h>

G_BEGIN_DECLS


#define BURGNER_TYPE_ISOFS_DIRECTORY		(burgner_isofs_directory_get_type ())
#define BURGNER_ISOFS_DIRECTORY(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), BURGNER_TYPE_ISOFS_DIRECTORY, BurgnerIsofsDirectory))
#define BURGNER_ISOFS_DIRECTORY_CLASS(k)	(G_TYPE_CHECK_CLASS_CAST ((k), BURGNER_TYPE_ISOFS_DIRECTORY, BurgnerIsofsDirectoryClass))
#define BURGNER_IS_ISOFS_DIRECTORY(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), BURGNER_TYPE_ISOFS_DIRECTORY))
#define BURGNER_IS_ISOFS_DIRECTORY_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), BURGNER_TYPE_ISOFS_DIRECTORY))
#define BURGNER_ISOFS_DIRECTORY_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), BURGNER_TYPE_ISOFS_DIRECTORY, BurgnerIsofsDirectoryClass))


typedef struct _BurgnerIsofsDirectory BurgnerIsofsDirectory;
typedef struct _BurgnerIsofsDirectoryClass BurgnerIsofsDirectoryClass;

struct _BurgnerIsofsDirectory {
	BurgnerIsofsItem __parent;

	BurgnerIsofsFileInfo *info;

	GSList *records;
	GSList *ca;
	BurgnerIsofsItem *ce;

	GSList *records_bytes;

	guint with_rrip : 1;
	guint fixed : 1;
};

struct _BurgnerIsofsDirectoryClass {
	BurgnerIsofsItemClass __parent_class;

	/*
	 * Virtual Functions
	 */
	gint (* records_cmp)(BurgnerIsofsDirectory *dir, BurgnerIsofsDirectoryRecord *a, BurgnerIsofsDirectoryRecord *b);
/*	BurgnerIsofsItem *(* get_continuation_area)(BurgnerIsofsDirectory *dir);*/
};


GType burgner_isofs_directory_get_type (void) G_GNUC_CONST;
GQuark burgner_isofs_directory_error_quark (void);

#define BURGNER_ISOFS_DIRECTORY_ERROR		(burgner_isofs_directory_error_quark ())


BurgnerIsofsItem *burgner_isofs_directory_new (void);

void burgner_isofs_directory_append_record (BurgnerIsofsDirectory *dir,
					    BurgnerIsofsItem *rec);
void burgner_isofs_directory_append_records_valist (BurgnerIsofsDirectory *dir,
						    BurgnerIsofsItem *first,
						    va_list args);
void burgner_isofs_directory_append_records (BurgnerIsofsDirectory *dir,
					     BurgnerIsofsItem *first,
					     ...);

						    

BurgnerIsofsItem *burgner_isofs_directory_get_continuation_area (BurgnerIsofsDirectory *dir);

G_END_DECLS

#endif /* __BURGNER_ISOFS_DIRECTORY_H__ */

