/*
 * BurgnerIsofsEltoritoBootCatalogInitialEntry - BurgnerIsofsEltoritoBootCatalogInitialEntry object
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "burgner-isofs-i18n.h"

#include <string.h>
#include <glib-object.h>
#include "burgner-isofs-eltorito-utils.h"
#include "burgner-isofs-eltorito-impl.h"
#include "burgner-isofs-eltorito-boot-catalog-initial-entry.h"


static BurgnerIsofsEltoritoBootCatalogEntryClass *parent_class = NULL;

enum {
	PROP_0,
	PROP_IMG,
	PROP_MEDIA_TYPE,
	PROP_LOAD_SEGMENT,
	PROP_SYSTEM_TYPE,
	PROP_BOOTABLE,
	LAST_ARG
};

static void
burgner_isofs_eltorito_boot_catalog_initial_entry_finalize (GObject *object)
{
	BurgnerIsofsEltoritoBootCatalogInitialEntry *self = BURGNER_ISOFS_ELTORITO_BOOT_CATALOG_INITIAL_ENTRY (object);

	if (self->img) {
		g_object_unref (self->img);
		self->img = NULL;
	}

	self->media_type = BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_NO_EMULATION;
	self->load_segment = 0x7c0;
	self->system_type = 0;
	self->bootable = 1;

	if (G_OBJECT_CLASS (parent_class)->finalize)
		G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
burgner_isofs_eltorito_boot_catalog_initial_entry_set_property (GObject *object,
	      guint prop_id,
	      const GValue *value,
	      GParamSpec *pspec)
{
	BurgnerIsofsEltoritoBootCatalogInitialEntry *self = BURGNER_ISOFS_ELTORITO_BOOT_CATALOG_INITIAL_ENTRY (object);

	switch (prop_id) {
		case PROP_IMG:
			if (self->img)
				g_object_unref (self->img);
			self->img = (BurgnerIsofsItem *) g_value_get_object (value);
			if (self->img)
				self->img = BURGNER_ISOFS_ITEM (g_object_ref (self->img));
			g_object_notify (object, "img");
			break;
		case PROP_MEDIA_TYPE:
			self->media_type = g_value_get_enum (value);
			g_object_notify (object, "media_type");
			break;
		case PROP_LOAD_SEGMENT:
			self->load_segment = g_value_get_uint (value);
			g_object_notify (object, "load_segment");
			break;
		case PROP_SYSTEM_TYPE:
			self->system_type = g_value_get_uint (value);
			g_object_notify (object, "system_type");
			break;
		case PROP_BOOTABLE:
			self->bootable = g_value_get_boolean (value)?1:0;
			g_object_notify (object, "bootable");
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
burgner_isofs_eltorito_boot_catalog_initial_entry_get_property (GObject *object,
	      guint prop_id,
	      GValue *value,
	      GParamSpec *pspec)
{
	BurgnerIsofsEltoritoBootCatalogInitialEntry *self = BURGNER_ISOFS_ELTORITO_BOOT_CATALOG_INITIAL_ENTRY (object);

	switch (prop_id) {
		case PROP_IMG:
			g_value_set_object (value, self->img);
			break;
		case PROP_MEDIA_TYPE:
			g_value_set_enum (value, self->media_type);
			break;
		case PROP_LOAD_SEGMENT:
			g_value_set_uint (value, self->load_segment);
			break;
		case PROP_SYSTEM_TYPE:
			g_value_set_uint (value, self->system_type);
			break;
		case PROP_BOOTABLE:
			g_value_set_boolean (value, self->bootable?TRUE:FALSE);
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static guint8
_biebcie_media_type_to_impl (BurgnerIsofsEltoritoMediaType t)
{
	guint8 r;

	switch (t) {
		case BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_NO_EMULATION:
			r = __BURGNER_ISOFS_ELTORITO_IMPL_INITIAL_ENTRY_BOOT_MEDIA_TYPE_NONE;
			break;
		case BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_1_2_MEG_DISKETTE:
			r = __BURGNER_ISOFS_ELTORITO_IMPL_INITIAL_ENTRY_BOOT_MEDIA_TYPE_1_2_MEG_DISKETTE;
			break;
		case BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_1_44_MEG_DISKETTE:
			r = __BURGNER_ISOFS_ELTORITO_IMPL_INITIAL_ENTRY_BOOT_MEDIA_TYPE_1_44_MEG_DISKETTE;
			break;
		case BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_2_88_MEG_DISKETTE:
			r = __BURGNER_ISOFS_ELTORITO_IMPL_INITIAL_ENTRY_BOOT_MEDIA_TYPE_2_88_MEG_DISKETTE;
			break;
		case BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_HARD_DISK:
			r = __BURGNER_ISOFS_ELTORITO_IMPL_INITIAL_ENTRY_BOOT_MEDIA_TYPE_HARD_DISK;
			break;
		default:
			g_return_val_if_reached (__BURGNER_ISOFS_ELTORITO_IMPL_INITIAL_ENTRY_BOOT_MEDIA_TYPE_NONE);
			break;
	}

	return r;
}

static void
_biebcie_write_sync (BurgnerIsofsItem *item,
		     BurgnerIsofsMapTable *table,
		     gpointer buf, guint32 size)
{
	__BurgnerIsofsEltoritoImplInitialEntry *impl;
	BurgnerIsofsEltoritoBootCatalogInitialEntry *self = BURGNER_ISOFS_ELTORITO_BOOT_CATALOG_INITIAL_ENTRY (item);
	gpointer buf_end = buf + size;

	g_return_if_fail (self->img != NULL);

	impl = (__BurgnerIsofsEltoritoImplInitialEntry *) buf;
	memset (impl, 0, 32);
	buf += 32;

	burgner_isofs_eltorito_set_byte (&impl->boot_indicator, self->bootable ? 0x88 : 0);
	burgner_isofs_eltorito_set_byte (&impl->boot_media_type, _biebcie_media_type_to_impl (self->media_type));

	burgner_isofs_eltorito_set_word (impl->load_segment, self->load_segment);
	burgner_isofs_eltorito_set_byte (&impl->system_type, self->system_type);

	{
		guint32 n_s, n_b;
		n_s = n_b = 0;
		burgner_isofs_item_get_size (self->img, &n_s, &n_b);
		g_return_if_fail ((n_b & 0x1ff) == 0);
		n_b += n_s<<11;
		n_b >>= 9;
		g_return_if_fail ((n_b & ~0xffff) == 0);
		burgner_isofs_eltorito_set_word (impl->sector_count, n_b);
	}

	{
		guint32 s_c = 0;
		gboolean ret;
		ret = burgner_isofs_map_table_get_addr (table, self->img, &s_c, NULL);
		g_return_if_fail (ret != FALSE);
		burgner_isofs_eltorito_set_d_word (impl->load_rba, s_c);
	}

#if 0
	burgner_isofs_writer_proxy_write (proxy, (guchar*) &impl, 32);
#endif

	g_assert (buf == buf_end);
}

static void
burgner_isofs_eltorito_boot_catalog_initial_entry_class_init (BurgnerIsofsEltoritoBootCatalogInitialEntryClass *klass)
{
	GObjectClass *object_class;
	BurgnerIsofsItemClass *item_class;

	object_class = (GObjectClass *) klass;
	object_class->set_property = burgner_isofs_eltorito_boot_catalog_initial_entry_set_property;
	object_class->get_property = burgner_isofs_eltorito_boot_catalog_initial_entry_get_property;
	object_class->finalize = burgner_isofs_eltorito_boot_catalog_initial_entry_finalize;

	item_class = (BurgnerIsofsItemClass *) klass;
	item_class->write_sync = _biebcie_write_sync;

	/* Properties */
	g_object_class_install_property (object_class,
					 PROP_IMG,
					 g_param_spec_object ("img",
					 		      _("img"),
							      _("Image"),
							      BURGNER_TYPE_ISOFS_ITEM,
							      G_PARAM_READABLE |
							      G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_MEDIA_TYPE,
					 g_param_spec_enum ("media_type",
					            	    _("media_type"),
							    _("Media Type"),
							    BURGNER_TYPE_ISOFS_ELTORITO_MEDIA_TYPE,
							    BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_NO_EMULATION,
							    G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_LOAD_SEGMENT,
					 g_param_spec_uint ("load_segment",
					            	    _("load_segment"),
							    _("Load Segment"),
							    0,
							    0xffff,
							    0x7c0,
							    G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_SYSTEM_TYPE,
					 g_param_spec_uint ("system_type",
					            	    _("system_type"),
							    _("Sytem Type"),
							    0,
							    0xff,
							    0,
							    G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_BOOTABLE,
					 g_param_spec_boolean ("bootable",
					            	       _("bootable"),
							       _("Bootable"),
							       TRUE,
							       G_PARAM_READABLE |
							       G_PARAM_WRITABLE));


	parent_class = g_type_class_peek_parent (klass);
}

static void
burgner_isofs_eltorito_boot_catalog_initial_entry_init (BurgnerIsofsEltoritoBootCatalogInitialEntry *self)
{
	self->img = NULL;
	self->media_type = BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_NO_EMULATION;
	self->load_segment = 0x7c0;
	self->system_type = 0;
	self->bootable = 1;
}

GType
burgner_isofs_eltorito_boot_catalog_initial_entry_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		static GTypeInfo info = {
			sizeof (BurgnerIsofsEltoritoBootCatalogInitialEntryClass),
			NULL,
			NULL,
			(GClassInitFunc) burgner_isofs_eltorito_boot_catalog_initial_entry_class_init,
			NULL,
			NULL,
			sizeof (BurgnerIsofsEltoritoBootCatalogInitialEntry),
			0,
			(GInstanceInitFunc) burgner_isofs_eltorito_boot_catalog_initial_entry_init,
		};

		type = g_type_register_static (BURGNER_TYPE_ISOFS_ELTORITO_BOOT_CATALOG_ENTRY, "BurgnerIsofsEltoritoBootCatalogInitialEntry", &info, 0 /*G_TYPE_FLAG_ABSTRACT*/);
	}

	return type;
}

BurgnerIsofsItem *
burgner_isofs_eltorito_boot_catalog_initial_entry_new (void)
{
	BurgnerIsofsEltoritoBootCatalogInitialEntry *self;

	self = g_object_new (BURGNER_TYPE_ISOFS_ELTORITO_BOOT_CATALOG_INITIAL_ENTRY, NULL);


	return BURGNER_ISOFS_ITEM (self);
}


GQuark
burgner_isofs_eltorito_boot_catalog_initial_entry_error_quark (void)
{
	return g_quark_from_string ("BurgnerIsofsEltoritoBootCatalogInitialEntry");
}

