/*
 * BurgnerIsofsEltoritoBootCatalogValidationEntry - BurgnerIsofsEltoritoBootCatalogValidationEntry object
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include <glib-object.h>
#include "burgner-isofs-eltorito-impl.h"
#include "burgner-isofs-eltorito-utils.h"
#include "burgner-isofs-eltorito-boot-catalog-validation-entry.h"


static BurgnerIsofsEltoritoBootCatalogEntryClass *parent_class = NULL;

enum {
	PROP_0,
	PROP_PLATFORM_ID,
	PROP_MANUFACTURER,
	LAST_ARG
};

static void
burgner_isofs_eltorito_boot_catalog_validation_entry_finalize (GObject *object)
{
	BurgnerIsofsEltoritoBootCatalogValidationEntry *self = BURGNER_ISOFS_ELTORITO_BOOT_CATALOG_VALIDATION_ENTRY (object);

	self->platform_id = BURGNER_ISOFS_ELTORITO_PLATFORM_ID_80x86;

	if (self->manufacturer) {
		g_free (self->manufacturer);
		self->manufacturer = NULL;
	}

	if (G_OBJECT_CLASS (parent_class)->finalize)
		G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
burgner_isofs_eltorito_boot_catalog_validation_entry_set_property (GObject *object,
	      guint prop_id,
	      const GValue *value,
	      GParamSpec *pspec)
{
	BurgnerIsofsEltoritoBootCatalogValidationEntry *self = BURGNER_ISOFS_ELTORITO_BOOT_CATALOG_VALIDATION_ENTRY (object);

	switch (prop_id) {
		case PROP_PLATFORM_ID:
			self->platform_id = g_value_get_enum (value);
			g_object_notify (object, "platform_id");
			break;
		case PROP_MANUFACTURER:
			if (self->manufacturer)
				g_free (self->manufacturer);
			self->manufacturer = g_value_dup_string (value);
			g_object_notify (object, "manufacturer");
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
burgner_isofs_eltorito_boot_catalog_validation_entry_get_property (GObject *object,
	      guint prop_id,
	      GValue *value,
	      GParamSpec *pspec)
{
	BurgnerIsofsEltoritoBootCatalogValidationEntry *self = BURGNER_ISOFS_ELTORITO_BOOT_CATALOG_VALIDATION_ENTRY (object);

	switch (prop_id) {
		case PROP_PLATFORM_ID:
			g_value_set_enum (value, self->platform_id);
			break;
		case PROP_MANUFACTURER:
			g_value_set_string (value, self->manufacturer);
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static guint8
_biebcve_platform_id_to_impl (BurgnerIsofsEltoritoPlatformId id)
{
	guint8 r;

	switch (id) {
		case BURGNER_ISOFS_ELTORITO_PLATFORM_ID_80x86:
			r = __BURGNER_ISOFS_ELTORITO_IMPL_SECTION_HEADER_PLATFORM_ID_80x86;
			break;
		case BURGNER_ISOFS_ELTORITO_PLATFORM_ID_POWER_PC:
			r = __BURGNER_ISOFS_ELTORITO_IMPL_SECTION_HEADER_PLATFORM_ID_POWER_PC;
			break;
		case BURGNER_ISOFS_ELTORITO_PLATFORM_ID_MAC:
			r = __BURGNER_ISOFS_ELTORITO_IMPL_SECTION_HEADER_PLATFORM_ID_MAC;
			break;
		default:
			g_return_val_if_reached (__BURGNER_ISOFS_ELTORITO_IMPL_SECTION_HEADER_PLATFORM_ID_80x86);
			break;
	}

	return r;
}

static void
_biebcve_write_sync (BurgnerIsofsItem *item,
		     BurgnerIsofsMapTable *table,
		     gpointer buf, guint32 size)
{
	__BurgnerIsofsEltoritoImplValidationEntry *impl;
	BurgnerIsofsEltoritoBootCatalogValidationEntry *self = BURGNER_ISOFS_ELTORITO_BOOT_CATALOG_VALIDATION_ENTRY (item);

	g_return_if_fail (size == 32);

	impl = (__BurgnerIsofsEltoritoImplValidationEntry *) buf;
	memset (impl,  0, 32);

	burgner_isofs_eltorito_set_byte (&impl->header_id, 1);
	burgner_isofs_eltorito_set_byte (&impl->platform_id, _biebcve_platform_id_to_impl (self->platform_id));

	if (self->manufacturer) {
		guint l = strlen (self->manufacturer);
		(void) memcpy (impl->manufacturer, self->manufacturer, MIN (l, 24));
	}

	burgner_isofs_eltorito_set_byte (&impl->key_55, 0x55);
	burgner_isofs_eltorito_set_byte (&impl->key_AA, 0xaa);

	{
		gint16 checksum = 0;
		guint i;
		for (i = 0; i < 32; i += 2) {
			checksum += ((guchar*) &impl)[i];
			checksum += ((guchar*) &impl)[i+1] * 256;
		}

		checksum = -checksum;
		burgner_isofs_eltorito_set_word (impl->checksum, *((guint16 *) &checksum));
	}
}

static void
burgner_isofs_eltorito_boot_catalog_validation_entry_class_init (BurgnerIsofsEltoritoBootCatalogValidationEntryClass *klass)
{
	GObjectClass *object_class;
	BurgnerIsofsItemClass *item_class;

	object_class = (GObjectClass *) klass;
	object_class->set_property = burgner_isofs_eltorito_boot_catalog_validation_entry_set_property;
	object_class->get_property = burgner_isofs_eltorito_boot_catalog_validation_entry_get_property;
	object_class->finalize = burgner_isofs_eltorito_boot_catalog_validation_entry_finalize;

	item_class = (BurgnerIsofsItemClass *) klass;
	item_class->write_sync = _biebcve_write_sync;

	/* Properties */
	g_object_class_install_property (object_class,
					 PROP_PLATFORM_ID,
					 g_param_spec_enum ("platform_id",
					            	    "platform_id",
							    "Platform Id",
							    BURGNER_TYPE_ISOFS_ELTORITO_PLATFORM_ID,
							    BURGNER_ISOFS_ELTORITO_PLATFORM_ID_80x86,
							    G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_MANUFACTURER,
					 g_param_spec_string ("manufacturer",
					            	      "manufacturer",
							      "Manufacturer",
							      NULL,
							      G_PARAM_READABLE | G_PARAM_WRITABLE));


	parent_class = g_type_class_peek_parent (klass);
}

static void
burgner_isofs_eltorito_boot_catalog_validation_entry_init (BurgnerIsofsEltoritoBootCatalogValidationEntry *self)
{
	self->platform_id = BURGNER_ISOFS_ELTORITO_PLATFORM_ID_80x86;
	self->manufacturer = NULL;
}

GType
burgner_isofs_eltorito_boot_catalog_validation_entry_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		static GTypeInfo info = {
			sizeof (BurgnerIsofsEltoritoBootCatalogValidationEntryClass),
			NULL,
			NULL,
			(GClassInitFunc) burgner_isofs_eltorito_boot_catalog_validation_entry_class_init,
			NULL,
			NULL,
			sizeof (BurgnerIsofsEltoritoBootCatalogValidationEntry),
			0,
			(GInstanceInitFunc) burgner_isofs_eltorito_boot_catalog_validation_entry_init,
		};

		type = g_type_register_static (BURGNER_TYPE_ISOFS_ELTORITO_BOOT_CATALOG_ENTRY, "BurgnerIsofsEltoritoBootCatalogValidationEntry", &info, 0 /*G_TYPE_FLAG_ABSTRACT*/);
	}

	return type;
}

BurgnerIsofsItem *
burgner_isofs_eltorito_boot_catalog_validation_entry_new (void)
{
	BurgnerIsofsEltoritoBootCatalogValidationEntry *self;

	self = g_object_new (BURGNER_TYPE_ISOFS_ELTORITO_BOOT_CATALOG_VALIDATION_ENTRY, NULL);


	return BURGNER_ISOFS_ITEM (self);
}


GQuark
burgner_isofs_eltorito_boot_catalog_validation_entry_error_quark (void)
{
	return g_quark_from_string ("BurgnerIsofsEltoritoBootCatalogValidationEntry");
}

