/*
 * BurgnerIsofsEltoritoBootRecord - BurgnerIsofsEltoritoBootRecord object
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include <glib-object.h>
#include "burgner-isofs-utils.h"
#include "burgner-isofs-impl.h"
#include "burgner-isofs-volume-descriptor-utils.h"
#include "burgner-isofs-eltorito-utils.h"
#include "burgner-isofs-eltorito-boot-record.h"


static BurgnerIsofsVolumeDescriptorClass *parent_class = NULL;

enum {
	PROP_0,
	PROP_BOOT_CATALOG,
	LAST_ARG
};

static void
burgner_isofs_eltorito_boot_record_finalize (GObject *object)
{
	BurgnerIsofsEltoritoBootRecord *self = BURGNER_ISOFS_ELTORITO_BOOT_RECORD (object);

	if (self->bc) {
		g_object_unref (self->bc);
		self->bc = NULL;
	}

	if (G_OBJECT_CLASS (parent_class)->finalize)
		G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
burgner_isofs_eltorito_boot_record_set_property (GObject *object,
	      guint prop_id,
	      const GValue *value,
	      GParamSpec *pspec)
{
	BurgnerIsofsEltoritoBootRecord *self = BURGNER_ISOFS_ELTORITO_BOOT_RECORD (object);

	switch (prop_id) {
		case PROP_BOOT_CATALOG:
			if (self->bc)
				g_object_unref (self->bc);
			self->bc = (BurgnerIsofsItem *) g_value_get_object (value);
			self->bc = BURGNER_ISOFS_ITEM (g_object_ref (self->bc));
			g_object_notify (object, "boot_catalog");
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
burgner_isofs_eltorito_boot_record_get_property (GObject *object,
	      guint prop_id,
	      GValue *value,
	      GParamSpec *pspec)
{
	BurgnerIsofsEltoritoBootRecord *self = BURGNER_ISOFS_ELTORITO_BOOT_RECORD (object);

	switch (prop_id) {
		case PROP_BOOT_CATALOG:
			g_value_set_object (value, self->bc);
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
_biebr_write_sync (BurgnerIsofsItem *item,
		   BurgnerIsofsMapTable *table,
		   gpointer buf, guint32 size)
{
	__BurgnerIsofsImplEltoritoBootRecord *impl;
	BurgnerIsofsEltoritoBootRecord *self = BURGNER_ISOFS_ELTORITO_BOOT_RECORD (item);
	gboolean ret;
	guint32 bc_addr;

	g_return_if_fail (self->bc != NULL);
	g_return_if_fail (size == 2048);

	impl = (__BurgnerIsofsImplEltoritoBootRecord *) buf;
	memset (impl, 0, sizeof (*impl));

	burgner_isofs_set_711 (&impl->header.type, BURGNER_ISOFS_VOLUME_DESCRIPTOR_TYPE_BOOT_RECORD);
	impl->header.std_id[0] = 'C';
	impl->header.std_id[1] = 'D';
	impl->header.std_id[2] = '0';
	impl->header.std_id[3] = '0';
	impl->header.std_id[4] = '1';
	burgner_isofs_set_711 (&impl->header.version, 1);

	memcpy (impl->boot_system_id, "EL TORITO SPECIFICATION", 23);

	bc_addr = 0;
	ret = burgner_isofs_map_table_get_addr (table, self->bc, &bc_addr, NULL);
	g_return_if_fail (ret != FALSE);
	burgner_isofs_eltorito_set_d_word (impl->boot_catalog_addr, bc_addr);
}

static void
burgner_isofs_eltorito_boot_record_class_init (BurgnerIsofsEltoritoBootRecordClass *klass)
{
	GObjectClass *object_class;
	BurgnerIsofsItemClass *item_class;

	object_class = (GObjectClass *) klass;
	object_class->set_property = burgner_isofs_eltorito_boot_record_set_property;
	object_class->get_property = burgner_isofs_eltorito_boot_record_get_property;
	object_class->finalize = burgner_isofs_eltorito_boot_record_finalize;

	item_class = (BurgnerIsofsItemClass *) klass;
	item_class->write_sync = _biebr_write_sync;

	/* Properties */
	g_object_class_install_property (object_class,
					 PROP_BOOT_CATALOG,
					 g_param_spec_object ("boot_catalog",
					            	      "boot_catalog",
							      "Boot Catalog",
							      BURGNER_TYPE_ISOFS_ITEM,
							      G_PARAM_READABLE | G_PARAM_WRITABLE));


	parent_class = g_type_class_peek_parent (klass);
}

static void
burgner_isofs_eltorito_boot_record_init (BurgnerIsofsEltoritoBootRecord *self)
{
	self->bc = NULL;
}

GType
burgner_isofs_eltorito_boot_record_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		static GTypeInfo info = {
			sizeof (BurgnerIsofsEltoritoBootRecordClass),
			NULL,
			NULL,
			(GClassInitFunc) burgner_isofs_eltorito_boot_record_class_init,
			NULL,
			NULL,
			sizeof (BurgnerIsofsEltoritoBootRecord),
			0,
			(GInstanceInitFunc) burgner_isofs_eltorito_boot_record_init,
		};

		type = g_type_register_static (BURGNER_TYPE_ISOFS_VOLUME_DESCRIPTOR, "BurgnerIsofsEltoritoBootRecord", &info, 0);
	}

	return type;
}

BurgnerIsofsItem *
burgner_isofs_eltorito_boot_record_new (void)
{
	BurgnerIsofsEltoritoBootRecord *self;

	self = g_object_new (BURGNER_TYPE_ISOFS_ELTORITO_BOOT_RECORD, NULL);


	return BURGNER_ISOFS_ITEM (self);
}


GQuark
burgner_isofs_eltorito_boot_record_error_quark (void)
{
	return g_quark_from_string ("BurgnerIsofsEltoritoBootRecord");
}

