/*
 * BurgnerIsofsEltoritoBuilder - BurgnerIsofsEltoritoBuilder object
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "burgner-isofs-i18n.h"

#include <glib-object.h>
#include "burgner-isofs-file.h"
#include "burgner-isofs-item-g-list.h"
#include "burgner-isofs-eltorito-builder.h"
#include "burgner-isofs-eltorito-boot-catalog.h"
#include "burgner-isofs-eltorito-boot-catalog-validation-entry.h"
#include "burgner-isofs-eltorito-boot-catalog-initial-entry.h"
#include "burgner-isofs-eltorito-boot-record.h"


static BurgnerIsofsTreeBuilderClass *parent_class = NULL;

enum {
	PROP_0,
	PROP_MANUFACTURER,
	PROP_INFO,
	PROP_PLATFORM_ID,
	PROP_MEDIA_TYPE,
	PROP_LOAD_SEGMENT,
	PROP_SYSTEM_TYPE,
	PROP_BOOTABLE,
	PROP_IS_IN_TREE,
	LAST_ARG
};

static void
burgner_isofs_eltorito_builder_finalize (GObject *object)
{
	BurgnerIsofsEltoritoBuilder *self = BURGNER_ISOFS_ELTORITO_BUILDER (object);

	if (self->manufacturer) {
		g_free (self->manufacturer);
		self->manufacturer = NULL;
	}

	if (self->info) {
		burgner_isofs_file_info_unref (self->info);
		self->info = NULL;
	}

	self->platform_id = BURGNER_ISOFS_ELTORITO_PLATFORM_ID_80x86;
	self->media_type = BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_NO_EMULATION;

	if (self->file) {
		g_object_unref (self->file);
		self->file = NULL;
	}

	if (self->vd) {
		burgner_isofs_item_g_list_unref (self->vd);
		self->vd = NULL;
	}

	if (self->tree) {
		burgner_isofs_item_g_list_unref (self->tree);
		self->tree = NULL;
	}

	self->load_segment = 0x7c0;
	self->system_type = 0;
	self->bootable = 1;
	self->is_in_tree = 0;

	if (G_OBJECT_CLASS (parent_class)->finalize)
		G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
burgner_isofs_eltorito_builder_set_property (GObject *object,
	      guint prop_id,
	      const GValue *value,
	      GParamSpec *pspec)
{
	BurgnerIsofsEltoritoBuilder *self = BURGNER_ISOFS_ELTORITO_BUILDER (object);

	switch (prop_id) {
		case PROP_MANUFACTURER:
			if (self->manufacturer)
				g_free (self->manufacturer);
			self->manufacturer = (gchar *) g_value_get_string (value);
			g_object_notify (object, "manufacturer");
			break;
		case PROP_INFO:
			if (self->info)
				burgner_isofs_file_info_unref (self->info);
			self->info = g_value_get_pointer (value);
			if (self->info)
				self->info = burgner_isofs_file_info_ref (self->info);
			g_object_notify (object, "info");
			break;
		case PROP_PLATFORM_ID:
			self->platform_id = g_value_get_enum (value);
			g_object_notify (object, "platform_id");
			break;
		case PROP_MEDIA_TYPE:
			self->media_type = g_value_get_enum (value);
			g_object_notify (object, "media_type");
			break;
		case PROP_LOAD_SEGMENT:
			self->load_segment = g_value_get_uint (value);
			g_object_notify (object, "load_segment");
			break;
		case PROP_SYSTEM_TYPE:
			self->system_type = g_value_get_uint (value);
			g_object_notify (object, "system_type");
			break;
		case PROP_BOOTABLE:
			self->bootable = g_value_get_boolean (value)?1:0;
			g_object_notify (object, "bootable");
			break;
		case PROP_IS_IN_TREE:
			self->is_in_tree = g_value_get_boolean (value)?1:0;
			g_object_notify (object, "is_in_tree");
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
burgner_isofs_eltorito_builder_get_property (GObject *object,
	      guint prop_id,
	      GValue *value,
	      GParamSpec *pspec)
{
	BurgnerIsofsEltoritoBuilder *self = BURGNER_ISOFS_ELTORITO_BUILDER (object);

	switch (prop_id) {
		case PROP_MANUFACTURER:
			g_value_set_string (value, self->manufacturer);
			break;
		case PROP_INFO:
			g_value_set_pointer (value, self->info);
			break;
		case PROP_PLATFORM_ID:
			g_value_set_enum (value, self->platform_id);
			break;
		case PROP_MEDIA_TYPE:
			g_value_set_enum (value, self->media_type);
			break;
		case PROP_LOAD_SEGMENT:
			g_value_set_uint (value, self->load_segment);
			break;
		case PROP_SYSTEM_TYPE:
			g_value_set_uint (value, self->system_type);
			break;
		case PROP_BOOTABLE:
			g_value_set_boolean (value, self->bootable?TRUE:FALSE);
			break;
		case PROP_IS_IN_TREE:
			g_value_set_boolean (value, self->is_in_tree?TRUE:FALSE);
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
_bieb_begin (BurgnerIsofsTreeBuilder *tb,
	     GNode *root,
	     BurgnerIsofsItem *last_item)
{
	BurgnerIsofsEltoritoBuilder *self = BURGNER_ISOFS_ELTORITO_BUILDER (tb);

	g_return_if_fail (self->file == NULL);
	g_return_if_fail (self->info != NULL);

	if (!self->is_in_tree) {
		self->file = burgner_isofs_file_new ();
		g_object_set (self->file,
			      "info", self->info,
			      NULL);
		self->tree = g_list_prepend (self->tree, g_object_ref (self->file));
	}
}

static void
_bieb_end (BurgnerIsofsTreeBuilder *tb,
	   GNode *root,
	   BurgnerIsofsItem *last_item)
{
	BurgnerIsofsEltoritoBuilder *eb = BURGNER_ISOFS_ELTORITO_BUILDER (tb);
	BurgnerIsofsItem *bc, *vent, *ient, *br;

	g_return_if_fail (eb->file != NULL);
	g_return_if_fail (eb->vd == NULL);

	bc = burgner_isofs_eltorito_boot_catalog_new ();
	vent = burgner_isofs_eltorito_boot_catalog_validation_entry_new ();
	g_object_set (vent,
		      "manufacturer", eb->manufacturer,
		      "platform_id", eb->platform_id,
		      NULL);
	ient = burgner_isofs_eltorito_boot_catalog_initial_entry_new ();
	g_object_set (ient,
		      "img", eb->file,
		      "media_type", eb->media_type,
		      "load_segment", eb->load_segment,
		      "system_type", eb->system_type,
		      "bootable", eb->bootable,
		      NULL);
	burgner_isofs_eltorito_boot_catalog_append_entries
				(BURGNER_ISOFS_ELTORITO_BOOT_CATALOG (bc), vent, ient, NULL);
	g_object_unref (vent);
	g_object_unref (ient);
	eb->tree = g_list_prepend (eb->tree, bc);

	br = burgner_isofs_eltorito_boot_record_new ();
	g_object_set (br,
		      "boot_catalog", bc,
		      NULL);
	burgner_isofs_item_set_mandatory_addr (br, TRUE, 0x11);
	eb->vd = g_list_prepend (NULL, br);
}

static void
_bieb_file (BurgnerIsofsTreeBuilder *tb,
	    BurgnerIsofsFileInfo *parent_info,
	    BurgnerIsofsFileInfo *info,
	    BurgnerIsofsItem *file)
{
	BurgnerIsofsEltoritoBuilder *eb = BURGNER_ISOFS_ELTORITO_BUILDER (tb);

	g_return_if_fail (eb->info != NULL);

	if (eb->is_in_tree && !eb->file && info == eb->info)
		eb->file = BURGNER_ISOFS_ITEM (g_object_ref (file));
}

static GList *
_bieb_get_volume_descriptors (BurgnerIsofsTreeBuilder *tb)
{
	BurgnerIsofsEltoritoBuilder *eb = BURGNER_ISOFS_ELTORITO_BUILDER (tb);
	return eb->vd;
}

static GList *
_bieb_get_items (BurgnerIsofsTreeBuilder *tb)
{
	BurgnerIsofsEltoritoBuilder *eb = BURGNER_ISOFS_ELTORITO_BUILDER (tb);
	return eb->tree;
}

static void
burgner_isofs_eltorito_builder_class_init (BurgnerIsofsEltoritoBuilderClass *klass)
{
	GObjectClass *object_class;
	BurgnerIsofsTreeBuilderClass *builder_class;

	object_class = (GObjectClass *) klass;
	object_class->set_property = burgner_isofs_eltorito_builder_set_property;
	object_class->get_property = burgner_isofs_eltorito_builder_get_property;
	object_class->finalize = burgner_isofs_eltorito_builder_finalize;

	builder_class = (BurgnerIsofsTreeBuilderClass *) klass;
	builder_class->begin = _bieb_begin;
	builder_class->end = _bieb_end;
	builder_class->file = _bieb_file;
	builder_class->get_volume_descriptors = _bieb_get_volume_descriptors;
	builder_class->get_items = _bieb_get_items;

	/* Properties */
	g_object_class_install_property (object_class,
					 PROP_MANUFACTURER,
					 g_param_spec_string ("manufacturer",
					            	      _("manufacturer"),
							      _("Manufacturer"),
							      NULL,
							      G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_INFO,
					 g_param_spec_pointer ("info",
					            	       _("info"),
							       _("Info"),
							       G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_PLATFORM_ID,
					 g_param_spec_enum ("platform_id",
					            	    _("platform_id"),
							    _("Platform Id"),
							    BURGNER_TYPE_ISOFS_ELTORITO_PLATFORM_ID,
							    BURGNER_ISOFS_ELTORITO_PLATFORM_ID_80x86,
							    G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_MEDIA_TYPE,
					 g_param_spec_enum ("media_type",
					            	    _("media_type"),
							    _("Media Type"),
							    BURGNER_TYPE_ISOFS_ELTORITO_MEDIA_TYPE,
							    BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_NO_EMULATION,
							    G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_LOAD_SEGMENT,
					 g_param_spec_uint ("load_segment",
					            	    _("load_segment"),
							    _("Load Segment"),
							    0,
							    0xffff,
							    0x7c0,
							    G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_SYSTEM_TYPE,
					 g_param_spec_uint ("system_type",
					            	    _("system_type"),
							    _("System Type"),
							    0,
							    0xff,
							    0,
							    G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_BOOTABLE,
					 g_param_spec_boolean ("bootable",
					            	       _("bootable"),
							       _("Bootable"),
							       TRUE,
							       G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_IS_IN_TREE,
					 g_param_spec_boolean ("is_in_tree",
					            	       _("is_in_tree"),
							       _("Is in tree"),
							       TRUE,
							       G_PARAM_READABLE | G_PARAM_WRITABLE));

	parent_class = g_type_class_peek_parent (klass);
}

static void
burgner_isofs_eltorito_builder_init (BurgnerIsofsEltoritoBuilder *self)
{
	self->manufacturer = NULL;
	self->info = NULL;
	self->platform_id = BURGNER_ISOFS_ELTORITO_PLATFORM_ID_80x86;
	self->media_type = BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_NO_EMULATION;
	self->file = NULL;
	self->vd = self->tree = NULL;
	self->load_segment = 0x7c0;
	self->system_type = 0;
	self->bootable = 1;
	self->is_in_tree = 1;
}

GType
burgner_isofs_eltorito_builder_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		static GTypeInfo info = {
			sizeof (BurgnerIsofsEltoritoBuilderClass),
			NULL,
			NULL,
			(GClassInitFunc) burgner_isofs_eltorito_builder_class_init,
			NULL,
			NULL,
			sizeof (BurgnerIsofsEltoritoBuilder),
			0,
			(GInstanceInitFunc) burgner_isofs_eltorito_builder_init,
		};

		type = g_type_register_static (BURGNER_TYPE_ISOFS_TREE_BUILDER, "BurgnerIsofsEltoritoBuilder", &info, 0);
	}

	return type;
}

BurgnerIsofsTreeBuilder *
burgner_isofs_eltorito_builder_new (void)
{
	BurgnerIsofsEltoritoBuilder *self;

	self = g_object_new (BURGNER_TYPE_ISOFS_ELTORITO_BUILDER, NULL);
	

	return BURGNER_ISOFS_TREE_BUILDER (self);
}


GQuark
burgner_isofs_eltorito_builder_error_quark (void)
{
	return g_quark_from_string ("BurgnerIsofsEltoritoBuilder");
}

