/*
 * BurgnerIsofsEltoritoBuilder - BurgnerIsofsEltoritoBuilder object
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifndef __BURGNER_ISOFS_ELTORITO_BUILDER_H__
#define __BURGNER_ISOFS_ELTORITO_BUILDER_H__

#include <glib-object.h>
#include <libburgnerisofs/burgner-isofs-eltorito-utils.h>
#include <libburgnerisofs/burgner-isofs-tree.h>
#include <libburgnerisofs/burgner-isofs-tree-builder.h>

G_BEGIN_DECLS


#define BURGNER_TYPE_ISOFS_ELTORITO_BUILDER		(burgner_isofs_eltorito_builder_get_type ())
#define BURGNER_ISOFS_ELTORITO_BUILDER(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), BURGNER_TYPE_ISOFS_ELTORITO_BUILDER, BurgnerIsofsEltoritoBuilder))
#define BURGNER_ISOFS_ELTORITO_BUILDER_CLASS(k)		(G_TYPE_CHECK_CLASS_CAST ((k), BURGNER_TYPE_ISOFS_ELTORITO_BUILDER, BurgnerIsofsEltoritoBuilderClass))
#define BURGNER_IS_ISOFS_ELTORITO_BUILDER(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), BURGNER_TYPE_ISOFS_ELTORITO_BUILDER))
#define BURGNER_IS_ISOFS_ELTORITO_BUILDER_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), BURGNER_TYPE_ISOFS_ELTORITO_BUILDER))
#define BURGNER_ISOFS_ELTORITO_BUILDER_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), BURGNER_TYPE_ISOFS_ELTORITO_BUILDER, BurgnerIsofsEltoritoBuilderClass))


typedef struct _BurgnerIsofsEltoritoBuilder BurgnerIsofsEltoritoBuilder;
typedef struct _BurgnerIsofsEltoritoBuilderClass BurgnerIsofsEltoritoBuilderClass;

struct _BurgnerIsofsEltoritoBuilder {
	BurgnerIsofsTreeBuilder __parent;

	gchar *manufacturer;
	BurgnerIsofsFileInfo *info;
	BurgnerIsofsEltoritoPlatformId platform_id;
	BurgnerIsofsEltoritoMediaType media_type;

	BurgnerIsofsItem *file;

	GList *vd;
	GList *tree;

	guint16 load_segment;
	guint8 system_type;
	guint bootable : 1;
	guint is_in_tree : 1;
};

struct _BurgnerIsofsEltoritoBuilderClass {
	BurgnerIsofsTreeBuilderClass __parent_class;

};


GType burgner_isofs_eltorito_builder_get_type (void) G_GNUC_CONST;
GQuark  burgner_isofs_eltorito_builder_error_quark (void);

#define BURGNER_ISOFS_ELTORITO_BUILDER_ERROR		(burgner_isofs_eltorito_builder_error_quark ())


BurgnerIsofsTreeBuilder *burgner_isofs_eltorito_builder_new (void);


G_END_DECLS

#endif /* __BURGNER_ISOFS_ELTORITO_BUILDER_H__ */

