/*
 * BurgnerIsofs Eltorito Impl
 *
 * This file is part of the libburgner library.
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifndef __BURGNER_ISOFS_ELTORITO_IMPL_H__
#define __BURGNER_ISOFS_ELTORITO_IMPL_H__

#include <glib.h>

G_BEGIN_DECLS

typedef struct {
	guchar header_id;
	guchar platform_id;

	guchar reserved[2];

	guchar manufacturer[24];
	guchar checksum[2];

	guchar key_55;
	guchar key_AA;
} __attribute__((packed)) __BurgnerIsofsEltoritoImplValidationEntry;

enum {
	__BURGNER_ISOFS_ELTORITO_IMPL_INITIAL_ENTRY_BOOT_INDICATOR_NOT_BOOTABLE = 0,
	__BURGNER_ISOFS_ELTORITO_IMPL_INITIAL_ENTRY_BOOT_INDICATOR_BOOTABLE = 0x88
};

enum {
	__BURGNER_ISOFS_ELTORITO_IMPL_INITIAL_ENTRY_BOOT_MEDIA_TYPE_NONE = 0,
	__BURGNER_ISOFS_ELTORITO_IMPL_INITIAL_ENTRY_BOOT_MEDIA_TYPE_1_2_MEG_DISKETTE = 1,
	__BURGNER_ISOFS_ELTORITO_IMPL_INITIAL_ENTRY_BOOT_MEDIA_TYPE_1_44_MEG_DISKETTE = 2,
	__BURGNER_ISOFS_ELTORITO_IMPL_INITIAL_ENTRY_BOOT_MEDIA_TYPE_2_88_MEG_DISKETTE = 3,
	__BURGNER_ISOFS_ELTORITO_IMPL_INITIAL_ENTRY_BOOT_MEDIA_TYPE_HARD_DISK = 4
};

typedef struct {
	guchar boot_indicator;
	guchar boot_media_type;

	guchar load_segment[2];

	guchar system_type;

	guchar unused0;

	guchar sector_count[2];
	guchar load_rba[4];

	guchar unused1[20];
} __attribute__((packed)) __BurgnerIsofsEltoritoImplInitialEntry;

enum {
	__BURGNER_ISOFS_ELTORITO_IMPL_SECTION_HEADER_ID_MORE = 0x90,
	__BURGNER_ISOFS_ELTORITO_IMPL_SECTION_HEADER_ID_FINAL = 0x91
};

enum {
	__BURGNER_ISOFS_ELTORITO_IMPL_SECTION_HEADER_PLATFORM_ID_80x86 = 0,
	__BURGNER_ISOFS_ELTORITO_IMPL_SECTION_HEADER_PLATFORM_ID_POWER_PC = 1,
	__BURGNER_ISOFS_ELTORITO_IMPL_SECTION_HEADER_PLATFORM_ID_MAC = 2
};

typedef struct {
	guchar header_id;
	guchar platform_id;
	guchar n_sections[2];

	guchar id_string[28];
} __attribute__((packed)) __BurgnerIsofsEltoritoImplSectionHeader;

enum {
	__BURGNER_ISOFS_ELTORITO_IMPL_SECTION_ENTRY_SELECTION_CRITERIA_TYPE_NONE = 0,
	__BURGNER_ISOFS_ELTORITO_IMPL_SECTION_ENTRY_SELECTION_CRITERIA_TYPE_LANG = 1
};

typedef struct {
	guchar boot_indicator;
	guchar boot_media_type;

	guchar load_segment[2];

	guchar system_type;

	guchar unused0;

	guchar sector_count[2];
	guchar load_rba[4];

	guchar selection_criteria_type;
	guchar selection_criteria[19];
} __attribute__((packed)) __BurgnerIsofsEltoritoImplSectionEntry;

enum {
	__BURGNER_ISOFS_ELTORITO_IMPL_SECTION_ENTRY_EXTENSION_BITS_FINAL = 0,
	__BURGNER_ISOFS_ELTORITO_IMPL_SECTION_ENTRY_EXTENSION_BITS_EE_FOLLOWS = 1 << 4
};

typedef struct {
	guchar ext_id;	/* Must be 0x44 */
	guchar bits;
	guchar selection_criteria[30];
} __attribute__((packed)) __BurgnerIsofsEltoritoImplSectionExtensionEntry;

G_END_DECLS

#endif /* __BURGNER_ISOFS_ELTORITO_IMPL_H__ */
