/*
 * 
 *
 * This file is part of the libburgner library.
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "burgner-isofs-i18n.h"

#include "burgner-isofs-hd-impl.h"
#include "burgner-isofs-eltorito-read-system-type.h"

gboolean
burgner_isofs_eltorito_read_system_type (guchar *mbr_buf,
					 guchar *system_type,
					 GError **error)
{
	_BurgnerIsofsHdImplMbr *mbr;
	guint i;
	guchar type = 0;

	g_return_val_if_fail (mbr != NULL, FALSE);

	mbr = (_BurgnerIsofsHdImplMbr *) mbr_buf;

	if (mbr->magic[0] != _BURGNER_ISOFS_HD_IMPL_MSDOS_MAGIC_0 ||
	    mbr->magic[1] != _BURGNER_ISOFS_HD_IMPL_MSDOS_MAGIC_1) {
		g_set_error (error,
			     BURGNER_ISOFS_ELTORITO_READ_SYSTEM_TYPE_ERROR,
			     BURGNER_ISOFS_ELTORITO_READ_SYSTEM_TYPE_ERROR_INVALID_MBR,
			     _("Invalid Master Boot Record"));
		return FALSE;
	}

	for (i = 0; i < 4; i++) {
		if (mbr->partitions[i].boot_indicator != _BURGNER_ISOFS_HD_IMPL_PARTITION_ACTIVE)
			continue;

		if (type != 0) {
			g_set_error (error,
				     BURGNER_ISOFS_ELTORITO_READ_SYSTEM_TYPE_ERROR,
				     BURGNER_ISOFS_ELTORITO_READ_SYSTEM_TYPE_ERROR_SEVERAL_PARTITIONS,
				     _("Hard disk contains several partitions."));
			return FALSE;
		}

		type = mbr->partitions[i].type;
	}

	if (type == 0) {
		g_set_error (error,
			     BURGNER_ISOFS_ELTORITO_READ_SYSTEM_TYPE_ERROR,
			     BURGNER_ISOFS_ELTORITO_READ_SYSTEM_TYPE_ERROR_NO_PARTITIONS,
			     _("Hard disk image contains no partitions."));
		return FALSE;
	}

	if (system_type)
		*system_type = type;

	return TRUE;
}

GQuark
burgner_isofs_eltorito_read_system_type_error_quark (void)
{
	return g_quark_from_string ("BurgnerIsofsEltorito read system type");
}

