/*
 * BurgnerIsofs Eltorito Utils
 *
 * This file is part of the libburgner library.
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "burgner-isofs-i18n.h"

#include "burgner-isofs-eltorito-utils.h"


GType
burgner_isofs_eltorito_platform_id_get_type (void)
{
	static GType etype = 0;

	if (etype == 0) {
		static const GEnumValue values[] = {
			{ BURGNER_ISOFS_ELTORITO_PLATFORM_ID_80x86, "BURGNER_ISOFS_ELTORITO_PLATFORM_ID_80x86", "80x86" },
			{ BURGNER_ISOFS_ELTORITO_PLATFORM_ID_POWER_PC, "BURGNER_ISOFS_ELTORITO_PLATFORM_ID_POWER_PC", "POWER_PC" },
			{ BURGNER_ISOFS_ELTORITO_PLATFORM_ID_MAC, "BURGNER_ISOFS_ELTORITO_PLATFORM_ID_MAC", "MAC" },
			{ 0, NULL, NULL }
		};
		etype = g_enum_register_static ("BurgnerIsofsEltoritoPlatformId", values);
	}

	return etype;
}

GType
burgner_isofs_eltorito_media_type_get_type (void)
{
	static GType etype = 0;

	if (etype == 0) {
		static const GEnumValue values[] = {
			{ BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_NO_EMULATION, "BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_NO_EMULATION", "NO_EMULATION" },
			{ BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_1_2_MEG_DISKETTE, "BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_1_2_MEG_DISKETTE", "1_2_MEG_DISKETTE" },
			{ BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_1_44_MEG_DISKETTE, "BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_1_44_MEG_DISKETTE", "1_44_MEG_DISKETTE" },
			{ BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_2_88_MEG_DISKETTE, "BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_2_88_MEG_DISKETTE", "2_88_MEG_DISKETTE" },
			{ BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_HARD_DISK, "BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_HARD_DISK", "HARD_DISK" },
			{ 0, NULL, NULL }
		};
		etype = g_enum_register_static ("BurgnerIsofsEltoritoMediaType", values);
	}

	return etype;
}

const gchar *
burgner_isofs_eltorito_media_type_to_string (BurgnerIsofsEltoritoMediaType t)
{
	const gchar *s = NULL;

	switch (t) {
		case BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_NO_EMULATION:
			s = _("no emulation");
			break;
		case BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_1_2_MEG_DISKETTE:
			s = _("1.2M diskette");
			break;
		case BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_1_44_MEG_DISKETTE:
			s = _("1.44M diskette");
			break;
		case BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_2_88_MEG_DISKETTE:
			s = _("2.88M diskette");
			break;
		case BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_HARD_DISK:
			s = _("hard disk");
			break;
		default:
			g_return_val_if_reached (NULL);
			break;
	}

	return s;
}

const gchar *
burgner_isofs_eltorito_platform_id_to_string (BurgnerIsofsEltoritoPlatformId p)
{
	gchar *s = NULL;

	switch (p) {
		case BURGNER_ISOFS_ELTORITO_PLATFORM_ID_80x86:
			s = _("80x86 (PC)");
			break;
		case BURGNER_ISOFS_ELTORITO_PLATFORM_ID_POWER_PC:
			s = _("PowerPC");
			break;
		case BURGNER_ISOFS_ELTORITO_PLATFORM_ID_MAC:
			s = _("Mac");
			break;
		default:
			g_return_val_if_reached (NULL);
			break;
	}

	return s;
}

