/*
 * BurgnerIsofsExtent - BurgnerIsofsExtent iface
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "burgner-isofs-extent.h"


GType
burgner_isofs_extent_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		static GTypeInfo info = {
			sizeof (BurgnerIsofsExtentIface),
			(GBaseInitFunc) NULL,
			(GBaseFinalizeFunc) NULL,
			
		};

		type = g_type_register_static (G_TYPE_INTERFACE, "BurgnerIsofsExtent", &info, 0);
	}

	return type;
}

guint32
burgner_isofs_extent_get_size (BurgnerIsofsExtent *extent)
{
	BurgnerIsofsExtentIface *iface;

	g_return_val_if_fail (extent != NULL, 0);
	g_return_val_if_fail (BURGNER_IS_ISOFS_EXTENT (extent), 0);

	iface = BURGNER_ISOFS_EXTENT_GET_IFACE (extent);
	g_return_val_if_fail (iface != NULL, 0);

	if (iface->get_size)
		return iface->get_size (extent);

	return 0;
}

#ifdef G_HAVE_GINT64
guint64
burgner_isofs_extent_get_size64 (BurgnerIsofsExtent *extent)
{
	BurgnerIsofsExtentIface *iface;

	g_return_val_if_fail (extent != NULL, G_GINT64_CONSTANT (0U));
	g_return_val_if_fail (BURGNER_IS_ISOFS_EXTENT (extent), G_GINT64_CONSTANT (0U));

	iface = BURGNER_ISOFS_EXTENT_GET_IFACE (extent);
	g_return_val_if_fail (iface != NULL, G_GINT64_CONSTANT (0U));

	if (iface->get_size64)
		return iface->get_size64 (extent);
	else if (iface->get_size)
		return (guint64) iface->get_size (extent);

	return G_GINT64_CONSTANT (0U);
}
#endif
