/*
 * BurgnerIsofsFile Buffer
 *
 * This file is part of the libburgner library.
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#include "burgner-isofs-file-buffer.h"

static gpointer _bifb_buffer = NULL;
static guint _bifb_refcount = 0;
static BurgnerIsofsItem *_bifb_current_owner = NULL;


void
burgner_isofs_file_buffer_ref (void)
{
	g_return_if_fail (_bifb_refcount != 0 || _bifb_buffer == NULL);

	_bifb_refcount += 1;

	if (!_bifb_buffer) {
		_bifb_buffer = g_malloc (BURGNER_ISOFS_FILE_BUFFER_SIZE);
	}
}

void
burgner_isofs_file_buffer_unref (void)
{
	g_return_if_fail (_bifb_refcount != 0 || _bifb_buffer == NULL);

	_bifb_refcount -= 1;

	if (_bifb_refcount == 0) {
		g_assert (_bifb_buffer != NULL);
		g_free (_bifb_buffer);
		_bifb_buffer = NULL;
	}
}

gboolean
burgner_isofs_file_buffer_is_locked (void)
{
	return (_bifb_current_owner != NULL);
}

gpointer
burgner_isofs_file_buffer_lock (BurgnerIsofsItem *item)
{
	g_return_val_if_fail (_bifb_current_owner == NULL, NULL);
	g_return_val_if_fail (_bifb_refcount != 0, NULL);
	g_return_val_if_fail (_bifb_buffer != NULL, NULL);

	_bifb_current_owner = item;
	return _bifb_buffer;
}

void
burgner_isofs_file_buffer_unlock (BurgnerIsofsItem *item)
{
	g_return_if_fail (_bifb_current_owner == item);
	g_assert (_bifb_refcount != 0);
	g_assert (_bifb_buffer != NULL);

	_bifb_current_owner = NULL;
}
