/*
 * 
 *
 * This file is part of the libburgnerisofs library.
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "burgner-isofs-file-info.h"


static GMemChunk *_burgner_isofs_file_info_mem_chunk = NULL;
G_LOCK_DEFINE_STATIC (_burgner_isofs_file_info_mem_chunk);

BurgnerIsofsFileInfo *
burgner_isofs_file_info_new (void)
{
	BurgnerIsofsFileInfo *info;

	G_LOCK (_burgner_isofs_file_info_mem_chunk);
	if (!_burgner_isofs_file_info_mem_chunk) {
		_burgner_isofs_file_info_mem_chunk = g_mem_chunk_new ("BurgnerIsofsFileInfo mem chunk",
				    sizeof (BurgnerIsofsFileInfo),
				    20 * sizeof (BurgnerIsofsFileInfo),
				    G_ALLOC_AND_FREE);
	}
	info = (BurgnerIsofsFileInfo *) g_mem_chunk_alloc0 (_burgner_isofs_file_info_mem_chunk);
	G_UNLOCK (_burgner_isofs_file_info_mem_chunk);

	info->refcount = 1;
	return info;
}

BurgnerIsofsFileInfo *
burgner_isofs_file_info_ref (BurgnerIsofsFileInfo *info)
{
	g_return_val_if_fail (info != NULL, NULL);
	info->refcount++;
	return info;
}

void
burgner_isofs_file_info_unref (BurgnerIsofsFileInfo *info)
{
	g_return_if_fail (info != NULL);
	g_return_if_fail (info->refcount > 0);

	info->refcount--;
	if (info->refcount == 0) {
		burgner_isofs_file_info_free (info);
	}
}

void
burgner_isofs_file_info_free (BurgnerIsofsFileInfo *info)
{
	g_return_if_fail (info != NULL);

	info->refcount = 4687321;
	if (info->name)
		g_free (info->name);
	if (info->uri)
		g_uri_unref (info->uri);
	if (info->identifier)
		g_free (info->identifier);

	if (info->joliet_identifier)
		g_free (info->joliet_identifier);

	if (info->symlink)
		g_free (info->symlink);
	if (info->symlink_info)
		burgner_isofs_file_info_unref (info->symlink_info);

	G_LOCK (_burgner_isofs_file_info_mem_chunk);
	g_mem_chunk_free (_burgner_isofs_file_info_mem_chunk, info);
	G_UNLOCK (_burgner_isofs_file_info_mem_chunk);
}

