/*
 * 
 *
 * This file is part of the libburgnerisofs library.
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */

#ifndef __BURGNER_ISOFS_FILE_INFO_H__
#define __BURGNER_ISOFS_FILE_INFO_H__

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <glib.h>
#include <libburgnerutils/g-uri.h>

G_BEGIN_DECLS

typedef enum {
	BURGNER_ISOFS_FILE_TYPE_UNKNOWN = 0,
	BURGNER_ISOFS_FILE_TYPE_REGULAR = S_IFREG,
	BURGNER_ISOFS_FILE_TYPE_DIRECTORY = S_IFDIR,
	BURGNER_ISOFS_FILE_TYPE_CHARACTER_DEVICE = S_IFCHR,
	BURGNER_ISOFS_FILE_TYPE_BLOCK_DEVICE = S_IFBLK,
	BURGNER_ISOFS_FILE_TYPE_FIFO = S_IFIFO,
	BURGNER_ISOFS_FILE_TYPE_SOCKET = S_IFSOCK,
	BURGNER_ISOFS_FILE_TYPE_SYMBOLIC_LINK = S_IFLNK
} BurgnerIsofsFileType;

typedef enum {
	BURGNER_ISOFS_FILE_PERMISSION_OWNER_READ = S_IRUSR,
	BURGNER_ISOFS_FILE_PERMISSION_OWNER_WRITE = S_IWUSR,
	BURGNER_ISOFS_FILE_PERMISSION_OWNER_EXEC = S_IXUSR,
	BURGNER_ISOFS_FILE_PERMISSION_GROUP_READ = S_IRGRP,
	BURGNER_ISOFS_FILE_PERMISSION_GROUP_WRITE = S_IWGRP,
	BURGNER_ISOFS_FILE_PERMISSION_GROUP_EXEC = S_IXGRP,
	BURGNER_ISOFS_FILE_PERMISSION_OTHERS_READ = S_IROTH,
	BURGNER_ISOFS_FILE_PERMISSION_OTHERS_WRITE = S_IWOTH,
	BURGNER_ISOFS_FILE_PERMISSION_OTHERS_EXEC = S_IXOTH,
	BURGNER_ISOFS_FILE_PERMISSION_SET_UID = S_ISUID,
	BURGNER_ISOFS_FILE_PERMISSION_SET_GID = S_ISGID,
	BURGNER_ISOFS_FILE_PERMISSION_STICKY = S_ISVTX
} BurgnerIsofsFilePermission;


typedef struct _BurgnerIsofsFileInfo BurgnerIsofsFileInfo;

struct _BurgnerIsofsFileInfo {
	gchar *name;
	GUri *uri;

	gchar *identifier;
	gunichar2 *joliet_identifier;

	BurgnerIsofsFileType type;
	BurgnerIsofsFilePermission permissions;

	guint uid;
	guint gid;

	guint link_count;
	guint64 inode_number;
	guint64 device;

	guint32 size;

	gchar *symlink;
	BurgnerIsofsFileInfo *symlink_info;

	guint32 addr;
	guint32 joliet_dir_addr;

	guint is_directory : 1;
	guint is_in_prev_session : 1;

	GTime creation_time;
	GTime modify_time;
	GTime access_time;
	GTime attributes_time;
	GTime backup_time;
	GTime expiration_time;
	GTime effective_time;

	/*<private>*/
	guint refcount;
	GNode *node;
};


BurgnerIsofsFileInfo *burgner_isofs_file_info_new (void);

BurgnerIsofsFileInfo *burgner_isofs_file_info_ref (BurgnerIsofsFileInfo *info);
void burgner_isofs_file_info_unref (BurgnerIsofsFileInfo *info);

void burgner_isofs_file_info_free (BurgnerIsofsFileInfo *info);


G_END_DECLS

#endif /* __BURGNER_ISOFS_FILE_INFO_H__ */

