/*
 * BurgnerIsofsImpl
 *
 * This file is part of the libburgner library.
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifndef __BURGNER_ISOFS_IMPL_H__
#define __BURGNER_ISOFS_IMPL_H__

#include <glib.h>

G_BEGIN_DECLS


typedef struct {
	guchar type;
	guchar std_id[5];
	guchar version;
} __attribute__((packed)) __BurgnerIsofsImplVolumeDescriptorHeader;

typedef struct {
	__BurgnerIsofsImplVolumeDescriptorHeader header;

	/* SVD only */ guchar volume_flags;

	guchar system_identifier[32];
	guchar volume_identifier[32];

	guchar unused[8];

	guchar volume_space_size[8];

	/* SVD only */ guchar escape_sequences[32];

	guchar volume_set_size[4];
	guchar volume_sequence_number[4];
	guchar logical_block_size[4];

	guchar path_table_size[8];
	guchar l_path_table_location[4];
	guchar opt_l_path_table_location[4];
	guchar m_path_table_location[4];
	guchar opt_m_path_table_location[4];

	guchar root_directory_record[34];

	guchar volume_set_identifier[128];
	guchar publisher_identifier[128];
	guchar data_preparer_identifier[128];
	guchar application_identifier[128];

	guchar copyright_file_identifier[37];
	guchar abstract_file_identifier[37];
	guchar bibliographic_file_identifier[37];

	guchar volume_creation_date[17];
	guchar volume_modification_date[17];
	guchar volume_expiration_date[17];
	guchar volume_effective_date[17];

	guchar file_structure_version;
	guchar reserved0;

	guchar application_use[512];
	guchar reserved1[653];
} __attribute__((packed)) __BurgnerIsofsImplTreeVolumeDescriptorData;


typedef struct {
	__BurgnerIsofsImplVolumeDescriptorHeader header;

	guchar boot_system_id[32];
	guchar boot_id[32];

	guchar boot_catalog_addr[4];

	guchar unused[1973];
} __attribute__((packed)) __BurgnerIsofsImplEltoritoBootRecord;


typedef struct {
	guchar len_di;
	guchar ear_len;

	guchar location_of_extent[4];
	guchar parent_dir_num[2];

	/* Then:
	 *	guchar directory_identifier[len_di];
	 *  If len_di is an odd number:
	 *	guchar padding;
	 */
} __attribute__((packed)) __BurgnerIsofsImplPathTableRecord;

typedef struct {
	guchar year;
	guchar month;
	guchar day;
	guchar hour;
	guchar minute;
	guchar second;
	guchar offset_from_gmt;
} __attribute__((packed)) __BurgnerIsofsImpl915Date;

enum {
	__BURGNER_ISOFS_IMPL_DIRECTORY_RECORD_FLAGS_NONE = 0,
	__BURGNER_ISOFS_IMPL_DIRECTORY_RECORD_FLAGS_HIDDEN = 1,
	__BURGNER_ISOFS_IMPL_DIRECTORY_RECORD_FLAGS_DIRECTORY = 2,
	__BURGNER_ISOFS_IMPL_DIRECTORY_RECORD_FLAGS_ASSOCIATED_FILE = 4,
	__BURGNER_ISOFS_IMPL_DIRECTORY_RECORD_FLAGS_RECORD = 8,
	__BURGNER_ISOFS_IMPL_DIRECTORY_RECORD_FLAGS_PROTECTION = 16,
	__BURGNER_ISOFS_IMPL_DIRECTORY_RECORD_FLAGS_MULTI_EXTENT = 128
};

typedef struct {
	guchar len_dr;
	guchar ear_len;

	guchar location[8];
	guchar data_len[8];

	__BurgnerIsofsImpl915Date recording_date;
	guchar file_flags;
	guchar file_unit_size;
	guchar interleave_gap_size;
	guchar volume_sequence_number[4];

	guchar len_fi;

	/* Then:
	 *	guchar file_identifier[len_fi];
	 *  If len_fi is an even number:
	 *	guchar padding;
	 *
	 *	guchar system_use[len_su];
	 */
} __attribute__((packed)) __BurgnerIsofsImplDirectoryRecord;

typedef struct {
	guchar sig[2];
	guchar len;
	guchar version;
} __attribute__((packed)) __BurgnerIsofsImplSuspEntryHeader;

typedef struct {
	__BurgnerIsofsImplSuspEntryHeader header;

	guchar flags;
} __attribute__((packed)) __BurgnerIsofsImplRripNM;
enum {
	__BURGNER_ISOFS_IMPL_RRIP_NM_FLAGS_CONTINUE = 1,
	__BURGNER_ISOFS_IMPL_RRIP_NM_FLAGS_CURRENT = 2,
	__BURGNER_ISOFS_IMPL_RRIP_NM_FLAGS_PARENT = 4
};

typedef struct {
	__BurgnerIsofsImplSuspEntryHeader header;

	guchar sector[8];
	guchar offset[8];
	guchar length[8];
} __attribute__((packed)) __BurgnerIsofsImplSuspCE;

typedef struct {
	__BurgnerIsofsImplSuspEntryHeader header;

	guchar be;
	guchar ef;

	guchar bytes_skipped;
} __attribute__((packed)) __BurgnerIsofsImplSuspSP;

typedef struct {
	__BurgnerIsofsImplSuspEntryHeader header;

	guchar mode[8];
	guchar links[8];
	guchar uid[8];
	guchar gid[8];
	guchar inode[8];
} __attribute__((packed)) __BurgnerIsofsImplRripPX;

typedef struct {
	__BurgnerIsofsImplSuspEntryHeader header;

	guchar loc[8];
} __attribute__((packed)) __BurgnerIsofsImplRripLoc;

typedef struct {
	__BurgnerIsofsImplSuspEntryHeader header;

	guchar be;
	guchar ef;

	guchar bs;
} __attribute__((packed)) __BurgnerIsofsImplRripSp;

typedef struct {
	__BurgnerIsofsImplSuspEntryHeader header;

	guchar id_len;
	guchar desc_len;
	guchar src_len;

	guchar ext_version;
} __attribute__((packed)) __BurgnerIsofsImplRripEr;

typedef struct {
	__BurgnerIsofsImplSuspEntryHeader header;

	guchar high[8];
	guchar low[8];
} __attribute__((packed)) __BurgnerIsofsImplRripPn;


enum {
	__BURGNER_ISOFS_IMPL_RRIP_TF_FLAG_CREATION = 1,
	__BURGNER_ISOFS_IMPL_RRIP_TF_FLAG_MODIFY = 1 << 1,
	__BURGNER_ISOFS_IMPL_RRIP_TF_FLAG_ACCESS = 1 << 2,
	__BURGNER_ISOFS_IMPL_RRIP_TF_FLAG_ATTRIBUTES = 1 << 3,
	__BURGNER_ISOFS_IMPL_RRIP_TF_FLAG_BACKUP = 1 << 4,
	__BURGNER_ISOFS_IMPL_RRIP_TF_FLAG_EXPIRATION = 1 << 5,
	__BURGNER_ISOFS_IMPL_RRIP_TF_FLAG_EFFECTIVE = 1 << 6,
	__BURGNER_ISOFS_IMPL_RRIP_TF_FLAG_LONG_FORM = 1 << 7,
	__BURGNER_ISOFS_IMPL_RRIP_TF_FLAG_SHORT_FORM = 0
};

typedef struct {
	__BurgnerIsofsImplSuspEntryHeader header;

	guchar flags;
} __attribute__((packed)) __BurgnerIsofsImplRripTf;


enum {
	__BURGNER_ISOFS_IMPL_RRIP_SL_FLAG_CONTINUE = 1
};

typedef struct {
	__BurgnerIsofsImplSuspEntryHeader header;

	guchar flags;
} __attribute__((packed)) __BurgnerIsofsImplRripSl;

enum {
	__BURGNER_ISOFS_IMPL_RRIP_SL_COMPONENT_FLAG_CONTINUE = 1,
	__BURGNER_ISOFS_IMPL_RRIP_SL_COMPONENT_FLAG_CURRENT = 1 << 1,
	__BURGNER_ISOFS_IMPL_RRIP_SL_COMPONENT_FLAG_PARENT = 1 << 2,
	__BURGNER_ISOFS_IMPL_RRIP_SL_COMPONENT_FLAG_ROOT = 1 << 3
};

typedef struct {
	guchar flags;
	guchar len;
} __attribute__((packed)) __BurgnerIsofsImplRripSlComponentHeader;


G_END_DECLS

#endif /* __BURGNER_ISOFS_IMPL_H__ */

