/*
 * BurgnerIsofsItemGList
 *
 * This file is part of the libburgner library.
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "burgner-isofs-item-g-list.h"


GSList *
burgner_isofs_item_g_slist_ref (GSList *slist)
{
	GSList *l = NULL, *it;

	if (slist == NULL)
		return NULL;

	for (it = slist; it != NULL; it = it->next) {
		l = g_slist_prepend (l, g_object_ref (it->data));
	}
	return g_slist_reverse (l);
}

GSList *
burgner_isofs_item_g_list_to_slist_ref (GList *list)
{
	GSList *l = NULL;
	GList *it;

	if (list == NULL)
		return NULL;

	for (it = list; it != NULL; it = it->next) {
		l = g_slist_prepend (l, g_object_ref (it->data));
	}
	return g_slist_reverse (l);
}

GList *
burgner_isofs_item_g_list_ref (GList *list)
{
	GList *l = NULL, *it;

	if (list == NULL)
		return NULL;

	for (it = list; it != NULL; it = it->next) {
		l = g_list_prepend (l, g_object_ref (it->data));
	}
	return g_list_reverse (l);
}

GList *
burgner_isofs_item_g_slist_to_list_ref (GSList *slist)
{
	GList *l = NULL;
	GSList *it;

	if (slist == NULL)
		return NULL;

	for (it = slist; it != NULL; it = it->next) {
		l = g_list_prepend (l, g_object_ref (it->data));
	}
	return g_list_reverse (l);
}

void
burgner_isofs_item_g_list_unref (GList *list)
{
	GList *it;

	if (list == NULL)
		return;

	for (it = list; it != NULL; it = it->next) {
		g_object_unref (it->data);
	}

	g_list_free (list);
}

void
burgner_isofs_item_g_slist_unref (GSList *list)
{
	GSList *it;

	if (list == NULL)
		return;

	for (it = list; it != NULL; it = it->next) {
		g_object_unref (it->data);
	}

	g_slist_free (list);
}
