/*
 * BurgnerIsofsItem
 *
 * This file is part of the libburgner library.
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifndef __BURGNER_ISOFS_ITEM_H__
#define __BURGNER_ISOFS_ITEM_H__

#include <glib-object.h>
#include <libburgnerisofs/burgner-isofs-types.h>
#include <libburgnerisofs/burgner-isofs-map-table.h>

G_BEGIN_DECLS


#define BURGNER_TYPE_ISOFS_ITEM		(burgner_isofs_item_get_type ())
#define BURGNER_ISOFS_ITEM(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), BURGNER_TYPE_ISOFS_ITEM, BurgnerIsofsItem))
#define BURGNER_ISOFS_ITEM_CLASS(k)	(G_TYPE_CHECK_CLASS_CAST ((k), BURGNER_TYPE_ISOFS_ITEM, BurgnerIsofsItemClass))
#define BURGNER_IS_ISOFS_ITEM(o)	(G_TYPE_CHECK_INSTANCE_TYPE ((o), BURGNER_TYPE_ISOFS_ITEM))
#define BURGNER_IS_ISOFS_ITEM_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), BURGNER_TYPE_ISOFS_ITEM))
#define BURGNER_ISOFS_ITEM_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), BURGNER_TYPE_ISOFS_ITEM, BurgnerIsofsItemClass))


typedef void (* BurgnerIsofsItemWriteAsyncInitCallback)
					(BurgnerIsofsItem *item,
					 gpointer write_async_data,
					 GError *error,
					 gpointer user_data);
typedef void (* BurgnerIsofsItemWriteAsyncCallback)
					(BurgnerIsofsItem *item,
					 gpointer write_async_data,
					 gpointer buf,
					 guint32 bytes_requested,
					 guint32 bytes_written,
					 GError *error,
					 gboolean end,
					 gpointer user_data);

typedef enum {
	BURGNER_ISOFS_ITEM_FLAGS_NONE = 0,
	BURGNER_ISOFS_ITEM_WRITE_ASYNC = 1,
	BURGNER_ISOFS_ITEM_SECTOR_START_ALIGNED = 1 << 1,
	BURGNER_ISOFS_ITEM_HAS_MANDATORY_ADDR = 1 << 2,
	BURGNER_ISOFS_ITEM_BLANK = 1 << 3
} BurgnerIsofsItemFlags;


typedef struct _BurgnerIsofsItemClass BurgnerIsofsItemClass;

struct _BurgnerIsofsItem {
	GObject object;

	guint32 mandatory_addr;

	guint has_mandatory_addr : 1;
	guint prepared : 1;
};

struct _BurgnerIsofsItemClass {
	GObjectClass object_class;

	/* Virtual functions */
	BurgnerIsofsItemFlags (* get_flags)(BurgnerIsofsItem *item);

	void (* set_mandatory_addr)(BurgnerIsofsItem *item,
				    gboolean has_mandatory_addr,
				    guint32 mandatory_addr);
	gboolean (* get_mandatory_addr)(BurgnerIsofsItem *item,
					guint32 *mandatory_addr);

	void (* get_size)(BurgnerIsofsItem *item,
			  guint32 *n_sectors,
			  guint32 *n_bytes);

	void (* prepare)(BurgnerIsofsItem *item,
			 BurgnerIsofsMapTable *table,
			 guint32 sector_n,
			 guint32 byte_o,
			 guint32 last_sector_n,
			 guint32 last_byte_o);

	void (* write_sync)(BurgnerIsofsItem *item,
			    BurgnerIsofsMapTable *table,
		            gpointer buf, guint32 size);
	void (* write_async_init)(BurgnerIsofsItem *item,
				  BurgnerIsofsMapTable *table,
				  BurgnerIsofsItemWriteAsyncInitCallback cb,
				  gpointer cb_data);
	void (* write_async)(BurgnerIsofsItem *item,
			     BurgnerIsofsMapTable *table,
		             gpointer write_async_data,
			     gpointer buf,
			     guint32 bytes_requested,
			     BurgnerIsofsItemWriteAsyncCallback cb,
			     gpointer cb_data);
	void (* write_async_cancel)(BurgnerIsofsItem *item,
				    gpointer write_async_data);

	GList *(* get_free_children)(BurgnerIsofsItem *item);
};


GType burgner_isofs_item_get_type (void) G_GNUC_CONST;


BurgnerIsofsItemFlags burgner_isofs_item_get_flags (BurgnerIsofsItem *item);

void burgner_isofs_item_set_mandatory_addr (BurgnerIsofsItem *item,
				    gboolean has_mandatory_addr,
				    guint32 mandatory_addr);
gboolean burgner_isofs_item_get_mandatory_addr (BurgnerIsofsItem *item,
					guint32 *mandatory_addr);

void burgner_isofs_item_get_size (BurgnerIsofsItem *item,
			  guint32 *n_sectors,
			  guint32 *n_bytes);

void burgner_isofs_item_prepare (BurgnerIsofsItem *item,
			 BurgnerIsofsMapTable *table,
			 guint32 sector_n,
			 guint32 byte_o,
			 guint32 last_sector_n,
			 guint32 last_byte_o);

void burgner_isofs_item_write_sync (BurgnerIsofsItem *item,
			    BurgnerIsofsMapTable *table,
		            gpointer buf, guint32 size);
void burgner_isofs_item_write_async_init (BurgnerIsofsItem *item,
				  BurgnerIsofsMapTable *table,
				  BurgnerIsofsItemWriteAsyncInitCallback cb,
				  gpointer cb_data);
void burgner_isofs_item_write_async (BurgnerIsofsItem *item,
			     BurgnerIsofsMapTable *table,
		             gpointer write_async_data,
			     gpointer buf,
			     guint32 bytes_requested,
			     BurgnerIsofsItemWriteAsyncCallback cb,
			     gpointer cb_data);
void burgner_isofs_item_write_async_cancel (BurgnerIsofsItem *item,
					    gpointer write_async_data);

GList *burgner_isofs_item_get_free_children (BurgnerIsofsItem *item);

G_END_DECLS

#endif /* __BURGNER_ISOFS_ITEM_H__ */

