/*
 * BurgnerIsofs List Iteration
 *
 * This file is part of the libburgner library.
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "burgner-isofs-list-iteration.h"


GList *
burgner_isofs_list_iteration_foreach (GList *list,
				      guint32 *s_c,
				      guint32 *b_o,
				      BurgnerIsofsListIterationFunc func,
				      gpointer user_data)
{
	GList *it;
	guint32 padding;

	g_return_val_if_fail (list != NULL, NULL);
	g_return_val_if_fail (s_c != NULL, NULL);
	g_return_val_if_fail (b_o != NULL, NULL);

	for (it = list; it != NULL; it = it->next) {
		BurgnerIsofsItem *item = BURGNER_ISOFS_ITEM (it->data);
		BurgnerIsofsItemFlags flags;
		guint32 n_s, n_b, o_s, o_b;

		flags = burgner_isofs_item_get_flags (item);
		n_s = n_b = 0;
		burgner_isofs_item_get_size (item, &n_s, &n_b);

#if 0
		if (n_s == 1 && n_b == 0) {
			n_s = 0;
			n_b = 2048;
		}
#endif

#if 0
		if (((prop & BURGNER_ISOFS_ITEM_PROPERTY_MUST_BEGIN_IN_NEW_SECTOR) && *b_o != 0) ||
		    (!(prop & BURGNER_ISOFS_ITEM_PROPERTY_CAN_BE_ACROSS_SECTOR) && (n_s == 0) && ((*b_o + n_b) > 2048)))
#else
		if ((flags & BURGNER_ISOFS_ITEM_SECTOR_START_ALIGNED) && *b_o != 0)
#endif
		{
			padding = 2048 - *b_o;
			*s_c += 1;
			*b_o = 0;
		} else {
			padding = 0;
		}

		o_s = *s_c;
		o_b = *b_o;

		*b_o += n_b;
		*s_c += n_s;

		if (*b_o & ~0x7ff) {
			*s_c += *b_o >> 11;
			*b_o &= 0x7ff;
		}

		if (func && func (list, it, item, flags, padding, o_s, o_b, n_s, n_b, *s_c, *b_o, user_data))
			return it;
	}

	return NULL;
}

