/*
 * BurgnerIsofsPathTableRecord - BurgnerIsofsPathTableRecord object
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifndef __BURGNER_ISOFS_PATH_TABLE_RECORD_H__
#define __BURGNER_ISOFS_PATH_TABLE_RECORD_H__

#include <glib-object.h>
#include <libburgnerisofs/burgner-isofs-item.h>

G_BEGIN_DECLS


#define BURGNER_TYPE_ISOFS_PATH_TABLE_RECORD		(burgner_isofs_path_table_record_get_type ())
#define BURGNER_ISOFS_PATH_TABLE_RECORD(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), BURGNER_TYPE_ISOFS_PATH_TABLE_RECORD, BurgnerIsofsPathTableRecord))
#define BURGNER_ISOFS_PATH_TABLE_RECORD_CLASS(k)	(G_TYPE_CHECK_CLASS_CAST ((k), BURGNER_TYPE_ISOFS_PATH_TABLE_RECORD, BurgnerIsofsPathTableRecordClass))
#define BURGNER_IS_ISOFS_PATH_TABLE_RECORD(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), BURGNER_TYPE_ISOFS_PATH_TABLE_RECORD))
#define BURGNER_IS_ISOFS_PATH_TABLE_RECORD_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), BURGNER_TYPE_ISOFS_PATH_TABLE_RECORD))
#define BURGNER_ISOFS_PATH_TABLE_RECORD_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), BURGNER_TYPE_ISOFS_PATH_TABLE_RECORD, BurgnerIsofsPathTableRecordClass))


typedef enum {
	BURGNER_ISOFS_PATH_TABLE_RECORD_TYPE_L,
	BURGNER_ISOFS_PATH_TABLE_RECORD_TYPE_M
} BurgnerIsofsPathTableRecordType;

GType burgner_isofs_path_table_record_type_get_type (void) G_GNUC_CONST;
#define BURGNER_TYPE_ISOFS_PATH_TABLE_RECORD_TYPE	(burgner_isofs_path_table_record_type_get_type ())


typedef struct _BurgnerIsofsPathTableRecord BurgnerIsofsPathTableRecord;
typedef struct _BurgnerIsofsPathTableRecordClass BurgnerIsofsPathTableRecordClass;

struct _BurgnerIsofsPathTableRecord {
	BurgnerIsofsItem __parent;

	gchar *id;

	BurgnerIsofsItem *dir;
	BurgnerIsofsItem *par;

	guint16 par_idx;

	guint is_root_dir : 1;
};

struct _BurgnerIsofsPathTableRecordClass {
	BurgnerIsofsItemClass __parent_class;

	/*
	 * Virtual Functions
	 */
	void (* write)(BurgnerIsofsItem *item,
		       BurgnerIsofsMapTable *table,
		       gpointer buf, guint32 size,
		       BurgnerIsofsPathTableRecordType type);
};


GType burgner_isofs_path_table_record_get_type (void) G_GNUC_CONST;
GQuark  burgner_isofs_path_table_record_error_quark (void);

#define BURGNER_ISOFS_PATH_TABLE_RECORD_ERROR		(burgner_isofs_path_table_record_error_quark ())


BurgnerIsofsItem *burgner_isofs_path_table_record_new (void);


void burgner_isofs_path_table_record_write (BurgnerIsofsPathTableRecord *rec,
					    BurgnerIsofsMapTable *table,
		                            gpointer buf, guint32 size,
		                            BurgnerIsofsPathTableRecordType type);

G_END_DECLS

#endif /* __BURGNER_ISOFS_PATH_TABLE_RECORD_H__ */

