/*
 * BurgnerIsofsPathTable - BurgnerIsofsPathTable object
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifndef __BURGNER_ISOFS_PATH_TABLE_H__
#define __BURGNER_ISOFS_PATH_TABLE_H__

#include <glib-object.h>
#include <libburgnerisofs/burgner-isofs-item.h>
#include <libburgnerisofs/burgner-isofs-path-table-record.h>

G_BEGIN_DECLS


#define BURGNER_TYPE_ISOFS_PATH_TABLE		(burgner_isofs_path_table_get_type ())
#define BURGNER_ISOFS_PATH_TABLE(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), BURGNER_TYPE_ISOFS_PATH_TABLE, BurgnerIsofsPathTable))
#define BURGNER_ISOFS_PATH_TABLE_CLASS(k)	(G_TYPE_CHECK_CLASS_CAST ((k), BURGNER_TYPE_ISOFS_PATH_TABLE, BurgnerIsofsPathTableClass))
#define BURGNER_IS_ISOFS_PATH_TABLE(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), BURGNER_TYPE_ISOFS_PATH_TABLE))
#define BURGNER_IS_ISOFS_PATH_TABLE_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), BURGNER_TYPE_ISOFS_PATH_TABLE))
#define BURGNER_ISOFS_PATH_TABLE_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), BURGNER_TYPE_ISOFS_PATH_TABLE, BurgnerIsofsPathTableClass))


typedef struct _BurgnerIsofsPathTable BurgnerIsofsPathTable;
typedef struct _BurgnerIsofsPathTableClass BurgnerIsofsPathTableClass;

struct _BurgnerIsofsPathTable {
	BurgnerIsofsItem __parent;

	GHashTable *dir_to_idx;

	GSList *records;
	GSList *n_bytes;

	BurgnerIsofsPathTableRecordType type;
};

struct _BurgnerIsofsPathTableClass {
	BurgnerIsofsItemClass __parent_class;

	/*
	 * Virtual Functions
	 */
	void (* add_record)(BurgnerIsofsPathTable *pt,
			    BurgnerIsofsItem *rec);
};


GType burgner_isofs_path_table_get_type (void) G_GNUC_CONST;
GQuark  burgner_isofs_path_table_error_quark (void);

#define BURGNER_ISOFS_PATH_TABLE_ERROR		(burgner_isofs_path_table_error_quark ())


BurgnerIsofsItem *burgner_isofs_path_table_new (void);

void burgner_isofs_path_table_add_record (BurgnerIsofsPathTable *pt, BurgnerIsofsItem *rec);

G_END_DECLS

#endif /* __BURGNER_ISOFS_PATH_TABLE_H__ */

