/*
 * BurgnerIsofsPrimaryBuilder - BurgnerIsofsPrimaryBuilder object
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifndef __BURGNER_ISOFS_PRIMARY_BUILDER_H__
#define __BURGNER_ISOFS_PRIMARY_BUILDER_H__

#include <glib-object.h>
#include <libburgnerisofs/burgner-isofs-tvd-info.h>
#include <libburgnerisofs/burgner-isofs-tree-builder.h>
#include <libburgnerisofs/burgner-isofs-path-table.h>

G_BEGIN_DECLS


#define BURGNER_TYPE_ISOFS_PRIMARY_BUILDER		(burgner_isofs_primary_builder_get_type ())
#define BURGNER_ISOFS_PRIMARY_BUILDER(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), BURGNER_TYPE_ISOFS_PRIMARY_BUILDER, BurgnerIsofsPrimaryBuilder))
#define BURGNER_ISOFS_PRIMARY_BUILDER_CLASS(k)		(G_TYPE_CHECK_CLASS_CAST ((k), BURGNER_TYPE_ISOFS_PRIMARY_BUILDER, BurgnerIsofsPrimaryBuilderClass))
#define BURGNER_IS_ISOFS_PRIMARY_BUILDER(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), BURGNER_TYPE_ISOFS_PRIMARY_BUILDER))
#define BURGNER_IS_ISOFS_PRIMARY_BUILDER_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), BURGNER_TYPE_ISOFS_PRIMARY_BUILDER))
#define BURGNER_ISOFS_PRIMARY_BUILDER_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), BURGNER_TYPE_ISOFS_PRIMARY_BUILDER, BurgnerIsofsPrimaryBuilderClass))


typedef struct _BurgnerIsofsPrimaryBuilder BurgnerIsofsPrimaryBuilder;
typedef struct _BurgnerIsofsPrimaryBuilderClass BurgnerIsofsPrimaryBuilderClass;

struct _BurgnerIsofsPrimaryBuilder {
	BurgnerIsofsTreeBuilder __parent;

	BurgnerIsofsItem *pvd;

	GList *vd_list;
	GList *items_list;

	GHashTable *fi_to_pt_d;
	gpointer root_pt_d;
	gpointer reloc_pt_d;
	GList *pt_list;

	GHashTable *fi_to_file;
	GHashTable *inode_to_file;

	gchar *reloc_dir_id;
	BurgnerIsofsItem *reloc_dir;
	guint32 reloc_id_count;

	GHashTable *fi_to_dir;

	BurgnerIsofsTvdInfo *tvd_info;

	BurgnerIsofsItem *root_dir;
	BurgnerIsofsItem *root_dir_rec;

	guint reloc_level;

	guint with_reloc : 1;
	guint with_rrip : 1;
	guint cache_inodes : 1;
	guint no_path_tables : 1;
};

struct _BurgnerIsofsPrimaryBuilderClass {
	BurgnerIsofsTreeBuilderClass __parent_class;

};


GType burgner_isofs_primary_builder_get_type (void) G_GNUC_CONST;
GQuark  burgner_isofs_primary_builder_error_quark (void);

#define BURGNER_ISOFS_PRIMARY_BUILDER_ERROR		(burgner_isofs_primary_builder_error_quark ())


BurgnerIsofsTreeBuilder *burgner_isofs_primary_builder_new (void);

G_END_DECLS

#endif /* __BURGNER_ISOFS_PRIMARY_BUILDER_H__ */

