/*
 * BurgnerIsofsRripNm - BurgnerIsofsRripNm object
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include <glib-object.h>
#include "burgner-isofs-impl.h"
#include "burgner-isofs-utils.h"
#include "burgner-isofs-rrip-nm.h"


static BurgnerIsofsSuspEntryClass *parent_class = NULL;

enum {
	PROP_0,
	PROP_STR,
	PROP_CONT,
	PROP_CURRENT,
	PROP_PARENT,
	LAST_ARG
};

static void
finalize (GObject *object)
{
	BurgnerIsofsRripNm *self = BURGNER_ISOFS_RRIP_NM (object);

	if (self->str) {
		g_free (self->str);
		self->str = NULL;
	}

	self->str_len = 0;

	self->cont = self->current = self->parent = 0;

	if (G_OBJECT_CLASS (parent_class)->finalize)
		G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
set_property (GObject *object,
	      guint prop_id,
	      const GValue *value,
	      GParamSpec *pspec)
{
	BurgnerIsofsRripNm *self = BURGNER_ISOFS_RRIP_NM (object);

	switch (prop_id) {
		case PROP_STR: {
			guint l;
			if (self->str) {
				g_free (self->str);
				self->str = NULL;
				self->str_len = 0;
			}

			self->str = g_value_dup_string (value);
			l = (self->str ? strlen (self->str) : 0);
			g_return_if_fail (l <= 250);
			self->str_len = l;
			g_object_notify (object, "str");
			} break;
		case PROP_CONT:
			if ((self->cont = g_value_get_boolean (value)?1:0)) {
				g_object_notify (object, "cont");
				if (self->current) {
					self->current = 0;
					g_object_notify (object, "current");
				}
				if (self->parent) {
					self->parent = 0;
					g_object_notify (object, "parent");
				}
			}
			break;
		case PROP_CURRENT:
			if ((self->current = g_value_get_boolean (value)?1:0)) {
				g_object_notify (object, "current");
				if (self->cont) {
					self->cont = 0;
					g_object_notify (object, "cont");
				}
				if (self->parent) {
					self->parent = 0;
					g_object_notify (object, "parent");
				}
			}
			break;
		case PROP_PARENT:
			if ((self->parent = g_value_get_boolean (value)?1:0)) {
				g_object_notify (object, "parent");
				if (self->cont) {
					self->cont = 0;
					g_object_notify (object, "cont");
				}
				if (self->current) {
					self->current = 0;
					g_object_notify (object, "current");
				}
			}
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
get_property (GObject *object,
	      guint prop_id,
	      GValue *value,
	      GParamSpec *pspec)
{
	BurgnerIsofsRripNm *self = BURGNER_ISOFS_RRIP_NM (object);

	switch (prop_id) {
		case PROP_STR:
			g_value_set_string (value, self->str);
			break;
		case PROP_CONT:
			g_value_set_boolean (value, self->cont);
			break;
		case PROP_CURRENT:
			g_value_set_boolean (value, self->current);
			break;
		case PROP_PARENT:
			g_value_set_boolean (value, self->parent);
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
_birn_get_size (BurgnerIsofsItem *item,
		guint32 *n_sectors,
		guint32 *n_bytes)
{
        BurgnerIsofsRripNm *nm = BURGNER_ISOFS_RRIP_NM (item);
	guint32 l;

	l = 5;
	if (!nm->current && !nm->parent)
		l += nm->str_len;

	if (n_sectors)
		*n_sectors = 0;
	if (n_bytes)
		*n_bytes = l;
}

static void
_birn_write_sync (BurgnerIsofsItem *item,
		  BurgnerIsofsMapTable *table,
		  gpointer buf, guint32 size)
{
	__BurgnerIsofsImplRripNM *impl;
	BurgnerIsofsRripNm *nm = BURGNER_ISOFS_RRIP_NM (item);
	guint32 l;

	l = 5;
	if (!nm->current && !nm->parent)
		l += nm->str_len;

	g_return_if_fail (size == l);

	impl = (__BurgnerIsofsImplRripNM *) buf;
	memset (impl, 0, sizeof (*impl));

	impl->header.sig[0] = 'N';
	impl->header.sig[1] = 'M';
	burgner_isofs_set_711 (&impl->header.len, (guint8) size);
	burgner_isofs_set_711 (&impl->header.version, 1);

	if (nm->cont)
		impl->flags = __BURGNER_ISOFS_IMPL_RRIP_NM_FLAGS_CONTINUE;
	else if (nm->current)
		impl->flags = __BURGNER_ISOFS_IMPL_RRIP_NM_FLAGS_CURRENT;
	else if (nm->parent)
		impl->flags = __BURGNER_ISOFS_IMPL_RRIP_NM_FLAGS_PARENT;
	else
		impl->flags = 0;

#if 0
	burgner_isofs_writer_proxy_write (proxy, (guchar*) &impl, 5);
#else
	buf += 5;
	size -= 5;
#endif

	if (!nm->current && !nm->parent) {
#if 0
		burgner_isofs_writer_proxy_write (proxy, nm->str, nm->str_len);
#else
		memcpy (buf, nm->str, nm->str_len);
		buf += nm->str_len;
		size -= nm->str_len;
#endif
	}

	g_assert (size == 0);
}

static void
class_init (BurgnerIsofsRripNmClass *klass)
{
	GObjectClass *object_class;
	BurgnerIsofsItemClass *item_class;

	object_class = (GObjectClass *) klass;
	object_class->set_property = set_property;
	object_class->get_property = get_property;
	object_class->finalize = finalize;

	item_class = (BurgnerIsofsItemClass *) klass;
	item_class->get_size = _birn_get_size;
	item_class->write_sync = _birn_write_sync;

	/* Properties */
	g_object_class_install_property (object_class,
					 PROP_STR,
					 g_param_spec_string ("str",
					            	      "str",
							      "String",
							      "",
							      G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_CONT,
					 g_param_spec_boolean ("continue",
					            	       "continue",
							       "Continue",
							       FALSE,
							       G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_CURRENT,
					 g_param_spec_boolean ("current",
					                       "current",
							       "Current",
							       FALSE,
							       G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_PARENT,
					 g_param_spec_boolean ("parent",
					                       "parent",
							       "Parent",
							       FALSE,
							       G_PARAM_READABLE | G_PARAM_WRITABLE));


	parent_class = g_type_class_peek_parent (klass);
}

static void
init (BurgnerIsofsRripNm *self)
{
	self->str = NULL;
	self->str_len = 0;
	self->cont = self->current = self->parent = 0;
}

GType
burgner_isofs_rrip_nm_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		static GTypeInfo info = {
			sizeof (BurgnerIsofsRripNmClass),
			NULL,
			NULL,
			(GClassInitFunc) class_init,
			NULL,
			NULL,
			sizeof (BurgnerIsofsRripNm),
			0,
			(GInstanceInitFunc) init,
		};

		type = g_type_register_static (BURGNER_TYPE_ISOFS_SUSP_ENTRY, "BurgnerIsofsRripNm", &info, 0);
	}

	return type;
}

BurgnerIsofsItem *
burgner_isofs_rrip_nm_new (void)
{
	BurgnerIsofsRripNm *self;

	self = g_object_new (BURGNER_TYPE_ISOFS_RRIP_NM, NULL);


	return BURGNER_ISOFS_ITEM (self);
}


GQuark
burgner_isofs_rrip_nm_error_quark (void)
{
	return g_quark_from_string ("BurgnerIsofsRripNm");
}

GSList *
burgner_isofs_rrip_nm_new_slist_from_id (const gchar *i)
{
	GSList *list = NULL;
	BurgnerIsofsItem *nm;
	const gchar *id;
	gchar *c;
	guint l, nm_l;

	g_return_val_if_fail (i != NULL, NULL);

	id = i;
	l = (id ? strlen (id) : 0);

	while (l) {
		nm_l = MIN (l, 250);
		c = g_strndup (id, nm_l);

		id += nm_l;
		l -= nm_l;

		nm = burgner_isofs_rrip_nm_new ();
		g_object_set (nm,
			      "str", c,
			      "continue", l ? TRUE : FALSE,
			      NULL);
		list = g_slist_prepend (list, nm);

		g_free (c);
	}

	return g_slist_reverse (list);
}

