/*
 * BurgnerIsofsRripPn - BurgnerIsofsRripPn object
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include <glib-object.h>
#include "burgner-isofs-impl.h"
#include "burgner-isofs-tree.h"
#include "burgner-isofs-utils.h"
#include "burgner-isofs-rrip-pn.h"


static BurgnerIsofsSuspEntryClass *parent_class = NULL;

enum {
	PROP_0,
	PROP_INFO,
	LAST_ARG
};

static void
finalize (GObject *object)
{
	BurgnerIsofsRripPn *self = BURGNER_ISOFS_RRIP_PN (object);

	if (self->info) {
		burgner_isofs_file_info_unref (self->info);
		self->info = NULL;
	}

	if (G_OBJECT_CLASS (parent_class)->finalize)
		G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
set_property (GObject *object,
	      guint prop_id,
	      const GValue *value,
	      GParamSpec *pspec)
{
	BurgnerIsofsRripPn *self = BURGNER_ISOFS_RRIP_PN (object);

	switch (prop_id) {
		case PROP_INFO:
			if (self->info)
				burgner_isofs_file_info_unref (self->info);
			self->info = g_value_get_pointer (value);
			if (self->info)
				self->info = burgner_isofs_file_info_ref (self->info);
			g_object_notify (object, "info");
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
get_property (GObject *object,
	      guint prop_id,
	      GValue *value,
	      GParamSpec *pspec)
{
	BurgnerIsofsRripPn *self = BURGNER_ISOFS_RRIP_PN (object);

	switch (prop_id) {
		case PROP_INFO:
			g_value_set_pointer (value, self->info);
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
_birpn_get_size (BurgnerIsofsItem *item,
		 guint32 *n_sectors,
		 guint32 *n_bytes)
{
	if (n_sectors)
		*n_sectors = 0;
	if (n_bytes)
		*n_bytes = 20;
}

static void
_birpn_write_sync (BurgnerIsofsItem *item,
		   BurgnerIsofsMapTable *table,
		   gpointer buf, guint32 size)
{
	__BurgnerIsofsImplRripPn *impl;
	BurgnerIsofsRripPn *self = BURGNER_ISOFS_RRIP_PN (item);

	g_return_if_fail (self->info != NULL);
	g_return_if_fail (size == 20);

	impl = (__BurgnerIsofsImplRripPn *) buf;
	memset (impl, 0, sizeof (*impl));

	impl->header.sig[0] = 'P';
	impl->header.sig[1] = 'N';
	burgner_isofs_set_711 (&impl->header.len, 20);
	burgner_isofs_set_711 (&impl->header.version, 1);

#if 0
	if (self->info->info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_DEVICE) {
		guint32 major, minor;
#ifdef __GNUC__
		major = (self->info->info->device >> 32) & 0xffffffff;
		minor = (self->info->info->device & 0xffffffff);
#else
#error __GNUC__ not defined
		/* TODO handle dev_t on non-GNU systems */
#endif
		burgner_isofs_set_733 (impl->high, major);
		burgner_isofs_set_733 (impl->low, minor);
	}
#else
	if (self->info->type == BURGNER_ISOFS_FILE_TYPE_CHARACTER_DEVICE ||
	    self->info->type == BURGNER_ISOFS_FILE_TYPE_BLOCK_DEVICE) {
		burgner_isofs_set_733 (impl->high, (guint32) ((self->info->device & 0xffffffff00000000) >> 32));
		burgner_isofs_set_733 (impl->low, (guint32) (self->info->device & 0xffffffff));
	}
#endif

#if 0
	burgner_isofs_writer_proxy_write (proxy, (guchar*) &impl, 20);
#else
	buf += 20;
	size -= 20;
#endif

	g_assert (size == 0);
}

static void
class_init (BurgnerIsofsRripPnClass *klass)
{
	GObjectClass *object_class;
	BurgnerIsofsItemClass *item_class;

	object_class = (GObjectClass *) klass;
	object_class->set_property = set_property;
	object_class->get_property = get_property;
	object_class->finalize = finalize;

	item_class = (BurgnerIsofsItemClass *) klass;
	item_class->get_size = _birpn_get_size;
	item_class->write_sync = _birpn_write_sync;

	/* Properties */
	g_object_class_install_property (object_class,
					 PROP_INFO,
					 g_param_spec_pointer ("info",
					            	       "info",
							       "Info",
							       G_PARAM_READABLE | G_PARAM_WRITABLE));


	parent_class = g_type_class_peek_parent (klass);
}

static void
init (BurgnerIsofsRripPn *self)
{
	self->info = NULL;
}

GType
burgner_isofs_rrip_pn_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		static GTypeInfo info = {
			sizeof (BurgnerIsofsRripPnClass),
			NULL,
			NULL,
			(GClassInitFunc) class_init,
			NULL,
			NULL,
			sizeof (BurgnerIsofsRripPn),
			0,
			(GInstanceInitFunc) init,
		};

		type = g_type_register_static (BURGNER_TYPE_ISOFS_SUSP_ENTRY, "BurgnerIsofsRripPn", &info, 0 /*G_TYPE_FLAG_ABSTRACT*/);
	}

	return type;
}

BurgnerIsofsItem *
burgner_isofs_rrip_pn_new (void)
{
	BurgnerIsofsRripPn *self;

	self = g_object_new (BURGNER_TYPE_ISOFS_RRIP_PN, NULL);


	return BURGNER_ISOFS_ITEM (self);
}


GQuark
burgner_isofs_rrip_pn_error_quark (void)
{
	return g_quark_from_string ("BurgnerIsofsRripPn");
}

