/*
 * BurgnerIsofsRripTf - BurgnerIsofsRripTf object
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "burgner-isofs-i18n.h"

#include <string.h>
#include <time.h>
#include <glib-object.h>
#include "burgner-isofs-impl.h"
#include "burgner-isofs-utils.h"
#include "burgner-isofs-rrip-tf.h"


static BurgnerIsofsSuspEntryClass *parent_class = NULL;

enum {
	PROP_0,
	PROP_INFO,
	BURGNER_ISOFS_RRIP_TF_LAST_ARG
};

static void
_burgner_isofs_rrip_tf_finalize (GObject *object)
{
	BurgnerIsofsRripTf *self = BURGNER_ISOFS_RRIP_TF (object);

	if (self->info) {
		burgner_isofs_file_info_unref (self->info);
		self->info = NULL;
	}

	if (G_OBJECT_CLASS (parent_class)->finalize)
		G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
_burgner_isofs_rrip_tf_set_property (GObject *object,
	      guint prop_id,
	      const GValue *value,
	      GParamSpec *pspec)
{
	BurgnerIsofsRripTf *self = BURGNER_ISOFS_RRIP_TF (object);

	switch (prop_id) {
		case PROP_INFO:
			if (self->info)
				burgner_isofs_file_info_unref (self->info);
			self->info = (BurgnerIsofsFileInfo *) g_value_get_pointer (value);
			if (self->info)
				self->info = burgner_isofs_file_info_ref (self->info);
			g_object_notify (object, "info");
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
_burgner_isofs_rrip_tf_get_property (GObject *object,
	      guint prop_id,
	      GValue *value,
	      GParamSpec *pspec)
{
	BurgnerIsofsRripTf *self = BURGNER_ISOFS_RRIP_TF (object);

	switch (prop_id) {
		case PROP_INFO:
			g_value_set_pointer (value, self->info);
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
_burgner_isofs_rrip_tf_get_size (BurgnerIsofsItem *item,
				 guint32 *n_s, guint32 *n_b)
{
	BurgnerIsofsRripTf *self = BURGNER_ISOFS_RRIP_TF (item);
	guint32 size = 5;

	g_return_if_fail (self->info != NULL);

	if (self->info->creation_time)
		size += 7;

	if (self->info->modify_time)
		size += 7;

	if (self->info->access_time)
		size += 7;

	if (self->info->attributes_time)
		size += 7;

	if (self->info->backup_time)
		size += 7;

	if (self->info->expiration_time)
		size += 7;

	if (self->info->effective_time)
		size += 7;

	if (n_s)
		*n_s = 0;
	if (n_b)
		*n_b = size;
}

static void
_burgner_isofs_rrip_tf_write_sync (BurgnerIsofsItem *item,
				   BurgnerIsofsMapTable *table,
				   gpointer buf, guint32 size)
{
	BurgnerIsofsRripTf *self = BURGNER_ISOFS_RRIP_TF (item);
	__BurgnerIsofsImplRripTf *impl;
	gpointer buf_end = buf + size;
	guchar flags = __BURGNER_ISOFS_IMPL_RRIP_TF_FLAG_SHORT_FORM;

	g_return_if_fail (self->info != NULL);

	impl = (__BurgnerIsofsImplRripTf *) buf;
	buf += 5;

	impl->header.sig[0] = 'T';
	impl->header.sig[1] = 'F';
	burgner_isofs_set_711 (&impl->header.len, size);
	burgner_isofs_set_711 (&impl->header.version, 1);

	if (self->info->creation_time) {
		burgner_isofs_rrip_tf_fill_951_from_time (buf, self->info->creation_time);
		buf += 7;

		flags |= __BURGNER_ISOFS_IMPL_RRIP_TF_FLAG_CREATION;
	}

	if (self->info->modify_time) {
		burgner_isofs_rrip_tf_fill_951_from_time (buf, self->info->modify_time);
		buf += 7;

		flags |= __BURGNER_ISOFS_IMPL_RRIP_TF_FLAG_MODIFY;
	}

	if (self->info->access_time) {
		burgner_isofs_rrip_tf_fill_951_from_time (buf, self->info->access_time);
		buf += 7;

		flags |= __BURGNER_ISOFS_IMPL_RRIP_TF_FLAG_ACCESS;
	}

	if (self->info->attributes_time) {
		burgner_isofs_rrip_tf_fill_951_from_time (buf, self->info->attributes_time);
		buf += 7;

		flags |= __BURGNER_ISOFS_IMPL_RRIP_TF_FLAG_ATTRIBUTES;
	}

	if (self->info->backup_time) {
		burgner_isofs_rrip_tf_fill_951_from_time (buf, self->info->backup_time);
		buf += 7;

		flags |= __BURGNER_ISOFS_IMPL_RRIP_TF_FLAG_BACKUP;
	}

	if (self->info->expiration_time) {
		burgner_isofs_rrip_tf_fill_951_from_time (buf, self->info->expiration_time);
		buf += 7;

		flags |= __BURGNER_ISOFS_IMPL_RRIP_TF_FLAG_EXPIRATION;
	}

	if (self->info->effective_time) {
		burgner_isofs_rrip_tf_fill_951_from_time (buf, self->info->effective_time);
		buf += 7;

		flags |= __BURGNER_ISOFS_IMPL_RRIP_TF_FLAG_EFFECTIVE;
	}

	burgner_isofs_set_711 (&impl->flags, flags);

	g_return_if_fail (buf == buf_end);
}

static void
_burgner_isofs_rrip_tf_class_init (BurgnerIsofsRripTfClass *klass)
{
	GObjectClass *object_class;
	BurgnerIsofsItemClass *item_class;

	object_class = (GObjectClass *) klass;
	object_class->set_property = _burgner_isofs_rrip_tf_set_property;
	object_class->get_property = _burgner_isofs_rrip_tf_get_property;
	object_class->finalize = _burgner_isofs_rrip_tf_finalize;

	item_class = (BurgnerIsofsItemClass *) klass;
	item_class->get_size = _burgner_isofs_rrip_tf_get_size;
	item_class->write_sync = _burgner_isofs_rrip_tf_write_sync;

	/* Properties */
	g_object_class_install_property (object_class,
					 PROP_INFO,
					 g_param_spec_pointer ("info",
					                _("info"),
							_("A BurgnerIsofsFileInfo"),
							G_PARAM_READABLE | G_PARAM_WRITABLE));


	parent_class = g_type_class_peek_parent (klass);
}

static void
_burgner_isofs_rrip_tf_init (BurgnerIsofsRripTf *self)
{
	self->info = NULL;
}

GType
burgner_isofs_rrip_tf_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		static GTypeInfo info = {
			sizeof (BurgnerIsofsRripTfClass),
			NULL,
			NULL,
			(GClassInitFunc) _burgner_isofs_rrip_tf_class_init,
			NULL,
			NULL,
			sizeof (BurgnerIsofsRripTf),
			0,
			(GInstanceInitFunc) _burgner_isofs_rrip_tf_init,
		};

		type = g_type_register_static (BURGNER_TYPE_ISOFS_SUSP_ENTRY, "BurgnerIsofsRripTf", &info, 0);
	}

	return type;
}

BurgnerIsofsItem *
burgner_isofs_rrip_tf_new (void)
{
	BurgnerIsofsRripTf *self;

	self = g_object_new (BURGNER_TYPE_ISOFS_RRIP_TF, NULL);
	

	return BURGNER_ISOFS_ITEM (self);
}


GQuark
burgner_isofs_rrip_tf_error_quark (void)
{
	return g_quark_from_string ("BurgnerIsofsRripTf");
}

void
burgner_isofs_rrip_tf_fill_951_from_time (gpointer buf, GTime time)
{
	__BurgnerIsofsImpl915Date *t = (__BurgnerIsofsImpl915Date *) buf;
	struct tm tm;
	time_t time_v = time;

	g_return_if_fail (buf != NULL);

	memset (&tm, 0, sizeof (tm));
	gmtime_r (&time_v, &tm);

	burgner_isofs_set_711 (&t->year, tm.tm_year);
	burgner_isofs_set_711 (&t->month, tm.tm_mon + 1);
	burgner_isofs_set_711 (&t->day, tm.tm_mday);
	burgner_isofs_set_711 (&t->hour, tm.tm_hour);
	burgner_isofs_set_711 (&t->minute, tm.tm_min);
	burgner_isofs_set_711 (&t->second, tm.tm_sec);
}

