/*
 * BurgnerIsofsSuspEr - BurgnerIsofsSuspEr object
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include <glib-object.h>
#include "burgner-isofs-impl.h"
#include "burgner-isofs-utils.h"
#include "burgner-isofs-susp-er.h"


static BurgnerIsofsSuspEntryClass *parent_class = NULL;

enum {
	PROP_0,
	PROP_ID,
	PROP_DESC,
	PROP_SRC,
	PROP_VER,
	LAST_ARG
};

static void
finalize (GObject *object)
{
	BurgnerIsofsSuspEr *self = BURGNER_ISOFS_SUSP_ER (object);

	if (self->id) {
		g_free (self->id);
		self->id = NULL;
		self->id_len = 0;
	}

	if (self->desc) {
		g_free (self->desc);
		self->desc = NULL;
		self->desc_len = 0;
	}

	if (self->src) {
		g_free (self->src);
		self->src = NULL;
		self->src_len = 0;
	}

	self->ext_ver = 1;

	if (G_OBJECT_CLASS (parent_class)->finalize)
		G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
set_property (GObject *object,
	      guint prop_id,
	      const GValue *value,
	      GParamSpec *pspec)
{
	BurgnerIsofsSuspEr *self = BURGNER_ISOFS_SUSP_ER (object);

	switch (prop_id) {
		case PROP_ID:
			if (self->id)
				g_free (self->id);
			self->id = g_value_dup_string (value);
			self->id_len = (self->id ? strlen (self->id) : 0);
			g_object_notify (object, "id");
			break;
		case PROP_DESC:
			if (self->desc)
				g_free (self->desc);
			self->desc = g_value_dup_string (value);
			self->desc_len = (self->desc ? strlen (self->desc) : 0);
			g_object_notify (object, "desc");
			break;
		case PROP_SRC:
			if (self->src)
				g_free (self->src);
			self->src = g_value_dup_string (value);
			self->src_len = (self->src ? strlen (self->src) : 0);
			g_object_notify (object, "src");
			break;
		case PROP_VER:
			self->ext_ver = g_value_get_uint (value);
			g_object_notify (object, "ext_ver");
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
get_property (GObject *object,
	      guint prop_id,
	      GValue *value,
	      GParamSpec *pspec)
{
	BurgnerIsofsSuspEr *self = BURGNER_ISOFS_SUSP_ER (object);

	switch (prop_id) {
		case PROP_ID:
			g_value_set_string (value, self->id);
			break;
		case PROP_DESC:
			g_value_set_string (value, self->desc);
			break;
		case PROP_SRC:
			g_value_set_string (value, self->src);
			break;
		case PROP_VER:
			g_value_set_uint (value, self->ext_ver);
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
_biser_get_size (BurgnerIsofsItem *item,
		 guint32 *n_sectors,
		 guint32 *n_bytes)
{
	BurgnerIsofsSuspEr *er = BURGNER_ISOFS_SUSP_ER (item);
	guint32 b = 8 + er->id_len + er->desc_len + er->src_len;

	if (n_sectors)
		*n_sectors = 0;
	if (n_bytes)
		*n_bytes = b;
}

static void
_biser_write_sync (BurgnerIsofsItem *item,
		   BurgnerIsofsMapTable *table,
		   gpointer buf, guint32 size)
{
	__BurgnerIsofsImplRripEr *impl;
	BurgnerIsofsSuspEr *er = BURGNER_ISOFS_SUSP_ER (item);

	g_return_if_fail (size == (8 + er->id_len + er->desc_len + er->src_len));

	impl = (__BurgnerIsofsImplRripEr *) buf;
	memset (impl, 0, sizeof (*impl));

	impl->header.sig[0] = 'E';
	impl->header.sig[1] = 'R';
	burgner_isofs_set_711 (&impl->header.len, size);
	burgner_isofs_set_711 (&impl->header.version, 1);

	burgner_isofs_set_711 (&impl->id_len, er->id_len);
	burgner_isofs_set_711 (&impl->desc_len, er->desc_len);
	burgner_isofs_set_711 (&impl->src_len, er->src_len);

	burgner_isofs_set_711 (&impl->ext_version, er->ext_ver);

#if 0
	burgner_isofs_writer_proxy_write (proxy, (guchar*) &impl, 8);
	burgner_isofs_writer_proxy_write (proxy, er->id, er->id_len);
	burgner_isofs_writer_proxy_write (proxy, er->desc, er->desc_len);
	burgner_isofs_writer_proxy_write (proxy, er->src, er->src_len);
#else
	buf += 8;

	memcpy (buf, er->id, er->id_len);
	buf += er->id_len;

	memcpy (buf, er->desc, er->desc_len);
	buf += er->desc_len;

	memcpy (buf, er->src, er->src_len);
#endif
}

static BurgnerIsofsItem *
_biser_dup (BurgnerIsofsSuspEntry *ent)
{
	BurgnerIsofsSuspEr *er = BURGNER_ISOFS_SUSP_ER (ent);
	BurgnerIsofsSuspEr *d;

	d = BURGNER_ISOFS_SUSP_ER (burgner_isofs_susp_er_new ());
	d->id = er->id ? g_strdup (er->id) : NULL;
	d->desc = er->desc ? g_strdup (er->desc) : NULL;
	d->src = er->src ? g_strdup (er->src) : NULL;

	d->id_len = er->id_len;
	d->desc_len = er->desc_len;
	d->src_len = er->src_len;
	d->ext_ver = er->ext_ver;

	return BURGNER_ISOFS_ITEM (d);
}

static void
class_init (BurgnerIsofsSuspErClass *klass)
{
	GObjectClass *object_class;
	BurgnerIsofsItemClass *item_class;
	BurgnerIsofsSuspEntryClass *ent_class;

	object_class = (GObjectClass *) klass;
	object_class->set_property = set_property;
	object_class->get_property = get_property;
	object_class->finalize = finalize;

	item_class = (BurgnerIsofsItemClass *) klass;
	item_class->get_size = _biser_get_size;
	item_class->write_sync = _biser_write_sync;

	ent_class = (BurgnerIsofsSuspEntryClass *) klass;
	ent_class->dup = _biser_dup;

	/* Properties */
	g_object_class_install_property (object_class,
					 PROP_ID,
					 g_param_spec_string ("id",
					            	      "id",
							      "Identifier",
							      NULL,
							      G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_DESC,
					 g_param_spec_string ("desc",
					                      "desc",
							      "Description",
							      NULL,
							      G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_SRC,
					 g_param_spec_string ("src",
					                      "src",
							      "Source",
							      NULL,
							      G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_VER,
					 g_param_spec_uint ("ext_ver",
					                    "ext_ver",
							    "Extension Verion",
							    0,
							    0xff,
							    1,
							    G_PARAM_READABLE | G_PARAM_WRITABLE));


	parent_class = g_type_class_peek_parent (klass);
}

static void
init (BurgnerIsofsSuspEr *self)
{
	self->id = self->desc = self->src = NULL;
	self->id_len = self->desc_len = self->src_len = 0;
	self->ext_ver = 1;
}

GType
burgner_isofs_susp_er_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		static GTypeInfo info = {
			sizeof (BurgnerIsofsSuspErClass),
			NULL,
			NULL,
			(GClassInitFunc) class_init,
			NULL,
			NULL,
			sizeof (BurgnerIsofsSuspEr),
			0,
			(GInstanceInitFunc) init,
		};

		type = g_type_register_static (BURGNER_TYPE_ISOFS_SUSP_ENTRY, "BurgnerIsofsSuspEr", &info, 0);
	}

	return type;
}

BurgnerIsofsItem *
burgner_isofs_susp_er_new (void)
{
	BurgnerIsofsSuspEr *self;

	self = g_object_new (BURGNER_TYPE_ISOFS_SUSP_ER, NULL);


	return BURGNER_ISOFS_ITEM (self);
}


GQuark
burgner_isofs_susp_er_error_quark (void)
{
	return g_quark_from_string ("BurgnerIsofsSuspEr");
}

