/*
 * BurgnerIsofsSuspSp - BurgnerIsofsSuspSp object
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include <glib-object.h>
#include "burgner-isofs-impl.h"
#include "burgner-isofs-utils.h"
#include "burgner-isofs-susp-sp.h"


static BurgnerIsofsSuspEntryClass *parent_class = NULL;

enum {
	PROP_0,
	PROP_BS,
	LAST_ARG
};

static void
finalize (GObject *object)
{
	BurgnerIsofsSuspSp *self = BURGNER_ISOFS_SUSP_SP (object);

	self->bs = 0;

	if (G_OBJECT_CLASS (parent_class)->finalize)
		G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
set_property (GObject *object,
	      guint prop_id,
	      const GValue *value,
	      GParamSpec *pspec)
{
	BurgnerIsofsSuspSp *self = BURGNER_ISOFS_SUSP_SP (object);

	switch (prop_id) {
		case PROP_BS:
			self->bs = g_value_get_uint (value);
			g_object_notify (object, "bs");
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
get_property (GObject *object,
	      guint prop_id,
	      GValue *value,
	      GParamSpec *pspec)
{
	BurgnerIsofsSuspSp *self = BURGNER_ISOFS_SUSP_SP (object);

	switch (prop_id) {
		case PROP_BS:
			g_value_set_uint (value, self->bs);
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
_bissp_get_size (BurgnerIsofsItem *item,
		 guint32 *n_sectors,
		 guint32 *n_bytes)
{
	if (n_sectors)
		*n_sectors = 0;
	if (n_bytes)
		*n_bytes = 7;
}

static void
_bissp_write_sync (BurgnerIsofsItem *item,
		   BurgnerIsofsMapTable *table,
		   gpointer buf, guint32 size)
{
	__BurgnerIsofsImplRripSp *impl;
	BurgnerIsofsSuspSp *sp = BURGNER_ISOFS_SUSP_SP (item);

	g_return_if_fail (size == 7);

	impl = (__BurgnerIsofsImplRripSp *) buf;
	memset (impl, 0, sizeof (*impl));

	impl->header.sig[0] = 'S';
	impl->header.sig[1] = 'P';
	burgner_isofs_set_711 (&impl->header.len, 7);
	burgner_isofs_set_711 (&impl->header.version, 1);

	impl->be = 0xbe;
	impl->ef = 0xef;
	burgner_isofs_set_711 (&impl->bs, sp->bs);

#if 0
	burgner_isofs_writer_proxy_write (proxy, (guchar*) &impl, 7);
#endif
}

static BurgnerIsofsItem *
_bissp_dup (BurgnerIsofsSuspEntry *ent)
{
	BurgnerIsofsSuspSp *sp = BURGNER_ISOFS_SUSP_SP (ent), *d;

	d = BURGNER_ISOFS_SUSP_SP (burgner_isofs_susp_sp_new ());
	g_object_set (d, "bs", sp->bs, NULL);

	return BURGNER_ISOFS_ITEM (d);
}

static void
class_init (BurgnerIsofsSuspSpClass *klass)
{
	GObjectClass *object_class;
	BurgnerIsofsItemClass *item_class;
	BurgnerIsofsSuspEntryClass *ent_class;

	object_class = (GObjectClass *) klass;
	object_class->set_property = set_property;
	object_class->get_property = get_property;
	object_class->finalize = finalize;

	item_class = (BurgnerIsofsItemClass *) klass;
	item_class->get_size = _bissp_get_size;
	item_class->write_sync = _bissp_write_sync;

	ent_class = (BurgnerIsofsSuspEntryClass *) klass;
	ent_class->dup = _bissp_dup;

	/* Properties */
	g_object_class_install_property (object_class,
					 PROP_BS,
					 g_param_spec_uint ("bs",
					            	    "bs",
							    "Bytes Skipped",
							    0,
							    0xff,
							    0,
							    G_PARAM_READABLE | G_PARAM_WRITABLE));


	parent_class = g_type_class_peek_parent (klass);
}

static void
init (BurgnerIsofsSuspSp *self)
{
	self->bs = 0;
}

GType
burgner_isofs_susp_sp_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		static GTypeInfo info = {
			sizeof (BurgnerIsofsSuspSpClass),
			NULL,
			NULL,
			(GClassInitFunc) class_init,
			NULL,
			NULL,
			sizeof (BurgnerIsofsSuspSp),
			0,
			(GInstanceInitFunc) init,
		};

		type = g_type_register_static (BURGNER_TYPE_ISOFS_SUSP_ENTRY, "BurgnerIsofsSuspSp", &info, 0);
	}

	return type;
}

BurgnerIsofsItem *
burgner_isofs_susp_sp_new (void)
{
	BurgnerIsofsSuspSp *self;

	self = g_object_new (BURGNER_TYPE_ISOFS_SUSP_SP, NULL);

	return BURGNER_ISOFS_ITEM (self);
}


GQuark
burgner_isofs_susp_sp_error_quark (void)
{
	return g_quark_from_string ("BurgnerIsofsSuspSp");
}

