/*
 * BurgnerIsofsTreeBuilder - BurgnerIsofsTreeBuilder object
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib-object.h>
#include "burgner-isofs-file.h"
#include "burgner-isofs-tree-builder.h"


static GObjectClass *parent_class = NULL;

static void
class_init (BurgnerIsofsTreeBuilderClass *klass)
{
	GObjectClass *object_class;

	object_class = (GObjectClass *) klass;

	parent_class = g_type_class_peek_parent (klass);
}

GType
burgner_isofs_tree_builder_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		static GTypeInfo info = {
			sizeof (BurgnerIsofsTreeBuilderClass),
			NULL,
			NULL,
			(GClassInitFunc) class_init,
			NULL,
			NULL,
			sizeof (BurgnerIsofsTreeBuilder),
			0,
			(GInstanceInitFunc) NULL,
		};

		type = g_type_register_static (G_TYPE_OBJECT, "BurgnerIsofsTreeBuilder", &info, G_TYPE_FLAG_ABSTRACT);
	}

	return type;
}

GQuark
burgner_isofs_tree_builder_error_quark (void)
{
	return g_quark_from_string ("BurgnerIsofsTreeBuilder");
}

void
burgner_isofs_tree_builder_begin (BurgnerIsofsTreeBuilder *tb,
		                  GNode *root,
				  BurgnerIsofsItem *last_item)
{
	g_return_if_fail (tb != NULL);
	g_return_if_fail (BURGNER_IS_ISOFS_TREE_BUILDER (tb));
	g_return_if_fail (root != NULL);
	g_return_if_fail (last_item != NULL);
	g_return_if_fail (BURGNER_IS_ISOFS_ITEM (last_item));

	if (BURGNER_ISOFS_TREE_BUILDER_GET_CLASS (tb)->begin)
		BURGNER_ISOFS_TREE_BUILDER_GET_CLASS (tb)->begin (tb, /*it,*/ root, last_item);
}

void
burgner_isofs_tree_builder_end (BurgnerIsofsTreeBuilder *tb,
		                GNode *root,
				BurgnerIsofsItem *last_item)
{
	g_return_if_fail (tb != NULL);
	g_return_if_fail (BURGNER_IS_ISOFS_TREE_BUILDER (tb));
	g_return_if_fail (root != NULL);
	g_return_if_fail (last_item != NULL);
	g_return_if_fail (BURGNER_IS_ISOFS_ITEM (last_item));

	if (BURGNER_ISOFS_TREE_BUILDER_GET_CLASS (tb)->end)
		BURGNER_ISOFS_TREE_BUILDER_GET_CLASS (tb)->end (tb, root, last_item);
}

void
burgner_isofs_tree_builder_directory (BurgnerIsofsTreeBuilder *tb,
			              BurgnerIsofsFileInfo *parent_info,
			              BurgnerIsofsFileInfo *info,
			              guint level)
{
	g_return_if_fail (tb != NULL);
	g_return_if_fail (BURGNER_IS_ISOFS_TREE_BUILDER (tb));
	g_return_if_fail (parent_info != NULL || (level == 1));
	g_return_if_fail (info != NULL);

	if (BURGNER_ISOFS_TREE_BUILDER_GET_CLASS (tb)->directory)
		BURGNER_ISOFS_TREE_BUILDER_GET_CLASS (tb)->directory (tb, parent_info, info, level);
}

void
burgner_isofs_tree_builder_file (BurgnerIsofsTreeBuilder *tb,
		                 BurgnerIsofsFileInfo *parent_info,
		                 BurgnerIsofsFileInfo *info,
		                 BurgnerIsofsItem *file)
{
	g_return_if_fail (tb != NULL);
	g_return_if_fail (BURGNER_IS_ISOFS_TREE_BUILDER (tb));
	g_return_if_fail (parent_info != NULL);
	g_return_if_fail (info != NULL);
	g_return_if_fail ((file == NULL) || BURGNER_IS_ISOFS_ITEM (file));

	if (BURGNER_ISOFS_TREE_BUILDER_GET_CLASS (tb)->file)
		BURGNER_ISOFS_TREE_BUILDER_GET_CLASS (tb)->file (tb, parent_info, info, file);
}

GList *
burgner_isofs_tree_builder_get_volume_descriptors (BurgnerIsofsTreeBuilder *tb)
{
	g_return_val_if_fail (tb != NULL, NULL);
	g_return_val_if_fail (BURGNER_IS_ISOFS_TREE_BUILDER (tb), NULL);

	if (BURGNER_ISOFS_TREE_BUILDER_GET_CLASS (tb)->get_volume_descriptors)
		return BURGNER_ISOFS_TREE_BUILDER_GET_CLASS (tb)->get_volume_descriptors (tb);

	return NULL;
}

GList *
burgner_isofs_tree_builder_get_items (BurgnerIsofsTreeBuilder *tb)
{
	g_return_val_if_fail (tb != NULL, NULL);
	g_return_val_if_fail (BURGNER_IS_ISOFS_TREE_BUILDER (tb), NULL);

	if (BURGNER_ISOFS_TREE_BUILDER_GET_CLASS (tb)->get_items)
		return BURGNER_ISOFS_TREE_BUILDER_GET_CLASS (tb)->get_items (tb);

	return NULL;
}

BurgnerIsofsItem *
burgner_isofs_tree_builder_create_file (BurgnerIsofsTreeBuilder *tb,
					BurgnerIsofsFileInfo *info)
{
	g_return_val_if_fail (tb != NULL, NULL);
	g_return_val_if_fail (BURGNER_IS_ISOFS_TREE_BUILDER (tb), NULL);
	g_return_val_if_fail (info != NULL, NULL);

	if (BURGNER_ISOFS_TREE_BUILDER_GET_CLASS (tb)->create_file)
		return BURGNER_ISOFS_TREE_BUILDER_GET_CLASS (tb)->create_file (tb, info);

	return NULL;
}

