/*
 * BurgnerIsofsTreeBuilder - BurgnerIsofsTreeBuilder object
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifndef __BURGNER_ISOFS_TREE_BUILDER_H__
#define __BURGNER_ISOFS_TREE_BUILDER_H__

#include <glib-object.h>
#include <libburgnerisofs/burgner-isofs-item.h>
#include <libburgnerisofs/burgner-isofs-tree.h>

G_BEGIN_DECLS


#define BURGNER_TYPE_ISOFS_TREE_BUILDER		(burgner_isofs_tree_builder_get_type ())
#define BURGNER_ISOFS_TREE_BUILDER(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), BURGNER_TYPE_ISOFS_TREE_BUILDER, BurgnerIsofsTreeBuilder))
#define BURGNER_ISOFS_TREE_BUILDER_CLASS(k)	(G_TYPE_CHECK_CLASS_CAST ((k), BURGNER_TYPE_ISOFS_TREE_BUILDER, BurgnerIsofsTreeBuilderClass))
#define BURGNER_IS_ISOFS_TREE_BUILDER(o)	(G_TYPE_CHECK_INSTANCE_TYPE ((o), BURGNER_TYPE_ISOFS_TREE_BUILDER))
#define BURGNER_IS_ISOFS_TREE_BUILDER_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), BURGNER_TYPE_ISOFS_TREE_BUILDER))
#define BURGNER_ISOFS_TREE_BUILDER_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), BURGNER_TYPE_ISOFS_TREE_BUILDER, BurgnerIsofsTreeBuilderClass))


typedef struct _BurgnerIsofsTreeBuilder BurgnerIsofsTreeBuilder;
typedef struct _BurgnerIsofsTreeBuilderClass BurgnerIsofsTreeBuilderClass;

struct _BurgnerIsofsTreeBuilder {
	GObject __parent;

};

struct _BurgnerIsofsTreeBuilderClass {
	GObjectClass __parent_class;

	/*
	 * Virtual Functions
	 */
	void (* begin)(BurgnerIsofsTreeBuilder *tb,
		       GNode *root,
		       BurgnerIsofsItem *last_item);
	void (* end)(BurgnerIsofsTreeBuilder *tb,
		     GNode *root,
		     BurgnerIsofsItem *last_item);

	void (* directory)(BurgnerIsofsTreeBuilder *tb,
			   BurgnerIsofsFileInfo *parent_info,
			   BurgnerIsofsFileInfo *info,
			   guint level);
	void (* file)(BurgnerIsofsTreeBuilder *tb,
		      BurgnerIsofsFileInfo *parent_info,
		      BurgnerIsofsFileInfo *info,
		      BurgnerIsofsItem *file);

	GList *(* get_volume_descriptors)(BurgnerIsofsTreeBuilder *tb);
	GList *(* get_items)(BurgnerIsofsTreeBuilder *tb);

	BurgnerIsofsItem *(* create_file)(BurgnerIsofsTreeBuilder *tb,
					  BurgnerIsofsFileInfo *info);
};


GType burgner_isofs_tree_builder_get_type (void) G_GNUC_CONST;
GQuark  burgner_isofs_tree_builder_error_quark (void);

#define BURGNER_ISOFS_TREE_BUILDER_ERROR		(burgner_isofs_tree_builder_error_quark ())


void burgner_isofs_tree_builder_begin (BurgnerIsofsTreeBuilder *tb,
		                       GNode *root,
		                       BurgnerIsofsItem *last_item);
void burgner_isofs_tree_builder_end (BurgnerIsofsTreeBuilder *tb,
		                     GNode *root,
		                     BurgnerIsofsItem *last_item);

void burgner_isofs_tree_builder_directory (BurgnerIsofsTreeBuilder *tb,
					   BurgnerIsofsFileInfo *parent_info,
			                   BurgnerIsofsFileInfo *info,
			                   guint level);
void burgner_isofs_tree_builder_file (BurgnerIsofsTreeBuilder *tb,
				      BurgnerIsofsFileInfo *parent_info,
		                      BurgnerIsofsFileInfo *info,
		                      BurgnerIsofsItem *file);

GList *burgner_isofs_tree_builder_get_volume_descriptors (BurgnerIsofsTreeBuilder *tb);
GList *burgner_isofs_tree_builder_get_items (BurgnerIsofsTreeBuilder *tb);

BurgnerIsofsItem *burgner_isofs_tree_builder_create_file (BurgnerIsofsTreeBuilder *tb,
							  BurgnerIsofsFileInfo *info);

G_END_DECLS

#endif /* __BURGNER_ISOFS_TREE_BUILDER_H__ */

