/*
 * 
 *
 * This file is part of the libburgner library.
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifndef __BURGNER_ISOFS_TREE_VOLUME_DESCRIPTOR_UTILS_H__
#define __BURGNER_ISOFS_TREE_VOLUME_DESCRIPTOR_UTILS_H__

#include <glib-object.h>

G_BEGIN_DECLS

#define BURGNER_TYPE_ISOFS_TREE_VOLUME_DESCRIPTOR_IDENTIFIER	(burgner_isofs_tree_volume_descriptor_identifier_get_type (void))
#define BURGNER_TYPE_ISOFS_TREE_VOLUME_DESCRIPTOR_DATE		(burgner_isofs_tree_volume_descriptor_date_get_type (void))


typedef enum {
	BURGNER_ISOFS_TREE_VOLUME_DESCRIPTOR_IDENTIFIER_TYPE_NONE = 0,
	BURGNER_ISOFS_TREE_VOLUME_DESCRIPTOR_IDENTIFIER_TYPE_STR,
	BURGNER_ISOFS_TREE_VOLUME_DESCRIPTOR_IDENTIFIER_TYPE_FILE
} BurgnerIsofsTreeVolumeDescriptorIdentifierType;

typedef struct {
	gchar *str;
	BurgnerIsofsTreeVolumeDescriptorIdentifierType type;
} BurgnerIsofsTreeVolumeDescriptorIdentifier;

typedef struct {
	guint16 year;		/* From 1 to 9999 */
	guint8 month;		/* From 1 to 12 */
	guint8 day;		/* From 1 to 31 */
	guint8 hour;		/* From 0 to 23 */
	guint8 minute;		/* From 0 to 59 */
	guint8 second;		/* From 0 to 59 */
	guint8 hundredth;	/* From 0 to 99 */
	gint8 offset_from_gmt;	/* From -48 to +52 */
} BurgnerIsofsTreeVolumeDescriptorDate;

#if 0
extern const BurgnerIsofsTreeVolumeDescriptorIdentifier burgner_isofs_tree_volume_descriptor_identifier_void;
extern const BurgnerIsofsTreeVolumeDescriptorDate burgner_isofs_tree_volume_descriptor_date_void;
#else
static const BurgnerIsofsTreeVolumeDescriptorIdentifier
burgner_isofs_tree_volume_descriptor_identifier_void = {
	NULL,
	BURGNER_ISOFS_TREE_VOLUME_DESCRIPTOR_IDENTIFIER_TYPE_NONE
};

static const BurgnerIsofsTreeVolumeDescriptorDate
burgner_isofs_tree_volume_descriptor_date_void = {
	0, 0, 0, 0, 0, 0, 0, 0
};
#endif


GType burgner_isofs_tree_volume_descriptor_identifier_get_type (void) G_GNUC_CONST;
GType burgner_isofs_tree_volume_descriptor_date_get_type (void) G_GNUC_CONST;

void burgner_isofs_tree_volume_descriptor_identifier_static_copy (BurgnerIsofsTreeVolumeDescriptorIdentifier *from,
							          BurgnerIsofsTreeVolumeDescriptorIdentifier *to);
void burgner_isofs_tree_volume_descriptor_identifier_static_free (BurgnerIsofsTreeVolumeDescriptorIdentifier *id);

G_END_DECLS

#endif /* __BURGNER_ISOFS_TREE_VOLUME_DESCRIPTOR_UTILS_H__ */
