/*
 * BurgnerIsofsTreeVolumeDescriptor - BurgnerIsofsTreeVolumeDescriptor object
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifndef __BURGNER_ISOFS_TREE_VOLUME_DESCRIPTOR_H__
#define __BURGNER_ISOFS_TREE_VOLUME_DESCRIPTOR_H__

#include <glib-object.h>
#include <libburgnerisofs/burgner-isofs-tree-volume-descriptor-utils.h>
#include <libburgnerisofs/burgner-isofs-volume-descriptor.h>

G_BEGIN_DECLS


#define BURGNER_TYPE_ISOFS_TREE_VOLUME_DESCRIPTOR		(burgner_isofs_tree_volume_descriptor_get_type ())
#define BURGNER_ISOFS_TREE_VOLUME_DESCRIPTOR(o)			(G_TYPE_CHECK_INSTANCE_CAST ((o), BURGNER_TYPE_ISOFS_TREE_VOLUME_DESCRIPTOR, BurgnerIsofsTreeVolumeDescriptor))
#define BURGNER_ISOFS_TREE_VOLUME_DESCRIPTOR_CLASS(k)		(G_TYPE_CHECK_CLASS_CAST ((k), BURGNER_TYPE_ISOFS_TREE_VOLUME_DESCRIPTOR, BurgnerIsofsTreeVolumeDescriptorClass))
#define BURGNER_IS_ISOFS_TREE_VOLUME_DESCRIPTOR(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), BURGNER_TYPE_ISOFS_TREE_VOLUME_DESCRIPTOR))
#define BURGNER_IS_ISOFS_TREE_VOLUME_DESCRIPTOR_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), BURGNER_TYPE_ISOFS_TREE_VOLUME_DESCRIPTOR))
#define BURGNER_ISOFS_TREE_VOLUME_DESCRIPTOR_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), BURGNER_TYPE_ISOFS_TREE_VOLUME_DESCRIPTOR, BurgnerIsofsTreeVolumeDescriptorClass))


typedef struct _BurgnerIsofsTreeVolumeDescriptor BurgnerIsofsTreeVolumeDescriptor;
typedef struct _BurgnerIsofsTreeVolumeDescriptorClass BurgnerIsofsTreeVolumeDescriptorClass;

struct _BurgnerIsofsTreeVolumeDescriptor {
	BurgnerIsofsVolumeDescriptor __parent;

	BurgnerIsofsTreeVolumeDescriptorDate c_date;
	BurgnerIsofsTreeVolumeDescriptorDate m_date;
	BurgnerIsofsTreeVolumeDescriptorDate ex_date;
	BurgnerIsofsTreeVolumeDescriptorDate eff_date;

	BurgnerIsofsTreeVolumeDescriptorIdentifier publisher_id;
	BurgnerIsofsTreeVolumeDescriptorIdentifier data_preparer_id;
	BurgnerIsofsTreeVolumeDescriptorIdentifier application_id;

	gchar *system_id;
	gchar *volume_id;

	BurgnerIsofsItem *last_item;

	BurgnerIsofsItem *l_pt;
	BurgnerIsofsItem *m_pt;

	BurgnerIsofsItem *root_dir_rec;

	gchar *volume_set_id;
	gchar *copyright_file_id;
	gchar *abstract_file_id;
	gchar *bibliographic_file_id;

	/* SVD only */ GString *escape_seq;

	guint16 volume_set_size;
	guint16 volume_sequence_number;
	guint16 lb_size;

	guint8 file_structure_version;

	/* SVD only */ guint8 volume_flags;

	guint is_svd : 1;
};

struct _BurgnerIsofsTreeVolumeDescriptorClass {
	BurgnerIsofsVolumeDescriptorClass __parent_class;

};


GType burgner_isofs_tree_volume_descriptor_get_type (void) G_GNUC_CONST;
GQuark  burgner_isofs_tree_volume_descriptor_error_quark (void);

#define BURGNER_ISOFS_TREE_VOLUME_DESCRIPTOR_ERROR		(burgner_isofs_tree_volume_descriptor_error_quark ())


BurgnerIsofsItem *burgner_isofs_tree_volume_descriptor_new (void);

G_END_DECLS

#endif /* __BURGNER_ISOFS_TREE_VOLUME_DESCRIPTOR_H__ */

