/*
 * 
 *
 * This file is part of the libburgnerisofs library.
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */

#ifndef __BURGNER_ISOFS_TREE_H__
#define __BURGNER_ISOFS_TREE_H__

#include <glib.h>
#include <libburgnerisofs/burgner-isofs-identifier.h>
#include <libburgnerisofs/burgner-isofs-file-info.h>

G_BEGIN_DECLS

typedef struct {
	GNode *root;
	GHashTable *name_to_node;

	guint refcount;
} BurgnerIsofsTree;


BurgnerIsofsTree *burgner_isofs_tree_new (void);

BurgnerIsofsTree *burgner_isofs_tree_ref (BurgnerIsofsTree *tree);
void burgner_isofs_tree_unref (BurgnerIsofsTree *tree);

void burgner_isofs_tree_free (BurgnerIsofsTree *tree);

GNode *burgner_isofs_tree_get_root_node (BurgnerIsofsTree *tree);

BurgnerIsofsFileInfo *burgner_isofs_tree_get_root_info (BurgnerIsofsTree *tree);

BurgnerIsofsFileInfo *burgner_isofs_tree_get_info_from_path (BurgnerIsofsTree *tree,
							     const gchar *path,
							     gboolean create);

BurgnerIsofsFileInfo *burgner_isofs_tree_add_file (BurgnerIsofsTree *tree,
						   BurgnerIsofsFileInfo *parent_info,
						   const gchar *name,
						   gboolean in_prev_session);
BurgnerIsofsFileInfo *burgner_isofs_tree_add_directory (BurgnerIsofsTree *tree,
							BurgnerIsofsFileInfo *parent_info,
							const gchar *name,
							gboolean in_prev_session);

void burgner_isofs_tree_create_identifiers (BurgnerIsofsTree *tree,
					    BurgnerIsofsIdentifierOptions options);

G_END_DECLS

#endif /* __BURGNER_ISOFS_TREE_H__ */

