/*
 * 
 *
 * This file is part of the libburgner library.
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "burgner-isofs-tree-volume-descriptor.h"
#include "burgner-isofs-tvd-info.h"


void
burgner_isofs_tvd_info_setup_tvd (BurgnerIsofsTvdInfo *info,
				  BurgnerIsofsItem *tvd)
{
	g_return_if_fail (info != NULL);
	g_return_if_fail (tvd != NULL);
	g_return_if_fail (BURGNER_IS_ISOFS_TREE_VOLUME_DESCRIPTOR (tvd));

	g_object_set (tvd,
		      "c_date", &info->c_date,
		      "m_date", &info->m_date,
		      "ex_date", &info->ex_date,
		      "eff_date", &info->eff_date,
		      "system_id", info->system_id,
		      "volume_id", info->volume_id,
		      "volume_set_id", info->volume_set_id,
		      "publisher_id", &info->publisher_id,
		      "data_preparer_id", &info->data_preparer_id,
		      "application_id", &info->application_id,
		      "copyright_file_id", info->copyright_file_id,
		      "abstract_file_id", info->abstract_file_id,
		      "bibliographic_file_id", info->copyright_file_id,
		      "volume_set_size", (guint) info->volume_set_size,
		      "volume_sequence_number", (guint) info->volume_sequence_number,
		      NULL);
}

void
burgner_isofs_tvd_info_static_copy (BurgnerIsofsTvdInfo *from,
			            BurgnerIsofsTvdInfo *to)
{
	g_return_if_fail (from != NULL);
	g_return_if_fail (to != NULL);

	to->c_date = from->c_date;
	to->m_date = from->m_date;
	to->ex_date = from->ex_date;
	to->eff_date = from->eff_date;

	burgner_isofs_tree_volume_descriptor_identifier_static_copy (&from->publisher_id, &to->publisher_id);
	burgner_isofs_tree_volume_descriptor_identifier_static_copy (&from->data_preparer_id, &to->data_preparer_id);
	burgner_isofs_tree_volume_descriptor_identifier_static_copy (&from->application_id, &to->application_id);

	to->system_id = g_strdup (from->system_id);
	to->volume_id = g_strdup (from->volume_id);
	to->volume_set_id = g_strdup (from->volume_set_id);

	to->copyright_file_id = g_strdup (from->copyright_file_id);
	to->abstract_file_id = g_strdup (from->abstract_file_id);
	to->bibliographic_file_id = g_strdup (from->bibliographic_file_id);

	to->volume_set_size = from->volume_set_size;
	to->volume_sequence_number = from->volume_sequence_number;
}

void
burgner_isofs_tvd_info_static_free (BurgnerIsofsTvdInfo *info)
{
	g_return_if_fail (info != NULL);

	info->c_date = info->m_date = info->ex_date = info->eff_date = burgner_isofs_tree_volume_descriptor_date_void;

	burgner_isofs_tree_volume_descriptor_identifier_static_free (&info->publisher_id);
	burgner_isofs_tree_volume_descriptor_identifier_static_free (&info->data_preparer_id);
	burgner_isofs_tree_volume_descriptor_identifier_static_free (&info->application_id);

	if (info->system_id) {
		g_free (info->system_id);
		info->system_id = NULL;
	}

	if (info->volume_id) {
		g_free (info->volume_id);
		info->volume_id = NULL;
	}

	if (info->volume_set_id) {
		g_free (info->volume_set_id);
		info->volume_set_id = NULL;
	}


	if (info->copyright_file_id) {
		g_free (info->copyright_file_id);
		info->copyright_file_id = NULL;
	}

	if (info->abstract_file_id) {
		g_free (info->abstract_file_id);
		info->abstract_file_id = NULL;
	}

	if (info->bibliographic_file_id) {
		g_free (info->bibliographic_file_id);
		info->bibliographic_file_id = NULL;
	}

	info->volume_set_size = 1;
	info->volume_sequence_number = 1;
}
