/*
 * BurgnerIsofsVolumeDescriptor Utils
 *
 * This file is part of the libburgner library.
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */

#ifndef __BURGNER_ISOFS_VOLUME_DESCRIPTOR_UTILS_H__
#define __BURGNER_ISOFS_VOLUME_DESCRIPTOR_UTILS_H__

#include <glib.h>

G_BEGIN_DECLS


typedef enum {
	BURGNER_ISOFS_VOLUME_DESCRIPTOR_IDENTIFIER_TYPE_NONE = 0,
	BURGNER_ISOFS_VOLUME_DESCRIPTOR_IDENTIFIER_TYPE_STR,
	BURGNER_ISOFS_VOLUME_DESCRIPTOR_IDENTIFIER_TYPE_FILE
} BurgnerIsofsVolumeDescriptorIdentifierType;

typedef struct {
	gchar *str;
	BurgnerIsofsVolumeDescriptorIdentifierType type;
} BurgnerIsofsVolumeDescriptorIdentifier;

typedef struct {
	guint16 year;		/* From 1 to 9999 */
	guint8 month;		/* From 1 to 12 */
	guint8 day;		/* From 1 to 31 */
	guint8 hour;		/* From 0 to 23 */
	guint8 minute;		/* From 0 to 59 */
	guint8 second;		/* From 0 to 59 */
	guint8 hundredth;	/* From 0 to 99 */
	gint8 offset_from_gmt;	/* From -48 to +52 */
} BurgnerIsofsVolumeDescriptorDate;

enum {
	BURGNER_ISOFS_VOLUME_DESCRIPTOR_TYPE_BOOT_RECORD = 0,
	BURGNER_ISOFS_VOLUME_DESCRIPTOR_TYPE_PRIMARY = 1,
	BURGNER_ISOFS_VOLUME_DESCRIPTOR_TYPE_SUPPLEMENTARY = 2,
	BURGNER_ISOFS_VOLUME_DESCRIPTOR_TYPE_PARTITION = 3,
	BURGNER_ISOFS_VOLUME_DESCRIPTOR_TYPE_SET_TERMINATOR = 255
};


extern const BurgnerIsofsVolumeDescriptorIdentifier burgner_isofs_volume_descriptor_identifier_void;
extern const BurgnerIsofsVolumeDescriptorDate burgner_isofs_volume_descriptor_date_void;


G_END_DECLS

#endif /* __BURGNER_ISOFS_VOLUME_DESCRIPTOR_UTILS_H__ */

