/*
 * BurgnerIsofsWriter - BurgnerIsofsWriter object
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifndef __BURGNER_ISOFS_WRITER_H__
#define __BURGNER_ISOFS_WRITER_H__

#include <glib.h>
#include <libburgnerisofs/burgner-isofs-item.h>

G_BEGIN_DECLS


typedef struct _BurgnerIsofsWriter BurgnerIsofsWriter;

typedef void (* BurgnerIsofsWriterWriteAsyncCallback)(BurgnerIsofsWriter *writer,
							gpointer buf, guint32 buf_size,
							guint32 bytes_written,
							GError *error,
							gpointer cb_data);

struct _BurgnerIsofsWriter {
	BurgnerIsofsMapTable *map_table;
	GList *items;

	GList *next;
	guint32 s_c;
	guint32 b_o;

	BurgnerIsofsItem *async_item;
	gpointer write_async_data;
	guint32 async_size;
	guint32 async_remaining;

	gpointer buf;
	guint32 buf_size;
	BurgnerIsofsWriterWriteAsyncCallback cb;
	gpointer cb_data;

	GString *tmp_buf;
	gsize tmp_buf_offset;

	guint32 blank;

	guint refcount;
};


GQuark  burgner_isofs_writer_error_quark (void) G_GNUC_CONST;
#define BURGNER_ISOFS_WRITER_ERROR		(burgner_isofs_writer_error_quark ())


BurgnerIsofsWriter *burgner_isofs_writer_new (GList *items);

BurgnerIsofsWriter *burgner_isofs_writer_ref (BurgnerIsofsWriter *self);
void burgner_isofs_writer_unref (BurgnerIsofsWriter *self);

void burgner_isofs_writer_free (BurgnerIsofsWriter *self);


void burgner_isofs_writer_prepare (BurgnerIsofsWriter *writer,
				   guint32 first_sector_lba);

guint32 burgner_isofs_writer_write_sync (BurgnerIsofsWriter *writer,
					 gpointer buf, guint32 buf_size,
					 gboolean *next_is_async);

void burgner_isofs_writer_write_async (BurgnerIsofsWriter *writer,
				       gpointer buf, guint32 buf_size,
				       BurgnerIsofsWriterWriteAsyncCallback cb,
				       gpointer cb_data);


guint32 burgner_isofs_writer_to_write_sync (BurgnerIsofsWriter *writer); /* Maximum */
guint32 burgner_isofs_writer_to_write_async (BurgnerIsofsWriter *writer); /* Minimum */
gboolean burgner_isofs_writer_end_reached (BurgnerIsofsWriter *writer);


G_END_DECLS

#endif /* __BURGNER_ISOFS_WRITER_H__ */

