/*
 * 
 *
 * This file is part of the libburgnerscsi library.
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>

#include "../libburgnerscsi/burgner-scsi-utils.h"
#include "burgner-mmc-impl.h"
#include "burgner-mmc-blank.h"

static guchar
_burgner_mmc_blank_type_to_impl (BurgnerMmcBlankType type)
{
	guchar r = 0;

	switch (type) {
		case BURGNER_MMC_BLANK_TYPE_DISC:
			r = _BURGNER_MMC_IMPL_BLANKING_TYPE_DISC;
			break;
		case BURGNER_MMC_BLANK_TYPE_MINIMALLY_DISC:
			r = _BURGNER_MMC_IMPL_BLANKING_TYPE_MINIMALLY_DISC;
			break;
		case BURGNER_MMC_BLANK_TYPE_TRACK:
			r = _BURGNER_MMC_IMPL_BLANKING_TYPE_TRACK;
			break;
		case BURGNER_MMC_BLANK_TYPE_UNRESERVE_TRACK:
			r = _BURGNER_MMC_IMPL_BLANKING_TYPE_UNRESERVE_TRACK;
			break;
		case BURGNER_MMC_BLANK_TYPE_TRACK_TAIL:
			r = _BURGNER_MMC_IMPL_BLANKING_TYPE_TRACK_TAIL;
			break;
		case BURGNER_MMC_BLANK_TYPE_UNCLOSE_LAST_SESSION:
			r = _BURGNER_MMC_IMPL_BLANKING_TYPE_UNCLOSE_LAST_SESSION;
			break;
		case BURGNER_MMC_BLANK_TYPE_LAST_SESSION:
			r = _BURGNER_MMC_IMPL_BLANKING_TYPE_LAST_SESSION;
			break;
		default:
			g_assert_not_reached ();
			break;
	}

	return r;
}

void
burgner_mmc_blank_write_cdb (gpointer cdb_buf, guint cdb_buf_size,
			     BurgnerMmcBlankType blank_type,
			     guint track_num_or_start_lba)
{
	_BurgnerMmcImplBlankCdb *impl;

	g_return_if_fail (cdb_buf != NULL);
	g_return_if_fail (cdb_buf_size >= BURGNER_MMC_BLANK_CDB_SIZE);

	impl = (_BurgnerMmcImplBlankCdb *) cdb_buf;
	memset (impl, 0, BURGNER_MMC_BLANK_CDB_SIZE);

	impl->op_code = BURGNER_MMC_BLANK_OP_CODE;
	impl->blanking_type = _burgner_mmc_blank_type_to_impl (blank_type);

	if (blank_type == BURGNER_MMC_BLANK_TYPE_TRACK || blank_type == BURGNER_MMC_BLANK_TYPE_TRACK)
		burgner_scsi_set_uint32 (impl->track_num_start_lba, track_num_or_start_lba);

}

