/*
 * 
 *
 * This file is part of the libburgnermmc library.
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "burgner-mmc-data-block-type.h"

GType
burgner_mmc_data_block_type_get_type (void)
{
	static GType etype = 0;

	if (etype == 0) {
		static const GEnumValue values[] = {
			{ BURGNER_MMC_DATA_BLOCK_TYPE_RAW_DATA, "BURGNER_MMC_DATA_BLOCK_TYPE_RAW_DATA", "RAW_DATA" },
			{ BURGNER_MMC_DATA_BLOCK_TYPE_RAW_DATA_WITH_P_Q_SUB_CHANNEL, "BURGNER_MMC_DATA_BLOCK_TYPE_RAW_DATA_WITH_P_Q_SUB_CHANNEL", "RAW_DATA_WITH_P_Q_SUB_CHANNEL" },
			{ BURGNER_MMC_DATA_BLOCK_TYPE_RAW_DARA_WITH_P_W_SUB_CHANNEL_1, "BURGNER_MMC_DATA_BLOCK_TYPE_RAW_DARA_WITH_P_W_SUB_CHANNEL_1", "RAW_DARA_WITH_P_W_SUB_CHANNEL_1" },
			{ BURGNER_MMC_DATA_BLOCK_TYPE_RAW_DATA_WITH_P_W_SUB_CHANNEL_2, "BURGNER_MMC_DATA_BLOCK_TYPE_RAW_DATA_WITH_P_W_SUB_CHANNEL_2", "RAW_DATA_WITH_P_W_SUB_CHANNEL_2" },
			{ BURGNER_MMC_DATA_BLOCK_TYPE_VENDOR_SPECIFIC_1, "BURGNER_MMC_DATA_BLOCK_TYPE_VENDOR_SPECIFIC_1", "VENDOR_SPECIFIC_1" },
			{ BURGNER_MMC_DATA_BLOCK_TYPE_MODE_1, "BURGNER_MMC_DATA_BLOCK_TYPE_MODE_1", "MODE_1" },
			{ BURGNER_MMC_DATA_BLOCK_TYPE_MODE_2, "BURGNER_MMC_DATA_BLOCK_TYPE_MODE_2", "MODE_2" },
			{ BURGNER_MMC_DATA_BLOCK_TYPE_CD_ROM_XA_MODE_2_FORM_1_1, "BURGNER_MMC_DATA_BLOCK_TYPE_CD_ROM_XA_MODE_2_FORM_1_1", "CD_ROM_XA_MODE_2_FORM_1_1" },
			{ BURGNER_MMC_DATA_BLOCK_TYPE_CD_ROM_XA_MODE_2_FORM_1_2, "BURGNER_MMC_DATA_BLOCK_TYPE_CD_ROM_XA_MODE_2_FORM_1_2", "CD_ROM_XA_MODE_2_FORM_1_2" },
			{ BURGNER_MMC_DATA_BLOCK_TYPE_CD_ROM_XA_MODE_2_FORM_2, "BURGNER_MMC_DATA_BLOCK_TYPE_CD_ROM_XA_MODE_2_FORM_2", "CD_ROM_XA_MODE_2_FORM_2" },
			{ BURGNER_MMC_DATA_BLOCK_TYPE_CD_ROM_XA_MIXED, "BURGNER_MMC_DATA_BLOCK_TYPE_CD_ROM_XA_MIXED", "CD_ROM_XA_MIXED" },
			{ BURGNER_MMC_DATA_BLOCK_TYPE_VENDOR_SPECIFIC_2, "BURGNER_MMC_DATA_BLOCK_TYPE_VENDOR_SPECIFIC_2", "VENDOR_SPECIFIC_2" },
			{ 0, NULL, NULL }
		};

		etype = g_enum_register_static ("BurgnerMmcDataBlockType", values);
	}

	return etype;
}

guint
burgner_mmc_data_block_type_get_data_block_size (BurgnerMmcDataBlockType type)
{
	guint r = 0;

	switch (type) {
		case BURGNER_MMC_DATA_BLOCK_TYPE_RAW_DATA:
			r = 2352;
			break;
		case BURGNER_MMC_DATA_BLOCK_TYPE_RAW_DATA_WITH_P_Q_SUB_CHANNEL:
			r = 2368;
			break;
		case BURGNER_MMC_DATA_BLOCK_TYPE_RAW_DARA_WITH_P_W_SUB_CHANNEL_1:
		case BURGNER_MMC_DATA_BLOCK_TYPE_RAW_DATA_WITH_P_W_SUB_CHANNEL_2:
			r = 2448;
			break;
		case BURGNER_MMC_DATA_BLOCK_TYPE_MODE_1:
		case BURGNER_MMC_DATA_BLOCK_TYPE_CD_ROM_XA_MODE_2_FORM_1_1:
			r = 2048;
			break;
		case BURGNER_MMC_DATA_BLOCK_TYPE_MODE_2:
			r = 2336;
			break;
		case BURGNER_MMC_DATA_BLOCK_TYPE_CD_ROM_XA_MODE_2_FORM_1_2:
			r = 2056;
			break;
		case BURGNER_MMC_DATA_BLOCK_TYPE_CD_ROM_XA_MODE_2_FORM_2:
			r = 2324;
			break;
		case BURGNER_MMC_DATA_BLOCK_TYPE_CD_ROM_XA_MIXED:
			r = 2332;
			break;
		default:
			break;
	}

	return r;
}

