/*
 * 
 *
 * This file is part of the libburgnerscsi library.
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */

#ifndef __BURGNER_MMC_IMPL_H__
#define __BURGNER_MMC_IMPL_H__

#include <glib.h>
#include <libburgnerscsi/burgner-scsi-bit-field-order.h>

G_BEGIN_DECLS


enum {
	_BURGNER_MMC_IMPL_BLANKING_TYPE_DISC = 0,
	_BURGNER_MMC_IMPL_BLANKING_TYPE_MINIMALLY_DISC = 1,
	_BURGNER_MMC_IMPL_BLANKING_TYPE_TRACK = 2,
	_BURGNER_MMC_IMPL_BLANKING_TYPE_UNRESERVE_TRACK = 3,
	_BURGNER_MMC_IMPL_BLANKING_TYPE_TRACK_TAIL = 4,
	_BURGNER_MMC_IMPL_BLANKING_TYPE_UNCLOSE_LAST_SESSION = 5,
	_BURGNER_MMC_IMPL_BLANKING_TYPE_LAST_SESSION = 6,
	_BURGNER_MMC_IMPL_BLANKING_TYPE_RESERVED = 7
};

typedef struct {
	guchar op_code;	/* 0xa1 */

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar res_01 : 5;
	guchar blanking_type : 3;
#else
	guchar blanking_type : 3;
	guchar res_01 : 5;
#endif

	guchar track_num_start_lba[4];

	guchar res_02[5];

	guchar control;
} __attribute__((packed)) _BurgnerMmcImplBlankCdb;


typedef struct {
	guchar op_code;	/* 0x5b */

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar res_01 : 7;
	guchar immed : 1;
#else
	guchar immed : 1;
	guchar res_01 : 7;
#endif

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar res_02 : 5;
	guchar session : 1;
	guchar track : 1;
#else
	guchar track : 1;
	guchar session : 1;
	guchar res_02 : 5;
#endif

	guchar res_03[2];

	guchar track_number;

	guchar res_04[3];

	guchar control;
} __attribute((packed)) _BurgnerMmcImplCloseTrackSessionCdb;


typedef struct {
	guchar op_code; /* 0x5c */

	guchar res_01[6];

	guchar alloc_len[2];

	guchar control;
} __attribute__((packed)) _BurgnerMmcImplReadBufferCapacityCdb;

typedef struct {
	guchar data_len[2];
	guchar res_01[2];

	guchar buffer_len[4];
	guchar blank_buffer_len[4];
} __attribute__((packed)) _BurgnerMmcImplReadBufferCapacityData;


typedef struct {
	guchar op_code;	/* 0x51 */

	guchar res_01[6];

	guchar alloc_len[2];

	guchar control;
} __attribute__((packed)) _BurgnerMmcImplReadDiscInformationCdb;

typedef struct {
	guchar speed[2];	/* In kB/s */
	guchar opc_values[6];
} __attribute__((packed)) _BurgnerMmcImplOpcTable;

typedef struct {
	guchar data_len[2];

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar res_01 : 3;
	guchar erasable : 1;
	guchar last_session_state : 2;
		/*
		 * 0: empty
		 * 1: incomplete
		 * 2: reserved
		 * 3: complete (only possible when disc_status is complete)
		 */
	guchar disc_status : 2;
		/*
		 * 0: empty
		 * 1: incomplete (appendable)
		 * 2: complete
		 * 3: reserved
		 */
#else
	guchar disc_status : 2;
	guchar last_session_state : 2;
	guchar erasable : 1;
	guchar res_01 : 3;
#endif

	guchar first_track_number;
	guchar number_of_sessions;

	guchar first_track_number_in_last_session;
	guchar last_track_number_in_last_session;

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar did_v : 1;
	guchar dbc_v : 1;
	guchar uru : 1;
	guchar res_02 : 5;
#else
	guchar res_02 : 5;
	guchar uru : 1;
	guchar dbc_v : 1;
	guchar did_v : 1;
#endif

	guchar disc_type;
		/*
		 * 0x00: CD-DA or CD-Rom
		 * 0x10: CD-I
		 * 0x20: CD-Rom XA
		 * 0xff: undefined
		 */

	guchar res_03[3];

	guchar disc_identification[4];
	guchar last_session_lead_in_start_time[4];	/* MSF */
	guchar last_possible_start_time_for_last_lead_out; /* MSF */

	guchar disc_bar_code[8];

	guchar res_04;

	guchar number_of_opc_entries;

	_BurgnerMmcImplOpcTable opc_entries[0];
} __attribute__((packed)) _BurgnerMmcImplReadDiscInformationData;


typedef struct {
	guchar op_code;	/* 0x52 */

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar res_01 : 7;
	guchar track : 1;
#else
	guchar track : 1;
	guchar res_01 : 7;
#endif

	guchar lba_or_track_num[4];

	guchar res_02;

	guchar alloc_len[2];

	guchar control;
} __attribute__((aligned)) _BurgnerMmcImplReadTrackInformationCdb;

typedef struct {
	guchar data_len[2];

	guchar track_num;
	guchar session_num;

	guchar res_01;

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar res_02 : 2;
	guchar damage : 1;
	guchar copy : 1;
	guchar track_mode : 4;
#else
	guchar track_mode : 4;
	guchar copy : 1;
	guchar damage : 1;
	guchar res_02 : 2;
#endif

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar rt : 1;
	guchar blank : 1;
	guchar packet : 1;
	guchar fp : 1;
	guchar data_mode : 4;
#else
	guchar data_mode : 4;
	guchar fp : 1;
	guchar packet : 1;
	guchar blank : 1;
	guchar rt : 1;
#endif

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar res_03 : 7;
	guchar nwa_v : 1;
#else
	guchar nwa_v : 1;
	guchar res_03 : 7;
#endif

	guchar start_address[4];
	guchar nwa[4];

	guchar free_blocks[4];
	guchar fixed_packet_size[4];
	guchar track_size[4];
} __attribute__((packed)) _BurgnerMmcImplReadTrackInformationData;


typedef struct {
	guchar op_code;	/* 0x53 */

	guchar res_01[4];

	guchar size[4];

	guchar control;
} __attribute__((packed)) _BurgnerMmcImplReserveTrackCdb;


typedef struct {
	guchar op_code;	/* 0x35 */

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar res_01 : 6;
	guchar immed : 1;
	guchar reladr : 1;
#else
	guchar reladr : 1;
	guchar immed : 1;
	guchar res_01 : 6;
#endif

	guchar lba[4];

	guchar res_02;

	guchar n_blocks[2];

	guchar control;
} __attribute__((packed)) _BurgnerMmcImplSynchronizeCacheCdb;


typedef struct {
	guchar op_code;	/* 0x2a */

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar res_01 : 3;
	guchar dpo : 1;
	guchar fua : 1;
	guchar res_02 : 2;
	guchar reladr : 1;
#else
	guchar reladr : 1;
	guchar res_02 : 2;
	guchar fua : 1;
	guchar dpo : 1;
	guchar res_01 : 3;
#endif

	guchar lba[4];

	guchar res_03;

	guchar length[2];

	guchar control;
} __attribute__((packed)) _BurgnerMmcImplWriteCdb;


typedef struct {
	guchar op_code;	/* 0xbb */

	guchar res_01;

	guchar read_speed[2];	/* kbytes/sec */
	guchar write_speed[2];	/* kbytes/sec */

	guchar res_02[5];

	guchar control;
} __attribute__((packed)) _BurgnerMmcImplSetCdSpeedCdb;


#define BURGNER_MMC_IMPL_CD_CAPABILITIES_PAGE_CODE	(0x2a)

typedef struct {
#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar ps : 1;
	guchar res_01 : 1;
	guchar page_code : 6;	/* 0x2a */
#else
	guchar page_code : 6;	/* 0x2a */
	guchar res_01 : 1;
	guchar ps : 1;
#endif

	guchar page_length;

	/* Begin of Individual Capabilities */
#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar res_02 : 5;
	guchar method_2 : 1;
	guchar cdrw_read : 1;
	guchar cdr_read : 1;
#else
	guchar cdr_read : 1;
	guchar cdrw_read : 1;
	guchar method_2 : 1;
	guchar res_02 : 5;
#endif

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar res_03 : 5;
	guchar test_write : 1;
	guchar cdrw_write : 1;
	guchar cdr_write : 1;
#else
	guchar cdr_write : 1;
	guchar cdrw_write : 1;
	guchar test_write : 1;
	guchar res_03 : 5;
#endif

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar res_04 : 1;
	guchar multi_session : 1;
	guchar mode_2_form_2 : 1;
	guchar mode_2_form_1 : 1;
	guchar digital_port_2 : 1;
	guchar digital_port_1 : 1;
	guchar composite : 1;
	guchar audio_play : 1;
#else
	guchar audio_play : 1;
	guchar composite : 1;
	guchar digital_port_1 : 1;
	guchar digital_port_2 : 1;
	guchar mode_2_form_1 : 1;
	guchar mode_2_form_2 : 1;
	guchar multi_session : 1;
	guchar res_04 : 1;
#endif

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar read_bar_code : 1;
	guchar upc : 1;
	guchar isrc : 1;
	guchar c2_pointers : 1;
	guchar r_w_corrected : 1;
	guchar r_w_supported : 1;
	guchar cdda_stream_accurate : 1;
	guchar cdda_commands : 1;
#else
	guchar cdda_commands : 1;
	guchar cdda_stream_accurate : 1;
	guchar r_w_supported : 1;
	guchar r_w_corrected : 1;
	guchar c2_pointers : 1;
	guchar isrc : 1;
	guchar upc : 1;
	guchar read_bar_code : 1;
#endif

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar mechanism_type : 3;
	guchar res_05 : 1;
	guchar eject : 1;
	guchar prevent_jumper : 1;
	guchar lock_state : 1;
	guchar lock : 1;
#else
	guchar lock : 1;
	guchar lock_state : 1;
	guchar prevent_jumper : 1;
	guchar eject : 1;
	guchar res_05 : 1;
	guchar mechanism_type : 3;
#endif

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar res_06 : 4;
	guchar sss : 1;
	guchar changer_disc_present_reporting : 1;
	guchar separate_channel_mute : 1;
	guchar separate_volume_levels : 1;
#else
	guchar separate_volume_levels : 1;
	guchar separate_channel_mute : 1;
	guchar changer_disc_present_reporting : 1;
	guchar sss : 1;
	guchar res_06 : 4;
#endif
	/* End of Individual Capabilities */

	guchar max_read_speed[2];	/* In kBps */
	guchar supported_volume_levels[2];
	guchar buffer_size[2];	/* In KBytes */
	guchar current_read_speed[2];	/* In kBps */

	guchar res_07;

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar res_08 : 2;
	guchar length : 2;
	guchar lsbf : 1;
	guchar rck : 1;
	guchar bck : 1;
	guchar res_09 : 1;
#else
	guchar res_09 : 1;
	guchar bck : 1;
	guchar rck : 1;
	guchar lsbf : 1;
	guchar length : 2;
	guchar res_08 : 2;
#endif

	guchar max_write_speed[2];
	guchar current_write_speed[2];
} __attribute__((packed)) _BurgnerMmcImplCdCapabilitiesPage;

typedef enum {
	_BURGNER_MMC_IMPL_LOADING_MECHANISM_TYPE_CADDY = 0,
	_BURGNER_MMC_IMPL_LOADING_MECHANISM_TYPE_TRAY = 1,
	_BURGNER_MMC_IMPL_LOADING_MECHANISM_TYPE_POP_UP = 2,
	_BURGNER_MMC_IMPL_LOADING_MECHANISM_TYPE_CHANGER_INDIVIDUALLY = 4,
	_BURGNER_MMC_IMPL_LOADING_MECHANISM_TYPE_CHANGER_CARTRIDGE = 5
} _BurgnerMmcImplLoadingMechanismType;


typedef struct {
#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar ps : 1;
	guchar res_01 : 1;
	guchar page_code : 6;	/* 0x05 */
#else
	guchar page_code : 6;	/* 0x05 */
	guchar res_01 : 1;
	guchar ps : 1;
#endif

	guchar page_length; /* 0x32 or 0x36 */

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar res_02 : 3;
	guchar test_write : 1;
	guchar write_type : 4;
#else
	guchar write_type : 4;
	guchar test_write : 1;
	guchar res_02 : 3;
#endif

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar multi_session : 2;
	guchar fp : 1;
	guchar copy : 1;
	guchar track_mode : 4;
#else
	guchar track_mode : 4;
	guchar copy : 1;
	guchar fp : 1;
	guchar multi_session : 2;
#endif

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar res_03 : 4;
	guchar data_block_type : 4;
#else
	guchar data_block_type : 4;
	guchar res_03 : 4;
#endif

	guchar res_04[2];

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar res_05 : 2;
	guchar host_application_code : 6;
#else
	guchar host_application_code : 6;
	guchar res_05 : 2;
#endif

	guchar session_format;

	guchar res_06;

	guchar packet_size[4];
	guchar audio_pause_length[2];

	guchar mcn[16];
	guchar isrc[16];

	guchar sub_header_0;
	guchar sub_header_1;
	guchar sub_header_2;
	guchar sub_header_3;

	guchar vendor_specific[4];
} __attribute__((packed)) _BurgnerMmcImplWritePage;

enum {
	_BURGNER_MMC_IMPL_WRITE_PAGE_WRITE_TYPE_PACKET = 0,
	_BURGNER_MMC_IMPL_WRITE_PAGE_WRITE_TYPE_TRACK_AT_ONCE = 1,
	_BURGNER_MMC_IMPL_WRITE_PAGE_WRITE_TYPE_SESSION_AT_ONCE = 2,
	_BURGNER_MMC_IMPL_WRITE_PAGE_WRITE_TYPE_RAW = 3
};

enum {
	_BURGNER_MMC_IMPL_WRITE_PAGE_MULTI_SESSION_NO_B0_POINTER = 0,
	_BURGNER_MMC_IMPL_WRITE_PAGE_MULTI_SESSION_B0_POINTER_FFFFFF = 1,
	_BURGNER_MMC_IMPL_WRITE_PAGE_MULTI_SESSION_NEXT_SESSION_ALLOWED = 3
};

enum {
	_BURGNER_MMC_IMPL_WRITE_PAGE_DATA_BLOCK_TYPE_RAW_DATA = 0,
	_BURGNER_MMC_IMPL_WRITE_PAGE_DATA_BLOCK_TYPE_RAW_DATA_WITH_P_Q_SUB_CHANNEL = 1,
	_BURGNER_MMC_IMPL_WRITE_PAGE_DATA_BLOCK_TYPE_RAW_DATA_WITH_P_W_SUB_CHANNEL_1 = 2,
	_BURGNER_MMC_IMPL_WRITE_PAGE_DATA_BLOCK_TYPE_RAW_DATA_WITH_P_W_SUB_CHANNEL_2 = 3,
	_BURGNER_MMC_IMPL_WRITE_PAGE_DATA_BLOCK_TYPE_VENDOR_SPECIFIC_1 = 7,
	_BURGNER_MMC_IMPL_WRITE_PAGE_DATA_BLOCK_TYPE_MODE_1 = 8,
	_BURGNER_MMC_IMPL_WRITE_PAGE_DATA_BLOCK_TYPE_MODE_2 = 9,
	_BURGNER_MMC_IMPL_WRITE_PAGE_DATA_BLOCK_TYPE_CD_ROM_XA_MODE_2_FORM_1_1 = 10,
	_BURGNER_MMC_IMPL_WRITE_PAGE_DATA_BLOCK_TYPE_CD_ROM_XA_MODE_2_FORM_1_2 = 11,
	_BURGNER_MMC_IMPL_WRITE_PAGE_DATA_BLOCK_TYPE_CD_ROM_XA_MODE_2_FORM_2 = 12,
	_BURGNER_MMC_IMPL_WRITE_PAGE_DATA_BLOCK_TYPE_CD_ROM_XA_MIXED = 13,
	_BURGNER_MMC_IMPL_WRITE_PAGE_DATA_BLOCK_TYPE_VENDOR_SPECIFIC_2 = 15
};

enum {
	_BURGNER_MMC_IMPL_WRITE_PAGE_SESSION_FORMAT_CD_DA_OR_CD_ROM = 0,
	_BURGNER_MMC_IMPL_WRITE_PAGE_SESSION_FORMAT_CD_I = 0x10,
	_BURGNER_MMC_IMPL_WRITE_PAGE_SESSION_FORMAT_CD_ROM_XA = 0x20
};

enum {
	/* Mode */
	_BURGNER_MMC_IMPL_TRACK_MODE_2_AUDIO_CHANNELS_WITHOUT_PRE_EMPHASIS = 0,
	_BURGNER_MMC_IMPL_TRACK_MODE_2_AUDIO_CHANNELS_WITH_PRE_EMPHASIS = 1,
	_BURGNER_MMC_IMPL_TRACK_MODE_AUDIO_CHANNELS_WITHOUT_PRE_EMPHASIS = 8,
	_BURGNER_MMC_IMPL_TRACK_MODE_AUDIO_CHANNELS_WITH_PRE_EMPHASIS = 9,
	_BURGNER_MMC_IMPL_TRACK_MODE_DATA_TRACK_UNINTERRUPTED = 4,
	_BURGNER_MMC_IMPL_TRACK_MODE_DATA_TRACK_INCREMENTAL = 5,
	/* Copy flag */
	_BURGNER_MMC_IMPL_TRACK_MODE_DIGITAL_COPY_PROHIBITED = 0,
	_BURGNER_MMC_IMPL_TRACK_MODE_DIGITAL_COPY_PERMITTED = 2
};


G_END_DECLS

#endif /* __BURGNER_MMC_IMPL_H__ */

