/*
 * BurgnerMmcReadBufferCapacity
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This file is part of the libburgnermmc library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include "../libburgnerscsi/burgner-scsi-utils.h"
#include "burgner-mmc-impl.h"
#include "burgner-mmc-read-buffer-capacity.h"

void
burgner_mmc_read_buffer_capacity_write_cdb (gpointer cdb_buf,
					    guint cdb_buf_size,
					    guint data_buf_size)
{
	_BurgnerMmcImplReadBufferCapacityCdb *impl;

	g_return_if_fail (cdb_buf != NULL);
	g_return_if_fail (cdb_buf_size >= BURGNER_MMC_READ_BUFFER_CAPACITY_CDB_SIZE);

	impl = (_BurgnerMmcImplReadBufferCapacityCdb *) cdb_buf;
	memset (impl, 0, BURGNER_MMC_READ_BUFFER_CAPACITY_CDB_SIZE);

	impl->op_code = BURGNER_MMC_READ_BUFFER_CAPACITY_OP_CODE;
	burgner_scsi_set_uint16 (impl->alloc_len, data_buf_size);
}


void
burgner_mmc_read_buffer_capacity_parse_data (gpointer data,
					     guint data_size,
					     guint32 *blank_length,
					     guint32 *total_length)
{
	_BurgnerMmcImplReadBufferCapacityData *impl;
	guint data_len;

	g_return_if_fail (data != NULL);

	impl = (_BurgnerMmcImplReadBufferCapacityData *) data;

	data_len = burgner_scsi_get_uint16 (impl->data_len);

	if (blank_length) {
		*blank_length = data_len >= 8 && data_size >= 8 ? burgner_scsi_get_uint32 (impl->blank_buffer_len) : 0;
	}

	if (total_length) {
		*total_length = data_len >= 12 && data_size >= 12 ? burgner_scsi_get_uint32 (impl->blank_buffer_len) : 0;
	}
}

