/*
 * BurgnerMmcReadTrackInformationStage - BurgnerMmcReadTrackInformationStage object
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "burgner-mmc-i18n.h"

#include <string.h>
#include <glib-object.h>
#include "burgner-mmc-read-track-information.h"
#include "burgner-mmc-read-track-information-stage.h"


static BurgnerScsiSendCommandStageClass *parent_class = NULL;

enum {
	PROP_0,
	PROP_DATA_BUF,
	PROP_DATA_BUF_SIZE,
	PROP_LBA,
	PROP_TRACK_NUM,
	BURGNER_MMC_READ_TRACK_INFORMATION_STAGE_LAST_ARG
};

static void
burgner_mmc_read_track_information_stage_finalize (GObject *object)
{
	BurgnerMmcReadTrackInformationStage *self = BURGNER_MMC_READ_TRACK_INFORMATION_STAGE (object);

	self->data_buf = NULL;
	self->data_buf_size = 0;
	self->a.lba = 0;
	self->with_lba = 0;

	if (G_OBJECT_CLASS (parent_class)->finalize)
		G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
burgner_mmc_read_track_information_stage_set_property (GObject *object,
	      guint prop_id,
	      const GValue *value,
	      GParamSpec *pspec)
{
	BurgnerMmcReadTrackInformationStage *self = BURGNER_MMC_READ_TRACK_INFORMATION_STAGE (object);

	switch (prop_id) {
		case PROP_DATA_BUF: {
			gpointer d = self->data_buf;
			self->data_buf = g_value_get_pointer (value);
			if (d != self->data_buf)
				g_object_notify (object, "data_buf");
			} break;
		case PROP_DATA_BUF_SIZE: {
			guint d = self->data_buf_size;
			self->data_buf_size = g_value_get_uint (value);
			if (d != self->data_buf_size)
				g_object_notify (object, "data_buf_size");
			} break;
		case PROP_LBA:
			self->a.lba = g_value_get_uint (value);
			self->with_lba = 1;
			g_object_notify (object, "lba");
			break;
		case PROP_TRACK_NUM:
			self->a.track_num = g_value_get_uint (value);
			self->with_lba = 0;
			g_object_notify (object, "track_num");
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
burgner_mmc_read_track_information_stage_get_property (GObject *object,
	      guint prop_id,
	      GValue *value,
	      GParamSpec *pspec)
{
	BurgnerMmcReadTrackInformationStage *self = BURGNER_MMC_READ_TRACK_INFORMATION_STAGE (object);

	switch (prop_id) {
		case PROP_DATA_BUF:
			g_value_set_pointer (value, self->data_buf);
			break;
		case PROP_DATA_BUF_SIZE:
			g_value_set_uint (value, self->data_buf_size);
			break;
		case PROP_LBA:
			g_value_set_uint (value, self->with_lba ? self->a.lba : 0);
			break;
		case PROP_TRACK_NUM:
			g_value_set_uint (value, self->with_lba ? 0 : self->a.track_num);
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
_burgner_mmc_read_track_information_get_command_parameters
				(BurgnerScsiSendCommandStage *sc_stage,
				 guchar *cdb_buf, guint cdb_buf_size,
				 BurgnerScsiDirection *direction,
				 guchar **data, guint *data_size,
				 guint *timeout)
{
	BurgnerMmcReadTrackInformationStage *self = BURGNER_MMC_READ_TRACK_INFORMATION_STAGE (sc_stage);

	g_return_if_fail (!self->data_buf_size || self->data_buf);
	g_return_if_fail (cdb_buf_size >= BURGNER_MMC_READ_TRACK_INFORMATION_CDB_SIZE);

	if (self->with_lba)
		burgner_mmc_read_track_information_with_lba_write_cdb
					(cdb_buf, cdb_buf_size,
					 self->data_buf_size,
					 self->a.lba);
	else
		burgner_mmc_read_track_information_write_cdb
					(cdb_buf, cdb_buf_size,
					 self->data_buf_size,
					 self->a.track_num);

	*direction = BURGNER_SCSI_DIRECTION_FROM_DEVICE;
	*data = self->data_buf;
	*data_size = self->data_buf_size;
	*timeout = BURGNER_MMC_READ_TRACK_INFORMATION_TIMEOUT;
}

static gboolean
_burgner_mmc_read_track_information_command_result
				(BurgnerScsiSendCommandStage *sc_stage,
				 guint status,
				 BurgnerScsiSense *sense,
				 guint data_used,
				 GError *command_error,
				 GError **error)
{
	BurgnerMmcReadTrackInformationStage *self = BURGNER_MMC_READ_TRACK_INFORMATION_STAGE (sc_stage);

	if (parent_class->command_result (sc_stage, status, sense, data_used,
					  command_error, error)) {
		memset (((guchar *) self->data_buf) + data_used,
			0,
			self->data_buf_size - data_used);
		return TRUE;
	}

	return FALSE;
}

static void
burgner_mmc_read_track_information_stage_class_init (BurgnerMmcReadTrackInformationStageClass *klass)
{
	GObjectClass *object_class;
	BurgnerScsiSendCommandStageClass *sc_stage_class;

	object_class = (GObjectClass *) klass;
	object_class->set_property = burgner_mmc_read_track_information_stage_set_property;
	object_class->get_property = burgner_mmc_read_track_information_stage_get_property;
	object_class->finalize = burgner_mmc_read_track_information_stage_finalize;

	sc_stage_class = (BurgnerScsiSendCommandStageClass *) klass;
	sc_stage_class->get_command_parameters = _burgner_mmc_read_track_information_get_command_parameters;
	sc_stage_class->command_result = _burgner_mmc_read_track_information_command_result;

	/* Properties */
	g_object_class_install_property (object_class,
					 PROP_DATA_BUF,
					 g_param_spec_pointer ("data_buf",
					                _("data_buf"),
							_("A buffer which will contain the track informations"),
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_DATA_BUF_SIZE,
					 g_param_spec_uint ("data_buf_size",
					                _("data_buf_size"),
							_("Size of data_buf (should be 28)"),
							0,
							0xffff,
							0,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_LBA,
					 g_param_spec_uint ("lba",
					                _("lba"),
							_("A LBA (Logical Block Address)"),
							0,
							0xffffffff,
							0,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_TRACK_NUM,
					 g_param_spec_uint ("track_num",
					                _("track_num"),
							_("The Track Number"),
							0,
							0xff,
							0,
							G_PARAM_READABLE | G_PARAM_WRITABLE));


	parent_class = g_type_class_peek_parent (klass);
}

static void
burgner_mmc_read_track_information_stage_init (BurgnerMmcReadTrackInformationStage *self)
{
	self->data_buf = NULL;
	self->data_buf_size = 0;
	self->a.lba = 0;
	self->with_lba = 0;
}

GType
burgner_mmc_read_track_information_stage_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		static GTypeInfo info = {
			sizeof (BurgnerMmcReadTrackInformationStageClass),
			NULL,
			NULL,
			(GClassInitFunc) burgner_mmc_read_track_information_stage_class_init,
			NULL,
			NULL,
			sizeof (BurgnerMmcReadTrackInformationStage),
			0,
			(GInstanceInitFunc) burgner_mmc_read_track_information_stage_init,
		};

		type = g_type_register_static (BURGNER_TYPE_SCSI_SEND_COMMAND_STAGE, "BurgnerMmcReadTrackInformationStage", &info, 0);
	}

	return type;
}

BurgnerScsiStage *
burgner_mmc_read_track_information_stage_new (void)
{
	BurgnerMmcReadTrackInformationStage *self;

	self = g_object_new (BURGNER_TYPE_MMC_READ_TRACK_INFORMATION_STAGE, NULL);


	return BURGNER_SCSI_STAGE (self);
}


GQuark
burgner_mmc_read_track_information_stage_error_quark (void)
{
	return g_quark_from_string ("BurgnerMmcReadTrackInformationStage");
}

