/*
 * BurgnerMmcSetCdSpeedStage - BurgnerMmcSetCdSpeedStage object
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "burgner-mmc-i18n.h"

#include <glib-object.h>
#include "burgner-mmc-utils.h"
#include "../libburgnerscsi/burgner-scsi-stage-error-utils.h"
#include "burgner-mmc-set-cd-speed.h"
#include "burgner-mmc-set-cd-speed-stage.h"


static BurgnerScsiSendCommandStageClass *parent_class = NULL;

enum {
	PROP_0,
	PROP_READ_SPEED,
	PROP_WRITE_SPEED,
	PROP_READ_X,
	PROP_WRITE_X,
	BURGNER_MMC_SET_CD_SPEED_STAGE_LAST_ARG
};

static void
burgner_mmc_set_cd_speed_stage_finalize (GObject *object)
{
	BurgnerMmcSetCdSpeedStage *self = BURGNER_MMC_SET_CD_SPEED_STAGE (object);

	self->read_speed = self->write_speed =
		BURGNER_MMC_SET_CD_SPEED_STAGE_MAX_SPEED;

	if (G_OBJECT_CLASS (parent_class)->finalize)
		G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
burgner_mmc_set_cd_speed_stage_set_property (GObject *object,
	      guint prop_id,
	      const GValue *value,
	      GParamSpec *pspec)
{
	BurgnerMmcSetCdSpeedStage *self = BURGNER_MMC_SET_CD_SPEED_STAGE (object);

	switch (prop_id) {
		case PROP_READ_SPEED:
			self->read_speed = g_value_get_uint (value);
			g_object_notify (object, "read_speed");
			break;
		case PROP_WRITE_SPEED:
			self->write_speed = g_value_get_uint (value);
			g_object_notify (object, "write_speed");
			break;
		case PROP_READ_X:
			self->read_speed = burgner_mmc_speed_x_to_kbps (g_value_get_uint (value));
			break;
		case PROP_WRITE_X:
			self->write_speed = burgner_mmc_speed_x_to_kbps (g_value_get_uint (value));
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
burgner_mmc_set_cd_speed_stage_get_property (GObject *object,
	      guint prop_id,
	      GValue *value,
	      GParamSpec *pspec)
{
	BurgnerMmcSetCdSpeedStage *self = BURGNER_MMC_SET_CD_SPEED_STAGE (object);

	switch (prop_id) {
		case PROP_READ_SPEED:
			g_value_set_uint (value, self->read_speed);
			break;
		case PROP_WRITE_SPEED:
			g_value_set_uint (value, self->write_speed);
			break;
		case PROP_READ_X:
			g_value_set_uint (value, burgner_mmc_speed_kbps_to_x (self->read_speed));
			break;
		case PROP_WRITE_X:
			g_value_set_uint (value, burgner_mmc_speed_kbps_to_x (self->write_speed));
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
_burgner_mmc_set_cd_speed_stage_get_command_parameters
				(BurgnerScsiSendCommandStage *sc_stage,
				 guchar *cdb_buf, guint cdb_size,
				 BurgnerScsiDirection *direction,
				 guchar **data, guint *data_size,
				 guint *timeout)
{
	BurgnerMmcSetCdSpeedStage *self = BURGNER_MMC_SET_CD_SPEED_STAGE (sc_stage);

	g_return_if_fail (cdb_buf != NULL);
	g_return_if_fail (cdb_size >= BURGNER_MMC_SET_CD_SPEED_CDB_SIZE);

	burgner_mmc_set_cd_speed_kbps_write_cdb (cdb_buf, cdb_size,
						 self->read_speed,
						 self->write_speed);
	*direction = BURGNER_SCSI_DIRECTION_NONE;
	*data = NULL;
	*data_size = 0;
	*timeout = 100;
}

#if 0
static gboolean
_burgner_mmc_set_cd_speed_stage_command_result
				(BurgnerScsiSendCommandStage *sc_stage,
				 guint status,
				 BurgnerScsiSense *sense,
				 guint data_used,
				 GError *command_error,
				 GError **error)
{
	return BURGNER_SCSI_SEND_COMMAND_STAGE_GET_CLASS (self)->command_result
		(sc_stage, status, sense, data_used, command_error, error);
}
#endif

static void
burgner_mmc_set_cd_speed_stage_class_init (BurgnerMmcSetCdSpeedStageClass *klass)
{
	GObjectClass *object_class;
	BurgnerScsiSendCommandStageClass *sc_stage_class;

	object_class = (GObjectClass *) klass;
	object_class->set_property = burgner_mmc_set_cd_speed_stage_set_property;
	object_class->get_property = burgner_mmc_set_cd_speed_stage_get_property;
	object_class->finalize = burgner_mmc_set_cd_speed_stage_finalize;

	sc_stage_class = (BurgnerScsiSendCommandStageClass *) klass;
	sc_stage_class->get_command_parameters = _burgner_mmc_set_cd_speed_stage_get_command_parameters;
#if 0
	sc_stage_class->command_result = _burgner_mmc_set_cd_speed_stage_command_result;
#endif

	/* Properties */
	g_object_class_install_property (object_class,
					 PROP_READ_SPEED,
					 g_param_spec_uint ("read_speed",
					                _("read_speed"),
							_("Read Speed"),
							0,
							BURGNER_MMC_SET_CD_SPEED_STAGE_MAX_SPEED,
							BURGNER_MMC_SET_CD_SPEED_STAGE_MAX_SPEED,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_WRITE_SPEED,
					 g_param_spec_uint ("write_speed",
					                _("write_speed"),
							_("Write Speed"),
							0,
							BURGNER_MMC_SET_CD_SPEED_STAGE_MAX_SPEED,
							BURGNER_MMC_SET_CD_SPEED_STAGE_MAX_SPEED,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_READ_X,
					 g_param_spec_uint ("read_x",
					                _("read_x"),
							_("Read Speed in x"),
							0,
							371,
							1,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_WRITE_SPEED,
					 g_param_spec_uint ("write_speed",
					                _("write_speed"),
							_("Write Speed"),
							0,
							371,
							1,
							G_PARAM_READABLE | G_PARAM_WRITABLE));


	parent_class = g_type_class_peek_parent (klass);
}

static void
burgner_mmc_set_cd_speed_stage_init (BurgnerMmcSetCdSpeedStage *self)
{
	self->read_speed = self->write_speed =
		BURGNER_MMC_SET_CD_SPEED_STAGE_MAX_SPEED;
}

GType
burgner_mmc_set_cd_speed_stage_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		static GTypeInfo info = {
			sizeof (BurgnerMmcSetCdSpeedStageClass),
			NULL,
			NULL,
			(GClassInitFunc) burgner_mmc_set_cd_speed_stage_class_init,
			NULL,
			NULL,
			sizeof (BurgnerMmcSetCdSpeedStage),
			0,
			(GInstanceInitFunc) burgner_mmc_set_cd_speed_stage_init,
		};

		type = g_type_register_static (BURGNER_TYPE_SCSI_SEND_COMMAND_STAGE, "BurgnerMmcSetCdSpeedStage", &info, 0);
	}

	return type;
}

BurgnerScsiStage *
burgner_mmc_set_cd_speed_stage_new (void)
{
	BurgnerMmcSetCdSpeedStage *self;

	self = g_object_new (BURGNER_TYPE_MMC_SET_CD_SPEED_STAGE, NULL);


	return BURGNER_SCSI_STAGE (self);
}


GQuark
burgner_mmc_set_cd_speed_stage_error_quark (void)
{
	return g_quark_from_string ("BurgnerMmcSetCdSpeedStage");
}

