/*
 * BurgnerMmcSetupSessionStage - BurgnerMmcSetupSessionStage object
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "burgner-mmc-i18n.h"

#include <glib-object.h>
#include "../libburgnerscsi/burgner-scsi-mode-parameter-list.h"
#include "burgner-mmc-get-write-page-stage.h"
#include "burgner-mmc-set-write-page-stage.h"
#include "burgner-mmc-write-page-utils.h"
#include "burgner-mmc-setup-session-stage.h"


static BurgnerScsiStageManagerClass *parent_class = NULL;

enum {
	BURGNER_MMC_SETUP_SESSION_STAGE_PROP_0,
	PROP_DEVICE,
	PROP_CDB_BUF,
	PROP_CDB_BUF_SIZE,
	PROP_SENSE_BUF,
	PROP_SENSE_BUF_SIZE,
	PROP_DATA_BUF,
	PROP_DATA_BUF_SIZE,
	PROP_CLOSURE,
	PROP_FORMAT,
	BURGNER_MMC_SETUP_SESSION_STAGE_LAST_ARG
};

static void
_burgner_mmc_setup_session_stage_finalize (GObject *object)
{
	BurgnerMmcSetupSessionStage *self = BURGNER_MMC_SETUP_SESSION_STAGE (object);

	if (self->device) {
		g_object_unref (self->device);
		self->device = NULL;
	}

	self->cdb_buf = self->sense_buf = NULL;
	self->cdb_buf_size = self->sense_buf_size = 0;

	self->closure = BURGNER_MMC_SESSION_CLOSURE_NEXT_SESSION_ALLOWED;
	self->format = BURGNER_MMC_SESSION_FORMAT_CD_DA_OR_CD_ROM;

	self->data_buf = NULL;
	self->data_buf_size = 0;

	if (self->mask_buf) {
		g_free (self->mask_buf);
		self->mask_buf = NULL;
	}

	if (G_OBJECT_CLASS (parent_class)->finalize)
		G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
_burgner_mmc_setup_session_stage_set_property (GObject *object,
	      guint prop_id,
	      const GValue *value,
	      GParamSpec *pspec)
{
	BurgnerMmcSetupSessionStage *self = BURGNER_MMC_SETUP_SESSION_STAGE (object);

	switch (prop_id) {
		case PROP_DEVICE:
			if (self->device)
				g_object_unref (self->device);
			self->device = (BurgnerScsiDevice *) g_value_get_object (value);
			if (self->device)
				self->device = BURGNER_SCSI_DEVICE (g_object_ref (self->device));
			g_object_notify (object, "device");
			break;
		case PROP_CDB_BUF:
			self->cdb_buf = g_value_get_pointer (value);
			g_object_notify (object, "cdb_buf");
			break;
		case PROP_CDB_BUF_SIZE:
			self->cdb_buf_size = g_value_get_uint (value);
			g_object_notify (object, "cdb_buf_size");
			break;
		case PROP_SENSE_BUF:
			self->sense_buf = g_value_get_pointer (value);
			g_object_notify (object, "sense_buf");
			break;
		case PROP_SENSE_BUF_SIZE:
			self->sense_buf_size = g_value_get_uint (value);
			g_object_notify (object, "sense_buf_size");
			break;
		case PROP_DATA_BUF:
			self->data_buf = g_value_get_pointer (value);
			g_object_notify (object, "data_buf");
			break;
		case PROP_DATA_BUF_SIZE:
			self->data_buf_size = g_value_get_uint (value);
			g_object_notify (object, "data_buf_size");
			break;
		case PROP_CLOSURE:
			self->closure = g_value_get_enum (value);
			g_object_notify (object, "closure");
			break;
		case PROP_FORMAT:
			self->format = g_value_get_enum (value);
			g_object_notify (object, "format");
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
_burgner_mmc_setup_session_stage_get_property (GObject *object,
	      guint prop_id,
	      GValue *value,
	      GParamSpec *pspec)
{
	BurgnerMmcSetupSessionStage *self = BURGNER_MMC_SETUP_SESSION_STAGE (object);

	switch (prop_id) {
		case PROP_DEVICE:
			g_value_set_object (value, self->device);
			break;
		case PROP_CDB_BUF:
			g_value_set_pointer (value, self->cdb_buf);
			break;
		case PROP_CDB_BUF_SIZE:
			g_value_set_uint (value, self->cdb_buf_size);
			break;
		case PROP_SENSE_BUF:
			g_value_set_pointer (value, self->sense_buf);
			break;
		case PROP_SENSE_BUF_SIZE:
			g_value_set_uint (value, self->sense_buf_size);
			break;
		case PROP_DATA_BUF:
			g_value_set_pointer (value, self->data_buf);
			break;
		case PROP_DATA_BUF_SIZE:
			g_value_set_uint (value, self->data_buf_size);
			break;
		case PROP_CLOSURE:
			g_value_set_enum (value, self->closure);
			break;
		case PROP_FORMAT:
			g_value_set_enum (value, self->format);
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}


enum {
	_BURGNER_MMC_SETUP_SESSION_STAGE_ID_NONE = -1,
	_BURGNER_MMC_SETUP_SESSION_STAGE_ID_GET_WRITE_PAGE_MASK,
	_BURGNER_MMC_SETUP_SESSION_STAGE_ID_GET_WRITE_PAGE,
	_BURGNER_MMC_SETUP_SESSION_STAGE_ID_SET_WRITE_PAGE
};

static BurgnerScsiStage *
_burgner_mmc_setup_session_stage_get_next_stage
				(BurgnerScsiStageManager *sm,
				 gint *stage_id,
				 BurgnerScsiStage *actual_stage,
				 gint actual_stage_id)
{
	BurgnerMmcSetupSessionStage *self = BURGNER_MMC_SETUP_SESSION_STAGE (sm);
	BurgnerScsiStage *stage = NULL;

	g_return_val_if_fail (self->device != NULL, NULL);
	g_return_val_if_fail (self->cdb_buf != NULL, NULL);
	g_return_val_if_fail (self->cdb_buf_size >= 10, NULL);
	g_return_val_if_fail (self->sense_buf != NULL, NULL);
	g_return_val_if_fail (self->sense_buf_size >= 18, NULL);
	g_return_val_if_fail (self->data_buf != NULL, NULL);
	g_return_val_if_fail (self->data_buf_size >= 60, NULL);

	switch (actual_stage_id) {
		case _BURGNER_MMC_SETUP_SESSION_STAGE_ID_NONE:
			g_assert (self->mask_buf == NULL);
			self->mask_buf = g_malloc0 (60);

			stage = burgner_mmc_get_write_page_stage_new ();
			g_object_set (stage,
				      "device", self->device,
				      "cdb_buf", self->cdb_buf,
				      "cdb_buf_size", self->cdb_buf_size,
				      "sense_buf", self->sense_buf,
				      "sense_buf_size", self->sense_buf_size,
				      "data_buf", self->mask_buf,
				      "data_buf_size", 60,
				      "values", BURGNER_SCSI_MODE_PAGE_VALUES_CHANGEABLE,
				      NULL);
			*stage_id = _BURGNER_MMC_SETUP_SESSION_STAGE_ID_GET_WRITE_PAGE_MASK;
			break;
		case _BURGNER_MMC_SETUP_SESSION_STAGE_ID_GET_WRITE_PAGE_MASK:
			stage = burgner_mmc_get_write_page_stage_new ();
			g_object_set (stage,
				      "device", self->device,
				      "cdb_buf", self->cdb_buf,
				      "cdb_buf_size", self->cdb_buf_size,
				      "sense_buf", self->sense_buf,
				      "sense_buf_size", self->sense_buf_size,
				      "data_buf", self->data_buf,
				      "data_buf_size", self->data_buf_size,
				      "values", BURGNER_SCSI_MODE_PAGE_VALUES_CURRENT,
				      NULL);
			*stage_id = _BURGNER_MMC_SETUP_SESSION_STAGE_ID_GET_WRITE_PAGE;
			break;
		case _BURGNER_MMC_SETUP_SESSION_STAGE_ID_GET_WRITE_PAGE:
			stage = burgner_mmc_set_write_page_stage_new ();
			g_object_set (stage,
				      "device", self->device,
				      "cdb_buf", self->cdb_buf,
				      "cdb_buf_size", self->cdb_buf_size,
				      "sense_buf", self->sense_buf,
				      "sense_buf_size", self->sense_buf_size,
				      "data_buf", self->data_buf,
				      "data_buf_size", self->data_buf_size,
				      NULL);
			*stage_id = _BURGNER_MMC_SETUP_SESSION_STAGE_ID_SET_WRITE_PAGE;
			break;
		case _BURGNER_MMC_SETUP_SESSION_STAGE_ID_SET_WRITE_PAGE:
			stage = NULL;
			*stage_id = -1;
			break;
		default:
			g_return_val_if_reached (NULL);
			break;
	}

	return stage;
}

static gboolean
_burgner_mmc_setup_session_stage_continue
				(BurgnerScsiStageManager *sm,
				 BurgnerScsiStage *stage,
				 gint stage_id,
				 GError *stage_error,
				 BurgnerScsiSense *sense,
				 GError **error)
{
	BurgnerMmcSetupSessionStage *self = BURGNER_MMC_SETUP_SESSION_STAGE (sm);
	gboolean r = TRUE;

	if (stage_error || sense) {
		if (stage_error)
			g_propagate_error (error, stage_error);
		return FALSE;
	}

	if (stage_id == _BURGNER_MMC_SETUP_SESSION_STAGE_ID_GET_WRITE_PAGE) {
		GList *p = NULL, *m = NULL;

		if ((r = burgner_scsi_mode_parameter_list_parse (TRUE,
								 self->data_buf,
								 self->data_buf_size,
								 NULL,
								 &p,
								 error)) &&
		    (r = burgner_scsi_mode_parameter_list_parse (TRUE,
				    				 self->mask_buf, 60,
								 NULL,
								 &m,
								 error))) {
			if (!(r = (p && p->data && m && m->data))) {
				g_set_error (error,
					     BURGNER_MMC_SETUP_SESSION_STAGE_ERROR,
					     BURGNER_MMC_SETUP_SESSION_STAGE_ERROR_INVALID_MODE_PARAMETER,
					     _("Device returned bad mode parameter"));
			} else {
				r = burgner_mmc_write_page_setup_session (p->data,
									  m->data,
									  self->format,
									  self->closure,
									  error);
			}
		}

		g_list_free (p);
		g_list_free (m);
		g_free (self->mask_buf);
	}

	return r;
}

static void
_burgner_mmc_setup_session_stage_class_init (BurgnerMmcSetupSessionStageClass *klass)
{
	GObjectClass *object_class;
	BurgnerScsiStageManagerClass *sm_class;

	object_class = (GObjectClass *) klass;
	object_class->set_property = _burgner_mmc_setup_session_stage_set_property;
	object_class->get_property = _burgner_mmc_setup_session_stage_get_property;
	object_class->finalize = _burgner_mmc_setup_session_stage_finalize;

	sm_class = (BurgnerScsiStageManagerClass *) klass;
	sm_class->get_next_stage = _burgner_mmc_setup_session_stage_get_next_stage;
	sm_class->_continue = _burgner_mmc_setup_session_stage_continue;

	/* Properties */
	g_object_class_install_property (object_class,
					 PROP_DEVICE,
					 g_param_spec_object ("device",
					                _("device"),
							_("A BurgnerScsiDevice"),
							BURGNER_TYPE_SCSI_DEVICE,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_CDB_BUF,
					 g_param_spec_pointer ("cdb_buf",
					                _("cdb_buf"),
							_("A buffer which will contain the CDB"),
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_CDB_BUF_SIZE,
					 g_param_spec_uint ("cdb_buf_size",
					                _("cdb_buf_size"),
							_("Size of cdb_buf (in bytes), should be at least 10"),
							0,
							G_MAXUINT,
							0,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_SENSE_BUF,
					 g_param_spec_pointer ("sense_buf",
					                _("sense_buf"),
							_("A buffer which will contain the sense"),
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_SENSE_BUF_SIZE,
					 g_param_spec_uint ("sense_buf_size",
					                _("sense_buf_size"),
							_("Size of sense_buf (in bytes), should be at least 18"),
							0,
							G_MAXUINT,
							0,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_DATA_BUF,
					 g_param_spec_pointer ("data_buf",
					                _("data_buf"),
							_("A buffer which will contain the current write page"),
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_DATA_BUF_SIZE,
					 g_param_spec_uint ("data_buf_size",
					                _("data_buf_size"),
							_("Size of data_buf (in bytes), should be at least 60"),
							0,
							G_MAXUINT,
							0,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_CLOSURE,
					 g_param_spec_enum ("closure",
					                _("closure"),
							_("BurgnerMmcSessionCLosure"),
							BURGNER_TYPE_MMC_SESSION_CLOSURE,
							BURGNER_MMC_SESSION_CLOSURE_NEXT_SESSION_ALLOWED,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_FORMAT,
					 g_param_spec_enum ("format",
					                _("format"),
							_("BurgnerMmcSessionFormat"),
							BURGNER_TYPE_MMC_SESSION_FORMAT,
							BURGNER_MMC_SESSION_FORMAT_CD_DA_OR_CD_ROM,
							G_PARAM_READABLE | G_PARAM_WRITABLE));


	parent_class = g_type_class_peek_parent (klass);
}

static void
_burgner_mmc_setup_session_stage_init (BurgnerMmcSetupSessionStage *self)
{
	self->device = NULL;
	self->cdb_buf = NULL;
	self->cdb_buf_size = 0;
	self->sense_buf = NULL;
	self->sense_buf_size = 0;
	self->data_buf = NULL;
	self->data_buf_size = 0;
	self->mask_buf = NULL;
	self->closure = BURGNER_MMC_SESSION_CLOSURE_NEXT_SESSION_ALLOWED;
	self->format = BURGNER_MMC_SESSION_FORMAT_CD_DA_OR_CD_ROM;
}

GType
burgner_mmc_setup_session_stage_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		static GTypeInfo info = {
			sizeof (BurgnerMmcSetupSessionStageClass),
			NULL,
			NULL,
			(GClassInitFunc) _burgner_mmc_setup_session_stage_class_init,
			NULL,
			NULL,
			sizeof (BurgnerMmcSetupSessionStage),
			0,
			(GInstanceInitFunc) _burgner_mmc_setup_session_stage_init,
		};

		type = g_type_register_static (BURGNER_TYPE_SCSI_STAGE_MANAGER, "BurgnerMmcSetupSessionStage", &info, 0);
	}

	return type;
}

BurgnerScsiStage *
burgner_mmc_setup_session_stage_new (void)
{
	BurgnerMmcSetupSessionStage *self;

	self = g_object_new (BURGNER_TYPE_MMC_SETUP_SESSION_STAGE, NULL);


	return BURGNER_SCSI_STAGE (self);
}


GQuark
burgner_mmc_setup_session_stage_error_quark (void)
{
	return g_quark_from_string ("BurgnerMmcSetupSessionStage");
}

