/*
 * BurgnerMmcTrackWriter - BurgnerMmcTrackWriter object
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "burgner-mmc-i18n.h"

#include <glib-object.h>
#include "../libburgnerscsi/burgner-scsi-mode-parameter-list.h"
#include "burgner-mmc-get-write-page-stage.h"
#include "burgner-mmc-set-write-page-stage.h"
#include "burgner-mmc-reserve-track-stage.h"
#include "burgner-mmc-write-stage.h"
#include "burgner-mmc-close-track-stage.h"
#include "burgner-mmc-write-page-utils.h"
#include "burgner-mmc-track-writer.h"


static BurgnerScsiStageManagerClass *parent_class = NULL;

enum {
	PROP_0,
	PROP_DEVICE,
	PROP_FIFO,
	PROP_CDB_BUF,
	PROP_CDB_BUF_SIZE,
	PROP_SENSE_BUF,
	PROP_SENSE_BUF_SIZE,
	PROP_DATA_BUF,
	PROP_DATA_BUF_SIZE,
	PROP_DATA_BLOCK_TYPE,
	PROP_TRACK_MODE,
	PROP_MAX_WRITE_BETWEEN_READ_BUF_CAP,
	PROP_MIN_TIME_BETWEEN_READ_BUF_CAP,
	PROP_START_LBA,
	PROP_TRACK_SIZE,
	PROP_PROGRESS_CB,
	PROP_BUFFER_CAPACITY_CB,
	PROP_USER_DATA,
	PROP_TRACK_NUM,
	PROP_RESERVE,
	PROP_TEST,
	PROP_READ_BUF_CAP,
	BURGNER_MMC_TRACK_WRITER_LAST_ARG
};

static void
burgner_mmc_track_writer_finalize (GObject *object)
{
	BurgnerMmcTrackWriter *self = BURGNER_MMC_TRACK_WRITER (object);

	if (self->device) {
		g_object_unref (self->device);
		self->device = NULL;
	}

	if (self->fifo) {
		g_object_unref (self->fifo);
		self->fifo = NULL;
	}

	if (G_OBJECT_CLASS (parent_class)->finalize)
		G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
burgner_mmc_track_writer_set_property (GObject *object,
	      guint prop_id,
	      const GValue *value,
	      GParamSpec *pspec)
{
	BurgnerMmcTrackWriter *self = BURGNER_MMC_TRACK_WRITER (object);

	switch (prop_id) {
		case PROP_DEVICE:
			if (self->device)
				g_object_unref (self->device);
			self->device = (BurgnerScsiDevice *) g_value_get_object (value);
			if (self->device)
				self->device = BURGNER_SCSI_DEVICE (g_object_ref (self->device));
			g_object_notify (object, "device");
			break;
		case PROP_FIFO:
			if (self->fifo)
				g_object_unref (self->fifo);
			self->fifo = (BurgnerFifo *) g_value_get_object (value);
			if (self->fifo)
				self->fifo = BURGNER_FIFO (g_object_ref (self->fifo));
			g_object_notify (object, "fifo");
			break;
		case PROP_CDB_BUF:
			self->cdb_buf = g_value_get_pointer (value);
			g_object_notify (object, "cdb_buf");
			break;
		case PROP_SENSE_BUF:
			self->sense_buf = g_value_get_pointer (value);
			g_object_notify (object, "sense_buf");
			break;
		case PROP_DATA_BUF:
			self->data_buf = g_value_get_pointer (value);
			g_object_notify (object, "data_buf");
			break;
		case PROP_PROGRESS_CB:
			self->progress_cb = g_value_get_pointer (value);
			g_object_notify (object, "progress_cb");
			break;
		case PROP_BUFFER_CAPACITY_CB:
			self->buffer_capacity_cb = g_value_get_pointer (value);
			g_object_notify (object, "buffer_capacity_cb");
			break;
		case PROP_USER_DATA:
			self->user_data = g_value_get_pointer (value);
			g_object_notify (object, "user_data");
			break;
		case PROP_TRACK_NUM:
			self->track_num = g_value_get_uint (value);
			g_object_notify (object, "track_num");
			break;
		case PROP_CDB_BUF_SIZE:
			self->cdb_buf_size = g_value_get_uint (value);
			g_object_notify (object, "cdb_buf_size");
			break;
		case PROP_SENSE_BUF_SIZE:
			self->sense_buf_size = g_value_get_uint (value);
			g_object_notify (object, "sense_buf_size");
			break;
		case PROP_DATA_BUF_SIZE:
			self->data_buf_size = g_value_get_uint (value);
			g_object_notify (object, "data_buf_size");
			break;
		case PROP_START_LBA:
			self->start_lba = g_value_get_uint (value);
			g_object_notify (object, "start_lba");
			break;
		case PROP_TRACK_SIZE:
			self->track_size = g_value_get_uint (value);
			g_object_notify (object, "track_size");
			break;
		case PROP_MAX_WRITE_BETWEEN_READ_BUF_CAP:
			self->max_write_between_read_buf_cap = g_value_get_int (value);
			g_object_notify (object, "max_write_between_read_buf_cap");
			break;
		case PROP_MIN_TIME_BETWEEN_READ_BUF_CAP:
			self->min_time_between_read_buf_cap = g_value_get_int (value);
			g_object_notify (object, "min_time_between_read_buf_cap");
			break;
		case PROP_DATA_BLOCK_TYPE:
			self->data_block_type = g_value_get_enum (value);
			g_object_notify (object, "data_block_type");
			break;
		case PROP_TRACK_MODE:
			self->track_mode = g_value_get_enum (value);
			g_object_notify (object, "track_mode");
			break;
		case PROP_RESERVE:
			self->reserve = g_value_get_boolean (value);
			g_object_notify (object, "reserve");
			break;
		case PROP_TEST:
			self->test = g_value_get_boolean (value);
			g_object_notify (object, "test");
			break;
		case PROP_READ_BUF_CAP:
			self->read_buf_cap = g_value_get_boolean (value);
			if (BURGNER_SCSI_STAGE_MANAGER (self)->stage_id == BURGNER_MMC_TRACK_WRITER_STAGE_ID_WRITE)
				g_object_set (BURGNER_SCSI_STAGE_MANAGER (self)->stage,
					      "read_buf_cap", self->read_buf_cap,
					      NULL);
			g_object_notify (object, "read_buf_cap");
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
burgner_mmc_track_writer_get_property (GObject *object,
	      guint prop_id,
	      GValue *value,
	      GParamSpec *pspec)
{
	BurgnerMmcTrackWriter *self = BURGNER_MMC_TRACK_WRITER (object);

	switch (prop_id) {
		case PROP_DEVICE:
			g_value_set_object (value, self->device);
			break;
		case PROP_FIFO:
			g_value_set_object (value, self->fifo);
			break;
		case PROP_CDB_BUF:
			g_value_set_pointer (value, self->cdb_buf);
			break;
		case PROP_SENSE_BUF:
			g_value_set_pointer (value, self->sense_buf);
			break;
		case PROP_DATA_BUF:
			g_value_set_pointer (value, self->data_buf);
			break;
		case PROP_PROGRESS_CB:
			g_value_set_pointer (value, self->progress_cb);
			break;
		case PROP_BUFFER_CAPACITY_CB:
			g_value_set_pointer (value, self->buffer_capacity_cb);
			break;
		case PROP_USER_DATA:
			g_value_set_pointer (value, self->user_data);
			break;
		case PROP_TRACK_NUM:
			g_value_set_uint (value, self->track_num);
			break;
		case PROP_CDB_BUF_SIZE:
			g_value_set_uint (value, self->cdb_buf_size);
			break;
		case PROP_SENSE_BUF_SIZE:
			g_value_set_uint (value, self->sense_buf_size);
			break;
		case PROP_DATA_BUF_SIZE:
			g_value_set_uint (value, self->data_buf_size);
			break;
		case PROP_START_LBA:
			g_value_set_uint (value, self->start_lba);
			break;
		case PROP_TRACK_SIZE:
			g_value_set_uint (value, self->track_size);
			break;
		case PROP_MAX_WRITE_BETWEEN_READ_BUF_CAP:
			g_value_set_int (value, self->max_write_between_read_buf_cap);
			break;
		case PROP_MIN_TIME_BETWEEN_READ_BUF_CAP:
			g_value_set_int (value, self->min_time_between_read_buf_cap);
			break;
		case PROP_DATA_BLOCK_TYPE:
			g_value_set_enum (value, self->data_block_type);
			break;
		case PROP_TRACK_MODE:
			g_value_set_enum (value, self->track_mode);
			break;
		case PROP_RESERVE:
			g_value_set_boolean (value, self->reserve);
			break;
		case PROP_READ_BUF_CAP:
			g_value_set_boolean (value, self->read_buf_cap);
			break;
		case PROP_TEST:
			g_value_set_boolean (value, self->test);
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
_burgner_mmc_track_writer_progress_cb (BurgnerMmcWriteStage *write_stage,
				       BurgnerScsiDevice *device,
				       guint written_blocks,
				       gpointer user_data)
{
	BurgnerMmcTrackWriter *self = BURGNER_MMC_TRACK_WRITER (user_data);

	self->total_written_blocks += written_blocks;

	if (self->progress_cb) {
		self->progress_cb (self,
				   self->total_written_blocks,
				   self->track_size,
				   self->user_data);
	}
}

static void
_burgner_mmc_track_writer_buffer_capacity_cb (BurgnerMmcWriteStage *write_stage,
					      BurgnerScsiDevice *device,
					      guint32 blank_length,
					      guint32 total_length,
					      GError *error,
					      guint sense_used,
					      gpointer user_data)
{
	BurgnerMmcTrackWriter *self = BURGNER_MMC_TRACK_WRITER (user_data);

	if (self->buffer_capacity_cb) {
		self->buffer_capacity_cb (self,
					  blank_length,
					  total_length,
					  error,
					  sense_used,
					  self->user_data);
	}
}

static BurgnerScsiStage *
_burgner_mmc_track_writer_create_write_stage (BurgnerMmcTrackWriter *self)
{
	BurgnerScsiStage *r = burgner_mmc_write_stage_new ();
	g_object_set (r,
		      "device", self->device,
		      "fifo", self->fifo,
		      "cdb_buf", self->cdb_buf,
		      "cdb_buf_size", self->cdb_buf_size,
		      "sense_buf", self->sense_buf,
		      "sense_buf_size", self->sense_buf_size,
		      "data_block_size", burgner_mmc_data_block_type_get_data_block_size (self->data_block_type),
		      "start_lba", self->start_lba,
		      "read_buf_cap", self->read_buf_cap,
		      "max_write_between_read_buf_cap", self->max_write_between_read_buf_cap,
		      "min_time_between_read_buf_cap", self->min_time_between_read_buf_cap,
		      "read_buf_cap_data_buf", self->data_buf,
		      "read_buf_cap_data_buf_size", self->data_buf_size,
		      "progress_cb", _burgner_mmc_track_writer_progress_cb,
		      "buffer_capacity_cb", _burgner_mmc_track_writer_buffer_capacity_cb,
		      "user_data", self,
		      NULL);
	return r;
}

#define __MASK_BUF_SIZE		(64)

static BurgnerScsiStage *
_burgner_mmc_track_writer_get_next_stage (BurgnerScsiStageManager *sm,
					  gint *stage_id,
					  BurgnerScsiStage *actual_stage,
					  gint actual_stage_id)
{
	BurgnerMmcTrackWriter *self = BURGNER_MMC_TRACK_WRITER (sm);
	BurgnerScsiStage *stage = NULL;

	g_assert (stage_id != NULL);

	g_return_val_if_fail (self->device != NULL, NULL);
	g_return_val_if_fail (self->fifo != NULL, NULL);
	g_return_val_if_fail (self->cdb_buf != NULL, NULL);
	g_return_val_if_fail (self->cdb_buf_size >= 10, NULL);
	g_return_val_if_fail (self->data_buf != NULL, NULL);
	g_return_val_if_fail (self->data_buf_size >= 56, NULL);
	g_return_val_if_fail (!self->reserve || self->track_size, NULL);

	switch (actual_stage_id) {
		case BURGNER_MMC_TRACK_WRITER_STAGE_ID_NONE:
			self->mask_buf = g_malloc0 (__MASK_BUF_SIZE);
			stage = burgner_mmc_get_write_page_stage_new ();
			g_object_set (stage,
				      "device", self->device,
				      "cdb_buf", self->cdb_buf,
				      "cdb_buf_size", self->cdb_buf_size,
				      "sense_buf", self->sense_buf,
				      "sense_buf_size", self->sense_buf_size,
				      "data_buf", self->mask_buf,
				      "data_buf_size", __MASK_BUF_SIZE,
				      "values", BURGNER_SCSI_MODE_PAGE_VALUES_CHANGEABLE,
				      NULL);
			*stage_id = BURGNER_MMC_TRACK_WRITER_STAGE_ID_GET_WRITE_PAGE_CHANGEABLE;
			break;
		case BURGNER_MMC_TRACK_WRITER_STAGE_ID_GET_WRITE_PAGE_CHANGEABLE:
			stage = BURGNER_SCSI_STAGE (g_object_ref (actual_stage));
			g_object_set (stage,
				      "data_buf", self->data_buf,
				      "data_buf_size", self->data_buf_size,
				      "values", BURGNER_SCSI_MODE_PAGE_VALUES_CURRENT,
				      NULL);
			*stage_id = BURGNER_MMC_TRACK_WRITER_STAGE_ID_GET_WRITE_PAGE_CURRENT;
			break;
		case BURGNER_MMC_TRACK_WRITER_STAGE_ID_GET_WRITE_PAGE_CURRENT:
			stage = burgner_mmc_set_write_page_stage_new ();
			g_object_set (stage,
				      "device", self->device,
				      "cdb_buf", self->cdb_buf,
				      "cdb_buf_size", self->cdb_buf_size,
				      "sense_buf", self->sense_buf,
				      "sense_buf_size", self->sense_buf_size,
				      "data_buf", self->data_buf,
#if 0
				      "data_buf_size", self->data_buf_size,
#endif
				      NULL);
			/*
			 * Don't know why, I got some times a GLib-GObject-WARNING saying
			 * that the BurgnerMmcSetWritePageStage has no property named data_buf_size
			 */
			((BurgnerScsiSetModePageStage *) stage)->data_buf_size = self->data_buf_size;
			*stage_id = BURGNER_MMC_TRACK_WRITER_STAGE_ID_SET_WRITE_PAGE;
			break;
		case BURGNER_MMC_TRACK_WRITER_STAGE_ID_SET_WRITE_PAGE:
			if (self->reserve) {
				stage = burgner_mmc_reserve_track_stage_new ();
				g_object_set (stage,
					      "device", self->device,
					      "cdb_buf", self->cdb_buf,
					      "cdb_buf_size", self->cdb_buf_size,
					      "sense_buf", self->sense_buf,
					      "sense_buf_size", self->sense_buf_size,
					      "track_size", self->track_size,
					      NULL);
				*stage_id = BURGNER_MMC_TRACK_WRITER_STAGE_ID_RESERVE_TRACK;
			} else {
				stage = _burgner_mmc_track_writer_create_write_stage (self);
				*stage_id = BURGNER_MMC_TRACK_WRITER_STAGE_ID_WRITE;
			}
			break; 
		case BURGNER_MMC_TRACK_WRITER_STAGE_ID_RESERVE_TRACK:
			stage = _burgner_mmc_track_writer_create_write_stage (self);
			*stage_id = BURGNER_MMC_TRACK_WRITER_STAGE_ID_WRITE;
			break;
		case BURGNER_MMC_TRACK_WRITER_STAGE_ID_WRITE:
			stage = burgner_mmc_close_track_stage_new ();
			g_object_set (stage,
				      "device", self->device,
				      "cdb_buf", self->cdb_buf,
				      "cdb_buf_size", self->cdb_buf_size,
				      "sense_buf", self->sense_buf,
				      "sense_buf_size", self->sense_buf_size,
				      "track_num", self->track_num,
				      "immed", FALSE,
				      NULL);
			*stage_id = BURGNER_MMC_TRACK_WRITER_STAGE_ID_CLOSE_TRACK;
			break;
		case BURGNER_MMC_TRACK_WRITER_STAGE_ID_CLOSE_TRACK:
			stage = NULL;
			*stage_id = -1;
			break;
		default:
			g_return_val_if_reached (NULL);
			break;
	}

	return stage;
}

static gboolean
_burgner_mmc_track_writer_continue (BurgnerScsiStageManager *sm,
				    BurgnerScsiStage *stage,
				    gint stage_id,
				    GError *stage_error,
				    BurgnerScsiSense *sense,
				    GError **error)
{
	BurgnerMmcTrackWriter *self = BURGNER_MMC_TRACK_WRITER (sm);

	if (stage_error || sense) {
		if (stage_error)
			g_propagate_error (error, stage_error);
		return FALSE;
	}

	if (stage_id == BURGNER_MMC_TRACK_WRITER_STAGE_ID_GET_WRITE_PAGE_CURRENT) {
		GList *p = NULL, *m = NULL;
		gboolean r;

		if ((r = burgner_scsi_mode_parameter_list_parse (TRUE,
								 self->data_buf,
								 self->data_buf_size,
								 NULL,
								 &p,
								 error)) &&
		    (r = burgner_scsi_mode_parameter_list_parse (TRUE,
				    				 self->mask_buf,
								 __MASK_BUF_SIZE,
								 NULL,
								 &m,
								 error))) {
			if (!(r = (p && p->data && m && m->data))) {
				g_set_error (error,
					     BURGNER_MMC_TRACK_WRITER_ERROR,
					     BURGNER_MMC_TRACK_WRITER_ERROR_INVALID_MODE_PARAMETER,
					     _("Device returned bad mode parameter"));
			} else {
				r = burgner_mmc_write_page_setup_tao_write (p->data,
							     m->data,
							     self->data_block_type,
							     self->track_mode,
							     self->test,
							     error);
			}
		}

		g_list_free (p);
		g_list_free (m);
		g_free (self->mask_buf);
		self->mask_buf = NULL;
		return r;
	}

	return TRUE;
}

static void
burgner_mmc_track_writer_class_init (BurgnerMmcTrackWriterClass *klass)
{
	GObjectClass *object_class;
	BurgnerScsiStageManagerClass *sm_class;

	object_class = (GObjectClass *) klass;
	object_class->set_property = burgner_mmc_track_writer_set_property;
	object_class->get_property = burgner_mmc_track_writer_get_property;
	object_class->finalize = burgner_mmc_track_writer_finalize;

	sm_class = (BurgnerScsiStageManagerClass *) klass;
	sm_class->get_next_stage = _burgner_mmc_track_writer_get_next_stage;
	sm_class->_continue = _burgner_mmc_track_writer_continue;

	/* Properties */
	g_object_class_install_property (object_class,
					 PROP_DEVICE,
					 g_param_spec_object ("device",
					                _("device"),
							_("A BurgnerScsiDevice"),
							BURGNER_TYPE_SCSI_DEVICE,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_FIFO,
					 g_param_spec_object ("fifo",
					                _("fifo"),
							_("A BurgnerFifo"),
							BURGNER_TYPE_FIFO,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_CDB_BUF,
					 g_param_spec_pointer ("cdb_buf",
					                _("cdb_buf"),
							_("A buffer for the CDB"),
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_SENSE_BUF,
					 g_param_spec_pointer ("sense_buf",
					                _("sense_buf"),
							_("A buffer for the sense"),
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_DATA_BUF,
					 g_param_spec_pointer ("data_buf",
					                _("data_buf"),
							_("A buffer for the data"),
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_PROGRESS_CB,
					 g_param_spec_pointer ("progress_cb",
					                _("progress_cb"),
							_("A BurgnerMmcTrackWriterProgressCallback"),
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_BUFFER_CAPACITY_CB,
					 g_param_spec_pointer ("buffer_capacity_cb",
					                _("buffer_capacity_cb"),
							_("A BurgnerMmcTrackWriterBufferCapacityCallback"),
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_USER_DATA,
					 g_param_spec_pointer ("user_data",
					                _("user_data"),
							_("User Data to be passed to progress_cb and buffer_capacity_cb"),
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_TRACK_NUM,
					 g_param_spec_uint ("track_num",
					                _("track_num"),
							_("Number of the track to be written (0xff for last track)"),
							0,
							0xff,
							0xff,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_CDB_BUF_SIZE,
					 g_param_spec_uint ("cdb_buf_size",
					                _("cdb_buf_size"),
							_("Size of cdb_buf (>= 10)"),
							0,
							G_MAXUINT,
							0,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_SENSE_BUF_SIZE,
					 g_param_spec_uint ("sense_buf_size",
					                _("sense_buf_size"),
							_("Size of sense_buf (>= 18)"),
							0,
							G_MAXUINT,
							0,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_DATA_BUF_SIZE,
					 g_param_spec_uint ("data_buf_size",
					                _("data_buf_size"),
							_("Size of data_buf (>= 56)"),
							0,
							G_MAXUINT,
							0,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_START_LBA,
					 g_param_spec_uint ("start_lba",
					                _("start_lba"),
							_("The start LBA"),
							0,
							0xffffffff,
							0,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_TRACK_SIZE,
					 g_param_spec_uint ("track_size",
					                _("track_size"),
							_("Size (in blocks) of the track to be written (only mandatory for reserving)"),
							0,
							0xffffffff,
							0,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_MAX_WRITE_BETWEEN_READ_BUF_CAP,
					 g_param_spec_int ("max_write_between_read_buf_cap",
					                _("max_write_between_read_buf_cap"),
							_("Maximum WRITE commands between two READ BUFFER CAPACITY commands"),
							-1,
							G_MAXINT,
							20,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_MIN_TIME_BETWEEN_READ_BUF_CAP,
					 g_param_spec_int ("min_time_between_read_buf_cap",
					                _("min_time_between_read_buf_cap"),
							_("Minimum time (in milliseconds) between two consecutive readings of buffer capacity"),
							-1,
							G_MAXINT,
							1000,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_DATA_BLOCK_TYPE,
					 g_param_spec_enum ("data_block_type",
					                _("data_block_type"),
							_("Data Block Type"),
							BURGNER_TYPE_MMC_DATA_BLOCK_TYPE,
							BURGNER_MMC_DATA_BLOCK_TYPE_MODE_1,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_TRACK_MODE,
					 g_param_spec_enum ("track_mode",
					                _("track_mode"),
							_("Track Mode"),
							BURGNER_TYPE_MMC_TRACK_MODE,
							BURGNER_MMC_TRACK_MODE_DATA_TRACK_UNINTERRUPTED,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_RESERVE,
					 g_param_spec_boolean ("reserve",
					                _("reserve"),
							_("Reserve Track (if TRUE, track size must be set)"),
							FALSE,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_TEST,
					 g_param_spec_boolean ("test",
					                _("test"),
							_("Write track in test mode"),
							FALSE,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_READ_BUF_CAP,
					 g_param_spec_boolean ("read_buf_cap",
					                _("read_buf_cap"),
							_("Read Buffer Capacity"),
							TRUE,
							G_PARAM_READABLE | G_PARAM_WRITABLE));


	parent_class = g_type_class_peek_parent (klass);
}

static void
burgner_mmc_track_writer_init (BurgnerMmcTrackWriter *self)
{
	self->device = NULL;
	self->fifo = NULL;
	self->cdb_buf = self->sense_buf = self->data_buf = self->mask_buf = NULL;
	self->cdb_buf_size = self->sense_buf_size = self->data_buf_size = 0;
	self->data_block_type = BURGNER_MMC_DATA_BLOCK_TYPE_MODE_1;
	self->track_mode = BURGNER_MMC_TRACK_MODE_DATA_TRACK_UNINTERRUPTED;
	self->max_write_between_read_buf_cap = 20;
	self->min_time_between_read_buf_cap = 1000;
	self->start_lba = 0;
	self->track_size = 0;
	self->total_written_blocks = 0;
	self->progress_cb = NULL;
	self->buffer_capacity_cb = NULL;
	self->user_data = NULL;
	self->track_num = 0xff;
	self->reserve = 0;
	self->test = 0;
	self->read_buf_cap = 1;
}

GType
burgner_mmc_track_writer_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		static GTypeInfo info = {
			sizeof (BurgnerMmcTrackWriterClass),
			NULL,
			NULL,
			(GClassInitFunc) burgner_mmc_track_writer_class_init,
			NULL,
			NULL,
			sizeof (BurgnerMmcTrackWriter),
			0,
			(GInstanceInitFunc) burgner_mmc_track_writer_init,
		};

		type = g_type_register_static (BURGNER_TYPE_SCSI_STAGE_MANAGER, "BurgnerMmcTrackWriter", &info, 0);
	}

	return type;
}

BurgnerScsiStage *
burgner_mmc_track_writer_new (void)
{
	BurgnerMmcTrackWriter *self;

	self = g_object_new (BURGNER_TYPE_MMC_TRACK_WRITER, NULL);


	return BURGNER_SCSI_STAGE (self);
}


GQuark
burgner_mmc_track_writer_error_quark (void)
{
	return g_quark_from_string ("BurgnerMmcTrackWriter");
}

