/*
 * BurgnerMmcTrackWriter - BurgnerMmcTrackWriter object
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifndef __BURGNER_MMC_TRACK_WRITER_H__
#define __BURGNER_MMC_TRACK_WRITER_H__

#include <glib-object.h>
#include <libburgnerscsi/burgner-scsi-device.h>
#include <libburgnerutils/burgner-fifo.h>
#include <libburgnermmc/burgner-mmc-data-block-type.h>
#include <libburgnermmc/burgner-mmc-track-mode.h>
#include <libburgnerscsi/burgner-scsi-stage-manager.h>

G_BEGIN_DECLS


#define BURGNER_TYPE_MMC_TRACK_WRITER		(burgner_mmc_track_writer_get_type ())
#define BURGNER_MMC_TRACK_WRITER(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), BURGNER_TYPE_MMC_TRACK_WRITER, BurgnerMmcTrackWriter))
#define BURGNER_MMC_TRACK_WRITER_CLASS(k)	(G_TYPE_CHECK_CLASS_CAST ((k), BURGNER_TYPE_MMC_TRACK_WRITER, BurgnerMmcTrackWriterClass))
#define BURGNER_IS_MMC_TRACK_WRITER(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), BURGNER_TYPE_MMC_TRACK_WRITER))
#define BURGNER_IS_MMC_TRACK_WRITER_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), BURGNER_TYPE_MMC_TRACK_WRITER))
#define BURGNER_MMC_TRACK_WRITER_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), BURGNER_TYPE_MMC_TRACK_WRITER, BurgnerMmcTrackWriterClass))


typedef struct _BurgnerMmcTrackWriter BurgnerMmcTrackWriter;
typedef struct _BurgnerMmcTrackWriterClass BurgnerMmcTrackWriterClass;

typedef void (* BurgnerMmcTrackWriterProgressCallback)
				(BurgnerMmcTrackWriter *track_writer,
				 guint32 total_written_blocks,
				 guint32 track_size,
				 gpointer user_data);
typedef void (* BurgnerMmcTrackWriterBufferCapacityCallback)
				(BurgnerMmcTrackWriter *track_writer,
				 guint32 blank_length,
				 guint32 total_length,
				 GError *error,
				 guint sense_used,
				 gpointer user_data);

struct _BurgnerMmcTrackWriter {
	BurgnerScsiStageManager __parent;

	BurgnerScsiDevice *device;
	BurgnerFifo *fifo;

	gpointer cdb_buf;
	guint cdb_buf_size;
	gpointer sense_buf;
	guint sense_buf_size;
	gpointer data_buf;
	guint data_buf_size;
	gpointer mask_buf;

	BurgnerMmcDataBlockType data_block_type;
	BurgnerMmcTrackMode track_mode;

	gint max_write_between_read_buf_cap;
	gint min_time_between_read_buf_cap;

	guint32 start_lba;
	guint32 track_size;
	guint32 total_written_blocks;

	BurgnerMmcTrackWriterProgressCallback progress_cb;
	BurgnerMmcTrackWriterBufferCapacityCallback buffer_capacity_cb;
	gpointer user_data;

	guint8 track_num;

	guint reserve : 1;
	guint test : 1;
	guint read_buf_cap : 1;
};

struct _BurgnerMmcTrackWriterClass {
	BurgnerScsiStageManagerClass __parent_class;

};

enum {
	BURGNER_MMC_TRACK_WRITER_STAGE_ID_NONE = -1,
	BURGNER_MMC_TRACK_WRITER_STAGE_ID_GET_WRITE_PAGE_CHANGEABLE,
	BURGNER_MMC_TRACK_WRITER_STAGE_ID_GET_WRITE_PAGE_CURRENT,
	BURGNER_MMC_TRACK_WRITER_STAGE_ID_SET_WRITE_PAGE,
	BURGNER_MMC_TRACK_WRITER_STAGE_ID_RESERVE_TRACK,
	BURGNER_MMC_TRACK_WRITER_STAGE_ID_WRITE,
	BURGNER_MMC_TRACK_WRITER_STAGE_ID_CLOSE_TRACK
};


GType burgner_mmc_track_writer_get_type (void) G_GNUC_CONST;
GQuark  burgner_mmc_track_writer_error_quark (void);

#define BURGNER_MMC_TRACK_WRITER_ERROR		(burgner_mmc_track_writer_error_quark ())
typedef enum {
	BURGNER_MMC_TRACK_WRITER_ERROR_INVALID_MODE_PARAMETER
} BurgnerMmcTrackWriterError;


BurgnerScsiStage *burgner_mmc_track_writer_new (void);

G_END_DECLS

#endif /* __BURGNER_MMC_TRACK_WRITER_H__ */

