/*
 * 
 *
 * This file is part of the libburgnermmc library.
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "burgner-mmc-utils.h"

/*
 *    DEFINITIONS
 * 1 kB/s <-> 1000 B/s (not 1024 here!)
 * 1kB <-> 1024 B
 * 1 sector <-> 2048 B of data
 *          <-> 2352 B of raw data
 * 1x <-> 150 kB of data per second (!= 150 kB/s)
 *
 *
 * => 1x <-> 150*1024=153600 B of data per second
 *       <-> 153600/2048=75 sectors per second
 *       <-> 75*2352=176400 B of raw data per second
 *       <-> 176400/1000=176.4 kB/s
 *
 *     +-------------------+
 * =>  | 1x <-> 176.4 kB/s |
 *     +-------------------+
 */

guint16
burgner_mmc_speed_x_to_kbps (guint x)
{
	guint r = 1764 * x, s;
	s = r / 10;
	return ((r % 10) >= 5) ? s + 1 : s;
}

guint
burgner_mmc_speed_kbps_to_x (guint16 speed)
{
	guint r = (100 * speed) / 1764, s;
	s = r / 10;
	return ((r % 10) >= 5) ? s + 1 : s;
}

