/*
 * 
 *
 * This file is part of the libburgnermmc library.
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "burgner-mmc-i18n.h"

#include "burgner-mmc-impl.h"
#include "burgner-mmc-write-page-utils.h"


static guchar
_burgner_mmc_write_page_session_format_to_impl
					(BurgnerMmcWritePageSessionFormat sf)
{
	guchar r = 0;

	switch (sf) {
		case BURGNER_MMC_WRITE_PAGE_SESSION_FORMAT_CD_DA_OR_CD_ROM:
			r = _BURGNER_MMC_IMPL_WRITE_PAGE_SESSION_FORMAT_CD_DA_OR_CD_ROM;
			break;
		case BURGNER_MMC_WRITE_PAGE_SESSION_FORMAT_CD_I:
			r = _BURGNER_MMC_IMPL_WRITE_PAGE_SESSION_FORMAT_CD_I;
			break;
		case BURGNER_MMC_WRITE_PAGE_SESSION_FORMAT_CD_ROM_XA:
			r = _BURGNER_MMC_IMPL_WRITE_PAGE_SESSION_FORMAT_CD_ROM_XA;
			break;
		default:
			g_return_val_if_reached (0);
	}

	return r;
}

static guchar
_burgner_mmc_write_page_session_closure_to_impl (BurgnerMmcWritePageSessionClosure sc)
{
	guchar r = 0;

	switch (sc) {
		case BURGNER_MMC_WRITE_PAGE_SESSION_CLOSURE_NO_B0_POINTER:
			r = _BURGNER_MMC_IMPL_WRITE_PAGE_MULTI_SESSION_NO_B0_POINTER;
			break;
		case BURGNER_MMC_WRITE_PAGE_SESSION_CLOSURE_NO_NEXT_SESSION:
			r = _BURGNER_MMC_IMPL_WRITE_PAGE_MULTI_SESSION_B0_POINTER_FFFFFF;
			break;
		case BURGNER_MMC_WRITE_PAGE_SESSION_CLOSURE_NEXT_SESSION_ALLOWED:
			r = _BURGNER_MMC_IMPL_WRITE_PAGE_MULTI_SESSION_NEXT_SESSION_ALLOWED;
			break;
		default:
			g_return_val_if_reached (0);
			break;
	}

	return r;
}

static guchar
_burgner_mmc_write_page_data_block_type_to_impl (BurgnerMmcDataBlockType type)
{
	guchar r = 0;

	switch (type) {
		case BURGNER_MMC_DATA_BLOCK_TYPE_RAW_DATA:
			r = _BURGNER_MMC_IMPL_WRITE_PAGE_DATA_BLOCK_TYPE_RAW_DATA;
			break;
		case BURGNER_MMC_DATA_BLOCK_TYPE_RAW_DATA_WITH_P_Q_SUB_CHANNEL:
			r = _BURGNER_MMC_IMPL_WRITE_PAGE_DATA_BLOCK_TYPE_RAW_DATA_WITH_P_Q_SUB_CHANNEL;
			break;
		case BURGNER_MMC_DATA_BLOCK_TYPE_RAW_DARA_WITH_P_W_SUB_CHANNEL_1:
			r = _BURGNER_MMC_IMPL_WRITE_PAGE_DATA_BLOCK_TYPE_RAW_DATA_WITH_P_W_SUB_CHANNEL_1;
			break;
		case BURGNER_MMC_DATA_BLOCK_TYPE_RAW_DATA_WITH_P_W_SUB_CHANNEL_2:
			r = _BURGNER_MMC_IMPL_WRITE_PAGE_DATA_BLOCK_TYPE_RAW_DATA_WITH_P_W_SUB_CHANNEL_2;
			break;
		case BURGNER_MMC_DATA_BLOCK_TYPE_VENDOR_SPECIFIC_1:
			r = _BURGNER_MMC_IMPL_WRITE_PAGE_DATA_BLOCK_TYPE_VENDOR_SPECIFIC_1;
			break;
		case BURGNER_MMC_DATA_BLOCK_TYPE_MODE_1:
			r = _BURGNER_MMC_IMPL_WRITE_PAGE_DATA_BLOCK_TYPE_MODE_1;
			break;
		case BURGNER_MMC_DATA_BLOCK_TYPE_MODE_2:
			r = _BURGNER_MMC_IMPL_WRITE_PAGE_DATA_BLOCK_TYPE_MODE_2;
			break;
		case BURGNER_MMC_DATA_BLOCK_TYPE_CD_ROM_XA_MODE_2_FORM_1_1:
			r = _BURGNER_MMC_IMPL_WRITE_PAGE_DATA_BLOCK_TYPE_CD_ROM_XA_MODE_2_FORM_1_1;
			break;
		case BURGNER_MMC_DATA_BLOCK_TYPE_CD_ROM_XA_MODE_2_FORM_1_2:
			r = _BURGNER_MMC_IMPL_WRITE_PAGE_DATA_BLOCK_TYPE_CD_ROM_XA_MODE_2_FORM_1_2;
			break;
		case BURGNER_MMC_DATA_BLOCK_TYPE_CD_ROM_XA_MODE_2_FORM_2:
			r = _BURGNER_MMC_IMPL_WRITE_PAGE_DATA_BLOCK_TYPE_CD_ROM_XA_MODE_2_FORM_2;
			break;
		case BURGNER_MMC_DATA_BLOCK_TYPE_CD_ROM_XA_MIXED:
			r = _BURGNER_MMC_IMPL_WRITE_PAGE_DATA_BLOCK_TYPE_CD_ROM_XA_MIXED;
			break;
		case BURGNER_MMC_DATA_BLOCK_TYPE_VENDOR_SPECIFIC_2:
			r = _BURGNER_MMC_IMPL_WRITE_PAGE_DATA_BLOCK_TYPE_VENDOR_SPECIFIC_2;
			break;
		default:
			g_return_val_if_reached (0);
			break;
	}

	return r;
}

static guint
_burgner_mmc_write_page_track_mode_to_impl (BurgnerMmcTrackMode track_mode)
{
	guint r = 0;

	switch (track_mode) {
		case BURGNER_MMC_TRACK_MODE_2_AUDIO_CHANNELS_WITHOUT_PRE_EMPHASIS:
			r = _BURGNER_MMC_IMPL_TRACK_MODE_2_AUDIO_CHANNELS_WITHOUT_PRE_EMPHASIS;
			break;
		case BURGNER_MMC_TRACK_MODE_2_AUDIO_CHANNELS_WITH_PRE_EMPHASIS:
			r = _BURGNER_MMC_IMPL_TRACK_MODE_2_AUDIO_CHANNELS_WITH_PRE_EMPHASIS;
			break;
		case BURGNER_MMC_TRACK_MODE_AUDIO_CHANNELS_WITHOUT_PRE_EMPHASIS:
			r = _BURGNER_MMC_IMPL_TRACK_MODE_AUDIO_CHANNELS_WITHOUT_PRE_EMPHASIS;
			break;
		case BURGNER_MMC_TRACK_MODE_AUDIO_CHANNELS_WITH_PRE_EMPHASIS:
			r = _BURGNER_MMC_IMPL_TRACK_MODE_AUDIO_CHANNELS_WITH_PRE_EMPHASIS;
			break;
		case BURGNER_MMC_TRACK_MODE_DATA_TRACK_UNINTERRUPTED:
			r = _BURGNER_MMC_IMPL_TRACK_MODE_DATA_TRACK_UNINTERRUPTED;
			break;
		case BURGNER_MMC_TRACK_MODE_DATA_TRACK_INCREMENTAL:
			r = _BURGNER_MMC_IMPL_TRACK_MODE_DATA_TRACK_INCREMENTAL;
			break;
		default:
			g_return_val_if_reached (0);
			break;
	}

	return r;
}


gboolean
burgner_mmc_write_page_setup_session (gpointer write_page,
				      gpointer mask_write_page,
				      BurgnerMmcWritePageSessionFormat sf,
				      BurgnerMmcWritePageSessionClosure closure,
				      GError **error)
{
	_BurgnerMmcImplWritePage *impl, *mask_impl;

	g_return_val_if_fail (write_page != NULL, FALSE);

	impl = (_BurgnerMmcImplWritePage *) write_page;
	mask_impl = (_BurgnerMmcImplWritePage *) mask_write_page;

	if (mask_write_page) {
		if ((mask_impl->multi_session & 3) != 3) {
			g_set_error (error,
				     BURGNER_MMC_WRITE_PAGE_UTILS_ERROR,
				     BURGNER_MMC_WRITE_PAGE_UTILS_ERROR_CANT_SET_SESSION_FORMAT,
				     _("Unable to set session format"));
			return FALSE;
		} else if ((mask_impl->session_format & 0x30) != 0x30) {
			g_set_error (error,
				     BURGNER_MMC_WRITE_PAGE_UTILS_ERROR,
				     BURGNER_MMC_WRITE_PAGE_UTILS_ERROR_CANT_SET_SESSION_CLOSURE,
				     _("Unable to set session closure"));
			return FALSE;
		}
	}

	impl->multi_session = _burgner_mmc_write_page_session_closure_to_impl (closure);
	impl->session_format = _burgner_mmc_write_page_session_format_to_impl (sf);

	return TRUE;
}

gboolean
burgner_mmc_write_page_setup_tao_write (gpointer write_page,
					gpointer mask_write_page,
					BurgnerMmcDataBlockType block_type,
					BurgnerMmcTrackMode track_mode,
					gboolean test_mode,
					GError **error)
{
	_BurgnerMmcImplWritePage *impl, *mask_impl;

	g_return_val_if_fail (write_page != NULL, FALSE);

	impl = (_BurgnerMmcImplWritePage *) write_page;
	mask_impl = (_BurgnerMmcImplWritePage *) mask_write_page;

	if (mask_write_page) {
		if (mask_impl->data_block_type != 0xf) {
			g_set_error (error,
				     BURGNER_MMC_WRITE_PAGE_UTILS_ERROR,
				     BURGNER_MMC_WRITE_PAGE_UTILS_ERROR_CANT_SET_DATA_BLOCK_TYPE,
				     _("Unable to set data block type"));
			return FALSE;
		} else if (mask_impl->test_write != 1) {
			g_set_error (error,
				     BURGNER_MMC_WRITE_PAGE_UTILS_ERROR,
				     BURGNER_MMC_WRITE_PAGE_UTILS_ERROR_CANT_SET_TEST_BIT,
				     _("Unable to set up test mode"));
			return FALSE;
		} else if ((mask_impl->track_mode & 0xd) != 0xd) {
			g_set_error (error,
				     BURGNER_MMC_WRITE_PAGE_UTILS_ERROR,
				     BURGNER_MMC_WRITE_PAGE_UTILS_ERROR_CANT_SET_TRACK_MODE,
				     _("Unable to set track mode"));
			return FALSE;
		}
	}

	impl->data_block_type = _burgner_mmc_write_page_data_block_type_to_impl (block_type);
	impl->track_mode = _burgner_mmc_write_page_track_mode_to_impl (track_mode);
	impl->test_write = test_mode ? 1 : 0;

	return TRUE;
}

GQuark
burgner_mmc_write_page_utils_error_quark (void)
{
	return g_quark_from_string ("BurgnerMmcWritePage utils");
}

