/*
 * BurgnerMmcWriteStage - BurgnerMmcWriteStage object
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifndef __BURGNER_MMC_WRITE_STAGE_H__
#define __BURGNER_MMC_WRITE_STAGE_H__

#include <glib-object.h>
#include <libburgnerutils/burgner-fifo.h>
#include <libburgnerscsi/burgner-scsi-device.h>
#include <libburgnerscsi/burgner-scsi-stage.h>

G_BEGIN_DECLS


#define BURGNER_TYPE_MMC_WRITE_STAGE		(burgner_mmc_write_stage_get_type ())
#define BURGNER_MMC_WRITE_STAGE(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), BURGNER_TYPE_MMC_WRITE_STAGE, BurgnerMmcWriteStage))
#define BURGNER_MMC_WRITE_STAGE_CLASS(k)	(G_TYPE_CHECK_CLASS_CAST ((k), BURGNER_TYPE_MMC_WRITE_STAGE, BurgnerMmcWriteStageClass))
#define BURGNER_IS_MMC_WRITE_STAGE(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), BURGNER_TYPE_MMC_WRITE_STAGE))
#define BURGNER_IS_MMC_WRITE_STAGE_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), BURGNER_TYPE_MMC_WRITE_STAGE))
#define BURGNER_MMC_WRITE_STAGE_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), BURGNER_TYPE_MMC_WRITE_STAGE, BurgnerMmcWriteStageClass))

#define BURGNER_MMC_WRITE_STAGE_CDB_BUF_MIN_SIZE	(10)


typedef struct _BurgnerMmcWriteStage BurgnerMmcWriteStage;
typedef struct _BurgnerMmcWriteStageClass BurgnerMmcWriteStageClass;


typedef void (* BurgnerMmcWriteStageProgressCallback)
				(BurgnerMmcWriteStage *write_stage,
				 BurgnerScsiDevice *device,
				 guint written_blocks,
				 gpointer user_data);
typedef void (* BurgnerMmcWriteStageBufferCapacityCallback)
				(BurgnerMmcWriteStage *write_stage,
				 BurgnerScsiDevice *device,
				 guint32 blank_length,
				 guint32 total_length,
				 GError *error,
				 guint sense_used,
				 gpointer user_data);

struct _BurgnerMmcWriteStage {
	BurgnerScsiStage __parent;

	BurgnerScsiDevice *device;
	BurgnerFifo *fifo;
	guint write_count;
	/* FIXME: next_lba may be negative */
	guint32 next_lba;
	guint data_block_size;

	gpointer cdb_buf;
	guint cdb_buf_size;

	gpointer sense_buf;
	guint sense_buf_size;

	gpointer read_buf_cap_data_buf;
	guint read_buf_cap_data_buf_size;

	guint pad;

	gint max_write_between_read_buf_cap;
	gint min_time_between_read_buf_cap;

	BurgnerMmcWriteStageProgressCallback progress_cb;
	BurgnerMmcWriteStageBufferCapacityCallback buffer_cb;
	gpointer user_data;

	guint timeout_id;
	guint idle_id;

	guint read_buf_cap : 1;
#if 0
	guint can_read : 1;
#endif
	guint end : 1;
	guint state : 2;
	guint cancel : 1;
};

struct _BurgnerMmcWriteStageClass {
	BurgnerScsiStageClass __parent_class;

};


GType burgner_mmc_write_stage_get_type (void) G_GNUC_CONST;
GQuark  burgner_mmc_write_stage_error_quark (void);

#define BURGNER_MMC_WRITE_STAGE_ERROR		(burgner_mmc_write_stage_error_quark ())

BurgnerScsiStage *burgner_mmc_write_stage_new (void);

G_END_DECLS

#endif /* __BURGNER_MMC_WRITE_STAGE_H__ */

