/*
 * BurgnerMmcWrite
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This file is part of the libburgnermmc library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include "../libburgnerscsi/burgner-scsi-utils.h"
#include "burgner-mmc-impl.h"
#include "burgner-mmc-write.h"

void
burgner_mmc_write_write_cdb (gpointer buf,
			     guint buf_size,
			     guint data_buf_size,
			     guint data_block_size,
			     guint32 lba,
			     gboolean force_unit_access)
{
	_BurgnerMmcImplWriteCdb *impl;
	guint n;

	g_return_if_fail (buf != NULL);
	g_return_if_fail (buf_size >= BURGNER_MMC_WRITE_CDB_SIZE);
	g_return_if_fail (data_block_size != 0);
	g_return_if_fail ((data_buf_size % data_block_size) == 0);

	n = data_buf_size / data_block_size;
	g_return_if_fail ((n & ~0xffff) == 0);

	impl = (_BurgnerMmcImplWriteCdb *) buf;
	memset (impl, 0, BURGNER_MMC_WRITE_CDB_SIZE);

	impl->op_code = BURGNER_MMC_WRITE_OP_CODE;
	impl->fua = force_unit_access ? 1 : 0;
	burgner_scsi_set_uint32 (impl->lba, lba);
	burgner_scsi_set_uint16 (impl->length, n);
}

