/*
 * BurgnerScsiDevice - BurgnerScsiDevice object
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib-object.h>
#include "burgner-scsi-device.h"


static GObjectClass *parent_class = NULL;

static void
burgner_scsi_device_finalize (GObject *object)
{
	if (G_OBJECT_CLASS (parent_class)->finalize)
		G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
burgner_scsi_device_class_init (BurgnerScsiDeviceClass *klass)
{
	GObjectClass *object_class;

	object_class = (GObjectClass *) klass;
	object_class->finalize = burgner_scsi_device_finalize;

	parent_class = g_type_class_peek_parent (klass);
}

static void
burgner_scsi_device_init (BurgnerScsiDevice *self)
{
	
}

GType
burgner_scsi_device_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		static GTypeInfo info = {
			sizeof (BurgnerScsiDeviceClass),
			NULL,
			NULL,
			(GClassInitFunc) burgner_scsi_device_class_init,
			NULL,
			NULL,
			sizeof (BurgnerScsiDevice),
			0,
			(GInstanceInitFunc) burgner_scsi_device_init,
		};

		type = g_type_register_static (G_TYPE_OBJECT, "BurgnerScsiDevice", &info, 0 /*G_TYPE_FLAG_ABSTRACT*/);
	}

	return type;
}


GQuark
burgner_scsi_device_error_quark (void)
{
	return g_quark_from_string ("BurgnerScsiDevice");
}

gboolean
burgner_scsi_device_send_cmd_sync (BurgnerScsiDevice *dev,
			   gpointer cdb, guint cdb_size,
			   gpointer sense, guint sense_size, guint *sense_used,
			   gpointer data, guint data_size, guint *data_used,
			   BurgnerScsiDirection direction,
			   guint timeout,
			   gint *status,
			   GError **err)
{
	g_return_val_if_fail (dev != NULL, FALSE);
	g_return_val_if_fail (BURGNER_IS_SCSI_DEVICE (dev), FALSE);

	if (BURGNER_SCSI_DEVICE_GET_CLASS (dev)->send_cmd_sync)
		return BURGNER_SCSI_DEVICE_GET_CLASS (dev)->send_cmd_sync (dev, cdb, cdb_size, sense, sense_size, sense_used, data, data_size, data_used, direction, timeout, status, err);

	return FALSE;
}

void
burgner_scsi_device_send_cmd (BurgnerScsiDevice *dev,
				    gpointer cdb, guint cdb_size,
				    gpointer sense, guint sense_size,
				    gpointer data, guint data_size,
				    BurgnerScsiDirection direction,
				    guint timeout,
				    BurgnerScsiDeviceSendCmdCallback cb,
				    gpointer user_data)
{
	g_return_if_fail (dev != NULL);
	g_return_if_fail (BURGNER_IS_SCSI_DEVICE (dev));
	g_return_if_fail (cdb != NULL);
	g_return_if_fail (cdb_size > 0);
	g_return_if_fail ((sense_size == 0) || (sense != NULL));
	g_return_if_fail ((data_size == 0) || (data != NULL));

	if (BURGNER_SCSI_DEVICE_GET_CLASS (dev)->send_cmd)
		BURGNER_SCSI_DEVICE_GET_CLASS (dev)->send_cmd (dev, cdb, cdb_size, sense, sense_size, data, data_size, direction, timeout, cb, user_data);
}

BurgnerFifo *
burgner_scsi_device_get_fifo (BurgnerScsiDevice *dev)
{
	g_return_val_if_fail (dev != NULL, NULL);
	g_return_val_if_fail (BURGNER_IS_SCSI_DEVICE (dev), NULL);

	if (BURGNER_SCSI_DEVICE_GET_CLASS (dev)->get_fifo)
		return BURGNER_SCSI_DEVICE_GET_CLASS (dev)->get_fifo (dev);

	return NULL;
}

