/*
 * BurgnerScsiDevice - BurgnerScsiDevice object
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifndef __BURGNER_SCSI_DEVICE_H__
#define __BURGNER_SCSI_DEVICE_H__

#include <glib-object.h>
#include <libburgnerutils/burgner-fifo.h>

G_BEGIN_DECLS


#define BURGNER_TYPE_SCSI_DEVICE		(burgner_scsi_device_get_type ())
#define BURGNER_SCSI_DEVICE(o)			(G_TYPE_CHECK_INSTANCE_CAST ((o), BURGNER_TYPE_SCSI_DEVICE, BurgnerScsiDevice))
#define BURGNER_SCSI_DEVICE_CLASS(k)		(G_TYPE_CHECK_CLASS_CAST ((k), BURGNER_TYPE_SCSI_DEVICE, BurgnerScsiDeviceClass))
#define BURGNER_IS_SCSI_DEVICE(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), BURGNER_TYPE_SCSI_DEVICE))
#define BURGNER_IS_SCSI_DEVICE_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), BURGNER_TYPE_SCSI_DEVICE))
#define BURGNER_SCSI_DEVICE_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), BURGNER_TYPE_SCSI_DEVICE, BurgnerScsiDeviceClass))


typedef enum {
	BURGNER_SCSI_DIRECTION_NONE = 0,
	BURGNER_SCSI_DIRECTION_TO_DEVICE,
	BURGNER_SCSI_DIRECTION_FROM_DEVICE
} BurgnerScsiDirection;


typedef struct _BurgnerScsiDevice BurgnerScsiDevice;
typedef struct _BurgnerScsiDeviceClass BurgnerScsiDeviceClass;

typedef void (* BurgnerScsiDeviceSendCmdCallback)
			(BurgnerScsiDevice *dev,
			 gpointer cdb, guint cdb_size,
			 gpointer sense, guint sense_size, guint sense_used,
			 gpointer data, guint data_size, guint data_used,
			 BurgnerScsiDirection direction,
		         guint timeout,
			 gint status,
			 GError *err,
			 gpointer user_data);


struct _BurgnerScsiDevice {
	GObject __parent;

};

struct _BurgnerScsiDeviceClass {
	GObjectClass __parent_class;

	/*
	 * Virtual Functions
	 */
	gboolean (* send_cmd_sync)(BurgnerScsiDevice *dev,
				   gpointer cdb, guint cdb_size,
				   gpointer sense, guint sense_size, guint *sense_used,
				   gpointer data, guint data_size, guint *data_used,
			 	   BurgnerScsiDirection direction,
				   guint timeout,
				   gint *status,
				   GError **err);

	void (* send_cmd)(BurgnerScsiDevice *dev,
				gpointer cdb, guint cdb_size,
				gpointer sense, guint sense_size,
				gpointer data, guint data_size,
			 	BurgnerScsiDirection direction,
				guint timeout,
				BurgnerScsiDeviceSendCmdCallback cb,
				gpointer user_data);

	BurgnerFifo *(* get_fifo)(BurgnerScsiDevice *dev);
};


GType burgner_scsi_device_get_type (void)
#if __GNUC__ <= 2
	G_GNUC_CONST
#endif
	;
GQuark  burgner_scsi_device_error_quark (void);

#define BURGNER_SCSI_DEVICE_ERROR		(burgner_scsi_device_error_quark ())
typedef enum {
	BURGNER_SCSI_DEVICE_ERROR_SENDING_CMD,
	BURGNER_SCSI_DEVICE_ERROR_CMD_TIMED_OUT,
	BURGNER_SCSI_DEVICE_ERROR_HOST_ERROR
} BurgnerScsiDeviceError;


gboolean burgner_scsi_device_send_cmd_sync (BurgnerScsiDevice *dev,
			   gpointer cdb, guint cdb_size,
			   gpointer sense, guint sense_size, guint *sense_used,
			   gpointer data, guint data_size, guint *data_used,
			   BurgnerScsiDirection direction,
			   guint timeout,
			   gint *status,
			   GError **err);

void burgner_scsi_device_send_cmd (BurgnerScsiDevice *dev,
					 gpointer cdb, guint cdb_size,
					 gpointer sense, guint sense_size,
					 gpointer data, guint data_size,
			 		 BurgnerScsiDirection direction,
					 guint timeout,
					 BurgnerScsiDeviceSendCmdCallback cb,
					 gpointer user_data);

BurgnerFifo *burgner_scsi_device_get_fifo (BurgnerScsiDevice *dev);

G_END_DECLS

#endif /* __BURGNER_SCSI_DEVICE_H__ */
