/*
 * 
 *
 * This file is part of the libburgnerscsi library.
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */

#ifndef __BURGNER_SCSI_DESCRIPTION_PARSER_H__
#define __BURGNER_SCSI_DESCRIPTION_PARSER_H__

#include <glib.h>
#include <libburgnerutils/burgner-module.h>
#include <libburgnerscsi/burgner-scsi-driver-info.h>

G_BEGIN_DECLS

#define BURGNER_SCSI_DRIVER_DESCRIPTION_PARSER_ERROR	(burgner_scsi_driver_description_parser_error_quark ())
GQuark burgner_scsi_driver_description_parser_error_quark (void);

typedef enum {
	BURGNER_SCSI_DRIVER_DESCRIPTION_PARSER_ERROR_ILLEGAL_ROOT_ELEMENT,
	BURGNER_SCSI_DRIVER_DESCRIPTION_PARSER_ERROR_MULTIPLE_INFO_ELEMENT,
	BURGNER_SCSI_DRIVER_DESCRIPTION_PARSER_ERROR_INCOMPLETE_INFO_ELEMENT,
	BURGNER_SCSI_DRIVER_DESCRIPTION_PARSER_ERROR_MULTIPLE_TYPES_ELEMENT,
	BURGNER_SCSI_DRIVER_DESCRIPTION_PARSER_ERROR_INCOMPLETE_TYPES_ELEMENT,
	BURGNER_SCSI_DRIVER_DESCRIPTION_PARSER_ERROR_INVALID_PARENT_TYPE,
	BURGNER_SCSI_DRIVER_DESCRIPTION_PARSER_ERROR_INCOMPLETE_TYPE_ELEMENT,
	BURGNER_SCSI_DRIVER_DESCRIPTION_PARSER_ERROR_INVALID_INTERFACE_TYPE,
	BURGNER_SCSI_DRIVER_DESCRIPTION_PARSER_ERROR_INVALID_INSTANCE_TYPE,
	BURGNER_SCSI_DRIVER_DESCRIPTION_PARSER_ERROR_INCOMPLETE_INTERFACE_ELEMENT,
	BURGNER_SCSI_DRIVER_DESCRIPTION_PARSER_ERROR_INCOMPLETE_DRIVER_DESCRIPTION
} BurgnerScsiDriverDescriptionError;

typedef struct _BurgnerScsiDriverDescriptionParser BurgnerScsiDriverDescriptionParser;

/**
 * burgner_scsi_driver_description_parser_new:
 * @filename: the name of the file to be parsed (optional, for error descriptions)
 *
 * Creates a new #BurgnerScsiDriverDescriptionParser that should
 * be freed with burgner_scsi_driver_description_parser_free() when
 * no longer needed.
 *
 * Returns: a new #BurgnerScsiDriverDescriptionParser
 */
BurgnerScsiDriverDescriptionParser *burgner_scsi_driver_description_parser_new (gchar *filename);

/**
 * burgner_scsi_driver_description_parser_free:
 * @dp: a #BurgnerScsiDriverDescriptionParser
 *
 * Frees @dp.
 */
void burgner_scsi_driver_description_parser_free (BurgnerScsiDriverDescriptionParser *dp);

/**
 * burgner_scsi_driver_description_parser_parse:
 * @dp: a #BurgnerScsiDriverDescriptionParser
 * @text: a text to parse
 * @size: size of @text
 * @error: return location for a #GError
 *
 * Parses @text. if an error occurred, returns %FALSE and set *@error; else
 * returns %TRUE. If an error occurred, then no more text can be fed in
 * @dp, and burgner_scsi_driver_description_parser_end mustn't be called.
 *
 * Returns: %TRUE if no error occurred
 */
gboolean burgner_scsi_driver_description_parser_parse (BurgnerScsiDriverDescriptionParser *dp, gchar *text, guint size, GError **error);

/**
  * burgner_scsi_driver_description_parser_end:
  * @dp: a #BurgnerScsiDriverDescriptionParser
  * @module: a location for a #BurgnerModule
  * @info: a location for a #BurgnerScsiDriverInfo
  * @error: return location for a #GError
  *
  * This function should be called when all the xml description has been fed
  * into @dp; put a #BurgnerModule in *@module and a #BurgnerScsiDriverInfo in
  * *@info, if no error occurred. If an error occurred, put an error in
  * *@error, and return %FALSE. This function must be called only once.
  * @module or @info (or both) may be NULL.
  *
  * Returns: %TRUE, if no error occurred (so *@module and *@info are set); else %FALSE, and *@error is set.
  */
gboolean burgner_scsi_driver_description_parser_end (BurgnerScsiDriverDescriptionParser *dp, BurgnerModule **module, BurgnerScsiDriverInfo **info, GError **error);

G_END_DECLS

#endif /* __BURGNER_SCSI_DRIVER_DESCRIPTION_PARSER_H__ */

