/*
 * 
 *
 * This file is part of the libburgnerscsi library.
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "burgner-scsi-driver-info.h"


BurgnerScsiDriverInfo *
burgner_scsi_driver_info_new (void)
{
	BurgnerScsiDriverInfo *info = g_new0 (BurgnerScsiDriverInfo, 1);
	info->refcount = 1;
	return info;
}

void
burgner_scsi_driver_info_free (BurgnerScsiDriverInfo *info)
{
	g_return_if_fail (info != NULL);

	if (info->id)
		g_free (info->id);
	if (info->name)
		g_free (info->name);
	if (info->version)
		g_free (info->version);
	if (info->authors)
		g_strfreev (info->authors);
	if (info->description)
		g_free (info->description);
	if (info->driver_type)
		g_free (info->driver_type);

	g_free (info);
}

BurgnerScsiDriverInfo *
burgner_scsi_driver_info_ref (BurgnerScsiDriverInfo *info)
{
	g_return_val_if_fail (info != NULL, NULL);

	info->refcount++;

	return info;
}

void
burgner_scsi_driver_info_unref (BurgnerScsiDriverInfo *info)
{
	g_return_if_fail (info != NULL);

	if ((info->refcount -= 1) == 0)
		burgner_scsi_driver_info_free (info);
}



void
burgner_scsi_driver_info_set_id (BurgnerScsiDriverInfo *info,
				 gchar *id)
{
	g_return_if_fail (info != NULL);

	if (info->id)
		g_free (info->id);
	info->id = g_strdup (id);
}

gchar *
burgner_scsi_driver_info_get_id (BurgnerScsiDriverInfo *info)
{
	g_return_val_if_fail (info != NULL, NULL);
	return info->id;
}


void
burgner_scsi_driver_info_set_name (BurgnerScsiDriverInfo *info,
				   gchar *name)
{
	g_return_if_fail (info != NULL);

	if (info->name)	
		g_free (info->name);
	info->name = g_strdup (name);
}

gchar *
burgner_scsi_driver_info_get_name (BurgnerScsiDriverInfo *info)
{
	g_return_val_if_fail (info != NULL, NULL);
	return info->name;
}


void
burgner_scsi_driver_info_set_version (BurgnerScsiDriverInfo *info,
				      gchar *version)
{
	g_return_if_fail (info != NULL);

	if (info->version)
		g_free (info->version);
	info->version = g_strdup (version);
}

gchar *
burgner_scsi_driver_info_get_version (BurgnerScsiDriverInfo *info)
{
	g_return_val_if_fail (info != NULL, NULL);
	return info->version;
}


void
burgner_scsi_driver_info_set_description (BurgnerScsiDriverInfo *info,
				          gchar *description)
{
	g_return_if_fail (info != NULL);

	if (info->description)
		g_free (info->description);
	info->description = g_strdup (description);
}

gchar *
burgner_scsi_driver_info_get_description (BurgnerScsiDriverInfo *info)
{
	g_return_val_if_fail (info != NULL, NULL);
	return info->description;
}


void
burgner_scsi_driver_info_set_driver_type (BurgnerScsiDriverInfo *info,
				          gchar *driver_type)
{
	g_return_if_fail (info != NULL);

	if (info->driver_type)
		g_free (info->driver_type);
	info->driver_type = g_strdup (driver_type);
}

gchar *
burgner_scsi_driver_info_get_driver_type (BurgnerScsiDriverInfo *info)
{
	g_return_val_if_fail (info != NULL, NULL);
	return info->driver_type;
}


void
burgner_scsi_driver_info_set_authors (BurgnerScsiDriverInfo *info,
				      gchar **authors)
{
	g_return_if_fail (info != NULL);

	if (info->authors)
		g_strfreev (info->authors);
	info->authors = g_strdupv (authors);
}

void
burgner_scsi_driver_info_set_authors_split (BurgnerScsiDriverInfo *info,
					    gchar *authors,
					    gchar *delimiter)
{
	g_return_if_fail (info != NULL);
	g_return_if_fail (delimiter != NULL);

	if (info->authors)
		g_strfreev (info->authors);
	info->authors = g_strsplit (authors, delimiter, -1);
}

gchar **
burgner_scsi_driver_info_get_authors (BurgnerScsiDriverInfo *info)
{
	g_return_val_if_fail (info != NULL, NULL);
	return info->authors;
}


