/*
 * 
 *
 * This file is part of the libburgnerscsi library.
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */

#ifndef __BURGNER_SCSI_DRIVER_INFO_H__
#define __BURGNER_SCSI_DRIVER_INFO_H__

#include <glib.h>

G_BEGIN_DECLS

typedef struct {
	gchar *id;
	gchar *name;
	gchar *version;
	gchar **authors;
	gchar *description;

	gboolean need_privilege;
	gchar *driver_type;

	guint refcount;
} BurgnerScsiDriverInfo;

BurgnerScsiDriverInfo *burgner_scsi_driver_info_new (void);
void burgner_scsi_driver_info_free (BurgnerScsiDriverInfo *info);

BurgnerScsiDriverInfo *burgner_scsi_driver_info_ref (BurgnerScsiDriverInfo *info);
void burgner_scsi_driver_info_unref (BurgnerScsiDriverInfo *info);


void burgner_scsi_driver_info_set_id (BurgnerScsiDriverInfo *info,
				      gchar *id);
gchar *burgner_scsi_driver_info_get_id (BurgnerScsiDriverInfo *info);

void burgner_scsi_driver_info_set_name (BurgnerScsiDriverInfo *info,
				      gchar *name);
gchar *burgner_scsi_driver_info_get_name (BurgnerScsiDriverInfo *info);

void burgner_scsi_driver_info_set_version (BurgnerScsiDriverInfo *info,
				      gchar *version);
gchar *burgner_scsi_driver_info_get_version (BurgnerScsiDriverInfo *info);

void burgner_scsi_driver_info_set_description (BurgnerScsiDriverInfo *info,
				      gchar *description);
gchar *burgner_scsi_driver_info_get_description (BurgnerScsiDriverInfo *info);

void burgner_scsi_driver_info_set_driver_type (BurgnerScsiDriverInfo *info,
				      gchar *driver_type);
gchar *burgner_scsi_driver_info_get_driver_type (BurgnerScsiDriverInfo *info);

void burgner_scsi_driver_info_set_authors (BurgnerScsiDriverInfo *info,
					   gchar **authors);
void burgner_scsi_driver_info_set_authors_split (BurgnerScsiDriverInfo *info,
						 gchar *authors,
						 gchar *delimiter);
gchar **burgner_scsi_driver_info_get_authors (BurgnerScsiDriverInfo *info);

G_END_DECLS

#endif /* __BURGNER_SCSI_DRIVER_INFO_H__ */

