/*
 * BurgnerScsiDriverManager - BurgnerScsiDriverManager object
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifndef __BURGNER_SCSI_DRIVER_MANAGER_H__
#define __BURGNER_SCSI_DRIVER_MANAGER_H__

#include <glib-object.h>
#include <libburgnerutils/g-uri.h>
#include <libburgnerscsi/burgner-scsi-driver.h>
#include <libburgnerscsi/burgner-scsi-privilege.h>
#include <libburgnerscsi/burgner-scsi-driver-info.h>

G_BEGIN_DECLS


#define BURGNER_TYPE_SCSI_DRIVER_MANAGER		(burgner_scsi_driver_manager_get_type ())
#define BURGNER_SCSI_DRIVER_MANAGER(o)			(G_TYPE_CHECK_INSTANCE_CAST ((o), BURGNER_TYPE_SCSI_DRIVER_MANAGER, BurgnerScsiDriverManager))
#define BURGNER_SCSI_DRIVER_MANAGER_CLASS(k)		(G_TYPE_CHECK_CLASS_CAST ((k), BURGNER_TYPE_SCSI_DRIVER_MANAGER, BurgnerScsiDriverManagerClass))
#define BURGNER_IS_SCSI_DRIVER_MANAGER(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), BURGNER_TYPE_SCSI_DRIVER_MANAGER))
#define BURGNER_IS_SCSI_DRIVER_MANAGER_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), BURGNER_TYPE_SCSI_DRIVER_MANAGER))
#define BURGNER_SCSI_DRIVER_MANAGER_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), BURGNER_TYPE_SCSI_DRIVER_MANAGER, BurgnerScsiDriverManagerClass))



typedef struct _BurgnerScsiDriverManager BurgnerScsiDriverManager;
typedef struct _BurgnerScsiDriverManagerClass BurgnerScsiDriverManagerClass;

typedef gpointer BurgnerScsiDriverManagerLoadDriverInfoListAsyncId;

struct _BurgnerScsiDriverManager {
	GObject __parent;

	BurgnerScsiPrivilege *privilege;

	GList *info_list;
	GHashTable *module_id_to_info;

	GList *async_data_list;
};

struct _BurgnerScsiDriverManagerClass {
	GObjectClass __parent_class;

	/*
	 * Signals
	 */
	void (* load_driver_info_list_async_done)(BurgnerScsiDriverManager *dm,
						  BurgnerScsiDriverManagerLoadDriverInfoListAsyncId async_id,
						  gchar *directory,
						  GList *driver_info_list,
						  GError *err);
};


GType burgner_scsi_driver_manager_get_type (void)
#if __GNUC__ <= 2
	G_GNUC_CONST
#endif
	;
GQuark  burgner_scsi_driver_manager_error_quark (void);

#define BURGNER_SCSI_DRIVER_MANAGER_ERROR		(burgner_scsi_driver_manager_error_quark ())
typedef enum {
	BURGNER_SCSI_DRIVER_MANAGER_ERROR_LOADING_DIRECTORY,
	BURGNER_SCSI_DRIVER_MANAGER_ERROR_OPENING_FILE,
	BURGNER_SCSI_DRIVER_MANAGER_ERROR_READING_FILE
} BurgnerScsiDriverManagerError;


BurgnerScsiDriverManager *burgner_scsi_driver_manager_new (void);

gboolean burgner_scsi_driver_manager_load_driver_info_list_sync (BurgnerScsiDriverManager *dm,
					gchar *directory,
				        GList **driver_info_list,
				        GError **err);
BurgnerScsiDriverManagerLoadDriverInfoListAsyncId burgner_scsi_driver_manager_load_driver_info_list_async (BurgnerScsiDriverManager *dm,
					gchar *directory);

GList *burgner_scsi_driver_manager_get_driver_info_list (BurgnerScsiDriverManager *dm);

BurgnerScsiDriver *burgner_scsi_driver_manager_get_driver (BurgnerScsiDriverManager *dm,
				  gchar *driver_id);
GList *burgner_scsi_driver_manager_get_all_drivers (BurgnerScsiDriverManager *dm);
BurgnerScsiDriver *burgner_scsi_driver_manager_get_driver_from_uri (BurgnerScsiDriverManager *dm,
					   GUri *uri);

/* Routines for emitting signals on the BurgnerScsiDriverManager */
void burgner_scsi_driver_manager_load_driver_info_list_async_done (BurgnerScsiDriverManager *dm,
						  BurgnerScsiDriverManagerLoadDriverInfoListAsyncId async_id,
						  gchar *directory,
						  GList *driver_info_list,
						  GError *err);

G_END_DECLS

#endif /* __BURGNER_SCSI_DRIVER_MANAGER_H__ */
