/*
 * BurgnerScsiDriver - BurgnerScsiDriver object
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib-object.h>
#include "burgner-scsi-driver.h"


static GObjectClass *parent_class = NULL;

static void
burgner_scsi_driver_finalize (GObject *object)
{
	if (G_OBJECT_CLASS (parent_class)->finalize)
		G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
burgner_scsi_driver_class_init (BurgnerScsiDriverClass *klass)
{
	GObjectClass *object_class;

	object_class = (GObjectClass *) klass;
	object_class->finalize = burgner_scsi_driver_finalize;

	parent_class = g_type_class_peek_parent (klass);
}

static void
burgner_scsi_driver_init (BurgnerScsiDriver *self)
{
}

GType
burgner_scsi_driver_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		static GTypeInfo info = {
			sizeof (BurgnerScsiDriverClass),
			NULL,
			NULL,
			(GClassInitFunc) burgner_scsi_driver_class_init,
			NULL,
			NULL,
			sizeof (BurgnerScsiDriver),
			0,
			(GInstanceInitFunc) burgner_scsi_driver_init,
		};

		type = g_type_register_static (G_TYPE_OBJECT, "BurgnerScsiDriver", &info, G_TYPE_FLAG_ABSTRACT);
	}

	return type;
}

GQuark
burgner_scsi_driver_error_quark (void)
{
	return g_quark_from_string ("BurgnerScsiDriver");
}

BurgnerScsiDriverOpId
burgner_scsi_driver_open_device
			(BurgnerScsiDriver *sd,
			 const GUri *uri,
			 gint requested_dma_size,
			 BurgnerScsiDriverOpenDeviceCallback cb,
			 gpointer user_data)
{
	g_return_val_if_fail (sd != NULL, NULL);
	g_return_val_if_fail (BURGNER_IS_SCSI_DRIVER (sd), NULL);
	g_return_val_if_fail (uri != NULL, NULL);

	if (BURGNER_SCSI_DRIVER_GET_CLASS (sd)->open_device)
		return BURGNER_SCSI_DRIVER_GET_CLASS (sd)->open_device (sd, uri, requested_dma_size, cb, user_data);

	return NULL;
}

BurgnerScsiDriverOpId
burgner_scsi_driver_close_device
			(BurgnerScsiDriver *sd,
			 BurgnerScsiDevice *dev,
			 BurgnerScsiDriverCloseDeviceCallback cb,
			 gpointer user_data)
{
	g_return_val_if_fail (sd != NULL, NULL);
	g_return_val_if_fail (BURGNER_IS_SCSI_DRIVER (sd), NULL);
	g_return_val_if_fail (dev != NULL, NULL);
	g_return_val_if_fail (BURGNER_IS_SCSI_DEVICE (dev), NULL);

	if (BURGNER_SCSI_DRIVER_GET_CLASS (sd)->close_device)
		return BURGNER_SCSI_DRIVER_GET_CLASS (sd)->close_device (sd, dev, cb, user_data);

	return NULL;
}

BurgnerScsiDevice *
burgner_scsi_driver_open_device_sync (BurgnerScsiDriver *sd,
					     const GUri *uri,
			 		gint requested_dma_size,
			 		gint *real_dma_size,
					     GError **err)
{
	g_return_val_if_fail (sd != NULL, NULL);
	g_return_val_if_fail (BURGNER_IS_SCSI_DRIVER (sd), NULL);
	g_return_val_if_fail (uri != NULL, NULL);

	if (BURGNER_SCSI_DRIVER_GET_CLASS (sd)->open_device_sync)
		return BURGNER_SCSI_DRIVER_GET_CLASS (sd)->open_device_sync (sd, uri, requested_dma_size, real_dma_size, err);

	return NULL;
}

gboolean
burgner_scsi_driver_close_device_sync (BurgnerScsiDriver *sd,
					      BurgnerScsiDevice *dev,
					      GError **err)
{
	g_return_val_if_fail (sd != NULL, FALSE);
	g_return_val_if_fail (BURGNER_IS_SCSI_DRIVER (sd), FALSE);
	g_return_val_if_fail (dev != NULL, FALSE);
	g_return_val_if_fail (BURGNER_IS_SCSI_DEVICE (dev), FALSE);

	if (BURGNER_SCSI_DRIVER_GET_CLASS (sd)->open_device_sync)
		return BURGNER_SCSI_DRIVER_GET_CLASS (sd)->close_device_sync (sd, dev, err);

	return FALSE;
}

GList *
burgner_scsi_driver_get_uri_list_sync (BurgnerScsiDriver *sd,
				       GError **error)
{
	g_return_val_if_fail (sd != NULL, NULL);
	g_return_val_if_fail (BURGNER_IS_SCSI_DRIVER (sd), NULL);

	if (BURGNER_SCSI_DRIVER_GET_CLASS (sd)->get_uri_list)
		return BURGNER_SCSI_DRIVER_GET_CLASS (sd)->get_uri_list_sync (sd, error);

	return NULL;
}

BurgnerScsiDriverOpId
burgner_scsi_driver_get_uri_list (BurgnerScsiDriver *driver,
					BurgnerScsiDriverGetUriListCallback cb,
					gpointer user_data)
{
	g_return_val_if_fail (driver != NULL, NULL);
	g_return_val_if_fail (BURGNER_IS_SCSI_DRIVER (driver), NULL);

	if (BURGNER_SCSI_DRIVER_GET_CLASS (driver)->get_uri_list)
		return BURGNER_SCSI_DRIVER_GET_CLASS (driver)->get_uri_list (driver, cb, user_data);

	return NULL;
}


gboolean
burgner_scsi_driver_need_privilege (BurgnerScsiDriver *sd)
{
	g_return_val_if_fail (sd != NULL, FALSE);
	g_return_val_if_fail (BURGNER_IS_SCSI_DRIVER (sd), FALSE);

	if (BURGNER_SCSI_DRIVER_GET_CLASS (sd)->need_privilege)
		return BURGNER_SCSI_DRIVER_GET_CLASS (sd)->need_privilege (sd);

	return FALSE;
}

void
burgner_scsi_driver_set_privilege (BurgnerScsiDriver *sd,
				   BurgnerScsiPrivilege *privilege)
{
	g_return_if_fail (sd != NULL);
	g_return_if_fail (BURGNER_IS_SCSI_DRIVER (sd));
	g_return_if_fail (privilege != NULL);
	g_return_if_fail (BURGNER_IS_SCSI_PRIVILEGE (privilege));

	if (BURGNER_SCSI_DRIVER_GET_CLASS (sd)->set_privilege)
		BURGNER_SCSI_DRIVER_GET_CLASS (sd)->set_privilege (sd, privilege);
}


