/*
 * BurgnerScsiDriver - BurgnerScsiDriver object
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifndef __BURGNER_SCSI_DRIVER_H__
#define __BURGNER_SCSI_DRIVER_H__

#include <glib-object.h>
#include <libburgnerutils/g-uri.h>
#include <libburgnerscsi/burgner-scsi-privilege.h>
#include <libburgnerscsi/burgner-scsi-device.h>

G_BEGIN_DECLS


#define BURGNER_TYPE_SCSI_DRIVER		(burgner_scsi_driver_get_type ())
#define BURGNER_SCSI_DRIVER(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), BURGNER_TYPE_SCSI_DRIVER, BurgnerScsiDriver))
#define BURGNER_SCSI_DRIVER_CLASS(k)	(G_TYPE_CHECK_CLASS_CAST ((k), BURGNER_TYPE_SCSI_DRIVER, BurgnerScsiDriverClass))
#define BURGNER_IS_SCSI_DRIVER(o)	(G_TYPE_CHECK_INSTANCE_TYPE ((o), BURGNER_TYPE_SCSI_DRIVER))
#define BURGNER_IS_SCSI_DRIVER_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), BURGNER_TYPE_SCSI_DRIVER))
#define BURGNER_SCSI_DRIVER_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), BURGNER_TYPE_SCSI_DRIVER, BurgnerScsiDriverClass))


typedef struct _BurgnerScsiDriver BurgnerScsiDriver;
typedef struct _BurgnerScsiDriverClass BurgnerScsiDriverClass;

typedef gpointer BurgnerScsiDriverOpId;

typedef void (* BurgnerScsiDriverOpenDeviceCallback)
			(BurgnerScsiDriver *sd,
			 BurgnerScsiDriverOpId id,
			 GUri *uri,
			 BurgnerScsiDevice *dev,
			 gint requested_dma_size,
			 gint real_dma_size,
			 GError *err,
			 gpointer user_data);
typedef void (* BurgnerScsiDriverCloseDeviceCallback)
			(BurgnerScsiDriver *sd,
			 BurgnerScsiDriverOpId id,
			 BurgnerScsiDevice *dev,
			 GError *err,
			 gpointer user_data);

typedef void (* BurgnerScsiDriverGetUriListCallback)
			(BurgnerScsiDriver *driver,
			 GList *uri_list,
			 GError *error,
			 gpointer user_data);


struct _BurgnerScsiDriver {
	GObject __parent;

};

struct _BurgnerScsiDriverClass {
	GObjectClass __parent_class;

	/*
	 * Virtual Functions
	 */
	BurgnerScsiDevice *(* open_device_sync)(BurgnerScsiDriver *sd,
						const GUri *uri,
						gint requested_dma_size,
						gint *real_dma_size,
						GError **err);
	gboolean (* close_device_sync)(BurgnerScsiDriver *sd,
				       BurgnerScsiDevice *dev,
				       GError **err);

	BurgnerScsiDriverOpId (* open_device)
			(BurgnerScsiDriver *sd,
			 const GUri *uri,
			 gint requested_dma_size,
			 BurgnerScsiDriverOpenDeviceCallback cb,
			 gpointer cb_data);
	BurgnerScsiDriverOpId (* close_device)
			(BurgnerScsiDriver *sd,
			 BurgnerScsiDevice *dev,
			 BurgnerScsiDriverCloseDeviceCallback cb,
			 gpointer cb_data);

	GList *(* get_uri_list_sync)(BurgnerScsiDriver *sd,
				     GError **error);
	BurgnerScsiDriverOpId (* get_uri_list)
			(BurgnerScsiDriver *driver,
			 BurgnerScsiDriverGetUriListCallback cb,
			 gpointer cb_data);

	gboolean (* need_privilege)(BurgnerScsiDriver *sd);
	void (* set_privilege)(BurgnerScsiDriver *sd,
			       BurgnerScsiPrivilege *privilege);
};


GType burgner_scsi_driver_get_type (void)
#if __GNUC__ <= 2
	G_GNUC_CONST
#endif
	;
GQuark  burgner_scsi_driver_error_quark (void);

#define BURGNER_SCSI_DRIVER_ERROR		(burgner_scsi_driver_error_quark ())
typedef enum {
	BURGNER_SCSI_DRIVER_ERROR_BAD_FILE,
	BURGNER_SCSI_DRIVER_ERROR_OPENING_DEVICE,
	BURGNER_SCSI_DRIVER_ERROR_CLOSING_DEVICE,
	BURGNER_SCSI_DRIVER_ERROR_SETTING_DMA_SIZE,
	BURGNER_SCSI_DRIVER_ERROR_GETTING_DMA_SIZE,
	BURGNER_SCSI_DRIVER_ERROR_UNSUPPORTED_URI_SCHEME
} BurgnerScsiDriverError;


BurgnerScsiDevice *burgner_scsi_driver_open_device_sync
			(BurgnerScsiDriver *sd,
			 const GUri *uri,
			 gint requested_dma_size,
			 gint *real_dma_size,
			 GError **err);
gboolean burgner_scsi_driver_close_device_sync
			(BurgnerScsiDriver *sd,
			 BurgnerScsiDevice *dev,
			 GError **err);

BurgnerScsiDriverOpId burgner_scsi_driver_open_device
			(BurgnerScsiDriver *sd,
			 const GUri *uri,
			 gint requested_dma_size,
			 BurgnerScsiDriverOpenDeviceCallback cb,
			 gpointer cb_data);
BurgnerScsiDriverOpId burgner_scsi_driver_close_device
			(BurgnerScsiDriver *sd,
			 BurgnerScsiDevice *dev,
			 BurgnerScsiDriverCloseDeviceCallback cb,
			 gpointer cb_data);

GList *burgner_scsi_driver_get_uri_list_sync (BurgnerScsiDriver *sd,
					      GError **error);
BurgnerScsiDriverOpId burgner_scsi_driver_get_uri_list
			(BurgnerScsiDriver *driver,
			 BurgnerScsiDriverGetUriListCallback cb,
			 gpointer cb_data);

gboolean burgner_scsi_driver_need_privilege (BurgnerScsiDriver *sd);
void burgner_scsi_driver_set_privilege (BurgnerScsiDriver *sd,
					BurgnerScsiPrivilege *privilege);

G_END_DECLS

#endif /* __BURGNER_SCSI_DRIVER_H__ */
