/*
 * BurgnerScsiGetModePageStage - BurgnerScsiGetModePageStage object
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "burgner-scsi-i18n.h"

#include <glib-object.h>
#include "../libburgnerscsi/burgner-scsi-mode-sense.h"
#include "burgner-scsi-get-mode-page-stage.h"


static BurgnerScsiSendCommandStageClass *parent_class = NULL;

enum {
	PROP_0,
	PROP_DATA_BUF,
	PROP_DATA_BUF_SIZE,
	PROP_VALUES,
	PROP_PAGE_CODE,
	PROP_ENABLE_BLOCK_DESCRIPTORS,
	BURGNER_SCSI_GET_MODE_PAGE_STAGE_LAST_ARG
};

static void
burgner_scsi_get_mode_page_stage_finalize (GObject *object)
{
	BurgnerScsiGetModePageStage *self = BURGNER_SCSI_GET_MODE_PAGE_STAGE (object);

	self->data_buf = NULL;
	self->data_buf_size = 0;
	self->values = BURGNER_SCSI_MODE_PAGE_VALUES_CURRENT;
	self->page_code = BURGNER_SCSI_GET_MODE_PAGE_STAGE_PAGE_CODE_ALL;
	self->enable_block_descriptors = 0;

	if (G_OBJECT_CLASS (parent_class)->finalize)
		G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
burgner_scsi_get_mode_page_stage_set_property (GObject *object,
	      guint prop_id,
	      const GValue *value,
	      GParamSpec *pspec)
{
	BurgnerScsiGetModePageStage *self = BURGNER_SCSI_GET_MODE_PAGE_STAGE (object);

	switch (prop_id) {
		case PROP_DATA_BUF:
			self->data_buf = g_value_get_pointer (value);
			g_object_notify (object, "data_buf");
			break;
		case PROP_DATA_BUF_SIZE:
			self->data_buf_size = g_value_get_uint (value);
			g_object_notify (object, "data_buf_size");
			break;
		case PROP_VALUES:
			self->values = g_value_get_enum (value);
			g_object_notify (object, "values");
			break;
		case PROP_PAGE_CODE:
			self->page_code = g_value_get_uint (value);
			g_object_notify (object, "page_code");
			break;
		case PROP_ENABLE_BLOCK_DESCRIPTORS:
			self->enable_block_descriptors = g_value_get_boolean (value) ? 1 : 0;
			g_object_notify (object, "enable_block_descriptors");
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
burgner_scsi_get_mode_page_stage_get_property (GObject *object,
	      guint prop_id,
	      GValue *value,
	      GParamSpec *pspec)
{
	BurgnerScsiGetModePageStage *self = BURGNER_SCSI_GET_MODE_PAGE_STAGE (object);

	switch (prop_id) {
		case PROP_DATA_BUF:
			g_value_set_pointer (value, self->data_buf);
			break;
		case PROP_DATA_BUF_SIZE:
			g_value_set_uint (value, self->data_buf_size);
			break;
		case PROP_VALUES:
			g_value_set_enum (value, self->values);
			break;
		case PROP_PAGE_CODE:
			g_value_set_uint (value, self->page_code);
			break;
		case PROP_ENABLE_BLOCK_DESCRIPTORS:
			g_value_set_boolean (value, self->enable_block_descriptors);
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
_burgner_scsi_get_mode_page_stage_get_command_parameters
				(BurgnerScsiSendCommandStage *sc_stage,
				 guchar *cdb_buf, guint cdb_buf_size,
				 BurgnerScsiDirection *direction,
				 guchar **data, guint *data_size,
				 guint *timeout)
{
	BurgnerScsiGetModePageStage *self = BURGNER_SCSI_GET_MODE_PAGE_STAGE (sc_stage);
#if 0
	gboolean do_10 = (self->data_buf_size & ~0xff) != 0;
#else
	gboolean do_10 = TRUE;
#endif

	g_return_if_fail (cdb_buf_size >= (do_10 ? BURGNER_SCSI_MODE_SENSE_10_CDB_SIZE : BURGNER_SCSI_MODE_SENSE_6_CDB_SIZE));

	if (do_10) {
		burgner_scsi_mode_sense_10_write_cdb (cdb_buf, cdb_buf_size,
						      self->enable_block_descriptors,
						      self->values,
						      self->page_code,
						      self->data_buf_size);
	} else {
		burgner_scsi_mode_sense_6_write_cdb (cdb_buf, cdb_buf_size,
						     self->enable_block_descriptors,
						     self->values,
						     self->page_code,
						     self->data_buf_size);
	}

	*direction = BURGNER_SCSI_DIRECTION_FROM_DEVICE;
	*data = self->data_buf;
	*data_size = self->data_buf_size;
	*timeout = 10000;
}

static void
burgner_scsi_get_mode_page_stage_class_init (BurgnerScsiGetModePageStageClass *klass)
{
	GObjectClass *object_class;
	BurgnerScsiSendCommandStageClass *sc_stage_class;

	object_class = (GObjectClass *) klass;
	object_class->set_property = burgner_scsi_get_mode_page_stage_set_property;
	object_class->get_property = burgner_scsi_get_mode_page_stage_get_property;
	object_class->finalize = burgner_scsi_get_mode_page_stage_finalize;

	sc_stage_class = (BurgnerScsiSendCommandStageClass *) klass;
	sc_stage_class->get_command_parameters = _burgner_scsi_get_mode_page_stage_get_command_parameters;

	/* Properties */
	g_object_class_install_property (object_class,
					 PROP_DATA_BUF,
					 g_param_spec_pointer ("data_buf",
					                _("data_buf"),
							_("A buffer which will contain the mode parameter"),
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_DATA_BUF_SIZE,
					 g_param_spec_uint ("data_buf_size",
					                _("data_buf_size"),
							_("Size of data_buf (should be less than 64 kB)"),
							0,
							0xffff,
							0,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_VALUES,
					 g_param_spec_enum ("values",
					                _("values"),
							_("BurgnerScsiModePageValues"),
							BURGNER_TYPE_SCSI_MODE_PAGE_VALUES,
							BURGNER_SCSI_MODE_PAGE_VALUES_CURRENT,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_PAGE_CODE,
					 g_param_spec_uint ("page_code",
					                _("page_code"),
							_("Page Code"),
							0,
							0x3f,
							0x3f,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_ENABLE_BLOCK_DESCRIPTORS,
					 g_param_spec_boolean ("enable_block_descriptors",
					                _("enable_block_descriptors"),
							_("Enable Block Descriptors"),
							FALSE,
							G_PARAM_READABLE | G_PARAM_WRITABLE));


	parent_class = g_type_class_peek_parent (klass);
}

static void
burgner_scsi_get_mode_page_stage_init (BurgnerScsiGetModePageStage *self)
{
	
}

GType
burgner_scsi_get_mode_page_stage_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		static GTypeInfo info = {
			sizeof (BurgnerScsiGetModePageStageClass),
			NULL,
			NULL,
			(GClassInitFunc) burgner_scsi_get_mode_page_stage_class_init,
			NULL,
			NULL,
			sizeof (BurgnerScsiGetModePageStage),
			0,
			(GInstanceInitFunc) burgner_scsi_get_mode_page_stage_init,
		};

		type = g_type_register_static (BURGNER_TYPE_SCSI_SEND_COMMAND_STAGE, "BurgnerScsiGetModePageStage", &info, 0);
	}

	return type;
}

BurgnerScsiStage *
burgner_scsi_get_mode_page_stage_new (void)
{
	BurgnerScsiGetModePageStage *self;

	self = g_object_new (BURGNER_TYPE_SCSI_GET_MODE_PAGE_STAGE, NULL);


	return BURGNER_SCSI_STAGE (self);
}


GQuark
burgner_scsi_get_mode_page_stage_error_quark (void)
{
	return g_quark_from_string ("BurgnerScsiGetModePageStage");
}

