/*
 * 
 *
 * This file is part of the libburgnerscsi library.
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */

#ifndef __BURGNER_SCSI_INQUIRY_IMPL_H__
#define __BURGNER_SCSI_INQUIRY_IMPL_H__

#include <glib.h>
#include <libburgnerscsi/burgner-scsi-bit-field-order.h>

G_BEGIN_DECLS

typedef struct {
	guchar op_code; /* 0x12 */

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar res_01 : 6;
	guchar cmd_dt : 1;
	guchar evpd : 1;
#else
	guchar evpd : 1;
	guchar cmd_dt : 1;
	guchar res_01 : 6;
#endif

	guchar page_code;

	guchar res_02;

	guchar alloc_length;

	guchar control;
} __attribute__((packed)) _BurgnerScsiInquiryImplCdb;

typedef struct {
#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar peripheral_qualifier : 3;
	guchar peripheral_device_type : 5;
#else
	guchar peripheral_device_type : 5;
	guchar peripheral_qualifier : 3;
#endif

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar rmb : 1;
	guchar res_01 : 7;
#else
	guchar res_01 : 7;
	guchar rmb : 1;
#endif

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar iso_iec_version : 2;
	guchar ecma_version : 3;
	guchar ansi_version : 3;
#else
	guchar ansi_version : 3;
	guchar ecma_version : 3;
	guchar iso_iec_version : 2;
#endif

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar aerc : 1;
	guchar trm_tsk : 1;
	guchar norm_aca : 1;
	guchar res_02 : 1;
	guchar response_data_format : 4;
#else
	guchar response_data_format : 4;
	guchar res_02 : 1;
	guchar norm_aca : 1;
	guchar trm_tsk : 1;
	guchar aerc : 1;
#endif

	guchar additional_length;
	guchar res_03;

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar res_04 : 1;
	guchar enc_serv : 1;
	guchar vs_01 : 1;
	guchar multi_p : 1;
	guchar m_chngr : 1;
	guchar ackreqq : 1;
	guchar addr32 : 1;
	guchar addr16 : 1;
#else
	guchar addr16 : 1;
	guchar addr32 : 1;
	guchar ackreqq : 1;
	guchar m_chngr : 1;
	guchar multi_p : 1;
	guchar vs_01 : 1;
	guchar enc_serv : 1;
	guchar res_04 : 1;
#endif

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar rel_adr : 1;
	guchar wbus32 : 1;
	guchar wbus16 : 1;
	guchar sync : 1;
	guchar linked : 1;
	guchar tran_dis : 1;
	guchar cmd_que : 1;
	guchar vs_02 : 1;
#else
	guchar vs_02 : 1;
	guchar cmd_que : 1;
	guchar tran_dis : 1;
	guchar linked : 1;
	guchar sync : 1;
	guchar wbus16 : 1;
	guchar wbus32 : 1;
	guchar rel_adr : 1;
#endif

	guchar vendor_id[8];
	guchar product_id[16];

	guchar product_revision_level[4];
} __attribute__((packed)) _BurgnerScsiInquiryImplData;


G_END_DECLS

#endif /* __BURGNER_SCSI_INQUIRY_IMPL_H__ */

