/*
 * 
 *
 * This file is part of the libburgnerscsi library.
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include "burgner-scsi-inquiry-impl.h"
#include "burgner-scsi-inquiry.h"


void
burgner_scsi_inquiry_simple_write_cdb (gpointer cdb_buf,
				       guint cdb_buf_size,
				       guint data_buf_size)
{
	_BurgnerScsiInquiryImplCdb *impl;

	g_return_if_fail (cdb_buf != NULL);
	g_return_if_fail (cdb_buf_size >= BURGNER_SCSI_INQUIRY_CDB_SIZE);

	impl = (_BurgnerScsiInquiryImplCdb *) cdb_buf;
	memset (impl, 0, BURGNER_SCSI_INQUIRY_CDB_SIZE);

	impl->op_code = BURGNER_SCSI_INQUIRY_OP_CODE;
	impl->alloc_length = data_buf_size & ~0xff ? 0xff : data_buf_size;
}

void
burgner_scsi_inquiry_parse_data (gpointer data_buf,
				 guint data_buf_size,
				 BurgnerScsiDeviceType *device_type,
				 gchar **vendor_id,
				 gchar **product_id)
{
	_BurgnerScsiInquiryImplData *data;

	g_return_if_fail (data_buf != NULL);

	data = (_BurgnerScsiInquiryImplData *) data_buf;

	if (data_buf_size >= 1) {
		if (device_type)
			*device_type = *((guchar *) data) & 0x1f;

		if (data_buf_size >= 16) {
			if (vendor_id)
				*vendor_id = g_strndup (data->vendor_id, 8);

			if (data_buf_size >= 32) {
				if (product_id)
					*product_id = g_strndup (data->product_id, 16);
			}
		}
	}
}

