/*
 * BurgnerScsiLinuxDevice - BurgnerScsiLinuxDevice object
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifndef __BURGNER_SCSI_LINUX_DEVICE_H__
#define __BURGNER_SCSI_LINUX_DEVICE_H__

#include <glib-object.h>
#include <libburgnerutils/burgner-thread.h>
#include <libburgnerscsi/burgner-scsi-device.h>

G_BEGIN_DECLS


#define BURGNER_TYPE_SCSI_LINUX_DEVICE		(burgner_scsi_linux_device_get_type ())
#define BURGNER_SCSI_LINUX_DEVICE(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), BURGNER_TYPE_SCSI_LINUX_DEVICE, BurgnerScsiLinuxDevice))
#define BURGNER_SCSI_LINUX_DEVICE_CLASS(k)	(G_TYPE_CHECK_CLASS_CAST ((k), BURGNER_TYPE_SCSI_LINUX_DEVICE, BurgnerScsiLinuxDeviceClass))
#define BURGNER_IS_SCSI_LINUX_DEVICE(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), BURGNER_TYPE_SCSI_LINUX_DEVICE))
#define BURGNER_IS_SCSI_LINUX_DEVICE_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), BURGNER_TYPE_SCSI_LINUX_DEVICE))
#define BURGNER_SCSI_LINUX_DEVICE_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), BURGNER_TYPE_SCSI_LINUX_DEVICE, BurgnerScsiLinuxDeviceClass))


typedef struct _BurgnerScsiLinuxDevice BurgnerScsiLinuxDevice;
typedef struct _BurgnerScsiLinuxDeviceClass BurgnerScsiLinuxDeviceClass;

struct _BurgnerScsiLinuxDevice {
	BurgnerScsiDevice __parent;

	gint fd;
	gint dma_size;

	BurgnerThread *thread;

	GList *send_cmd_data_list;
};

struct _BurgnerScsiLinuxDeviceClass {
	BurgnerScsiDeviceClass __parent_class;

	/*
	 * Virtual Functions
	 */
	gboolean (* send_cmd_real)(BurgnerScsiLinuxDevice *device,
				   gint fd,
				   gpointer cdb, guint cdb_size,
				   gpointer sense, guint sense_size, guint *sense_used,
				   gpointer data, guint data_size, guint *data_used,
				   BurgnerScsiDirection direction,
				   guint timeount, gint *status,
				   GError **error);
};


GType burgner_scsi_linux_device_get_type (void)
#if __GNUC__ <= 2
	G_GNUC_CONST
#endif
	;
GQuark  burgner_scsi_linux_device_error_quark (void) G_GNUC_CONST;

#define BURGNER_SCSI_LINUX_DEVICE_ERROR		(burgner_scsi_linux_device_error_quark ())


BurgnerScsiLinuxDevice *burgner_scsi_linux_device_new (void);

gboolean burgner_scsi_linux_device_send_cmd_real (BurgnerScsiLinuxDevice *device,
				   gint fd,
				   gpointer cdb, guint cdb_size,
				   gpointer sense, guint sense_size, guint *sense_used,
				   gpointer data, guint data_size, guint *data_used,
				   BurgnerScsiDirection direction,
				   guint timeout, gint *status,
				   GError **error);

G_END_DECLS

#endif /* __BURGNER_SCSI_LINUX_DEVICE_H__ */

