/*
 * BurgnerScsiLinuxDriver - BurgnerScsiLinuxDriver object
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifndef __BURGNER_SCSI_LINUX_DRIVER_H__
#define __BURGNER_SCSI_LINUX_DRIVER_H__

#include <glib-object.h>
#include <libburgnerutils/burgner-thread.h>
#include <libburgnerscsi/burgner-scsi-driver.h>

G_BEGIN_DECLS


#define BURGNER_TYPE_SCSI_LINUX_DRIVER		(burgner_scsi_linux_driver_get_type ())
#define BURGNER_SCSI_LINUX_DRIVER(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), BURGNER_TYPE_SCSI_LINUX_DRIVER, BurgnerScsiLinuxDriver))
#define BURGNER_SCSI_LINUX_DRIVER_CLASS(k)	(G_TYPE_CHECK_CLASS_CAST ((k), BURGNER_TYPE_SCSI_LINUX_DRIVER, BurgnerScsiLinuxDriverClass))
#define BURGNER_IS_SCSI_LINUX_DRIVER(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), BURGNER_TYPE_SCSI_LINUX_DRIVER))
#define BURGNER_IS_SCSI_LINUX_DRIVER_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), BURGNER_TYPE_SCSI_LINUX_DRIVER))
#define BURGNER_SCSI_LINUX_DRIVER_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), BURGNER_TYPE_SCSI_LINUX_DRIVER, BurgnerScsiLinuxDriverClass))


typedef struct _BurgnerScsiLinuxDriver BurgnerScsiLinuxDriver;
typedef struct _BurgnerScsiLinuxDriverClass BurgnerScsiLinuxDriverClass;

struct _BurgnerScsiLinuxDriver {
	BurgnerScsiDriver __parent;

	GList *devices;

	GList *open_device_data_list;
	GList *close_device_data_list;
	GList *get_uri_data_list;
};

struct _BurgnerScsiLinuxDriverClass {
	BurgnerScsiDriverClass __parent_class;

	/*
	 * Virtual Functions
	 */
	gchar *(* get_file_name_from_uri)(BurgnerScsiLinuxDriver *driver,
					  const GUri *uri,
					  GError **error);

	gint (* open_device_real)(BurgnerScsiLinuxDriver *driver,
				  const gchar *file,
				  gint requested_dma_size,
				  gint *real_dma_size,
				  GError **error);
	gboolean (* close_device_real)(BurgnerScsiLinuxDriver *driver,
				       gint fd,
				       GError **error);

	GList *(* get_uri_list_real)(BurgnerScsiLinuxDriver *driver,
				     GError **error);

	BurgnerScsiDevice *(* create_device)(BurgnerScsiLinuxDriver *driver);
};


GType burgner_scsi_linux_driver_get_type (void)
#if __GNUC__ <= 2
	G_GNUC_CONST
#endif
	;

GQuark  burgner_scsi_linux_driver_error_quark (void) G_GNUC_CONST;

#define BURGNER_SCSI_LINUX_DRIVER_ERROR		(burgner_scsi_linux_driver_error_quark ())


gchar *burgner_scsi_linux_driver_get_file_name_from_uri (BurgnerScsiLinuxDriver *driver,
					  const GUri *uri,
					  GError **error);

gint burgner_scsi_linux_driver_open_device_real (BurgnerScsiLinuxDriver *driver,
				  const gchar *file,
				  gint requested_dma_size,
				  gint *real_dma_size,
				  GError **error);
gboolean burgner_scsi_linux_driver_close_device_real (BurgnerScsiLinuxDriver *driver,
				       gint fd,
				       GError **error);

GList *burgner_scsi_linux_driver_get_uri_list_real (BurgnerScsiLinuxDriver *driver,
				     GError **error);

BurgnerScsiDevice *burgner_scsi_linux_driver_create_device (BurgnerScsiLinuxDriver *driver);

G_END_DECLS

#endif /* __BURGNER_SCSI_LINUX_DRIVER_H__ */

