/*
 * 
 *
 * This file is part of the libburgnerscsi library.
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "burgner-scsi-spc-impl.h"
#include "burgner-scsi-mode-page-utils.h"

GType
burgner_scsi_mode_page_values_get_type (void)
{
	static GType etype = 0;

	if (etype == 0) {
		static const GEnumValue values[] = {
			{ BURGNER_SCSI_MODE_PAGE_VALUES_CURRENT, "BURGNER_SCSI_MODE_PAGE_VALUES_CURRENT", "CURRENT" },
			{ BURGNER_SCSI_MODE_PAGE_VALUES_CHANGEABLE, "BURGNER_SCSI_MODE_PAGE_VALUES_CHANGEABLE", "CHANGEABLE" },
			{ BURGNER_SCSI_MODE_PAGE_VALUES_DEFAULT, "BURGNER_SCSI_MODE_PAGE_VALUES_DEFAULT", "DEFAULT" },
			{ BURGNER_SCSI_MODE_PAGE_VALUES_SAVED, "BURGNER_SCSI_MODE_PAGE_VALUES_SAVED", "SAVED" },
			{ 0, NULL, NULL }
		};
		etype = g_enum_register_static ("BurgnerScsiModePageValues", values);
	}

	return etype;
}

guint
burgner_scsi_mode_page_values_to_pc (BurgnerScsiModePageValues values)
{
	switch (values) {
		case BURGNER_SCSI_MODE_PAGE_VALUES_CURRENT:
			return 0;
		case BURGNER_SCSI_MODE_PAGE_VALUES_CHANGEABLE:
			return 1;
		case BURGNER_SCSI_MODE_PAGE_VALUES_DEFAULT:
			return 2;
		case BURGNER_SCSI_MODE_PAGE_VALUES_SAVED:
			return 3;
		default:
			break;
	}

	g_return_val_if_reached (0);
	return 0;
}

guint
burgner_scsi_mode_page_list_get_length (GList *pages)
{
	GList *it;
	guint l = 0;

	g_return_val_if_fail (pages != NULL, 0);

	for (it = pages; it != NULL; it = it->next) {
		_BurgnerScsiSpcImplModePageHeader *h;

		g_return_val_if_fail (it->data != NULL, l);

		h = (_BurgnerScsiSpcImplModePageHeader *) it->data;
		l += h->page_length + 2;
	}

	return l;
}

