/*
 * BurgnerScsiModeSense
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include "burgner-scsi-utils.h"
#include "burgner-scsi-spc-impl.h"
#include "burgner-scsi-mode-sense.h"

void
burgner_scsi_mode_sense_6_write_cdb (gpointer buf, guint buf_size,
				     gboolean enable_block_descriptors,
				     BurgnerScsiModePageValues values,
				     guint8 page_code,
				     guint data_buf_size)
{
	_BurgnerScsiSpcImplModeSense6Cdb *impl;

	g_return_if_fail (buf != NULL);
	g_return_if_fail (buf_size >= BURGNER_SCSI_MODE_SENSE_6_CDB_SIZE);
	g_return_if_fail ((data_buf_size & ~0xff) == 0);
	g_return_if_fail ((page_code & ~0x3f) == 0);

	impl = (_BurgnerScsiSpcImplModeSense6Cdb *) buf;
	memset (impl, 0, BURGNER_SCSI_MODE_SENSE_6_CDB_SIZE);

	impl->op_code = BURGNER_SCSI_MODE_SENSE_6_OP_CODE;
	impl->dbd = !enable_block_descriptors;
	impl->pc = burgner_scsi_mode_page_values_to_pc (values);
	impl->page_code = page_code;
	impl->alloc_length = data_buf_size;
}
					  
void
burgner_scsi_mode_sense_10_write_cdb (gpointer buf, guint buf_size,
				      gboolean enable_block_descriptors,
				      BurgnerScsiModePageValues values,
				      guint8 page_code,
				      guint data_buf_size)
{
	_BurgnerScsiSpcImplModeSense10Cdb *impl;

	g_return_if_fail (buf != NULL);
	g_return_if_fail (buf_size >= BURGNER_SCSI_MODE_SENSE_10_CDB_SIZE);
	g_return_if_fail ((data_buf_size & ~0xffff) == 0);
	g_return_if_fail ((page_code & ~0x3f) == 0);

	impl = (_BurgnerScsiSpcImplModeSense10Cdb *) buf;
	memset (impl, 0, BURGNER_SCSI_MODE_SENSE_10_CDB_SIZE);

	impl->op_code = BURGNER_SCSI_MODE_SENSE_10_OP_CODE;
	impl->dbd = !enable_block_descriptors;
	impl->pc = burgner_scsi_mode_page_values_to_pc (values);
	impl->page_code = page_code;
	burgner_scsi_set_uint16 (impl->alloc_length, data_buf_size);
}

