/*
 * BurgnerScsiPrivilege - BurgnerScsiPrivilege object
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifdef PRIVILEGE_POSIX
#include <unistd.h>
#include <sys/types.h>
#include <string.h>
#endif

#include <glib-object.h>
#include "burgner-scsi-privilege.h"

const gboolean burgner_scsi_privilege_supported = 
#ifdef PRIVILEGE_POSIX
	TRUE
#else
	FALSE
#endif
;

#define check(o)		G_STMT_START {	\
	g_return_if_fail (o);	\
	g_return_if_fail (BURGNER_IS_SCSI_PRIVILEGE (o));	\
				} G_STMT_END
#define check_with_val(o,v)	G_STMT_START {	\
	g_return_val_if_fail ((o), (v));	\
	g_return_val_if_fail (BURGNER_IS_SCSI_PRIVILEGE (o), (v));	\
				} G_STMT_END

struct _BurgnerScsiPrivilege {
	GObject __parent;

#ifdef PRIVILEGE_POSIX
	uid_t orig_e_uid;
	uid_t r_uid;
#endif

	guint count;
};


static GObjectClass *parent_class = NULL;

static void
burgner_scsi_privilege_finalize (GObject *object)
{
	if (G_OBJECT_CLASS (parent_class)->finalize)
		G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
burgner_scsi_privilege_class_init (BurgnerScsiPrivilegeClass *klass)
{
	GObjectClass *object_class;

	object_class = (GObjectClass *) klass;
	object_class->finalize = burgner_scsi_privilege_finalize;

	parent_class = g_type_class_peek_parent (klass);
}

static void
burgner_scsi_privilege_instance_init (BurgnerScsiPrivilege *self)
{
	self->count = 1;
}

GType
burgner_scsi_privilege_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		static GTypeInfo info = {
			sizeof (BurgnerScsiPrivilegeClass),
			NULL,
			NULL,
			(GClassInitFunc) burgner_scsi_privilege_class_init,
			NULL,
			NULL,
			sizeof (BurgnerScsiPrivilege),
			0,
			(GInstanceInitFunc) burgner_scsi_privilege_instance_init,
		};

		type = g_type_register_static (G_TYPE_OBJECT, "BurgnerScsiPrivilege", &info, 0);
	}

	return type;
}

BurgnerScsiPrivilege *
burgner_scsi_privilege_new (void)
{
	BurgnerScsiPrivilege *self;

	self = g_object_new (BURGNER_TYPE_SCSI_PRIVILEGE, NULL);


	return self;
}


GQuark
burgner_scsi_privilege_error_quark (void)
{
	return g_quark_from_string ("BurgnerScsiPrivilege");
}

gboolean
burgner_scsi_privilege_init (BurgnerScsiPrivilege *privilege,
			     GError **error)
{
	check_with_val (privilege, FALSE);

#ifdef PRIVILEGE_POSIX
	privilege->orig_e_uid = geteuid ();
	privilege->r_uid = getuid ();
#endif

	return TRUE;
}

gboolean
burgner_scsi_privilege_set_up (BurgnerScsiPrivilege *privilege,
			       GError **error)
{
	check_with_val (privilege, FALSE);

	if (privilege->count == 0) {
#ifdef PRIVILEGE_POSIX
		if (seteuid (privilege->orig_e_uid) < 0) {
			g_set_error (error,
				     BURGNER_SCSI_PRIVILEGE_ERROR,
				     BURGNER_SCSI_PRIVILEGE_ERROR_WHILE_SETTING_UP,
				     _("Error while setting up privileges: %s"),
				     strerror (errno));
			return FALSE;
		}
#endif
	}

	privilege->count++;
	return TRUE;
}

gboolean
burgner_scsi_privilege_remove (BurgnerScsiPrivilege *privilege,
			       GError **error)
{
	check_with_val (privilege, FALSE);

	if (privilege->count == 1) {
#ifdef PRIVILEGE_POSIX
		if (seteuid (privilege->r_uid) < 0) {
			g_set_error (error,
				     BURGNER_SCSI_PRIVILEGE_ERROR,
				     BURGNER_SCSI_PRIVILEGE_ERROR_WHILE_REMOVING,
				     _("Error while removing privileges: %s"),
				     strerror (errno));
			return FALSE;
		}
#endif
	}

	privilege->count--;
	return TRUE;
}


