/*
 * BurgnerScsiSendCommandStage - BurgnerScsiSendCommandStage object
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifndef __BURGNER_SCSI_SEND_COMMAND_STAGE_H__
#define __BURGNER_SCSI_SEND_COMMAND_STAGE_H__

#include <glib-object.h>
#include <libburgnerscsi/burgner-scsi-device.h>
#include <libburgnerscsi/burgner-scsi-sense.h>
#include <libburgnerscsi/burgner-scsi-stage.h>

G_BEGIN_DECLS


#define BURGNER_TYPE_SCSI_SEND_COMMAND_STAGE		(burgner_scsi_send_command_stage_get_type ())
#define BURGNER_SCSI_SEND_COMMAND_STAGE(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), BURGNER_TYPE_SCSI_SEND_COMMAND_STAGE, BurgnerScsiSendCommandStage))
#define BURGNER_SCSI_SEND_COMMAND_STAGE_CLASS(k)	(G_TYPE_CHECK_CLASS_CAST ((k), BURGNER_TYPE_SCSI_SEND_COMMAND_STAGE, BurgnerScsiSendCommandStageClass))
#define BURGNER_IS_SCSI_SEND_COMMAND_STAGE(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), BURGNER_TYPE_SCSI_SEND_COMMAND_STAGE))
#define BURGNER_IS_SCSI_SEND_COMMAND_STAGE_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), BURGNER_TYPE_SCSI_SEND_COMMAND_STAGE))
#define BURGNER_SCSI_SEND_COMMAND_STAGE_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), BURGNER_TYPE_SCSI_SEND_COMMAND_STAGE, BurgnerScsiSendCommandStageClass))


typedef struct _BurgnerScsiSendCommandStage BurgnerScsiSendCommandStage;
typedef struct _BurgnerScsiSendCommandStageClass BurgnerScsiSendCommandStageClass;

struct _BurgnerScsiSendCommandStage {
	BurgnerScsiStage __parent;

	BurgnerScsiDevice *device;

	guchar *cdb_buf;
	guint cdb_buf_size;

	guchar *sense_buf;
	guint sense_buf_size;
};

struct _BurgnerScsiSendCommandStageClass {
	BurgnerScsiStageClass __parent_class;

	/*
	 * Virtual Functions
	 */
	void (* get_command_parameters)(BurgnerScsiSendCommandStage *self,
					guchar *cdb_buf, guint cdb_buf_size,
					BurgnerScsiDirection *direction,
					guchar **data, guint *data_size,
					guint *timeout);
	gboolean (* command_result)(BurgnerScsiSendCommandStage *self,
				    guint status,
				    BurgnerScsiSense *sense,
				    guint data_used,
				    GError *command_error,
				    GError **error);
};


GType burgner_scsi_send_command_stage_get_type (void) G_GNUC_CONST;
GQuark burgner_scsi_send_command_stage_error_quark (void);

#define BURGNER_SCSI_SEND_COMMAND_STAGE_ERROR		(burgner_scsi_send_command_stage_error_quark ())

void burgner_scsi_send_command_stage_get_command_parameters (BurgnerScsiSendCommandStage *self,
							    guchar *cdb_buf, guint cdb_buf_size,
							    BurgnerScsiDirection *direction,
							    guchar **data, guint *data_size,
							    guint *timeout);
gboolean burgner_scsi_send_command_stage_command_result (BurgnerScsiSendCommandStage *self,
							guint status,
							BurgnerScsiSense *sense,
							guint data_used,
							GError *command_error,
							GError **error);

G_END_DECLS

#endif /* __BURGNER_SCSI_SEND_COMMAND_STAGE_H__ */

