/*
 * BurgnerScsiSense
 *
 * This file is part of the libburgnerscsi library.
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */

#ifndef __BURGNER_SCSI_SENSE_H__
#define __BURGNER_SCSI_SENSE_H__

#include <glib.h>

G_BEGIN_DECLS


typedef enum {
	BURGNER_SCSI_SENSE_RESPONSE_CODE_CURRENT_ERROR = 0,
	BURGNER_SCSI_SENSE_RESPONSE_CODE_DEFERRED_ERROR,
	BURGNER_SCSI_SENSE_RESPONSE_CODE_VENDOR_SPECIFIC
} BurgnerScsiSenseResponseCode;

typedef enum {
	BURGNER_SCSI_SENSE_KEY_NO_SENSE = 0,
	BURGNER_SCSI_SENSE_KEY_RECOVERED_ERROR,
	BURGNER_SCSI_SENSE_KEY_NOT_READY,
	BURGNER_SCSI_SENSE_KEY_MEDIUM_ERROR,
	BURGNER_SCSI_SENSE_KEY_HARDWARE_ERROR,
	BURGNER_SCSI_SENSE_KEY_ILLEGAL_REQUEST,
	BURGNER_SCSI_SENSE_KEY_UNIT_ATTENTION,
	BURGNER_SCSI_SENSE_KEY_DATA_PROTECT,
	BURGNER_SCSI_SENSE_KEY_BLANK_CHECK,
	BURGNER_SCSI_SENSE_KEY_VENDOR_SPECIFIC,
	BURGNER_SCSI_SENSE_KEY_COPY_ABORTED,
	BURGNER_SCSI_SENSE_KEY_ABORTED_COMMAND,
	BURGNER_SCSI_SENSE_KEY_OBSOLETE,
	BURGNER_SCSI_SENSE_KEY_VOLUME_OVERFLOW,
	BURGNER_SCSI_SENSE_KEY_MISCOMPARE,
	BURGNER_SCSI_SENSE_KEY_RESERVED
} BurgnerScsiSenseKey;


typedef struct {
	guint32 information;

	guchar asc;
	guchar ascq;

	guchar valid : 1;
	BurgnerScsiSenseResponseCode response_code : 2;
	BurgnerScsiSenseKey sense_key : 4;

	guchar sksv : 1;

	union {
		struct {
			guint16 field_pointer;
			guchar bit_pointer_valid : 1;
			guchar bit_pointer : 3;
			guchar in_cdb : 1; /* Else in data */
		} illegal_request;
		guint16 actual_retry_count;
		guint16 progress_indication;
	} sks;


	guint ref_count;
} BurgnerScsiSense;


GQuark burgner_scsi_sense_error_quark (void);

#define BURGNER_SCSI_SENSE_ERROR	(burgner_scsi_sense_error_quark ())
typedef enum {
	BURGNER_SCSI_SENSE_ERROR_BUFFER_TOO_SMALL,
	BURGNER_SCSI_SENSE_ERROR_INVALID_RESPONSE_CODE
} BurgnerScsiSenseError;


BurgnerScsiSense *burgner_scsi_sense_new (void);
void burgner_scsi_sense_free (BurgnerScsiSense *sense);

BurgnerScsiSense *burgner_scsi_sense_ref (BurgnerScsiSense *sense);
void burgner_scsi_sense_unref (BurgnerScsiSense *sense);

gboolean burgner_scsi_sense_parse (BurgnerScsiSense *sense,
				   guchar *sense_buf,
				   guint sense_buf_len,
				   GError **error);

gboolean burgner_scsi_sense_new_from_buf (BurgnerScsiSense **sense,
					  guchar *sense_buf,
					  guint sense_buf_len,
					  GError **error);

gchar *burgner_scsi_sense_get_detailed_description_string (BurgnerScsiSense *sense);

G_END_DECLS

#endif /* __BURGNER_SCSI_SENSE_H__ */

