/*
 * BurgnerScsiSetModePageStage - BurgnerScsiSetModePageStage object
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "burgner-scsi-i18n.h"

#include <glib-object.h>
#include "../libburgnerscsi/burgner-scsi-mode-select.h"
#include "burgner-scsi-set-mode-page-stage.h"


static BurgnerScsiSendCommandStageClass *parent_class = NULL;

enum {
	PROP_0,
	PROP_DATA_BUF,
	PROP_DATA_BUF_SIZE,
	PROP_STD_FORMAT,
	PROP_SAVE_PAGES,
	BURGNER_SCSI_SET_MODE_PAGE_STAGE_LAST_ARG
};

static void
burgner_scsi_set_mode_page_stage_finalize (GObject *object)
{
	BurgnerScsiSetModePageStage *self = BURGNER_SCSI_SET_MODE_PAGE_STAGE (object);

	self->data_buf = NULL;
	self->data_buf_size = 0;
	self->std_format = 1;
	self->save_pages = 0;

	if (G_OBJECT_CLASS (parent_class)->finalize)
		G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
burgner_scsi_set_mode_page_stage_set_property (GObject *object,
	      guint prop_id,
	      const GValue *value,
	      GParamSpec *pspec)
{
	BurgnerScsiSetModePageStage *self = BURGNER_SCSI_SET_MODE_PAGE_STAGE (object);

	switch (prop_id) {
		case PROP_DATA_BUF:
			self->data_buf = g_value_get_pointer (value);
			g_object_notify (object, "data_buf");
			break;
		case PROP_DATA_BUF_SIZE:
			self->data_buf_size = g_value_get_uint (value);
			g_object_notify (object, "data_buf_size");
			break;
		case PROP_STD_FORMAT:
			self->std_format = g_value_get_boolean (value) ? 1 : 0;
			break;
		case PROP_SAVE_PAGES:
			self->save_pages = g_value_get_boolean (value) ? 1 : 0;
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
burgner_scsi_set_mode_page_stage_get_property (GObject *object,
	      guint prop_id,
	      GValue *value,
	      GParamSpec *pspec)
{
	BurgnerScsiSetModePageStage *self = BURGNER_SCSI_SET_MODE_PAGE_STAGE (object);

	switch (prop_id) {
		case PROP_DATA_BUF:
			g_value_set_pointer (value, self->data_buf);
			break;
		case PROP_DATA_BUF_SIZE:
			g_value_set_uint (value, self->data_buf_size);
			break;
		case PROP_STD_FORMAT:
			g_value_set_boolean (value, self->std_format);
			break;
		case PROP_SAVE_PAGES:
			g_value_set_boolean (value, self->save_pages);
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
_burgner_scsi_set_mode_page_stage_get_command_parameters
				(BurgnerScsiSendCommandStage *sc_stage,
				 guchar *cdb_buf, guint cdb_buf_size,
				 BurgnerScsiDirection *direction,
				 guchar **data, guint *data_size,
				 guint *timeout)
{
	BurgnerScsiSetModePageStage *self = BURGNER_SCSI_SET_MODE_PAGE_STAGE (sc_stage);
#if 0
	gboolean do_10 = ((self->data_buf_size & ~0xff) != 0);
#else
	gboolean do_10 = TRUE;
#endif

	g_return_if_fail (self->data_buf != NULL);
	g_return_if_fail (self->data_buf_size != 0);
	g_return_if_fail (cdb_buf_size >= do_10
					? BURGNER_SCSI_MODE_SELECT_10_CDB_SIZE
					: BURGNER_SCSI_MODE_SELECT_6_CDB_SIZE);

	if (do_10) {
		burgner_scsi_mode_select_10_write_cdb (cdb_buf, cdb_buf_size,
						       self->std_format,
						       self->save_pages,
						       self->data_buf_size);
	} else {
		burgner_scsi_mode_select_6_write_cdb (cdb_buf, cdb_buf_size,
						      self->std_format,
						      self->save_pages,
						      self->data_buf_size);
	}

	*direction = BURGNER_SCSI_DIRECTION_TO_DEVICE;
	*data = self->data_buf;
	*data_size = self->data_buf_size;
	*timeout = 10000;
}

static void
burgner_scsi_set_mode_page_stage_class_init (BurgnerScsiSetModePageStageClass *klass)
{
	GObjectClass *object_class;
	BurgnerScsiSendCommandStageClass *sc_stage_class;

	object_class = (GObjectClass *) klass;
	object_class->set_property = burgner_scsi_set_mode_page_stage_set_property;
	object_class->get_property = burgner_scsi_set_mode_page_stage_get_property;
	object_class->finalize = burgner_scsi_set_mode_page_stage_finalize;

	sc_stage_class = (BurgnerScsiSendCommandStageClass *) klass;
	sc_stage_class->get_command_parameters = _burgner_scsi_set_mode_page_stage_get_command_parameters;

	/* Properties */
	g_object_class_install_property (object_class,
					 PROP_DATA_BUF,
					 g_param_spec_pointer ("data_buf",
					                _("data_buf"),
							_("A buffer containing the mode parameter list"),
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_DATA_BUF_SIZE,
					 g_param_spec_uint ("data_buf_size",
					                _("data_buf_size"),
							_("Size of data_buf"),
							0,
							0xffff,
							0,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_STD_FORMAT,
					 g_param_spec_boolean ("std_format",
					                _("std_format"),
							_("Whether mode parameter is in standard format"),
							TRUE,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_SAVE_PAGES,
					 g_param_spec_boolean ("save_pages",
					                _("save_pages"),
							_("Save Pages"),
							FALSE,
							G_PARAM_READABLE | G_PARAM_WRITABLE));


	parent_class = g_type_class_peek_parent (klass);
}

static void
burgner_scsi_set_mode_page_stage_init (BurgnerScsiSetModePageStage *self)
{
	self->data_buf = NULL;
	self->data_buf_size = 0;
	self->std_format = 1;
	self->save_pages = 0;
}

GType
burgner_scsi_set_mode_page_stage_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		static GTypeInfo info = {
			sizeof (BurgnerScsiSetModePageStageClass),
			NULL,
			NULL,
			(GClassInitFunc) burgner_scsi_set_mode_page_stage_class_init,
			NULL,
			NULL,
			sizeof (BurgnerScsiSetModePageStage),
			0,
			(GInstanceInitFunc) burgner_scsi_set_mode_page_stage_init,
		};

		type = g_type_register_static (BURGNER_TYPE_SCSI_SEND_COMMAND_STAGE, "BurgnerScsiSetModePageStage", &info, 0);
	}

	return type;
}

BurgnerScsiStage *
burgner_scsi_set_mode_page_stage_new (void)
{
	BurgnerScsiSetModePageStage *self;

	self = g_object_new (BURGNER_TYPE_SCSI_SET_MODE_PAGE_STAGE, NULL);


	return BURGNER_SCSI_STAGE (self);
}


GQuark
burgner_scsi_set_mode_page_stage_error_quark (void)
{
	return g_quark_from_string ("BurgnerScsiSetModePageStage");
}

