/*
 * 
 *
 * This file is part of the libburgnerscsi library.
 *
 * Copyright (c) 2002 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */

#ifndef __BURGNER_SCSI_SPC_IMPL_H__
#define __BURGNER_SCSI_SPC_IMPL_H__

#include <glib.h>
#include <libburgnerscsi/burgner-scsi-bit-field-order.h>

G_BEGIN_DECLS

typedef struct {
	guchar op_code; /* 0x0 */

	guchar res_01[4];
	guchar control;
} __attribute__((packed)) _BurgnerScsiSpcImplTestUnitReadyCdb;


typedef struct {
	guchar op_code; /* 15h */

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar res_01 : 3;
	guchar pf : 1;
	guchar res_02 : 3;
	guchar sp : 1;
#else
	guchar sp : 1;
	guchar res_02 : 3;
	guchar pf : 1;
	guchar res_01 : 3;
#endif

	guchar res_03[2];

	guchar param_list_length;

	guchar control;
} __attribute__((packed)) _BurgnerScsiSpcImplModeSelect6Cdb;

typedef struct {
	guchar op_code; /* 0x55 */

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar res_01 : 3;
	guchar pf : 1;
	guchar res_02 : 3;
	guchar sp : 1;
#else
	guchar sp : 1;
	guchar res_02 : 3;
	guchar pf : 1;
	guchar res_01 : 3;
#endif

	guchar res_03[5];

	guchar param_list_length[2];

	guchar control;
} __attribute__((packed)) _BurgnerScsiSpcImplModeSelect10Cdb;

typedef struct {
	guchar op_code; /* 0x1a */

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar res_01 : 4;
	guchar dbd : 1;
	guchar res_02 : 3;
#else
	guchar res_02 : 3;
	guchar dbd : 1;
	guchar res_01 : 4;
#endif

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar pc : 2;
	guchar page_code : 6;
#else
	guchar page_code : 6;
	guchar pc : 2;
#endif

	guchar res_03;

	guchar alloc_length;

	guchar control;
} __attribute__((packed)) _BurgnerScsiSpcImplModeSense6Cdb;

typedef struct {
	guchar op_code; /* 0x5a */

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar res_01 : 4;
	guchar dbd : 1;
	guchar res_02 : 3;
#else
	guchar res_02 : 3;
	guchar dbd : 1;
	guchar res_01 : 4;
#endif

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar pc : 2;
	guchar page_code : 6;
#else
	guchar page_code : 6;
	guchar pc : 2;
#endif

	guchar res_03[4];

	guchar alloc_length[2];

	guchar control;
} __attribute__((packed)) _BurgnerScsiSpcImplModeSense10Cdb;

typedef struct {
	guchar op_code; /* 0x1e */

	guchar res_01[3];

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar res_02 : 6;
	guchar prevent : 2;
#else
	guchar prevent : 2;
	guchar res_02 : 6;
#endif

	guchar control;
} __attribute__((packed)) _BurgnerScsiSpcImplPreventAllowMediumRemovalCdb;


typedef struct {
	guchar mode_data_length;

	guchar medium_type;
	guchar device_specific_param;

	guchar block_descriptor_length;
} __attribute__((packed)) _BurgnerScsiSpcImplModeParameterHeader6;

typedef struct {
	guchar mode_data_length[2];

	guchar medium_type;
	guchar device_specific_param;

	guchar res_01[2];

	guchar block_descriptor_length[2];
} __attribute__((packed)) _BurgnerScsiSpcImplModeParameterHeader10;

typedef struct {
#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar ps : 1;
	guchar res_01 : 1;
	guchar page_code : 6;
#else
	guchar page_code : 6;
	guchar res_01 : 1;
	guchar ps : 1;
#endif

	guchar page_length;
} __attribute__((packed)) _BurgnerScsiSpcImplModePageHeader;

typedef struct {
	guchar op_code;	/* 0x2a */

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar res_01 : 3;
	guchar dpo : 1;
	guchar fua : 1;
	guchar res_02 : 2;
	guchar reladr : 1;
#else
	guchar reladr : 1;
	guchar res_02 : 2;
	guchar fua : 1;
	guchar dpo : 1;
	guchar res_01 : 3;
#endif

	guchar lba[4];

	guchar res_03;

	guchar transfert_length[2];

	guchar control;
} __attribute__((packed)) _BurgnerScsiSpcImplWriteCdb;


typedef struct {
#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar information_valid : 1;
	guchar response_code : 7;
#else
	guchar response_code : 7;
	guchar information_valid : 1;
#endif
	/*
	 * response_code:
	 *	0x70: current errors
	 *	0x71: deferred errors
	 *	0x7F: vendor-specific
	 */

	guchar segment_number;

#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
	guchar filemark : 1;
	guchar eom : 1;
	guchar ili : 1;
	guchar res_01 : 1;
	guchar sense_key : 4;
#else
	guchar sense_key : 4;
	guchar res_01 : 1;
	guchar ili : 1;
	guchar eom : 1;
	guchar filemark : 1;
#endif

	guchar information[4];

	guchar additional_sense_length;

	guchar command_specific_information[4];

	guchar asc;
	guchar ascq;

	guchar field_replaceable_unit_code;

	union {
		struct {
#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
			guchar sksv : 1:
			guint sense_key_specific : 23;
#else
			guint sense_key_specific : 23;
			guchar sksv : 1;
#endif
		} unknown;

		/* sense_key: ILLEGAL_REQUEST */
		struct {
#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
			guchar valid : 1;
			guchar c_d : 1;
			guchar res_01 : 2;
			guchar bit_pointer_valid : 1;
			guchar bit_pointer : 3;
#else
			guchar bit_pointer : 3;
			guchar bit_pointer_valid : 1;
			guchar res_01 : 2;
			guchar c_d : 1;
			guchar valid : 1;
#endif

			guchar field_pointer[2];
		} field_pointer;

		/* sense_key: RECOVERED/HARDWARE/MEDIUM ERROR */
		struct {
#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
			guchar valid : 1;
			guchar res_01 : 7;
#else
			guchar res_01 : 7;
			guchar valid : 1;
#endif

			guchar actual_retry_count[2];
		} actual_retry_count;

		/* sense_key: NOT_READY/NO_SENSE */
		struct {
#ifdef __BURGNER_SCSI_BIT_FIELDS_HTOL
			guchar valid : 1;
			guchar res_01 : 2;
#else
			guchar res_01 : 2;
			guchar valid : 1;
#endif

			guchar progress_indication[2];
		} progress_indication;
	} sks;
} __attribute__((packed)) _BurgnerScsiSpcImplSenseData;

G_END_DECLS

#endif /* __BURGNER_SCSI_SPC_IMPL_H__ */
