/*
 * 
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "burgner-scsi-i18n.h"

#include "../libburgnerscsi/burgner-scsi-status-code.h"
#include "burgner-scsi-stage.h"
#include "burgner-scsi-stage-error-utils.h"


BurgnerScsiStageError
burgner_scsi_stage_error_from_status_code (gint status_code)
{
	BurgnerScsiStageError err = BURGNER_SCSI_STAGE_ERROR_CONDITION_MET_STATUS_CODE;

	switch (status_code) {
		case BURGNER_SCSI_STATUS_CODE_CHECK_CONDITION:
			err = BURGNER_SCSI_STAGE_ERROR_CHECK_CONDITION_STATUS_CODE;
			break;
		case BURGNER_SCSI_STATUS_CODE_CONDITION_MET:
			err = BURGNER_SCSI_STAGE_ERROR_CONDITION_MET_STATUS_CODE;
			break;
		case BURGNER_SCSI_STATUS_CODE_BUSY:
			err = BURGNER_SCSI_STAGE_ERROR_BUSY_STATUS_CODE;
			break;
		case BURGNER_SCSI_STATUS_CODE_INTERMEDIATE:
			err = BURGNER_SCSI_STAGE_ERROR_INTERMEDIATE_STATUS_CODE;
			break;
		case BURGNER_SCSI_STATUS_CODE_INTERMEDIATE_CONDITION_MET:
			err = BURGNER_SCSI_STAGE_ERROR_INTERMEDIATE_CONDITION_MET_STATUS_CODE;
			break;
		case BURGNER_SCSI_STATUS_CODE_RESERVATION_CONFLICT:
			err = BURGNER_SCSI_STAGE_ERROR_RESERVATION_CONFLICT_STATUS_CODE;
			break;
		case BURGNER_SCSI_STATUS_CODE_COMMAND_TERMINATED:
			err = BURGNER_SCSI_STAGE_ERROR_COMMAND_TERMINATED_STATUS_CODE;
			break;
		case BURGNER_SCSI_STATUS_CODE_TASK_SET_FULL:
			err = BURGNER_SCSI_STAGE_ERROR_TASK_SET_FULL_STATUS_CODE;
			break;
		case BURGNER_SCSI_STATUS_CODE_ACA_ACTIVE:
			err = BURGNER_SCSI_STAGE_ERROR_ACA_ACTIVE_STATUS_CODE;
			break;
		case BURGNER_SCSI_STATUS_CODE_GOOD:
		default:
			g_return_val_if_reached (BURGNER_SCSI_STAGE_ERROR_CONDITION_MET_STATUS_CODE);
			break;
	}

	return err;
}

void
burgner_scsi_stage_set_error_from_status_code (GError **error,
					      guint status_code)
{
	g_return_if_fail (!error || !*error);
	g_return_if_fail (status_code != BURGNER_SCSI_STATUS_CODE_GOOD);

	if (!error)
		return;

	g_set_error (error,
		     BURGNER_SCSI_STAGE_ERROR,
		     burgner_scsi_stage_error_from_status_code (status_code),
		     _("Got bad status code: %s"),
		     burgner_scsi_status_code_to_string (status_code));
}

