/*
 * BurgnerScsiStageManagerCallback - BurgnerScsiStageManagerCallback object
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifndef __BURGNER_SCSI_STAGE_MANAGER_CALLBACK_H__
#define __BURGNER_SCSI_STAGE_MANAGER_CALLBACK_H__

#include <glib-object.h>
#include <libburgnerscsi/burgner-scsi-stage-manager.h>

G_BEGIN_DECLS


#define BURGNER_TYPE_SCSI_STAGE_MANAGER_CALLBACK		(burgner_scsi_stage_manager_callback_get_type ())
#define BURGNER_SCSI_STAGE_MANAGER_CALLBACK(o)			(G_TYPE_CHECK_INSTANCE_CAST ((o), BURGNER_TYPE_SCSI_STAGE_MANAGER_CALLBACK, BurgnerScsiStageManagerCallback))
#define BURGNER_SCSI_STAGE_MANAGER_CALLBACK_CLASS(k)		(G_TYPE_CHECK_CLASS_CAST ((k), BURGNER_TYPE_SCSI_STAGE_MANAGER_CALLBACK, BurgnerScsiStageManagerCallbackClass))
#define BURGNER_IS_SCSI_STAGE_MANAGER_CALLBACK(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), BURGNER_TYPE_SCSI_STAGE_MANAGER_CALLBACK))
#define BURGNER_IS_SCSI_STAGE_MANAGER_CALLBACK_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), BURGNER_TYPE_SCSI_STAGE_MANAGER_CALLBACK))
#define BURGNER_SCSI_STAGE_MANAGER_CALLBACK_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), BURGNER_TYPE_SCSI_STAGE_MANAGER_CALLBACK, BurgnerScsiStageManagerCallbackClass))


typedef struct _BurgnerScsiStageManagerCallback BurgnerScsiStageManagerCallback;
typedef struct _BurgnerScsiStageManagerCallbackClass BurgnerScsiStageManagerCallbackClass;

typedef BurgnerScsiStage *(* BurgnerScsiStageManagerCallbackGetNextStageFunc)
				(BurgnerScsiStageManagerCallback *self,
				 gint *stage_id,
				 BurgnerScsiStage *actual_stage,
				 gint actual_stage_id,
				 gpointer user_data);
typedef gboolean (* BurgnerScsiStageManagerCallbackContinueFunc)
				(BurgnerScsiStageManagerCallback *self,
				 BurgnerScsiStage *stage,
				 gint stage_id,
				 GError *stage_error,
				 BurgnerScsiSense *sense,
				 GError **error,
				 gpointer user_data);


struct _BurgnerScsiStageManagerCallback {
	BurgnerScsiStageManager __parent;

	BurgnerScsiStageManagerCallbackGetNextStageFunc get_next_stage_func;
	BurgnerScsiStageManagerCallbackContinueFunc continue_func;
	gpointer user_data;
};

struct _BurgnerScsiStageManagerCallbackClass {
	BurgnerScsiStageManagerClass __parent_class;

};


GType burgner_scsi_stage_manager_callback_get_type (void) G_GNUC_CONST;
GQuark  burgner_scsi_stage_manager_callback_error_quark (void) G_GNUC_CONST;

#define BURGNER_SCSI_STAGE_MANAGER_CALLBACK_ERROR		(burgner_scsi_stage_manager_callback_error_quark ())


BurgnerScsiStage *burgner_scsi_stage_manager_callback_new (void);

G_END_DECLS

#endif /* __BURGNER_SCSI_STAGE_MANAGER_CALLBACK_H__ */

