/*
 * BurgnerScsiStageManager - BurgnerScsiStageManager object
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifndef __BURGNER_SCSI_STAGE_MANAGER_H__
#define __BURGNER_SCSI_STAGE_MANAGER_H__

#include <glib-object.h>
#include <libburgnerscsi/burgner-scsi-sense.h>
#include <libburgnerscsi/burgner-scsi-stage.h>

G_BEGIN_DECLS


#define BURGNER_TYPE_SCSI_STAGE_MANAGER		(burgner_scsi_stage_manager_get_type ())
#define BURGNER_SCSI_STAGE_MANAGER(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), BURGNER_TYPE_SCSI_STAGE_MANAGER, BurgnerScsiStageManager))
#define BURGNER_SCSI_STAGE_MANAGER_CLASS(k)	(G_TYPE_CHECK_CLASS_CAST ((k), BURGNER_TYPE_SCSI_STAGE_MANAGER, BurgnerScsiStageManagerClass))
#define BURGNER_IS_SCSI_STAGE_MANAGER(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), BURGNER_TYPE_SCSI_STAGE_MANAGER))
#define BURGNER_IS_SCSI_STAGE_MANAGER_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), BURGNER_TYPE_SCSI_STAGE_MANAGER))
#define BURGNER_SCSI_STAGE_MANAGER_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), BURGNER_TYPE_SCSI_STAGE_MANAGER, BurgnerScsiStageManagerClass))


typedef struct _BurgnerScsiStageManager BurgnerScsiStageManager;
typedef struct _BurgnerScsiStageManagerClass BurgnerScsiStageManagerClass;

struct _BurgnerScsiStageManager {
	BurgnerScsiStage __parent;

	gulong end_signal_id;

	BurgnerScsiStage *stage;
	gint stage_id;

	guint cancel : 1;
};

struct _BurgnerScsiStageManagerClass {
	BurgnerScsiStageClass __parent_class;

	/*
	 * Virtual Functions
	 */
	BurgnerScsiStage *(* get_next_stage)(BurgnerScsiStageManager *self,
					    gint *stage_id,
					    BurgnerScsiStage *actual_stage,
					    gint actual_stage_id);
	gboolean (* _continue)(BurgnerScsiStageManager *self,
			       BurgnerScsiStage *stage,
			       gint stage_id,
			       GError *stage_error,
			       BurgnerScsiSense *sense,
			       GError **error);

	/*
 	 * Signals
	 */
	void (* start_stage)(BurgnerScsiStageManager *self,
			     BurgnerScsiStage *stage,
			     gint stage_id);
	void (* end_stage)(BurgnerScsiStageManager *self,
			   BurgnerScsiStage *stage,
			   gint stage_id,
			   GError *error,
			   BurgnerScsiSense *sense);
};


GType burgner_scsi_stage_manager_get_type (void) G_GNUC_CONST;
GQuark  burgner_scsi_stage_manager_error_quark (void);

#define BURGNER_SCSI_STAGE_MANAGER_ERROR		(burgner_scsi_stage_manager_error_quark ())

BurgnerScsiStage *burgner_scsi_stage_manager_get_next_stage (BurgnerScsiStageManager *self,
							   gint *stage_id,
							   BurgnerScsiStage *actual_stage,
							   gint actual_stage_id);
gboolean burgner_scsi_stage_manager_continue (BurgnerScsiStageManager *self,
					     BurgnerScsiStage *stage,
					     gint stage_id,
					     GError *stage_error,
					     BurgnerScsiSense *sense,
					     GError **error);

/* Routines for emitting signals on the BurgnerScsiStageManager */
void burgner_scsi_stage_manager_start_stage (BurgnerScsiStageManager *self,
					    BurgnerScsiStage *stage,
					    gint stage_id);
void burgner_scsi_stage_manager_end_stage (BurgnerScsiStageManager *self,
					  BurgnerScsiStage *stage,
					  gint stage_id,
					  GError *error,
					  BurgnerScsiSense *sense);


G_END_DECLS

#endif /* __BURGNER_SCSI_STAGE_MANAGER_H__ */

