/*
 * BurgnerScsiStage - BurgnerScsiStage object
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifndef __BURGNER_SCSI_STAGE_H__
#define __BURGNER_SCSI_STAGE_H__

#include <glib-object.h>
#include <libburgnerscsi/burgner-scsi-sense.h>

G_BEGIN_DECLS


#define BURGNER_TYPE_SCSI_STAGE		(burgner_scsi_stage_get_type ())
#define BURGNER_SCSI_STAGE(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), BURGNER_TYPE_SCSI_STAGE, BurgnerScsiStage))
#define BURGNER_SCSI_STAGE_CLASS(k)	(G_TYPE_CHECK_CLASS_CAST ((k), BURGNER_TYPE_SCSI_STAGE, BurgnerScsiStageClass))
#define BURGNER_IS_SCSI_STAGE(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), BURGNER_TYPE_SCSI_STAGE))
#define BURGNER_IS_SCSI_STAGE_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), BURGNER_TYPE_SCSI_STAGE))
#define BURGNER_SCSI_STAGE_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), BURGNER_TYPE_SCSI_STAGE, BurgnerScsiStageClass))


typedef struct _BurgnerScsiStage BurgnerScsiStage;
typedef struct _BurgnerScsiStageClass BurgnerScsiStageClass;

struct _BurgnerScsiStage {
	GObject __parent;

	guint idle_id;
};

struct _BurgnerScsiStageClass {
	GObjectClass __parent_class;

	/*
	 * Virtual Functions
	 */
	void (* start)(BurgnerScsiStage *stage,
			   gboolean from_idle);
	void (* real_start)(BurgnerScsiStage *stage);

	void (* cancel)(BurgnerScsiStage *stage);
	void (* real_cancel)(BurgnerScsiStage *stage);

	gboolean (* cancellable)(BurgnerScsiStage *stage);

	/*
	 * Signals
	 */
	void (* end)(BurgnerScsiStage *stage,
		     gboolean cancelled,
		     GError *error,
		     BurgnerScsiSense *sense);
};


GType burgner_scsi_stage_get_type (void) G_GNUC_CONST;
GQuark  burgner_scsi_stage_error_quark (void);

#define BURGNER_SCSI_STAGE_ERROR		(burgner_scsi_stage_error_quark ())
typedef enum {
	BURGNER_SCSI_STAGE_ERROR_CHECK_CONDITION_STATUS_CODE,
	BURGNER_SCSI_STAGE_ERROR_BUSY_STATUS_CODE,
	BURGNER_SCSI_STAGE_ERROR_CONDITION_MET_STATUS_CODE,
	BURGNER_SCSI_STAGE_ERROR_INTERMEDIATE_STATUS_CODE,
	BURGNER_SCSI_STAGE_ERROR_INTERMEDIATE_CONDITION_MET_STATUS_CODE,
	BURGNER_SCSI_STAGE_ERROR_RESERVATION_CONFLICT_STATUS_CODE,
	BURGNER_SCSI_STAGE_ERROR_COMMAND_TERMINATED_STATUS_CODE,
	BURGNER_SCSI_STAGE_ERROR_TASK_SET_FULL_STATUS_CODE,
	BURGNER_SCSI_STAGE_ERROR_ACA_ACTIVE_STATUS_CODE
} BurgnerScsiStageError;


void burgner_scsi_stage_start (BurgnerScsiStage *stage,
				  gboolean from_idle);
void burgner_scsi_stage_real_start (BurgnerScsiStage *stage);

void burgner_scsi_stage_cancel (BurgnerScsiStage *stage);
void burgner_scsi_stage_real_cancel (BurgnerScsiStage *stage);

gboolean burgner_scsi_stage_cancellable (BurgnerScsiStage *stage);

/* Routines for emitting signals on the BurgnerScsiStage */
void burgner_scsi_stage_end (BurgnerScsiStage *stage,
			    gboolean cancelled,
			    GError *error,
			    BurgnerScsiSense *sense);

G_END_DECLS

#endif /* __BURGNER_SCSI_STAGE_H__ */

