/*
 * BurgnerScsiStopUnitStage - BurgnerScsiStopUnitStage object
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "burgner-scsi-i18n.h"

#include <glib-object.h>
#include "../libburgnerscsi/burgner-scsi-start-stop-unit.h"
#include "burgner-scsi-stop-unit-stage.h"


static BurgnerScsiSendCommandStageClass *parent_class = NULL;

enum {
	PROP_0,
	PROP_IMMED,
	PROP_EJECT,
	BURGNER_SCSI_STOP_UNIT_STAGE_LAST_ARG
};

static void
burgner_scsi_stop_unit_stage_finalize (GObject *object)
{
	BurgnerScsiStopUnitStage *self = BURGNER_SCSI_STOP_UNIT_STAGE (object);

	self->immed = 0;
	self->eject = 1;

	if (G_OBJECT_CLASS (parent_class)->finalize)
		G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
burgner_scsi_stop_unit_stage_set_property (GObject *object,
	      guint prop_id,
	      const GValue *value,
	      GParamSpec *pspec)
{
	BurgnerScsiStopUnitStage *self = BURGNER_SCSI_STOP_UNIT_STAGE (object);

	switch (prop_id) {
		case PROP_IMMED:
			self->immed = g_value_get_boolean (value) ? 1 : 0;
			g_object_notify (object, "immed");
			break;
		case PROP_EJECT:
			self->eject = g_value_get_boolean (value) ? 1 : 0;
			g_object_notify (object, "eject");
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
burgner_scsi_stop_unit_stage_get_property (GObject *object,
	      guint prop_id,
	      GValue *value,
	      GParamSpec *pspec)
{
	BurgnerScsiStopUnitStage *self = BURGNER_SCSI_STOP_UNIT_STAGE (object);

	switch (prop_id) {
		case PROP_IMMED:
			g_value_set_boolean (value, self->immed);
			break;
		case PROP_EJECT:
			g_value_set_boolean (value, self->eject);
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
_burgner_scsi_stop_unit_stage_get_command_parameters
				(BurgnerScsiSendCommandStage *sc_stage,
				 guchar *cdb_buf, guint cdb_buf_size,
				 BurgnerScsiDirection *direction,
				 guchar **data, guint *data_size,
				 guint *timeout)
{
	BurgnerScsiStopUnitStage *self = BURGNER_SCSI_STOP_UNIT_STAGE (sc_stage);

	g_return_if_fail (cdb_buf_size >= BURGNER_SCSI_START_STOP_UNIT_CDB_SIZE);

	burgner_scsi_stop_unit_write_cdb (cdb_buf, cdb_buf_size,
					  self->immed, self->eject);

	*direction = BURGNER_SCSI_DIRECTION_NONE;
	*data = NULL;
	*data_size = 0;
	*timeout = 10000;
}

static void
burgner_scsi_stop_unit_stage_class_init (BurgnerScsiStopUnitStageClass *klass)
{
	GObjectClass *object_class;
	BurgnerScsiSendCommandStageClass *sc_stage_class;

	object_class = (GObjectClass *) klass;
	object_class->set_property = burgner_scsi_stop_unit_stage_set_property;
	object_class->get_property = burgner_scsi_stop_unit_stage_get_property;
	object_class->finalize = burgner_scsi_stop_unit_stage_finalize;

	sc_stage_class = (BurgnerScsiSendCommandStageClass *) klass;
	sc_stage_class->get_command_parameters = _burgner_scsi_stop_unit_stage_get_command_parameters;

	/* Properties */
	g_object_class_install_property (object_class,
					 PROP_IMMED,
					 g_param_spec_boolean ("immed",
					                _("immed"),
							_("Whether the command should return immediatly"),
							FALSE,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_EJECT,
					 g_param_spec_boolean ("eject",
					                _("eject"),
							_("Eject the medium"),
							TRUE,
							G_PARAM_READABLE | G_PARAM_WRITABLE));


	parent_class = g_type_class_peek_parent (klass);
}

static void
burgner_scsi_stop_unit_stage_init (BurgnerScsiStopUnitStage *self)
{
	self->immed = 0;
	self->eject = 1;
}

GType
burgner_scsi_stop_unit_stage_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		static GTypeInfo info = {
			sizeof (BurgnerScsiStopUnitStageClass),
			NULL,
			NULL,
			(GClassInitFunc) burgner_scsi_stop_unit_stage_class_init,
			NULL,
			NULL,
			sizeof (BurgnerScsiStopUnitStage),
			0,
			(GInstanceInitFunc) burgner_scsi_stop_unit_stage_init,
		};

		type = g_type_register_static (BURGNER_TYPE_SCSI_SEND_COMMAND_STAGE, "BurgnerScsiStopUnitStage", &info, 0);
	}

	return type;
}

BurgnerScsiStage *
burgner_scsi_stop_unit_stage_new (void)
{
	BurgnerScsiStopUnitStage *self;

	self = g_object_new (BURGNER_TYPE_SCSI_STOP_UNIT_STAGE, NULL);


	return BURGNER_SCSI_STAGE (self);
}


GQuark
burgner_scsi_stop_unit_stage_error_quark (void)
{
	return g_quark_from_string ("BurgnerScsiStopUnitStage");
}

