/*
 * BurgnerScsiWaitFifoFullStage - BurgnerScsiWaitFifoFullStage object
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "burgner-scsi-i18n.h"

#include <glib-object.h>
#include "burgner-scsi-wait-fifo-full-stage.h"


static BurgnerScsiStageClass *parent_class = NULL;

enum {
	PROP_0,
	PROP_FIFO,
	PROP_INTERVAL,
	PROP_PRIORITY,
	PROP_MIN_PERCENT,
	BURGNER_SCSI_WAIT_FIFO_FULL_STAGE_LAST_ARG
};

static void
_burgner_scsi_wait_fifo_full_stage_finalize (GObject *object)
{
	BurgnerScsiWaitFifoFullStage *self = BURGNER_SCSI_WAIT_FIFO_FULL_STAGE (object);

	if (self->fifo) {
		g_object_unref (self->fifo);
		self->fifo = NULL;
	}

	if (self->timeout_id) {
		g_source_remove (self->timeout_id);
		self->timeout_id = 0;
	}

	self->interval = 1000;
	self->priority = G_PRIORITY_DEFAULT_IDLE;
	self->min_percent = 90;

	if (G_OBJECT_CLASS (parent_class)->finalize)
		G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
_burgner_scsi_wait_fifo_full_stage_set_property (GObject *object,
	      guint prop_id,
	      const GValue *value,
	      GParamSpec *pspec)
{
	BurgnerScsiWaitFifoFullStage *self = BURGNER_SCSI_WAIT_FIFO_FULL_STAGE (object);

	switch (prop_id) {
		case PROP_FIFO:
			if (self->fifo)
				g_object_unref (self->fifo);
			self->fifo = (BurgnerFifo *) g_value_get_object (value);
			if (self->fifo)
				self->fifo = BURGNER_FIFO (g_object_ref (self->fifo));
			g_object_notify (object, "fifo");
			break;
		case PROP_INTERVAL:
			self->interval = g_value_get_uint (value);
			g_object_notify (object, "interval");
			break;
		case PROP_PRIORITY:
			self->priority = g_value_get_int (value);
			g_object_notify (object, "priority");
			break;
		case PROP_MIN_PERCENT:
			self->min_percent = g_value_get_uint (value);
			g_object_notify (object, "min_percent");
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
_burgner_scsi_wait_fifo_full_stage_get_property (GObject *object,
	      guint prop_id,
	      GValue *value,
	      GParamSpec *pspec)
{
	BurgnerScsiWaitFifoFullStage *self = BURGNER_SCSI_WAIT_FIFO_FULL_STAGE (object);

	switch (prop_id) {
		case PROP_FIFO:
			g_value_set_object (value, self->fifo);
			break;
		case PROP_INTERVAL:
			g_value_set_uint (value, self->interval);
			break;
		case PROP_PRIORITY:
			g_value_set_int (value, self->priority);
			break;
		case PROP_MIN_PERCENT:
			g_value_set_uint (value, self->min_percent);
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static gboolean
_burgner_scsi_wait_fifo_full_stage_timeout_func (gpointer user_data)
{
	BurgnerScsiWaitFifoFullStage *self = BURGNER_SCSI_WAIT_FIFO_FULL_STAGE (user_data);

	g_return_val_if_fail (self->fifo != NULL, FALSE);
	g_return_val_if_fail (self->timeout_id != 0, FALSE);

	if (burgner_fifo_end_reached (self->fifo) ||
	    burgner_fifo_get_percent (self->fifo) >= self->min_percent) {
		self->timeout_id = 0;
		burgner_scsi_stage_end (BURGNER_SCSI_STAGE (self), FALSE, NULL, NULL);
		return FALSE;
	}

	return TRUE;
}

static void
_burgner_scsi_wait_fifo_full_stage_start (BurgnerScsiStage *stage,
					  gboolean from_idle)
{
	BurgnerScsiWaitFifoFullStage *self = BURGNER_SCSI_WAIT_FIFO_FULL_STAGE (stage);

	g_return_if_fail (self->fifo != NULL);
	g_return_if_fail (self->timeout_id == 0);

	self->timeout_id = g_timeout_add_full (self->priority,
					       self->interval,
					       _burgner_scsi_wait_fifo_full_stage_timeout_func,
					       self, NULL);
}

static void
_burgner_scsi_wait_fifo_full_stage_cancel (BurgnerScsiStage *stage)
{
	BurgnerScsiWaitFifoFullStage *self = BURGNER_SCSI_WAIT_FIFO_FULL_STAGE (stage);

	g_return_if_fail (self->fifo != NULL);
	g_return_if_fail (self->timeout_id != 0);

	g_source_remove (self->timeout_id);
}

static gboolean
_burgner_scsi_wait_fifo_full_stage_cancellable (BurgnerScsiStage *stage)
{
	return TRUE;
}

static void
_burgner_scsi_wait_fifo_full_stage_class_init (BurgnerScsiWaitFifoFullStageClass *klass)
{
	GObjectClass *object_class;
	BurgnerScsiStageClass *stage_class;

	object_class = (GObjectClass *) klass;
	object_class->set_property = _burgner_scsi_wait_fifo_full_stage_set_property;
	object_class->get_property = _burgner_scsi_wait_fifo_full_stage_get_property;
	object_class->finalize = _burgner_scsi_wait_fifo_full_stage_finalize;

	stage_class = (BurgnerScsiStageClass *) klass;
	stage_class->start = _burgner_scsi_wait_fifo_full_stage_start;
	stage_class->cancel = _burgner_scsi_wait_fifo_full_stage_cancel;
	stage_class->cancellable = _burgner_scsi_wait_fifo_full_stage_cancellable;

	/* Properties */
	g_object_class_install_property (object_class,
					 PROP_FIFO,
					 g_param_spec_object ("fifo",
					                _("fifo"),
							_("A BurgnerFifo"),
							BURGNER_TYPE_FIFO,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_INTERVAL,
					 g_param_spec_uint ("interval",
					                _("interval"),
							_("the time between each check"),
							0,
							G_MAXUINT,
							1000,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_PRIORITY,
					 g_param_spec_int ("priority",
					                _("priority"),
							_("the priority of the timeout source"),
							G_MININT,
							G_MAXINT,
							G_PRIORITY_DEFAULT_IDLE,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_MIN_PERCENT,
					 g_param_spec_uint ("min_percent",
					                _("min_percent"),
							_("the minimum percent the fifo is assumed full at"),
							0,
							100,
							90,
							G_PARAM_READABLE | G_PARAM_WRITABLE));


	parent_class = g_type_class_peek_parent (klass);
}

static void
_burgner_scsi_wait_fifo_full_stage_init (BurgnerScsiWaitFifoFullStage *self)
{
	self->fifo = NULL;
	self->timeout_id = 0;
	self->interval = 1000;
	self->priority = G_PRIORITY_DEFAULT_IDLE;
	self->min_percent = 90;
}

GType
burgner_scsi_wait_fifo_full_stage_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		static GTypeInfo info = {
			sizeof (BurgnerScsiWaitFifoFullStageClass),
			NULL,
			NULL,
			(GClassInitFunc) _burgner_scsi_wait_fifo_full_stage_class_init,
			NULL,
			NULL,
			sizeof (BurgnerScsiWaitFifoFullStage),
			0,
			(GInstanceInitFunc) _burgner_scsi_wait_fifo_full_stage_init,
		};

		type = g_type_register_static (BURGNER_TYPE_SCSI_STAGE, "BurgnerScsiWaitFifoFullStage", &info, 0);
	}

	return type;
}

BurgnerScsiStage *
burgner_scsi_wait_fifo_full_stage_new (void)
{
	BurgnerScsiWaitFifoFullStage *self;

	self = g_object_new (BURGNER_TYPE_SCSI_WAIT_FIFO_FULL_STAGE, NULL);


	return BURGNER_SCSI_STAGE (self);
}


GQuark
burgner_scsi_wait_fifo_full_stage_error_quark (void)
{
	return g_quark_from_string ("BurgnerScsiWaitFifoFullStage");
}

