/*
 * BurgnerScsiWaitUnitReadyStage - BurgnerScsiWaitUnitReadyStage object
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "burgner-scsi-i18n.h"

#include <glib-object.h>
#include "burgner-scsi-test-unit-ready-stage.h"
#include "burgner-scsi-wait-stage.h"
#include "burgner-scsi-wait-unit-ready-stage.h"


static BurgnerScsiStageManagerClass *parent_class = NULL;

enum {
	PROP_0,
	PROP_DEVICE,
	PROP_CDB_BUF,
	PROP_CDB_BUF_SIZE,
	PROP_SENSE_BUF,
	PROP_SENSE_BUF_SIZE,
	PROP_INTERVAL_TIME,
	PROP_TIME,
	PROP_ELAPSED_TIME,
	BURGNER_SCSI_WAIT_UNIT_READY_STAGE_LAST_ARG
};


enum {
	_BURGNER_SCSI_WAIT_UNIT_READY_STAGE_ID_NONE = -1,
	_BURGNER_SCSI_WAIT_UNIT_READY_STAGE_ID_TEST_UNIT_READY,
	_BURGNER_SCSI_WAIT_UNIT_READY_STAGE_ID_WAIT
};

static void
_burgner_scsi_wait_unit_ready_stage_finalize (GObject *object)
{
	BurgnerScsiWaitUnitReadyStage *self = BURGNER_SCSI_WAIT_UNIT_READY_STAGE (object);

	if (self->device) {
		g_object_unref (self->device);
		self->device = NULL;
	}

	self->cdb_buf = self->sense_buf = NULL;
	self->cdb_buf_size = self->sense_buf_size = 0;

	if (self->test_unit_ready_stage) {
		g_object_unref (self->test_unit_ready_stage);
		self->test_unit_ready_stage = NULL;
	}

	if (self->wait_stage) {
		g_object_unref (self->wait_stage);
		self->wait_stage = NULL;
	}

	self->interval_time = 1000;
	self->time = 1000000;
	self->elapsed_time = 0;

	self->got_no_sense = 0;

	if (G_OBJECT_CLASS (parent_class)->finalize)
		G_OBJECT_CLASS (parent_class)->finalize (object);
}

static void
_burgner_scsi_wait_unit_ready_stage_set_property (GObject *object,
	      guint prop_id,
	      const GValue *value,
	      GParamSpec *pspec)
{
	BurgnerScsiWaitUnitReadyStage *self = BURGNER_SCSI_WAIT_UNIT_READY_STAGE (object);

	switch (prop_id) {
		case PROP_DEVICE:
			g_return_if_fail (self->test_unit_ready_stage == NULL);
			if (self->device)
				g_object_unref (self->device);
			self->device = (BurgnerScsiDevice *) g_value_get_object (value);
			if (self->device)
				self->device = BURGNER_SCSI_DEVICE (g_object_ref (self->device));
			g_object_notify (object, "device");
			break;
		case PROP_CDB_BUF:
			g_return_if_fail (self->test_unit_ready_stage == NULL);
			self->cdb_buf = g_value_get_pointer (value);
			g_object_notify (object, "cdb_buf");
			break;
		case PROP_CDB_BUF_SIZE:
			g_return_if_fail (self->test_unit_ready_stage == NULL);
			self->cdb_buf_size = g_value_get_uint (value);
			g_object_notify (object, "cdb_buf_size");
			break;
		case PROP_SENSE_BUF:
			g_return_if_fail (self->test_unit_ready_stage == NULL);
			self->sense_buf = g_value_get_pointer (value);
			g_object_notify (object, "sense_buf");
			break;
		case PROP_SENSE_BUF_SIZE:
			g_return_if_fail (self->test_unit_ready_stage == NULL);
			self->sense_buf_size = g_value_get_uint (value);
			g_object_notify (object, "sense_buf_size");
			break;
		case PROP_INTERVAL_TIME:
			g_return_if_fail (self->elapsed_time == 0);
			g_return_if_fail (self->wait_stage == NULL);
			self->interval_time = g_value_get_uint (value);
			g_object_notify (object, "interval_time");
			break;
		case PROP_TIME:
			g_return_if_fail (self->elapsed_time == 0);
			g_return_if_fail (self->wait_stage == NULL);
			self->time = g_value_get_uint (value);
			g_object_notify (object, "time");
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static void
_burgner_scsi_wait_unit_ready_stage_get_property (GObject *object,
	      guint prop_id,
	      GValue *value,
	      GParamSpec *pspec)
{
	BurgnerScsiWaitUnitReadyStage *self = BURGNER_SCSI_WAIT_UNIT_READY_STAGE (object);

	switch (prop_id) {
		case PROP_DEVICE:
			g_value_set_object (value, self->device);
			break;
		case PROP_CDB_BUF:
			g_value_set_pointer (value, self->cdb_buf);
			break;
		case PROP_CDB_BUF_SIZE:
			g_value_set_uint (value, self->cdb_buf_size);
			break;
		case PROP_SENSE_BUF:
			g_value_set_pointer (value, self->sense_buf);
			break;
		case PROP_SENSE_BUF_SIZE:
			g_value_set_uint (value, self->sense_buf_size);
			break;
		case PROP_INTERVAL_TIME:
			g_value_set_uint (value, self->interval_time);
			break;
		case PROP_TIME:
			g_value_set_uint (value, self->time);
			break;
		case PROP_ELAPSED_TIME:
			g_value_set_uint (value, self->elapsed_time);
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
			break;
	}
}

static BurgnerScsiStage *
_burgner_scsi_wait_unit_ready_stage_get_next_stage
					(BurgnerScsiStageManager *sm,
					 gint *stage_id,
					 BurgnerScsiStage *actual_stage,
					 gint actual_stage_id)
{
	BurgnerScsiWaitUnitReadyStage *self = BURGNER_SCSI_WAIT_UNIT_READY_STAGE (sm);
	BurgnerScsiStage *stage = NULL;

	switch (actual_stage_id) {
		case _BURGNER_SCSI_WAIT_UNIT_READY_STAGE_ID_NONE:
			g_assert (!actual_stage);
			g_assert (!self->test_unit_ready_stage);
			g_assert (!self->wait_stage);
			g_assert (!self->elapsed_time);
			g_assert (self->got_no_sense == 0);

			g_return_val_if_fail (self->interval_time <= self->time, NULL);

			self->test_unit_ready_stage = burgner_scsi_test_unit_ready_stage_new ();
			g_object_set (self->test_unit_ready_stage,
					"device", self->device,
					"cdb_buf", self->cdb_buf,
					"cdb_buf_size", self->cdb_buf_size,
					"sense_buf", self->sense_buf,
					"sense_buf_size", self->sense_buf_size,
					NULL);

			self->wait_stage = burgner_scsi_wait_stage_new ();
			g_object_set (self->wait_stage,
					"time", self->interval_time,
					"priority", G_PRIORITY_HIGH_IDLE,
					NULL);

			stage = BURGNER_SCSI_STAGE (g_object_ref (self->test_unit_ready_stage));
			*stage_id = _BURGNER_SCSI_WAIT_UNIT_READY_STAGE_ID_TEST_UNIT_READY;
			break;
		case _BURGNER_SCSI_WAIT_UNIT_READY_STAGE_ID_WAIT:
			stage = BURGNER_SCSI_STAGE (g_object_ref (self->test_unit_ready_stage));
			*stage_id = _BURGNER_SCSI_WAIT_UNIT_READY_STAGE_ID_TEST_UNIT_READY;
			break;
		case _BURGNER_SCSI_WAIT_UNIT_READY_STAGE_ID_TEST_UNIT_READY:
			//if (self->elapsed_time >= self->time) {
			if (self->got_no_sense) {
				g_object_unref (self->test_unit_ready_stage);
				self->test_unit_ready_stage = NULL;
				g_object_unref (self->wait_stage);
				self->wait_stage = NULL;
				*stage_id = _BURGNER_SCSI_WAIT_UNIT_READY_STAGE_ID_NONE;
				return NULL;
			}

			stage = BURGNER_SCSI_STAGE (g_object_ref (self->wait_stage));
			*stage_id = _BURGNER_SCSI_WAIT_UNIT_READY_STAGE_ID_WAIT;
			break;
		default:
			g_return_val_if_reached (NULL);
			break;
	}

	return stage;
}

static gboolean
_burgner_scsi_wait_unit_ready_stage_continue
					(BurgnerScsiStageManager *sm,
					 BurgnerScsiStage *stage,
					 gint stage_id,
					 GError *stage_error,
					 BurgnerScsiSense *sense,
					 GError **error)
{
	BurgnerScsiWaitUnitReadyStage *self = BURGNER_SCSI_WAIT_UNIT_READY_STAGE (sm);

	if (stage_error) {
		g_object_unref (self->test_unit_ready_stage);
		self->test_unit_ready_stage = NULL;
		g_object_unref (self->wait_stage);
		self->wait_stage = NULL;

		*error = g_error_copy (stage_error);
		return FALSE;
	}

	switch (stage_id) {
		case _BURGNER_SCSI_WAIT_UNIT_READY_STAGE_ID_TEST_UNIT_READY:
			if (sense == NULL) {
				self->got_no_sense = 1;
				return TRUE;
			}

			if (self->elapsed_time >= self->time) {
				g_object_unref (self->test_unit_ready_stage);
				self->test_unit_ready_stage = NULL;
				g_object_unref (self->wait_stage);
				self->wait_stage = NULL;
				g_set_error (error,
						BURGNER_SCSI_WAIT_UNIT_READY_STAGE_ERROR,
						BURGNER_SCSI_WAIT_UNIT_READY_STAGE_ERROR_UNIT_NOT_READY,
						_("Unit is not ready"));
				return FALSE;
			}

			if (sense &&
			    (sense->asc != 0x4 ||
			     (sense->ascq != 0x0 && sense->ascq != 0x1))) {
				g_object_unref (self->test_unit_ready_stage);
				self->test_unit_ready_stage = NULL;
				g_object_unref (self->wait_stage);
				self->wait_stage = NULL;
				g_set_error (error,
						BURGNER_SCSI_WAIT_UNIT_READY_STAGE_ERROR,
						BURGNER_SCSI_WAIT_UNIT_READY_STAGE_ERROR_UNIT_NOT_READY,
						_("Unit is not ready"));
				return FALSE;
			}
			break;
		case _BURGNER_SCSI_WAIT_UNIT_READY_STAGE_ID_WAIT:
			self->elapsed_time += self->interval_time;
			break;
		default:
			g_return_val_if_reached (FALSE);
			break;
	}

	return TRUE;
}

static void
_burgner_scsi_wait_unit_ready_stage_class_init (BurgnerScsiWaitUnitReadyStageClass *klass)
{
	GObjectClass *object_class;
	BurgnerScsiStageManagerClass *sm_class;

	object_class = (GObjectClass *) klass;
	object_class->set_property = _burgner_scsi_wait_unit_ready_stage_set_property;
	object_class->get_property = _burgner_scsi_wait_unit_ready_stage_get_property;
	object_class->finalize = _burgner_scsi_wait_unit_ready_stage_finalize;

	sm_class = (BurgnerScsiStageManagerClass *) klass;
	sm_class->get_next_stage = _burgner_scsi_wait_unit_ready_stage_get_next_stage;
	sm_class->_continue = _burgner_scsi_wait_unit_ready_stage_continue;

	/* Properties */
	g_object_class_install_property (object_class,
					 PROP_DEVICE,
					 g_param_spec_object ("device",
					                _("device"),
							_("A BurgnerScsiDevice"),
							BURGNER_TYPE_SCSI_DEVICE,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_CDB_BUF,
					 g_param_spec_pointer ("cdb_buf",
					                _("cdb_buf"),
							_("A buffer which will contain the CDB"),
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_CDB_BUF_SIZE,
					 g_param_spec_uint ("cdb_buf_size",
					                _("cdb_buf_size"),
							_("Size of cdb_buf (should be at least 6)"),
							0,
							G_MAXUINT,
							0,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_SENSE_BUF,
					 g_param_spec_pointer ("sense_buf",
					                _("sense_buf"),
							_("A buffer which will contain the sense"),
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_SENSE_BUF_SIZE,
					 g_param_spec_uint ("sense_buf_size",
					                _("sense_buf_size"),
							_("Size of sense_buf (should be at least 18)"),
							0,
							G_MAXUINT,
							0,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_INTERVAL_TIME,
					 g_param_spec_uint ("interval_time",
					                _("interval_time"),
							_("Time between each test"),
							0,
							G_MAXUINT,
							1000,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_TIME,
					 g_param_spec_uint ("time",
					                _("time"),
							_("Maximum time to wait"),
							0,
							G_MAXUINT,
							1000000,
							G_PARAM_READABLE | G_PARAM_WRITABLE));
	g_object_class_install_property (object_class,
					 PROP_ELAPSED_TIME,
					 g_param_spec_uint ("elapsed_time",
					                _("elapsed_time"),
							_("Elapsed time since first test"),
							0,
							G_MAXUINT,
							0,
							G_PARAM_READABLE));


	parent_class = g_type_class_peek_parent (klass);
}

static void
_burgner_scsi_wait_unit_ready_stage_init (BurgnerScsiWaitUnitReadyStage *self)
{
	self->device = NULL;
	self->cdb_buf = self->sense_buf = NULL;
	self->cdb_buf_size = self->sense_buf_size = 0;

	self->test_unit_ready_stage = NULL;
	self->wait_stage = NULL;

	self->interval_time = 1000;
	self->time = 1000000;
	self->elapsed_time = 0;

	self->got_no_sense = 0;
}

GType
burgner_scsi_wait_unit_ready_stage_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		static GTypeInfo info = {
			sizeof (BurgnerScsiWaitUnitReadyStageClass),
			NULL,
			NULL,
			(GClassInitFunc) _burgner_scsi_wait_unit_ready_stage_class_init,
			NULL,
			NULL,
			sizeof (BurgnerScsiWaitUnitReadyStage),
			0,
			(GInstanceInitFunc) _burgner_scsi_wait_unit_ready_stage_init,
		};

		type = g_type_register_static (BURGNER_TYPE_SCSI_STAGE_MANAGER, "BurgnerScsiWaitUnitReadyStage", &info, 0);
	}

	return type;
}

BurgnerScsiStage *
burgner_scsi_wait_unit_ready_stage_new (void)
{
	BurgnerScsiWaitUnitReadyStage *self;

	self = g_object_new (BURGNER_TYPE_SCSI_WAIT_UNIT_READY_STAGE, NULL);


	return BURGNER_SCSI_STAGE (self);
}


GQuark
burgner_scsi_wait_unit_ready_stage_error_quark (void)
{
	return g_quark_from_string ("BurgnerScsiWaitUnitReadyStage");
}

