/*
 * BurgnerScsiLinuxScsi3Device - BurgnerScsiLinuxScsi3Device object
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "../../burgner-scsi-i18n.h"

#include <string.h>
#include <errno.h>
#include <scsi/sg.h>
#include <sys/ioctl.h>
#include <glib-object.h>
#include "burgner-scsi-linux-scsi-3-device.h"


static BurgnerScsiLinuxDeviceClass *parent_class = NULL;


static int
_burgner_scsi_linux_scsi_3_device_direction_to_scsi3_dir (BurgnerScsiDirection direction)
{
	switch (direction) {
		case BURGNER_SCSI_DIRECTION_TO_DEVICE:
			return SG_DXFER_TO_DEV;
		case BURGNER_SCSI_DIRECTION_FROM_DEVICE:
			return SG_DXFER_FROM_DEV;
		case BURGNER_SCSI_DIRECTION_NONE:
			return SG_DXFER_NONE;
		default:
			break;
	}

	g_return_val_if_reached (SG_DXFER_TO_FROM_DEV);
}


static gboolean
_burgner_scsi_linux_scsi_3_device_send_cmd_real (BurgnerScsiLinuxDevice *device,
						 gint fd,
						 gpointer cdb, guint cdb_size,
						 gpointer sense, guint sense_size, guint *sense_used,
						 gpointer data, guint data_size, guint *data_used,
						 BurgnerScsiDirection direction,
						 guint timeout,
						 gint *status,
						 GError **error)
{
	sg_io_hdr_t cmd;

	memset (&cmd, 0, sizeof (sg_io_hdr_t));

	cmd.interface_id = 'S';
	cmd.dxfer_direction = _burgner_scsi_linux_scsi_3_device_direction_to_scsi3_dir (direction);
	cmd.cmdp = cdb;
	cmd.cmd_len = cdb_size;
	cmd.sbp = sense;
	cmd.mx_sb_len = sense_size;
	cmd.dxferp = data;
	cmd.dxfer_len = data_size;

	/*
	 * When a command timed out, the ide-scsi driver often crashes...
	 */
	cmd.timeout = G_MAXUINT;

	cmd.flags = SG_FLAG_DIRECT_IO;
	cmd.pack_id = 0;
	cmd.usr_ptr = NULL;
	cmd.iovec_count = 0;

	if (ioctl (fd, SG_IO, &cmd) != 0) {
		g_set_error (error,
			     BURGNER_SCSI_DEVICE_ERROR,
			     BURGNER_SCSI_DEVICE_ERROR_SENDING_CMD,
			     _("Error while sending command: %s"),
			     		strerror (errno));
		return FALSE;
	}

	if (sense_used)
		*sense_used = cmd.sb_len_wr;
	if (data_used)
		*data_used = data_size - cmd.resid;
	if (status)
		*status = cmd.masked_status << 1;

	return TRUE;
}


static void
_burgner_scsi_linux_scsi_3_device_class_init (BurgnerScsiLinuxScsi3DeviceClass *klass)
{
	BurgnerScsiLinuxDeviceClass *device_class;

	device_class = (BurgnerScsiLinuxDeviceClass *) klass;
	device_class->send_cmd_real = _burgner_scsi_linux_scsi_3_device_send_cmd_real;

	parent_class = g_type_class_peek_parent (klass);
}


GTypeInfo burgner_scsi_linux_scsi_3_device_info = {
	sizeof (BurgnerScsiLinuxScsi3DeviceClass),
	NULL,
	NULL,
	(GClassInitFunc) _burgner_scsi_linux_scsi_3_device_class_init,
	NULL,
	NULL,
	sizeof (BurgnerScsiLinuxScsi3Device),
	0,
	NULL,
};

GType
burgner_scsi_linux_scsi_3_device_get_type (void)
{
	return g_type_from_name ("BurgnerScsiLinuxScsi3Device");
}


BurgnerScsiDevice *
burgner_scsi_linux_scsi_3_device_new (void)
{
	BurgnerScsiLinuxScsi3Device *self;

	self = g_object_new (BURGNER_TYPE_SCSI_LINUX_SCSI_3_DEVICE, NULL);
	

	return BURGNER_SCSI_DEVICE (self);
}


GQuark
burgner_scsi_linux_scsi_3_device_error_quark (void)
{
	return g_quark_from_string ("BurgnerScsiLinuxScsi3Device");
}

