/*
 * BurgnerScsiLinuxScsi3Driver - BurgnerScsiLinuxScsi3Driver object
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "../../burgner-scsi-i18n.h"

#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <scsi/sg.h>

#include <glib-object.h>
#include "burgner-scsi-linux-scsi-3-device.h"
#include "burgner-scsi-linux-scsi-3-driver.h"


#define DRIVER_ID	"linux-scsi-3"


static BurgnerScsiLinuxDriverClass *parent_class = NULL;


#if 0
static gchar *
_burgner_scsi_linux_scsi_3_driver_get_file_name_from_uri (BurgnerScsiLinuxDriver *driver,
							  const GUri *uri,
							  GError **error)
{
	g_return_val_if_fail (uri != NULL, NULL);

	if (!g_uri_get_scheme (uri) ||
	    g_ascii_strcasecmp (g_uri_get_scheme (uri), "file") != 0) {
		g_set_error (error,
			     BURGNER_SCSI_DRIVER_ERROR,
			     BURGNER_SCSI_DRIVER_ERROR_UNSUPPORTED_URI_SCHEME,
			     _("Unsupported URI scheme: '%s'"),
			             uri->scheme);
		return NULL;
	}

	return g_strdup (g_uri_get_path (uri));
}
#endif

static gint
_burgner_scsi_linux_scsi_3_driver_open_device_real (BurgnerScsiLinuxDriver *driver,
						    const gchar *file,
						    gint requested_dma_size,
						    gint *real_dma_size,
						    GError **error)
{
	struct stat stat_buf;
	int fd, s;

	memset (&stat_buf, 0, sizeof (stat_buf));
	if (stat (file, &stat_buf) != 0) {
		g_set_error (error,
			     BURGNER_SCSI_DRIVER_ERROR,
			     BURGNER_SCSI_DRIVER_ERROR_BAD_FILE,
			     _("Error while getting informations about %s: %s"),
			     		file,
			     		strerror (errno));
		return -1;
	}

	if (!S_ISCHR (stat_buf.st_mode)) {
		g_set_error (error,
			     BURGNER_SCSI_DRIVER_ERROR,
			     BURGNER_SCSI_DRIVER_ERROR_BAD_FILE,
			     _("%s is not a character device"),
			     file);
		return -1;
	}

	if ((fd = open (file, O_RDWR | O_NONBLOCK)) < 0) {
		g_set_error (error,
			     BURGNER_SCSI_DRIVER_ERROR,
			     BURGNER_SCSI_DRIVER_ERROR_OPENING_DEVICE,
			     _("Error while opening %s: %s"),
			     		file,
			     		strerror (errno));
		return -1;
	}

	s = requested_dma_size >= 0 ? requested_dma_size :
#ifdef SG_BIG_BUFF
		SG_BIG_BUFF
#else
		4 * 1024  /*
			   * Value found in
			   * cdrtools-2.0/libscg/scsi-linux-sg.c
			   */
#endif
		;
	if (ioctl (fd, SG_SET_RESERVED_SIZE, &s) != 0) {
		g_set_error (error,
			     BURGNER_SCSI_DRIVER_ERROR,
			     BURGNER_SCSI_DRIVER_ERROR_SETTING_DMA_SIZE,
			     _("Error while setting DMA size to %u B: %s"),
			     		s, strerror (errno));
		close (fd);
		return -1;
	}

	s = 0;
	if (ioctl (fd, SG_GET_RESERVED_SIZE, &s) != 0) {
		g_set_error (error,
			     BURGNER_SCSI_DRIVER_ERROR,
			     BURGNER_SCSI_DRIVER_ERROR_GETTING_DMA_SIZE,
			     _("Error while getting DMA size: %s"),
			     		strerror (errno));
		close (fd);
		return -1;
	}

	if (real_dma_size)
		*real_dma_size = s;

	return fd;
}

static GList *
_burgner_scsi_linux_scsi_3_driver_get_uri_list_real (BurgnerScsiLinuxDriver *driver,
						     GError **error)
{
	struct stat stat_buf;
	GList *l = NULL;
	GString *s;
	gint i;

	s = g_string_sized_new (50);

	for (i = 0; i < 58; i++) {
		GUri *uri;

		if (i < 32) {
			g_string_printf (s, "/dev/sg%d", i);
		} else {
			g_string_printf (s, "/dev/sg%c", 'a' + i - 32);
		}

		if (stat (s->str, &stat_buf) != 0)
			continue;

		g_string_prepend (s, "file://" DRIVER_ID);
		uri = g_uri_new ();
		g_uri_set (uri, s->str, FALSE, FALSE, NULL);
		l = g_list_prepend (l, uri);
	}

	g_string_free (s, TRUE);
	return g_list_reverse (l);
}

static BurgnerScsiDevice *
_burgner_scsi_linux_scsi_3_driver_create_device (BurgnerScsiLinuxDriver *driver)
{
	return burgner_scsi_linux_scsi_3_device_new ();
}


static void
_burgner_scsi_linux_scsi_3_driver_class_init (BurgnerScsiLinuxScsi3DriverClass *klass)
{
	BurgnerScsiLinuxDriverClass *driver_class;

	driver_class = (BurgnerScsiLinuxDriverClass *) klass;
	driver_class->open_device_real = _burgner_scsi_linux_scsi_3_driver_open_device_real;
	driver_class->get_uri_list_real = _burgner_scsi_linux_scsi_3_driver_get_uri_list_real;
	driver_class->create_device = _burgner_scsi_linux_scsi_3_driver_create_device;

	parent_class = g_type_class_peek_parent (klass);
}

GTypeInfo burgner_scsi_linux_scsi_3_driver_info = {
	sizeof (BurgnerScsiLinuxScsi3DriverClass),
	NULL,
	NULL,
	(GClassInitFunc) _burgner_scsi_linux_scsi_3_driver_class_init,
	NULL,
	NULL,
	sizeof (BurgnerScsiLinuxScsi3Driver),
	0,
	NULL,
};

GType
burgner_scsi_linux_scsi_3_driver_get_type (void)
{
	return g_type_from_name ("BurgnerScsiLinuxScsi3Driver");
}

BurgnerScsiDriver *
burgner_scsi_linux_scsi_3_driver_new (void)
{
	BurgnerScsiLinuxScsi3Driver *self;

	self = g_object_new (BURGNER_TYPE_SCSI_LINUX_SCSI_3_DRIVER, NULL);


	return BURGNER_SCSI_DRIVER (self);
}


GQuark
burgner_scsi_linux_scsi_3_driver_error_quark (void)
{
	return g_quark_from_string ("BurgnerScsiLinuxScsi3Driver");
}

