/*
 * BurgnerFifoFilter - BurgnerFifoFilter object
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib-object.h>
#include "burgner-fifo-filter.h"


static GObjectClass *parent_class = NULL;

static void
_burgner_fifo_filter_class_init (BurgnerFifoFilterClass *klass)
{
	parent_class = g_type_class_peek_parent (klass);
}

GType
burgner_fifo_filter_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		static GTypeInfo info = {
			sizeof (BurgnerFifoFilterClass),
			NULL,
			NULL,
			(GClassInitFunc) _burgner_fifo_filter_class_init,
			NULL,
			NULL,
			sizeof (BurgnerFifoFilter),
			0,
			NULL,
		};

		type = g_type_register_static (G_TYPE_OBJECT, "BurgnerFifoFilter", &info, 0);
	}

	return type;
}


GQuark
burgner_fifo_filter_error_quark (void)
{
	return g_quark_from_string ("BurgnerFifoFilter");
}

void
burgner_fifo_filter_begin (BurgnerFifoFilter *filter)
{
	g_return_if_fail (filter != NULL);
	g_return_if_fail (BURGNER_IS_FIFO_FILTER (filter));

	if (BURGNER_FIFO_FILTER_GET_CLASS (filter)->begin)
		BURGNER_FIFO_FILTER_GET_CLASS (filter)->begin (filter);
}

void
burgner_fifo_filter_filter (BurgnerFifoFilter *filter, gpointer buf, guint32 buf_size)
{
	g_return_if_fail (filter != NULL);
	g_return_if_fail (BURGNER_IS_FIFO_FILTER (filter));

	if (BURGNER_FIFO_FILTER_GET_CLASS (filter)->filter)
		BURGNER_FIFO_FILTER_GET_CLASS (filter)->filter (filter, buf, buf_size);
}

void
burgner_fifo_filter_end (BurgnerFifoFilter *filter)
{
	g_return_if_fail (filter != NULL);
	g_return_if_fail (BURGNER_IS_FIFO_FILTER (filter));

	if (BURGNER_FIFO_FILTER_GET_CLASS (filter)->end)
		BURGNER_FIFO_FILTER_GET_CLASS (filter)->end (filter);
}

