/*
 * BurgnerFifoReader - BurgnerFifoReader iface
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "burgner-fifo-reader.h"


GType
burgner_fifo_reader_get_type (void)
{
	static GType type = 0;

	if (type == 0) {
		GTypeInfo info = {
			sizeof (BurgnerFifoReaderIface),
			(GBaseInitFunc) NULL,
			(GBaseFinalizeFunc) NULL,
			
		};

		type = g_type_register_static (G_TYPE_INTERFACE, "BurgnerFifoReader", &info, 0);
	}

	return type;
}

void
burgner_fifo_reader_to_read (BurgnerFifoReader *reader,
		   BurgnerFifo *fifo,
		   const guchar *to_read,
		   guint size)
{
	BurgnerFifoReaderIface *iface;

	g_return_if_fail (reader != NULL);
	g_return_if_fail (BURGNER_IS_FIFO_READER (reader));
	g_return_if_fail (fifo != NULL);
	g_return_if_fail (BURGNER_IS_FIFO (fifo));

	iface = BURGNER_FIFO_READER_GET_IFACE (reader);
	g_return_if_fail (iface != NULL);

	if (iface->to_read)
		iface->to_read (reader, fifo, to_read, size);
}

void
burgner_fifo_reader_end_reached (BurgnerFifoReader *reader,
		     BurgnerFifo *fifo)
{
	BurgnerFifoReaderIface *iface;

	g_return_if_fail (reader != NULL);
	g_return_if_fail (BURGNER_IS_FIFO_READER (reader));
	g_return_if_fail (fifo != NULL);
	g_return_if_fail (BURGNER_IS_FIFO (fifo));

	iface = BURGNER_FIFO_READER_GET_IFACE (reader);
	g_return_if_fail (iface != NULL);

	if (iface->end_reached)
		iface->end_reached (reader, fifo);
}

