/*
 * BurgnerFifo - BurgnerFifo object
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifndef __BURGNER_FIFO_H__
#define __BURGNER_FIFO_H__

#include <glib-object.h>
#include <libburgnerutils/burgner-fifo-filter.h>

G_BEGIN_DECLS


#define BURGNER_TYPE_FIFO		(burgner_fifo_get_type ())
#define BURGNER_FIFO(o)			(G_TYPE_CHECK_INSTANCE_CAST ((o), BURGNER_TYPE_FIFO, BurgnerFifo))
#define BURGNER_FIFO_CLASS(k)		(G_TYPE_CHECK_CLASS_CAST ((k), BURGNER_TYPE_FIFO, BurgnerFifoClass))
#define BURGNER_IS_FIFO(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), BURGNER_TYPE_FIFO))
#define BURGNER_IS_FIFO_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), BURGNER_TYPE_FIFO))
#define BURGNER_FIFO_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), BURGNER_TYPE_FIFO, BurgnerFifoClass))


typedef struct _BurgnerFifo BurgnerFifo;
typedef struct _BurgnerFifoClass BurgnerFifoClass;

typedef struct _BurgnerFifoReader BurgnerFifoReader;
typedef struct _BurgnerFifoWriter BurgnerFifoWriter;

struct _BurgnerFifo {
	GObject __parent;

	BurgnerFifoReader *reader;
	BurgnerFifoWriter *writer;

	BurgnerFifoFilter *filter;

	guchar end_reached : 1;
};

struct _BurgnerFifoClass {
	GObjectClass __parent_class;

	/*
	 * Virtual Functions
	 */
	guint (* to_read)(BurgnerFifo *fifo, guchar **buf);
	void (* read)(BurgnerFifo *fifo, guint size);
	gboolean (* end_reached)(BurgnerFifo *fifo);

	guint (* to_write)(BurgnerFifo *fifo, guchar **buf);
	void (* wrote)(BurgnerFifo *fifo, guint size);
	void (* end)(BurgnerFifo *fifo);

	void (* reset)(BurgnerFifo *fifo);

	void (* set_data_block_size)(BurgnerFifo *fifo, guint size);

	/* FIXME: rename this fucntion */
	guint (* get_percent)(BurgnerFifo *fifo);

	void (* wake_up)(BurgnerFifo *fifo);
};


GType burgner_fifo_get_type (void)
#if __GNUC__ <= 2
	G_GNUC_CONST
#endif
	;
GQuark  burgner_fifo_error_quark (void);

#define BURGNER_FIFO_ERROR		(burgner_fifo_error_quark ())


guint burgner_fifo_to_read (BurgnerFifo *fifo, guchar **buf);
void burgner_fifo_read (BurgnerFifo *fifo, guint size);
gboolean burgner_fifo_end_reached (BurgnerFifo *fifo);

guint burgner_fifo_to_write (BurgnerFifo *fifo, guchar **buf);
void burgner_fifo_wrote (BurgnerFifo *fifo, guint size);
void burgner_fifo_end (BurgnerFifo *fifo);

void burgner_fifo_reset (BurgnerFifo *fifo);

void burgner_fifo_set_data_block_size (BurgnerFifo *fifo, guint size);

guint burgner_fifo_get_percent (BurgnerFifo *fifo);

void burgner_fifo_do_filter (BurgnerFifo *fifo, gpointer buf, guint32 size);

void burgner_fifo_wake_up (BurgnerFifo *fifo);

G_END_DECLS

#endif /* __BURGNER_FIFO_H__ */
