/*
 * 
 *
 * This file is part of the libburgnerutils library.
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "burgner-math.h"


guint
burgner_math_get_nearest_sum (GList *list, guint total,
			      BurgnerMathGetNumFunc get_num_func,
			      gpointer get_num_func_data,
			      GList **sum_list)
{
	GList *it = list, *best_list = NULL;
	guint best_total = 0, num = 0;

	g_return_val_if_fail (get_num_func != NULL, 0);
	g_return_val_if_fail (total > 0, 0);

	while (it && (num = get_num_func (it->data, get_num_func_data)) > total) {
		it = it->next;
	}

	if (!it) {
		if (sum_list)
			*sum_list = NULL;
		return 0;
	}

	if (num == total) {
		if (sum_list)
			*sum_list = g_list_prepend (NULL, it->data);
		return num;
	}

	for (; it != NULL; it = it->next) {
		GList *l = NULL;
		guint nt, s;

		num = get_num_func (it->data, get_num_func_data);
		nt = total - num;
		s = burgner_math_get_nearest_sum (it->next, nt, get_num_func, get_num_func_data, &l);

		if ((s + num) > best_total) {
			if (best_list)
				g_list_free (best_list);

			if (s == nt) {
				if (sum_list) {
					*sum_list = g_list_prepend (l, it->data);
				} else {
					g_list_free (l);
				}

				return total;
			} else {
				best_list = g_list_prepend (l, it->data);
				best_total = s + num;
			}
		}

		while (it->next && get_num_func (it->next->data, get_num_func_data) == num) {
			it = it->next;
		}
	}

	if (sum_list) {
		*sum_list = best_list;
	} else if (best_list) {
		g_list_free (best_list);
	}

	return best_total;
}

typedef struct {
	BurgnerMathGetNumFunc func;
	gpointer data;
} _BurgnerMathGetNearestSumSortListData;

static gint
_burgner_math_get_nearest_sum_sort_list_sort_func (gconstpointer a, gconstpointer b,
						   gpointer user_data)
{
	_BurgnerMathGetNearestSumSortListData *d = (_BurgnerMathGetNearestSumSortListData *) user_data;
	guint ai = d->func ((gpointer) a, d->data), bi = d->func ((gpointer) b, d->data);

	if (ai > bi)
		return -1;

	return (ai != bi);
}

GList *
burgner_math_get_nearest_sum_sort_list (GList *list,
					BurgnerMathGetNumFunc get_num_func,
					gpointer get_num_func_data)
{
	_BurgnerMathGetNearestSumSortListData d;
	d.func = get_num_func;
	d.data = get_num_func_data;
	return g_list_sort_with_data (list, _burgner_math_get_nearest_sum_sort_list_sort_func, &d);
}

