/*
 * BurgnerModule - BurgnerModule object
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifndef __BURGNER_MODULE_H__
#define __BURGNER_MODULE_H__

#include <glib-object.h>
#include <gmodule.h>

G_BEGIN_DECLS


#define BURGNER_TYPE_MODULE		(burgner_module_get_type ())
#define BURGNER_MODULE(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), BURGNER_TYPE_MODULE, BurgnerModule))
#define BURGNER_MODULE_CLASS(k)		(G_TYPE_CHECK_CLASS_CAST ((k), BURGNER_TYPE_MODULE, BurgnerModuleClass))
#define BURGNER_IS_MODULE(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), BURGNER_TYPE_MODULE))
#define BURGNER_IS_MODULE_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), BURGNER_TYPE_MODULE))
#define BURGNER_MODULE_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), BURGNER_TYPE_MODULE, BurgnerModuleClass))


typedef struct _BurgnerModule BurgnerModule;
typedef struct _BurgnerModuleClass BurgnerModuleClass;

struct _BurgnerModule {
	GObject __parent;

	/*< private >*/
	GList *type_list;
	GList *iface_list;

	GHashTable *type_to_info_sym;
	GHashTable *iface_to_info_sym;

	gchar *module_path;
	GModule *module;

	guint use_count;
};

struct _BurgnerModuleClass {
	GObjectClass __parent_class;

	/*
	 * Virtual Functions
	 */
	void (* add_type)(BurgnerModule *module,
			   gchar *name,
			   GType parent_type,
			   gchar *info_symbol);
	void (* add_interface)(BurgnerModule *module,
				GType instance_type,
				GType interface_type,
				gchar *info_symbol);
	void (* register_types_ifaces)(BurgnerModule *module);

	/*
	 * Signals
	 */
	void (* load)(BurgnerModule *module,
		      gchar *module_path,
		      GModule *gmodule,
		      GError *err);
	void (* unload)(BurgnerModule *module,
			gchar *module_path,
			GError *err);
};


GType burgner_module_get_type (void) G_GNUC_CONST;
GQuark  burgner_module_error_quark (void);

#define BURGNER_MODULE_ERROR		(burgner_module_error_quark ())
typedef enum {
	BURGNER_MODULE_ERROR_G_MODULE_ERROR,
	BURGNER_MODULE_ERROR_CANT_GET_TYPE_INFO,
	BURGNER_MODULE_ERROR_CANT_GET_INTERFACE_INFO
} BurgnerModuleError;


BurgnerModule *burgner_module_new (void);

void burgner_module_use (BurgnerModule *module);
void burgner_module_unuse (BurgnerModule *module);


void burgner_module_add_type (BurgnerModule *module,
		   	      gchar *name,
			      GType parent_type,
			      gchar *info_symbol);
void burgner_module_add_interface (BurgnerModule *module,
				   GType instance_type,
				   GType interface_type,
				   gchar *info_symbol);
void burgner_module_register_types_ifaces (BurgnerModule *module);

/* Routines for emitting signals on the BurgnerModule */
void burgner_module_load (BurgnerModule *module,
			  gchar *module_path,
			  GModule *gmodule,
			  GError *err);
void burgner_module_unload (BurgnerModule *module,
			    gchar *module_path,
			    GError *err);

G_END_DECLS

#endif /* __BURGNER_MODULE_H__ */
