/*
 * BurgnerRingFifo - BurgnerRingFifo object
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifndef __BURGNER_RING_FIFO_H__
#define __BURGNER_RING_FIFO_H__

#include <glib-object.h>
#include <libburgnerutils/burgner-fifo.h>

G_BEGIN_DECLS


#define BURGNER_TYPE_RING_FIFO		(burgner_ring_fifo_get_type ())
#define BURGNER_RING_FIFO(o)			(G_TYPE_CHECK_INSTANCE_CAST ((o), BURGNER_TYPE_RING_FIFO, BurgnerRingFifo))
#define BURGNER_RING_FIFO_CLASS(k)		(G_TYPE_CHECK_CLASS_CAST ((k), BURGNER_TYPE_RING_FIFO, BurgnerRingFifoClass))
#define BURGNER_IS_RING_FIFO(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), BURGNER_TYPE_RING_FIFO))
#define BURGNER_IS_RING_FIFO_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), BURGNER_TYPE_RING_FIFO))
#define BURGNER_RING_FIFO_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), BURGNER_TYPE_RING_FIFO, BurgnerRingFifoClass))


typedef struct _BurgnerRingFifo BurgnerRingFifo;
typedef struct _BurgnerRingFifoClass BurgnerRingFifoClass;

struct _BurgnerRingFifo {
	BurgnerFifo __parent;

	guchar *buf;
	guchar *buf_end;
	guchar *real_buf_end;

	guchar *to_read;
	guchar *to_write;

	guint read_chunk_size;
	guint real_read_chunk_size;
	guint write_chunk_size;
	guint real_write_chunk_size;

	guint data_block_size;

	guint reading;
	guint writing;

	guint idle_id;

	enum {
		TO_WRITE,
		TO_READ
	} equal_case : 1;
};

struct _BurgnerRingFifoClass {
	BurgnerFifoClass __parent_class;

};


GType burgner_ring_fifo_get_type (void)
#if __GNUC__ <= 2
	G_GNUC_CONST
#endif
	;
GQuark  burgner_ring_fifo_error_quark (void);

#define BURGNER_RING_FIFO_ERROR		(burgner_ring_fifo_error_quark ())


BurgnerFifo *burgner_ring_fifo_new (void);

G_END_DECLS

#endif /* __BURGNER_RING_FIFO_H__ */

