/*
 * BurgnerThread - BurgnerThread object
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifndef __BURGNER_THREAD_H__
#define __BURGNER_THREAD_H__

#include <glib-object.h>

G_BEGIN_DECLS


#define BURGNER_TYPE_THREAD		(burgner_thread_get_type ())
#define BURGNER_THREAD(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), BURGNER_TYPE_THREAD, BurgnerThread))
#define BURGNER_THREAD_CLASS(k)		(G_TYPE_CHECK_CLASS_CAST ((k), BURGNER_TYPE_THREAD, BurgnerThreadClass))
#define BURGNER_IS_THREAD(o)		(G_TYPE_CHECK_INSTANCE_TYPE ((o), BURGNER_TYPE_THREAD))
#define BURGNER_IS_THREAD_CLASS(k)	(G_TYPE_CHECK_CLASS_TYPE ((k), BURGNER_TYPE_THREAD))
#define BURGNER_THREAD_GET_CLASS(o)	(G_TYPE_INSTANCE_GET_CLASS ((o), BURGNER_TYPE_THREAD, BurgnerThreadClass))


typedef struct _BurgnerThread BurgnerThread;
typedef struct _BurgnerThreadClass BurgnerThreadClass;

typedef struct _BurgnerThreadCall _BurgnerThreadCall;

typedef gpointer BurgnerThreadCallId;

struct _BurgnerThread {
	GObject __parent;

	GThread *thread;

	gint wpipe_fd;
	gint rpipe_fd;

	GIOChannel *pipe_channel;
	guint pipe_watch_id;

	_BurgnerThreadCall *actual_call;
	GList *call_list;

	guint thread_busy : 1;
};

struct _BurgnerThreadClass {
	GObjectClass __parent_class;

};

typedef gboolean (* BurgnerThreadFunction)(gpointer function_data);
typedef gboolean (* BurgnerThreadCallback)(BurgnerThread *thread,
					   BurgnerThreadCallId id,
					   BurgnerThreadFunction function,
					   gpointer function_data,
					   gpointer user_data);

GType burgner_thread_get_type (void) G_GNUC_CONST;
GQuark  burgner_thread_error_quark (void) G_GNUC_CONST;

#define BURGNER_THREAD_ERROR		(burgner_thread_error_quark ())
typedef enum {
	BURGNER_THREAD_ERROR_CREATING_PIPE
} BurgnerThreadError;


BurgnerThread *burgner_thread_new (GError **error);

BurgnerThreadCallId burgner_thread_call (BurgnerThread *thread,
			  BurgnerThreadFunction function,
			  gpointer function_data,
			  BurgnerThreadCallback callback,
			  gpointer callback_data);


G_END_DECLS

#endif /* __BURGNER_THREAD_H__ */

