/*
 * 
 *
 * This file is part of the libburgnerutils library.
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "burgner-utils-i18n.h"

#include <string.h>

#include "burgner-wav-impl.h"
#include "burgner-wav.h"

/* 
 * This function have been highly inspired by
 * cdrtools-2.0/cdrecord/audiosize.c's wavsize
 */
gboolean
burgner_wav_setup_channel (GIOChannel *channel,
			   guint32 stat_size,
			   guint *channels,
			   guint *sample_rate,
			   guint *bits_per_sample,
			   guint32 *data_size,
			   guint32 *data_offset,
			   GError **error)
{
	_BurgnerWavImplHeader header;
	guint32 offset;
	gboolean got_format;
	guint32 size;

	g_return_val_if_fail (channel != NULL, FALSE);
	g_return_val_if_fail (stat_size >= 4, FALSE);

	offset = 0;
	got_format = FALSE;

	while (1) {
		gsize bytes_read = 0;
		if (g_io_channel_read_chars (channel, (gchar *) &header, 8, &bytes_read, error) != G_IO_STATUS_NORMAL || bytes_read != 8) {
			return FALSE;
		}

		size = GUINT32_FROM_LE (*((guint32 *) header.size));

		if (strncmp (header.id, "RIFF", 4) == 0) {
			_BurgnerWavImplRiffData riff_data;

#if 0
			if (size != 4) {
				g_set_error (error,
					     BURGNER_WAV_ERROR,
					     BURGNER_WAV_ERROR_BAD_HEADER,
					     _("Bad RIFF header"));
				return FALSE;
			}
#endif

			bytes_read = 0;
			if (g_io_channel_read_chars (channel, (gchar *) &riff_data, sizeof (riff_data), &bytes_read, error) != G_IO_STATUS_NORMAL || bytes_read != sizeof (riff_data)) {
				return FALSE;
			}

			if (strncmp (riff_data.wave, "WAVE", 4) != 0) {
				g_set_error (error,
					     BURGNER_WAV_ERROR,
					     BURGNER_WAV_ERROR_BAD_HEADER,
					     _("Bad \"RIFF\" header"));
				return FALSE;
			}

			offset += 12;
		} else if (strncmp (header.id, "fmt ", 4) == 0) {
			_BurgnerWavImplFmtData fmt_data;

			if (size != 16) {
				g_set_error (error,
					     BURGNER_WAV_ERROR,
					     BURGNER_WAV_ERROR_BAD_HEADER,
					     _("Bad \"fmt \" header"));
				return FALSE;
			}

			bytes_read = 0;
			if (g_io_channel_read_chars (channel, (gchar *) &fmt_data, sizeof (fmt_data), &bytes_read, error) != G_IO_STATUS_NORMAL || bytes_read != sizeof (fmt_data)) {
				return FALSE;
			}

			if (channels)
				*channels = GUINT16_FROM_LE (*((guint16 *) fmt_data.channels));
			if (sample_rate)
				*sample_rate = GUINT32_FROM_LE (*((guint32 *) fmt_data.sample_rate));
			if (bits_per_sample)
				*bits_per_sample = GUINT16_FROM_LE (*((guint16 *) fmt_data.bits_per_sample));

			got_format = TRUE;
			offset += 16;
		} else if (strncmp (header.id, "data", 4) == 0) {
			if (!got_format) {
				g_set_error (error,
					     BURGNER_WAV_ERROR,
					     BURGNER_WAV_ERROR_BAD_HEADER,
					     _("Bad header"));
				return FALSE;
			}

			offset += 8;

			if (data_size) {
				if ((offset + size) > stat_size)
					size = stat_size - offset;

				*data_size = size;
			}

			if (data_offset)
				*data_offset = offset;

			return TRUE;
		} else {
			if (g_io_channel_seek_position (channel, size, G_SEEK_CUR, error) != G_IO_STATUS_NORMAL)
				return FALSE;
			offset += size + 8;
		}
	}

	g_set_error (error,
		     BURGNER_WAV_ERROR,
		     BURGNER_WAV_ERROR_BAD_HEADER,
		     _("Incomplete WAV header"));

	return FALSE;
}

GQuark
burgner_wav_error_quark (void)
{
	return g_quark_from_string ("BurgnerWav");
}

