/*
 * GUri Path Parser
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <string.h>
#include <glib.h>
#include "g-uri-path-parser.h"


GList *
_g_uri_path_parser_static_parse_path (gchar *path,
				      gboolean include_last_segment,
				      gboolean *absolute)
{
	GList *l = NULL;
	gchar *s, *it = path;

	if (*it == '/') {
		if (absolute)
			*absolute = TRUE;
		++it;
	} else {
		if (absolute)
			*absolute = FALSE;
	}

	while ((s = strchr (it, '/'))) {
		*(s++) = '\0';
		l = g_list_prepend (l, it);
		it = s;
	}

#if 0
	if (include_last_segment && *it)
#else
	if (include_last_segment)
#endif
		l = g_list_prepend (l, it);

	return g_list_reverse (l);
}

GList *
_g_uri_path_parser_dynamic_parse_path (gchar *path,
					   gboolean include_last_segment,
					   gboolean *absolute)
{
	GList *l = NULL;
	gchar *s, *it = path;

	if (*it == '/') {
		if (absolute)
			*absolute = TRUE;
		++it;
	}

	while ((s = strchr (it, '/'))) {
		l = g_list_prepend (l, (gpointer) g_strndup (it, s - it));
		it = s + 1;
	}

	if (include_last_segment)
		l = g_list_prepend (l, (gpointer) g_strdup (it));

	return g_list_reverse (l);
}


void
_g_uri_path_static_to_dynamic (GList *path)
{
	for (; path != NULL; path = path->next)
		path->data = g_strdup (path->data);
}


GList *
_g_uri_path_parser_static_reduce (GList *path)
{
	GList *it = path, *start = path, *tr, *tr2;
	guint level = 0;

	while (it != NULL) {
		if (strcmp ((gchar *) it->data, ".") != 0 &&
		    strcmp ((gchar *) it->data, "..") != 0) {
			++level;
			it = it->next;
			continue;
		}

		if (level == 0) {
			start = it->next;
			it = start;
			continue;
		}

		if (((gchar *) it->data)[1] == 0) { /* '.' */
			tr = it;

			if ((it = it->next)) {
				start = g_list_remove_link (start, tr);
				g_list_free_1 (tr);
			} else {
				((gchar *) tr->data)[0] = 0;
			}

			continue;
		}

		/* '..' */
		tr = it->prev;
		tr2 = it;
		start = g_list_remove_link (start, tr);
		g_list_free_1 (tr);
		if ((it = it->next)) {
			start = g_list_remove_link (start, tr2);
			g_list_free_1 (tr2);
		} else {
			((gchar *) tr2->data)[0] = 0;
		}

		level -= 1;
	}

	return g_list_first (start);
}

gchar *
_g_uri_path_format (GList *path, gboolean absolute)
{
	GString *s = g_string_new (NULL);
	gchar *p;

	if (absolute)
		g_string_append_c (s, '/');

	for (; path != NULL; path = path->next) {
		g_string_append (s, (gchar *) path->data);
		if (path->next)
			g_string_append_c (s, '/');
	}

	p = s->str;
	g_string_free (s, FALSE);

	return p;
}

