/*
 * GUri Resolver
 *
 * Copyright (c) 2001 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib.h>
#include "g-uri.h"
#include "g-uri-path-parser.h"
#include "g-uri-resolver.h"

gboolean
_g_uri_relative_resolver_step_2 (GUri *e, const GUri *base)
{
	if (g_uri_is_void (e))
		return TRUE;

	return FALSE;
}

gboolean
_g_uri_relative_resolver_step_3 (GUri *e, const GUri *base)
{
	if (e->scheme)
		return TRUE;

	if (base->scheme)
		e->scheme = g_strdup (base->scheme);

	return FALSE;
}

	
gboolean
_g_uri_relative_resolver_step_4 (GUri *e, const GUri *base)
{
	if (e->authority)
		return TRUE;

	if (base->authority)
		e->authority = g_strdup (base->authority);

	return FALSE;
}

gboolean
_g_uri_relative_resolver_step_5 (GUri *e, const GUri *base)
{
	if (e->path && *(e->path) == '/')
		return TRUE;

	return FALSE;
}

gboolean
_g_uri_relative_resolver_step_6 (GUri *e, const GUri *base)
{
	GList *base_p, *e_p;
	gchar *bps, *eps, *path;
	gboolean absolute = FALSE;

	if (!e->path) {
		e->path = g_strdup (base->path);
		e->path_is_absolute = base->path_is_absolute;
	} else {
		bps = g_strdup (base->path);
		base_p = _g_uri_path_parser_static_parse_path (bps, FALSE, &absolute);

		eps = g_strdup (e->path);
		e_p = _g_uri_path_parser_static_parse_path (eps, TRUE, NULL);

		base_p = g_list_concat (base_p, e_p);
		base_p = _g_uri_path_parser_static_reduce (base_p);

		path = _g_uri_path_format (base_p, absolute);

		g_list_free (base_p);
		g_free (bps);
		g_free (eps);

		g_free (e->path);
		e->path = path;
		e->path_is_absolute = absolute;
	}

	return TRUE;
}

