/*
 * 
 *
 * This file is part of the libburgnerutils library.
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */

#include <string.h>
#include "g-uri-char-utils.h"
#include "g-uri-string.h"

guint8
_g_uri_hex_char_to_uint (gchar c)
{
	if (IS_DIGIT (c))
		return c - '0';
	if (c >= 'a' && c <= 'f')
		return c - 'a' + 10;
	if (c >= 'A' && c <= 'F')
		return c - 'A' + 10;

	g_return_val_if_reached (0);
	return 0;
}

gchar *
g_uri_string_compress (const gchar *source)
{
	const gchar *p = source;
	gchar *dest, *d;

	g_return_val_if_fail (source != NULL, NULL);

	dest = g_malloc (strlen (source) + 1);
	d = dest;

	while (*p) {
		if (*p == '%' && p[1] && p[2] && IS_HEX (p[1]) && IS_HEX (p[2])) {
			*(d++) = (_g_uri_hex_char_to_uint (p[1]) << 4) +
				 _g_uri_hex_char_to_uint (p[2]);
			p += 2;
		} else {
			*(d++) = *p;
		}
		p++;
	}

	*d = 0;

	return dest;
}

