/*
 * GUri - GUri struct
 *
 * Copyright (c) 2001 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */


#ifndef __G_URI_H__
#define __G_URI_H__

#include <glib.h>

G_BEGIN_DECLS


typedef enum {
	G_URI_FORMAT_OPTIONS_NONE
} GUriFormatOptions;


typedef struct _GUri GUri;
struct _GUri {
	gchar *scheme;

	gchar *authority;
	struct {
		gchar *userinfo;
		gchar *host;
		gchar *port;
	} parsed_authority;

	gchar *path;

	gchar *query;
	gchar *fragment;

	gint refcount;

	guchar has_parsed_auth : 1;
	guchar path_is_absolute : 1;
};

#define G_URI_ERROR	(g_uri_error_quark ())
GQuark g_uri_error_quark (void);
typedef enum {
	G_URI_ERROR_INVALID_FRAGMENT,
	G_URI_ERROR_INVALID_QUERY,
	G_URI_ERROR_INCOMPLETE_ESCAPED_CHAR,
	G_URI_ERROR_INVALID_ESCAPED_CHAR
} GUriError;


GUri *g_uri_new (void);
void g_uri_destroy (GUri *self);

GUri *g_uri_ref (GUri *self);
void g_uri_unref (GUri *self);


GUri *g_uri_clone (const GUri *uri);

gboolean g_uri_is_void (const GUri *uri);

gchar *g_uri_format (const GUri *uri, GUriFormatOptions options);
gboolean g_uri_set (GUri *uri, gchar *str, gboolean validate, gboolean parse_auth, GError **error);

void g_uri_set_scheme (GUri *uri, const gchar *scheme);
void g_uri_set_authority (GUri *uri, const gchar *authority);
void g_uri_set_path (GUri *uri, const gchar *path);
void g_uri_set_query (GUri *uri, const gchar *query);
void g_uri_set_fragment (GUri *uri, const gchar *fragment);

const gchar *g_uri_get_scheme (const GUri *uri);
const gchar *g_uri_get_authority (const GUri *uri);
const gchar *g_uri_get_path (const GUri *uri);
const gchar *g_uri_get_query (const GUri *uri);
const gchar *g_uri_get_fragment (const GUri *uri);

gboolean g_uri_parse_authority (GUri *uri, GError **error);


gboolean g_uri_relative_resolve (GUri **uri, const GUri *r, const GUri *base, GError **err);



G_END_DECLS

#endif /* __G_URI_H__ */

