/*
 * burgner-cdrecord 
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "../libburgnermmc/burgner-mmc-i18n.h"

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <glib-object.h>
#include <libgnomevfs/gnome-vfs.h>

#include <popt.h>


#include "../libburgnermmc/burgner-mmc-impl.h"
#include "../libburgnerscsi/burgner-scsi-mode-parameter-list.h"
#include "../libburgnerscsi/burgner-scsi-prevent-allow-medium-removal.h"

#include "../libburgnerscsi/burgner-scsi-inquiry.h"
#include "../libburgnerscsi/burgner-scsi-inquiry-stage.h"
#include "../libburgnermmc/burgner-mmc-get-capabilities-page-stage.h"
#include "../libburgnerscsi/burgner-scsi-start-unit-stage.h"
#include "../libburgnerscsi/burgner-scsi-wait-unit-ready-stage.h"
#include "../libburgnerscsi/burgner-scsi-prevent-allow-medium-removal-stage.h"
#include "../libburgnerscsi/burgner-scsi-wait-fifo-full-stage.h"
#include "../libburgnermmc/burgner-mmc-read-disc-information-stage.h"
#include "../libburgnermmc/burgner-mmc-read-track-information-stage.h"
#include "../libburgnermmc/burgner-mmc-setup-session-stage.h"
#include "../libburgnermmc/burgner-mmc-track-writer.h"
#include "../libburgnermmc/burgner-mmc-close-session-stage.h"
#include "../libburgnerscsi/burgner-scsi-stop-unit-stage.h"
#include "../libburgnerscsi/burgner-scsi-stage-manager-callback.h"

#include "../libburgnerscsi/burgner-scsi-driver-manager.h"
#include "../libburgnerscsi/burgner-scsi-driver.h"
#include "../libburgnerscsi/burgner-scsi-driver-info.h"
#include "../libburgnerscsi/burgner-scsi-device.h"

#include "../libburgnerutils/burgner-fifo.h"
#include "../libburgnerutils/burgner-fifo-fd-writer.h"
#include "../libburgnerutils/burgner-fifo-gnome-vfs-writer.h"

#include "../libburgnerscsi/burgner-scsi-init.h"
#include "../libburgnerscsi/burgner-scsi-utils.h"
#include "../libburgnermmc/burgner-mmc-utils.h"
#include "../libburgnerutils/burgner-wav.h"
#include "../libburgnerutils/burgner-fifo-big-endian-filter.h"


#include "burgner-cdrecord-error.h"


static const char *argv_0 = NULL;

static GMainLoop *main_loop = NULL;
static int ret_val = 0;

static gboolean cdrecord_comp = FALSE;

static gchar *device_uri = NULL;
static gchar *blank_type = NULL;
static gchar *data_track = NULL;
static gchar *audio_wav_track = NULL;
static gchar *session = NULL;

static gboolean eject = FALSE;
#if 0
static gboolean list_devices = FALSE;
#endif
static gboolean read_buf_cap = FALSE;
static gboolean reserve = FALSE;
static gboolean test = FALSE;
static gboolean swap_audio = FALSE;

static gboolean version = FALSE;

static gint speed = 0;
static gint buffer_size = 4*1024*1024;

static enum {
	_VERBOSITY_QUIET,
	_VERBOSITY_NORMAL,
	_VERBOSITY_VERBOSE
} verbosity = _VERBOSITY_NORMAL;

static enum {
	_SESSION_ALLOW_NEXT,
	_SESSION_LAST,
	_SESSION_NO_INFO,
	_SESSION_DO_NOT_CLOSE
} session_closure = _SESSION_ALLOW_NEXT;

static inline BurgnerMmcSessionClosure
_get_session_closure (void)
{
	BurgnerMmcSessionClosure r = BURGNER_MMC_SESSION_CLOSURE_NEXT_SESSION_ALLOWED;

	switch (session_closure) {
		case _SESSION_ALLOW_NEXT:
			r = BURGNER_MMC_SESSION_CLOSURE_NEXT_SESSION_ALLOWED;
			break;
		case _SESSION_LAST:
			r = BURGNER_MMC_SESSION_CLOSURE_NO_NEXT_SESSION;
			break;
		case _SESSION_NO_INFO:
			r = BURGNER_MMC_SESSION_CLOSURE_NO_B0_POINTER;
			break;
		case _SESSION_DO_NOT_CLOSE:
			r = BURGNER_MMC_SESSION_CLOSURE_NEXT_SESSION_ALLOWED;
			break;
		default:
			g_assert_not_reached ();
	}

	return r;
}


static enum {
	_BLANK_NONE = 0,
	_BLANK_COMPLETE,
	_BLANK_MINIMAL,
	_BLANK_SESSION,
	_BLANK_UNCLOSE_SESSION
} blank = _BLANK_NONE;


enum {
	_STAGE_ID_NONE = -1,
	_STAGE_ID_INQUIRY,
	_STAGE_ID_GET_CAPABILITIES_PAGE,
	_STAGE_ID_PREVENT_MEDIUM_REMOVAL,
	_STAGE_ID_START_UNIT,
	_STAGE_ID_WAIT_UNIT_READY,
	_STAGE_ID_BLANK,
	_STAGE_ID_READ_DISC_INFORMATION,
	_STAGE_ID_READ_TRACK_INFORMATION,
	_STAGE_ID_SETUP_SESSION,
	_STAGE_ID_WAIT_FIFO_FULL,
	_STAGE_ID_TRACK_WRITER,
	_STAGE_ID_CLOSE_SESSION,
	_STAGE_ID_ALLOW_MEDIUM_REMOVAL,
	_STAGE_ID_STOP_UNIT
};


static gchar **directories = NULL;
static gint directories_i = -1;

static GUri *device_guri = NULL;

static BurgnerScsiDriverManager *driver_manager = NULL;
static BurgnerScsiDriver *driver = NULL;
static BurgnerScsiDevice *device = NULL;

static BurgnerScsiStage *smc = NULL;


static gulong start_stage_id = 0;

static void
_burgner_cdrecord_track_writer_start_stage_signal_cb (BurgnerScsiStageManager *stage_manager,
						      BurgnerScsiStage *stage,
						      gint stage_id,
						      gpointer user_data)
{
	switch (stage_id) {
		case BURGNER_MMC_TRACK_WRITER_STAGE_ID_RESERVE_TRACK:
			if (verbosity == _VERBOSITY_VERBOSE)
				fprintf (stderr, _("\nReserving track ...\n"));
			break;
		case BURGNER_MMC_TRACK_WRITER_STAGE_ID_CLOSE_TRACK:
			if (verbosity == _VERBOSITY_VERBOSE)
				fprintf (stderr, _("\nClosing track ...\n"));
			break;
		default:
			break;
	}
}

#define _CDB_BUF_SIZE	(12)
static guchar *cdb_buf[_CDB_BUF_SIZE];

#define _SENSE_BUF_SIZE	(18)
static guchar *sense_buf[_SENSE_BUF_SIZE];

#define _DATA_BUF_SIZE	(60)
static guchar *data_buf[_DATA_BUF_SIZE];


static BurgnerFifo *fifo = NULL;
static guint32 start_lba = 0;
static guint8 track_num = 0xff;

static BurgnerFifoWriter *fifo_writer;
static BurgnerFifoFilter *big_endian_filter = NULL;

static gboolean cdrw_write = FALSE;

static guint32 track_size = 0;


typedef struct {
	gchar *file;

	guint32 size;
	guint32 offset;

	BurgnerMmcDataBlockType block_type;
	BurgnerMmcTrackMode track_mode;
} _BurgnerCdrecordTrack;

static GList *track_list = NULL;
static GList *next_track = NULL;
static _BurgnerCdrecordTrack *actual_track = NULL;


static void _burgner_cdrecord_track_option_cb (poptContext ctx,
				   enum poptCallbackReason reason,
				   const struct poptOption *opt,
				   const char *arg, void *data);

static struct poptOption track_options_table[] = {
	{ NULL, '\0', POPT_ARG_CALLBACK, _burgner_cdrecord_track_option_cb, 0, NULL, NULL },

	{ "data", '\0', POPT_ARG_STRING, &data_track, 0,
	  N_("Specify a data image to burn as a data track"), N_("data-file") },
	{ "audio-wav", '\0', POPT_ARG_STRING, &audio_wav_track, 0,
	  N_("Specify a wav file to burn as an audio track"), N_("wav-file") },

	POPT_TABLEEND
};


static struct poptOption options_table[] = {
	{ "device", 'd', POPT_ARG_STRING, &device_uri, 0,
	  N_("Specify the device to use"), N_("device_uri") },

#if 0
	{ "blank", '\0', POPT_ARG_STRING, &blank_type, 0,
	  N_("Blank the disc or parts of the disc"),
	  N_("complete/minimal/session/unclose_session") },
#endif

	{ NULL, '\0', POPT_ARG_INCLUDE_TABLE, track_options_table, 0,
	  N_("Options to add a track to write"), NULL },

	{ "session", '\0', POPT_ARG_STRING, &session, 0,
	  N_("Specify the session behavior"),
	  N_("[allow_next]/last/no_info/do_not_close") },

	{ "eject", '\0', POPT_ARG_NONE, &eject, 0,
	  N_("Eject the disc after the operations"), NULL },

	{ "speed", '\0', POPT_ARG_INT | POPT_ARGFLAG_SHOW_DEFAULT,
	  &speed, 0, N_("Specify write speed (in x, 0 for maximum speed)"),
	  N_("n") },

	{ "reserve", '\0', POPT_ARG_NONE, &reserve, 0,
	  N_("Reserve track before writing"), NULL },

	{ "test", '\0', POPT_ARG_NONE, &test, 0,
	  N_("Write track in test mode"), NULL },

#if 0
	{ "list-devices", '\0', POPT_ARG_NONE, &list_devices, 0,
	  N_("List available devices"), NULL },
#endif

	{ "buffer-size", '\0', POPT_ARG_INT | POPT_ARGFLAG_SHOW_DEFAULT,
	  &buffer_size, 0, N_("Specify internal buffer size, in bytes"),
	  N_("size") },

	{ "dont-read-device-buffer-state", '\0', POPT_ARG_VAL,
	  &read_buf_cap, FALSE, N_("Do not read device internal buffer state while writing"), NULL },

	{ "read-device-buffer-state", '\0', POPT_ARG_VAL,
	  &read_buf_cap, TRUE, N_("Read device internal buffer size while writing"), NULL },

	{ "swap-audio", '\0', POPT_ARG_NONE, &swap_audio, 0,
	  N_("Swap audio bytes"), NULL },

	{ "verbose", 'v', POPT_ARG_VAL, &verbosity, _VERBOSITY_VERBOSE,
	  N_("Be verbose"), NULL },

	{ "quiet", '\0', POPT_ARG_VAL, &verbosity, _VERBOSITY_QUIET,
	  N_("Be quiet"), NULL },

	{ "version", '\0', POPT_ARG_NONE, &version, 0,
	  N_("Print version and exit"), NULL },

	POPT_AUTOHELP
	POPT_TABLEEND
};

static struct poptOption cdrecord_options_table[] = {
	{ "dev", 'd', POPT_ARG_STRING | POPT_ARGFLAG_ONEDASH, &device_uri, 0,
	  N_("Specify the device to use"), N_("device_uri") },

#if 0
	{ "blank", '\0', POPT_ARG_STRING | POPT_ARGFLAG_ONEDASH, &blank_type, 0,
	  N_("Blank the disc or parts of the disc"),
	  N_("all/minimal/session/unclose") },
#endif

	{ "speed", '\0',
	  POPT_ARG_INT | POPT_ARGFLAG_SHOW_DEFAULT | POPT_ARGFLAG_ONEDASH,
	  &speed, 0, N_("Specify write speed (in x, 0 for maximum speed)"),
	  N_("n") },

	{ "data", '\0', POPT_ARG_STRING | POPT_ARGFLAG_ONEDASH, &data_track, 0,
	  N_("Specify the data image to burn"), N_("file") },

#if 0
	{ "scan-bus", '\0', POPT_ARG_NONE | POPT_ARGFLAG_ONEDASH,
	  &list_devices, 0, N_("List available devices"), NULL },
#endif

	{ "dummy", '\0', POPT_ARG_NONE, &test, 0,
	  N_("Write track in test mode"), NULL },

#if 0
	{ "fix", '\0', POPT_ARG_VAL, &session, TRUE,
	  N_("Close the session after burning"), NULL },

	{ "nofix", '\0', POPT_ARG_VAL, &session, FALSE,
	  N_("Do not close the session after burning"), NULL },
#endif

	{ "fs", '\0', POPT_ARG_INT | POPT_ARGFLAG_ONEDASH, &buffer_size, 0,
	  N_("Specify internal buffer (fifo) size, in bytes"), N_("size") },

	POPT_AUTOHELP
	POPT_TABLEEND
};


static void
_burgner_cdrecord_track_option_cb (poptContext ctx,
				   enum poptCallbackReason reason,
				   const struct poptOption *opt,
				   const char *arg, void *data)
{
	_BurgnerCdrecordTrack *t = NULL;

	if (reason != POPT_CALLBACK_REASON_OPTION)
		return;

	if (!arg || !*arg) {
		poptPrintUsage (ctx, stderr, 0);
		fprintf (stderr, _("You must specify a track to write after a --data or --audio-wav argument\n"));
		exit (EXIT_FAILURE);
	}

	if (opt == &track_options_table[1]) {
		struct stat stat_buf;

		memset (&stat_buf, 0, sizeof (stat_buf));
		if (stat (arg, &stat_buf) != 0) {
			fprintf (stderr, _("Unable to get informations about %s: %s\n"),
					arg, strerror (errno));
			exit (EXIT_FAILURE);
		}

		t = g_new0 (_BurgnerCdrecordTrack, 1);

		t->file = g_strdup (arg);
		t->size = stat_buf.st_size;
		t->block_type = BURGNER_MMC_DATA_BLOCK_TYPE_MODE_1;
		t->track_mode = BURGNER_MMC_TRACK_MODE_DATA_TRACK_UNINTERRUPTED;
	} else if (opt == &track_options_table[2]) {
		GIOChannel *channel;
		GIOFlags f;
		GError *error = NULL;
		struct stat stat_buf;
		guint channels, sample_rate, bits_per_sample;

		memset (&stat_buf, 0, sizeof (stat_buf));
		if (stat (arg, &stat_buf) != 0) {
			fprintf (stderr, _("Unable to get informations about %s: %s\n"),
					arg, strerror (errno));
			exit (EXIT_FAILURE);
		}

		if (!(channel = g_io_channel_new_file (arg, "r", &error))) {
			fprintf (stderr, _("Error while opening %s: %s\n"),
					arg, error && error->message ? error->message : "");
			exit (EXIT_FAILURE);
		}

		if ((f = g_io_channel_get_flags (channel)) & G_IO_FLAG_NONBLOCK)
			g_io_channel_set_flags (channel, f & ~G_IO_FLAG_NONBLOCK, NULL);

		g_io_channel_set_encoding (channel, NULL, NULL);
		g_io_channel_set_buffered (channel, FALSE);

		t = g_new0 (_BurgnerCdrecordTrack, 1);

		if (!burgner_wav_setup_channel (channel, stat_buf.st_size, &channels,
						&sample_rate, &bits_per_sample,
						&t->size, &t->offset, &error)) {
			fprintf (stderr, _("%s: %s\n"), arg, error && error->message ? error->message : "");
			exit (EXIT_FAILURE);
		}

		g_io_channel_unref (channel);

		if (channels != 2 || sample_rate != 44100 || bits_per_sample != 16) {
			fprintf (stderr, _("%s has not two channels, a sample rate of 44100 Hz, and 16 bits/sample.\n"), arg);
			exit (EXIT_FAILURE);
		}

		t->file = g_strdup (arg);
		t->block_type = BURGNER_MMC_DATA_BLOCK_TYPE_RAW_DATA;
		t->track_mode = BURGNER_MMC_TRACK_MODE_2_AUDIO_CHANNELS_WITHOUT_PRE_EMPHASIS;
	} else {
		g_warning ("Unknown option: %s %c",
				opt->longName ? opt->longName : "",
				opt->shortName);
	}

	if (t)
		track_list = g_list_prepend (track_list, t);
}


static void
_close_device_cb (BurgnerScsiDriver *drv,
		  BurgnerScsiDriverOpId op_id,
		  BurgnerScsiDevice *dev,
		  GError *err,
		  gpointer user_data)
{
	if (err) {
		fprintf (stderr, _("Error while closing device: %s\n"),
				err && err->message ? err->message : _("[No description]"));
	}

	g_main_loop_quit (main_loop);
}

static void
_allow_medium_removal_cb (BurgnerScsiDevice *device,
			  gpointer cdb_buf, guint cdb_buf_size,
			  gpointer sense_buf, guint sense_buf_size, guint sense_buf_used,
			  gpointer data_buf, guint data_buf_size, guint data_buf_used,
			  BurgnerScsiDirection direction, guint timeout, gint status,
			  GError *error,
			  gpointer user_data)
{
	if (verbosity == _VERBOSITY_VERBOSE)
		fprintf (stderr, _("Closing device ...\n"));

	burgner_scsi_driver_close_device (driver, device,
					  _close_device_cb, NULL);
}

static gboolean
_quit_idle_func (gpointer user_data)
{
	if (device) {
		if (verbosity == _VERBOSITY_VERBOSE)
			fprintf (stderr, _("Allowing medium removal ...\n"));

		burgner_scsi_prevent_allow_medium_removal_write_cdb (cdb_buf, _CDB_BUF_SIZE, FALSE);
		burgner_scsi_device_send_cmd (device,
					      cdb_buf, BURGNER_SCSI_PREVENT_ALLOW_MEDIUM_REMOVAL_CDB_SIZE,
					      sense_buf, _SENSE_BUF_SIZE,
					      NULL, 0,
					      BURGNER_SCSI_DIRECTION_NONE,
					      40000,
					      _allow_medium_removal_cb, NULL);
	} else {
		g_main_loop_quit (main_loop);
	}

	return FALSE;
}

static gboolean _idle_func (gpointer user_data);

static void
_load_driver_info_list_async_done_cb (BurgnerScsiDriverManager *dm,
				      BurgnerScsiDriverManagerLoadDriverInfoListAsyncId id,
				      gchar *directory,
				      GList *driver_info_list,
				      GError *err,
				      gpointer user_data)
{
	if (err) {
		if (verbosity != _VERBOSITY_QUIET) {
			fprintf (stderr, _("Error while driving drivers from %s: %s\n"),
						directory,
						err->message ? err->message : _("[No description]"));
		}
	} else if (verbosity == _VERBOSITY_VERBOSE) {
		GList *it;
		for (it = driver_info_list; it != NULL; it = it->next) {
			BurgnerScsiDriverInfo *info = (BurgnerScsiDriverInfo *) it->data;

			fprintf (stderr, _("Found driver %s in directory %s\n"),
						info->id, directory);
		}
	}

	directories_i++;
	g_idle_add (_idle_func, NULL);
}

static guint32 blocks_written = 0;
static guint32 buf_blank_len = 0;
static guint32 buf_len = 0;

static void
update_line (void)
{
	guint p = track_size ? (blocks_written * 100 / (track_size / burgner_mmc_data_block_type_get_data_block_size (actual_track->block_type))) : 0;
	guint f = burgner_fifo_get_percent (fifo);
	guint h = buf_len ? (100 - (100 * buf_blank_len /buf_len)) : 0;

	fprintf (stderr, _("\rWrote %2.2u%% [%c%c%c%c%c%c%c%c%c%c] soft: %2.2u%% hard: %2.2u%%"),
			p,
			(int) (p >= 9 ? '#' : ' '),
			(int) (p >= 18 ? '#' : ' '),
			(int) (p >= 27 ? '#' : ' '),
			(int) (p >= 36 ? '#' : ' '),
			(int) (p >= 45 ? '#' : ' '),
			(int) (p >= 54 ? '#' : ' '),
			(int) (p >= 63 ? '#' : ' '),
			(int) (p >= 72 ? '#' : ' '),
			(int) (p >= 81 ? '#' : ' '),
			(int) (p >= 91 ? '#' : ' '),
			f, h);
}


static void
_progress_cb (BurgnerMmcTrackWriter *track_writer,
	      guint32 total_written_blocks,
	      guint32 track_size,
	      gpointer user_data)
{
	blocks_written = total_written_blocks;
	update_line ();
}

static void
_buffer_capacity_cb (BurgnerMmcTrackWriter *track_writer,
		     guint32 blank_length,
		     guint32 total_length,
		     GError *error,
		     guint sense_used,
		     gpointer user_data)
{
	if (error || sense_used) {
		fprintf (stderr, _("Reading of driver internal buffer capacity failed, not reading it any more\n"));
		if (sense_used) {
			BurgnerScsiSense *s;
			if (burgner_scsi_sense_new_from_buf (&s,
							     (guchar *) sense_buf,
							     sense_used,
							     NULL)) {
				gchar *d = burgner_scsi_sense_get_detailed_description_string (s);
				fprintf (stderr, _("Sense: %s\n"), d);
				g_free (d);
				burgner_scsi_sense_free (s);
			}
		}

		read_buf_cap = FALSE;
		g_object_set (BURGNER_SCSI_STAGE_MANAGER (smc)->stage,
			      "read_buf_cap", FALSE,
			      NULL);
		buf_len = buf_blank_len = 0;
		update_line ();
	}

	buf_blank_len = blank_length;
	buf_len = total_length;
	update_line ();
}


static BurgnerScsiStage *
_get_next_stage_func (BurgnerScsiStageManagerCallback *stage_mc,
		      gint *stage_id,
		      BurgnerScsiStage *actual_stage,
		      gint actual_stage_id,
		      gpointer user_data)
{
	BurgnerScsiStage *stage = NULL;

	g_assert (stage_id);

	switch (actual_stage_id) {
		case _STAGE_ID_NONE:
			stage = burgner_scsi_inquiry_stage_new ();
			g_object_set (stage,
				      "device", device,
				      "cdb_buf", cdb_buf,
				      "cdb_buf_size", _CDB_BUF_SIZE,
				      "sense_buf", sense_buf,
				      "sense_buf_size", _SENSE_BUF_SIZE,
				      "data_buf", data_buf,
				      "data_buf_size", BURGNER_SCSI_INQUIRY_DATA_SIZE,
				      NULL);
			*stage_id = _STAGE_ID_INQUIRY;
			break;
		case _STAGE_ID_INQUIRY:
			stage = burgner_mmc_get_capabilities_page_stage_new ();
			g_object_set (stage,
				      "device", device,
				      "cdb_buf", cdb_buf,
				      "cdb_buf_size", _CDB_BUF_SIZE,
				      "sense_buf", sense_buf,
				      "sense_buf_size", _SENSE_BUF_SIZE,
				      "data_buf", data_buf,
				      "data_buf_size", _DATA_BUF_SIZE,
				      NULL);
			*stage_id = _STAGE_ID_GET_CAPABILITIES_PAGE;
			break;
		case _STAGE_ID_GET_CAPABILITIES_PAGE:
			if (verbosity == _VERBOSITY_VERBOSE)
				fprintf (stderr, _("Preventing medium  removal ...\n"));

			stage = burgner_scsi_prevent_allow_medium_removal_stage_new ();
			g_object_set (stage,
				      "device", device,
				      "cdb_buf", cdb_buf,
				      "cdb_buf_size", _CDB_BUF_SIZE,
				      "sense_buf", sense_buf,
				      "sense_buf_size", _SENSE_BUF_SIZE,
				      "prevent", TRUE,
				      NULL);
			*stage_id = _STAGE_ID_PREVENT_MEDIUM_REMOVAL;
			break;
		case _STAGE_ID_PREVENT_MEDIUM_REMOVAL:
			if (verbosity == _VERBOSITY_VERBOSE)
				fprintf (stderr, _("Starting unit ...\n"));

			stage = burgner_scsi_start_unit_stage_new ();
			g_object_set (stage,
				      "device", device,
				      "cdb_buf", cdb_buf,
				      "cdb_buf_size", _CDB_BUF_SIZE,
				      "sense_buf", sense_buf,
				      "sense_buf_size", _SENSE_BUF_SIZE,
				      "immed", FALSE,
				      "load", TRUE,
				      NULL);
			*stage_id = _STAGE_ID_START_UNIT;
			break;
		case _STAGE_ID_START_UNIT:
			if (verbosity == _VERBOSITY_VERBOSE)
				fprintf (stderr, _("Waiting unit ready ...\n"));

			stage = burgner_scsi_wait_unit_ready_stage_new ();
			g_object_set (stage,
				      "device", device,
				      "cdb_buf", cdb_buf,
				      "cdb_buf_size", _CDB_BUF_SIZE,
				      "sense_buf", sense_buf,
				      "sense_buf_size", _SENSE_BUF_SIZE,
				      "interval_time", 1000,
				      "time", 60000,
				      NULL);
			*stage_id = _STAGE_ID_WAIT_UNIT_READY;
			break;
		case _STAGE_ID_WAIT_UNIT_READY:
#if 0
			if (blank != _BLANK_NONE) {
				stage = burgner_mmc_blank_stage_new ();
				/* TODO */
				*stage_id = _STAGE_ID_BLANK;
			} else {
#endif
			if (verbosity == _VERBOSITY_VERBOSE)
				fprintf (stderr, _("Reading disc information ...\n"));

				stage = burgner_mmc_read_disc_information_stage_new ();
				g_object_set (stage,
					      "device", device,
					      "cdb_buf", cdb_buf,
					      "cdb_buf_size", _CDB_BUF_SIZE,
					      "sense_buf", sense_buf,
					      "sense_buf_size", _SENSE_BUF_SIZE,
					      "data_buf", data_buf,
					      "data_buf_size", _DATA_BUF_SIZE,
					      NULL);
				*stage_id = _STAGE_ID_READ_DISC_INFORMATION;
#if 0
			}
#endif
			break;
		case _STAGE_ID_BLANK:
#if 0
			if (fifo_writer) {
				stage = burgner_mmc_read_disc_information_stage_new ();
				g_object_set (stage,
					      "device", device,
					      "cdb_buf", cdb_buf,
					      "cdb_buf_size", _CDB_BUF_SIZE,
					      "sense_buf", sense_buf,
					      "sense_buf_size", _SENSE_BUF_SIZE,
					      "data_buf", data_buf,
					      "data_buf_size", _DATA_BUF_SIZE,
					      NULL);
				*stage_id = _STAGE_ID_READ_DISC_INFORMATION;
			} else {
				stage = burgner_scsi_prevent_allow_medium_removal_stage_new ();
				g_object_set (stage,
					      "device", device,
					      "cdb_buf", cdb_buf,
					      "cdb_buf_size", _CDB_BUF_SIZE,
					      "sense_buf", sense_buf,
					      "sense_buf_size", _SENSE_BUF_SIZE,
					      "prevent", FALSE,
					      NULL);
				*stage_id = _STAGE_ID_ALLOW_MEDIUM_REMOVAL;
			}
#endif
			break;
		case _STAGE_ID_READ_DISC_INFORMATION:
			if (verbosity == _VERBOSITY_VERBOSE)
				fprintf (stderr, _("Setting up session ...\n"));

			stage = burgner_mmc_setup_session_stage_new ();
			g_object_set (stage,
				      "device", device,
				      "cdb_buf", cdb_buf,
				      "cdb_buf_size", _CDB_BUF_SIZE,
				      "sense_buf", sense_buf,
				      "sense_buf_size", _SENSE_BUF_SIZE,
				      "data_buf", data_buf,
				      "data_buf_size", _DATA_BUF_SIZE,
				      "closure", _get_session_closure (),
				      "format", BURGNER_MMC_SESSION_FORMAT_CD_DA_OR_CD_ROM,
				      NULL);
			*stage_id = _STAGE_ID_SETUP_SESSION;
			break;
		case _STAGE_ID_SETUP_SESSION:
			if (verbosity == _VERBOSITY_VERBOSE)
				fprintf (stderr, _("Reading track information ...\n"));

			stage = burgner_mmc_read_track_information_stage_new ();
			g_object_set (stage,
				      "device", device,
				      "cdb_buf", cdb_buf,
				      "cdb_buf_size", _CDB_BUF_SIZE,
				      "sense_buf", sense_buf,
				      "sense_buf_size", _SENSE_BUF_SIZE,
				      "data_buf", data_buf,
				      "data_buf_size", _DATA_BUF_SIZE,
				      "track_num", 0xff,
				      NULL);
			*stage_id = _STAGE_ID_READ_TRACK_INFORMATION;
			break;
		case _STAGE_ID_READ_TRACK_INFORMATION:
			g_assert (fifo);
			g_assert (fifo_writer);

			if (verbosity == _VERBOSITY_VERBOSE)
				fprintf (stderr, _("Waiting fifo to be full ...\n"));

			stage = burgner_scsi_wait_fifo_full_stage_new ();
			g_object_set (stage,
					"fifo", fifo,
					NULL);
			*stage_id = _STAGE_ID_WAIT_FIFO_FULL;
			break;
		case _STAGE_ID_WAIT_FIFO_FULL:
			g_assert (fifo);
			g_assert (fifo_writer);
			g_assert (actual_track);

			if (verbosity == _VERBOSITY_VERBOSE)
				fprintf (stderr, _("Writing track %u at lba %" G_GUINT32_FORMAT " (%s from %s) ...\n"),
							track_num,
							start_lba,
							actual_track->block_type == BURGNER_MMC_DATA_BLOCK_TYPE_MODE_1 ? _("data") : _("audio"),
							actual_track->file);

			stage = burgner_mmc_track_writer_new ();
			g_object_set (stage,
				      "device", device,
				      "cdb_buf", cdb_buf,
				      "cdb_buf_size", _CDB_BUF_SIZE,
				      "sense_buf", sense_buf,
				      "sense_buf_size", _SENSE_BUF_SIZE,
				      "data_buf", data_buf,
				      "data_buf_size", _DATA_BUF_SIZE,
				      "fifo", fifo,
				      "data_block_type", actual_track->block_type,
				      "track_mode", actual_track->track_mode,
				      "start_lba", start_lba,
				      "read_buf_cap", read_buf_cap,
				      "progress_cb", _progress_cb,
				      "buffer_capacity_cb", _buffer_capacity_cb,
#if 0
				      "track_num", track_num,
#else
				      "track_num", 0xff,
#endif
				      "max_write_between_read_buf_cap", 100,
				      "min_time_between_read_buf_cap", 1000,
				      "track_size", actual_track->size,
				      "reserve", reserve,
				      "test", test,
				      NULL);
			*stage_id = _STAGE_ID_TRACK_WRITER;

			start_stage_id = g_signal_connect_data (stage, "start_stage",
								(GCallback) _burgner_cdrecord_track_writer_start_stage_signal_cb, NULL,
								NULL, 0);
			break;
		case _STAGE_ID_TRACK_WRITER:
			if (!next_track) {
				if (verbosity == _VERBOSITY_VERBOSE)
					fprintf (stderr, _("\nClosing session ...\n"));

				stage = burgner_mmc_close_session_stage_new ();
				g_object_set (stage,
					      "device", device,
					      "cdb_buf", cdb_buf,
					      "cdb_buf_size", _CDB_BUF_SIZE,
					      "sense_buf", sense_buf,
					      "sense_buf_size", _SENSE_BUF_SIZE,
					      "immed", FALSE,
					      NULL);
				*stage_id = _STAGE_ID_CLOSE_SESSION;
			} else {
				if (verbosity == _VERBOSITY_VERBOSE)
					fprintf (stderr, _("Reading track information ...\n"));

				stage = burgner_mmc_read_track_information_stage_new ();
				g_object_set (stage,
					      "device", device,
					      "cdb_buf", cdb_buf,
					      "cdb_buf_size", _CDB_BUF_SIZE,
					      "sense_buf", sense_buf,
					      "sense_buf_size", _SENSE_BUF_SIZE,
					      "data_buf", data_buf,
					      "data_buf_size", _DATA_BUF_SIZE,
#if 0
					      "track_num", track_num + 1,
#else
					      "track_num", 0xff,
#endif
					      NULL);
				*stage_id = _STAGE_ID_READ_TRACK_INFORMATION;
			}
			break;
		case _STAGE_ID_CLOSE_SESSION:
			if (verbosity == _VERBOSITY_VERBOSE)
				fprintf (stderr, _("Allowing medium removal ...\n"));

			stage = burgner_scsi_prevent_allow_medium_removal_stage_new ();
			g_object_set (stage,
				      "device", device,
				      "cdb_buf", cdb_buf,
				      "cdb_buf_size", _CDB_BUF_SIZE,
				      "sense_buf", sense_buf,
				      "sense_buf_size", _SENSE_BUF_SIZE,
				      "prevent", FALSE,
				      NULL);
			*stage_id = _STAGE_ID_ALLOW_MEDIUM_REMOVAL;
			break;
		case _STAGE_ID_ALLOW_MEDIUM_REMOVAL:
			if (verbosity == _VERBOSITY_VERBOSE)
				fprintf (stderr, _("Stopping unit ...\n"));

			stage = burgner_scsi_stop_unit_stage_new ();
			g_object_set (stage,
				      "device", device,
				      "cdb_buf", cdb_buf,
				      "cdb_buf_size", _CDB_BUF_SIZE,
				      "sense_buf", sense_buf,
				      "sense_buf_size", _SENSE_BUF_SIZE,
				      "immed", FALSE,
				      "eject", eject,
				      NULL);
			*stage_id = _STAGE_ID_STOP_UNIT;
			break;
		case _STAGE_ID_STOP_UNIT:
			if (verbosity != _VERBOSITY_QUIET) {
				fprintf (stderr, _("Blanking/burning ended with no errors\n"));
			}

			stage = NULL;
			*stage_id = -1;
			break;
		default:
			g_return_val_if_reached (NULL);
			break;
	}

	return stage;
}

static const gchar *
_mechanism_impl_to_string (guint impl)
{
	switch (impl) {
		case _BURGNER_MMC_IMPL_LOADING_MECHANISM_TYPE_CADDY:
			return _("Caddy");
		case _BURGNER_MMC_IMPL_LOADING_MECHANISM_TYPE_TRAY:
			return _("Tray");
		case _BURGNER_MMC_IMPL_LOADING_MECHANISM_TYPE_POP_UP:
			return _("Pop-up");
		case _BURGNER_MMC_IMPL_LOADING_MECHANISM_TYPE_CHANGER_INDIVIDUALLY:
			return _("Changer, individually changeable");
		case _BURGNER_MMC_IMPL_LOADING_MECHANISM_TYPE_CHANGER_CARTRIDGE:
			return _("Changer, changeable by cartridge");
		default:
			g_return_val_if_reached ("");
	}

	return "";
}

static gboolean
_continue_func (BurgnerScsiStageManagerCallback *stage_mc,
		BurgnerScsiStage *stage,
		gint stage_id,
		GError *stage_error,
		BurgnerScsiSense *sense,
		GError **error,
		gpointer user_data)
{
	if (stage_error || sense) {
		if (stage_error)
			g_propagate_error (error, stage_error);
		return FALSE;
	}

	switch (stage_id) {
		case _STAGE_ID_INQUIRY: {
			BurgnerScsiDeviceType dt = BURGNER_SCSI_DEVICE_TYPE_UNKNOWN;
			gchar *vendor, *product;

			burgner_scsi_inquiry_parse_data (data_buf,
							 BURGNER_SCSI_INQUIRY_DATA_SIZE,
							 &dt,
							 &vendor,
							 &product);

			fprintf (stderr, _("Device (%s) : %s %s\n"),
					burgner_scsi_device_type_to_string (dt),
					vendor ? vendor : "",
					product ? product : "");

			if (dt != BURGNER_SCSI_DEVICE_TYPE_CD_ROM) {
				g_set_error (error,
					     BURGNER_CDRECORD_ERROR,
					     BURGNER_CDRECORD_ERROR_NOT_A_CDROM_DRIVE,
					     _("Device is not a CD-ROM drive.\n"));
				return FALSE;
			}

		}	break;
		case _STAGE_ID_GET_CAPABILITIES_PAGE: {
			_BurgnerMmcImplCdCapabilitiesPage *impl;
			GList *pages = NULL;

			if (!burgner_scsi_mode_parameter_list_parse (TRUE,
								     data_buf,
								     _DATA_BUF_SIZE,
								     NULL,
								     &pages,
								     error)) {
				return FALSE;
			}

			if (!pages || !pages->data ||
			    ((impl = (_BurgnerMmcImplCdCapabilitiesPage *) pages->data)->page_code != BURGNER_MMC_IMPL_CD_CAPABILITIES_PAGE_CODE)) {
				g_set_error (error,
					     g_quark_from_string ("BurgnerCdrecordError"),
					     0,
					     _("Invalid mode parameter list"));
				return FALSE;
			}

			;

			if (verbosity == _VERBOSITY_VERBOSE) {
				fprintf (stderr, _(
				   "Supports Method 2 : %s\n"
				   "CD-R : reading: %s, writing: %s\n"
				   "CD-RW : reading: %s, wrtiting: %s\n"
				   "Supports Test Writing : %s\n"
				   "Supports Multi-Session : %s\n"
				   "Loading Mechanism : %s\n"
				   "Supports Ejecting : %s\n"
				   "Supports Locking : %s\n"
				   "Actually locked : %s\n"
				   "Current Read Speed : %u kBps (%ux)\n"
				   "Maximum Read Speed : %u kBps (%ux)\n"
				   "Current Write Speed : %u kBps (%ux)\n"
				   "Maximum Write Speed : %u kBps (%ux)\n"
				   "Buffer Size : %u KB\n"),
				   	impl->method_2 ? _("yes") : _("no"),
					impl->cdr_read ? _("yes") : _("no"),
					impl->cdr_write ? _("yes") : _("no"),
					impl->cdrw_read ? _("yes") : _("no"),
					impl->cdrw_write ? _("yes") : _("no"),
					impl->test_write ? _("yes") : _("no"),
					impl->multi_session ? _("yes") :_("no"),
					_mechanism_impl_to_string (impl->mechanism_type),
					impl->eject ? _("yes") : _("no"),
					impl->lock ? _("yes") : _("no"),
					impl->lock_state ? _("yes") : _("no"),
					(guint) burgner_scsi_get_uint16 (impl->current_read_speed),
					burgner_mmc_speed_kbps_to_x (burgner_scsi_get_uint16 (impl->current_read_speed)),
					(guint) burgner_scsi_get_uint16 (impl->max_read_speed),
					burgner_mmc_speed_kbps_to_x (burgner_scsi_get_uint16 (impl->max_read_speed)),
					(guint) burgner_scsi_get_uint16 (impl->current_write_speed),
					burgner_mmc_speed_kbps_to_x (burgner_scsi_get_uint16 (impl->current_write_speed)),
					(guint) burgner_scsi_get_uint16 (impl->max_write_speed),
					burgner_mmc_speed_kbps_to_x (burgner_scsi_get_uint16 (impl->max_write_speed)),
					(guint) burgner_scsi_get_uint16 (impl->buffer_size));
			}

			if (!impl->cdr_write) {
				g_set_error (error,
					     BURGNER_CDRECORD_ERROR,
					     BURGNER_CDRECORD_ERROR_NOT_A_CDR_WRITER,
					     _("Device is not able to write CD-R"));
				return FALSE;
			}

			cdrw_write = impl->cdrw_write;
			if (!cdrw_write && blank) {
				g_set_error (error,
					     BURGNER_CDRECORD_ERROR,
					     BURGNER_CDRECORD_ERROR_NOT_A_CDRW_WRITER,
					     _("Device is not able to write CD-RW"));
				return FALSE;
			}

			if (!impl->test_write && test) {
				g_set_error (error,
					     BURGNER_CDRECORD_ERROR,
					     BURGNER_CDRECORD_ERROR_DEVICE_CANT_TEST_WRITE,
					     _("Device is not able to write in test mode"));
				return FALSE;
			}

			if (!impl->eject && eject) {
				g_set_error (error,
					     BURGNER_CDRECORD_ERROR,
					     BURGNER_CDRECORD_ERROR_DEVICE_CANT_EJECT_DISC,
					     _("Device is not able to eject disc"));
				return FALSE;
			}

		}	break;
		case _STAGE_ID_START_UNIT:
		case _STAGE_ID_PREVENT_MEDIUM_REMOVAL:
		case _STAGE_ID_BLANK:
			break;
		case _STAGE_ID_READ_DISC_INFORMATION: {
			_BurgnerMmcImplReadDiscInformationData *impl = (_BurgnerMmcImplReadDiscInformationData *) data_buf;

			if (verbosity == _VERBOSITY_VERBOSE) {
				/* TODO */
			}

			if (impl->disc_status == 2) {
				g_set_error (error,
					     BURGNER_CDRECORD_ERROR,
					     BURGNER_CDRECORD_ERROR_DISC_COMPLETE,
					     _("Disc complete (unappendable)"));
				return FALSE;
			}

			if (impl->disc_type != 0 &&
			    impl->disc_type != 0xff) {
				g_set_error (error,
					     BURGNER_CDRECORD_ERROR,
					     BURGNER_CDRECORD_ERROR_BAD_DISC_TYPE,
					     _("Bad disc type (CD-I or CD-ROM XA)"));
				return FALSE;
			}

			if (impl->erasable && !cdrw_write &&
			    verbosity == _VERBOSITY_VERBOSE) {
				fprintf (stderr, _("The CD seems to be a CD-RW, but the drive is not able to write CD-RW\n"));
			}
		}	break;
		case _STAGE_ID_SETUP_SESSION:
			break;
		case _STAGE_ID_READ_TRACK_INFORMATION: {
			_BurgnerMmcImplReadTrackInformationData *impl = (_BurgnerMmcImplReadTrackInformationData *) data_buf;
			guint32 free_blocks;
			gint fd;

			if (next_track == track_list) {
				g_object_set (fifo_writer,
						"fifo", fifo,
						NULL);
			} else {
				burgner_fifo_reset (fifo);
			}

			track_num = impl->track_num;
			start_lba = impl->nwa_v ? burgner_scsi_get_uint32 (impl->nwa) : 0;

			free_blocks = burgner_scsi_get_uint32 (impl->free_blocks);
			if (verbosity == _VERBOSITY_VERBOSE) {
				fprintf (stderr, _("Available space: %u KB\n"),
							(guint) (free_blocks*2));
			}

			g_assert (next_track && next_track->data);
			actual_track = (_BurgnerCdrecordTrack *) next_track->data;
			next_track = next_track->next;

			track_size = actual_track->size;

			burgner_fifo_set_data_block_size (fifo, burgner_mmc_data_block_type_get_data_block_size (actual_track->block_type));

			if ((fd = open (actual_track->file, O_RDONLY | O_NONBLOCK)) < 0) {
				g_set_error (error,
					     BURGNER_CDRECORD_ERROR,
					     BURGNER_CDRECORD_ERROR_OPENING_FILE,
					     _("opening %s: %s\n"),
					     actual_track->file, strerror (errno));
				return FALSE;
			}

			if (actual_track->offset &&
			    (lseek (fd, actual_track->offset, SEEK_SET) != actual_track->offset)) {
				g_set_error (error,
					     BURGNER_CDRECORD_ERROR,
					     BURGNER_CDRECORD_ERROR_SEEKING_FILE,
					     _("seeking %s to %" G_GUINT32_FORMAT ": %s\n"),
					     actual_track->file,
					     actual_track->offset,
					     strerror (errno));
				return FALSE;
			}

			g_object_set (fifo_writer,
					"fd", fd,
					"size", actual_track->size,
					NULL);

			if (actual_track->track_mode == BURGNER_MMC_TRACK_MODE_2_AUDIO_CHANNELS_WITHOUT_PRE_EMPHASIS &&
			    swap_audio) {
				if (!big_endian_filter) {
					big_endian_filter = burgner_fifo_big_endian_filter_new ();
					g_object_set (big_endian_filter,
							"sample_size", 2,
							"sample_signed", TRUE,
							NULL);
				}

				g_object_set (fifo, "filter", big_endian_filter, NULL);
			} else {
				g_object_set (fifo, "filter", NULL, NULL);
			}

			burgner_fifo_wake_up (fifo);

			if (track_size > (free_blocks << 11)) {
				fprintf (stderr, _("Warning: all data may not fit on disc\n"));
			}
		}	break;
		case _STAGE_ID_TRACK_WRITER: {
			gint fd = -1;
			g_object_get (fifo_writer, "fd", &fd, NULL);
			g_assert (fd >= 0);
			close (fd);

			g_object_set (fifo_writer, "fd", -1, "size", 0, NULL);

			g_object_set (fifo, "reader", NULL, NULL);

			g_signal_handler_disconnect (stage, start_stage_id);
			start_stage_id = 0;
		} break;
		case _STAGE_ID_CLOSE_SESSION:
		case _STAGE_ID_ALLOW_MEDIUM_REMOVAL:
		case _STAGE_ID_STOP_UNIT:
		default:
			break;
	}

	return TRUE;
}

static void
_end_cb (BurgnerScsiStage *scsi_stage,
	 gboolean cancelled,
	 GError *error,
	 BurgnerScsiSense *sense,
	 gpointer user_data)
{
	if (error && error->message)
		fprintf (stderr, _("Error: %s\n"), error->message);
	if (sense) {
		gchar *s = burgner_scsi_sense_get_detailed_description_string (sense);
		fprintf (stderr, _("Sense: %s\n"), s ? s : "");
		g_free (s);
	}

	if (error || sense)
		fprintf (stderr, _("Exiting\n"));

	ret_val = 1;
	g_idle_add (_quit_idle_func, NULL);
}

static void
start_smc (void)
{
	g_assert (smc == NULL);
	smc = burgner_scsi_stage_manager_callback_new ();
	g_assert (smc);
	g_object_set (smc,
		      "get_next_stage_func", _get_next_stage_func,
		      "continue_func", _continue_func,
		      NULL);

	g_signal_connect_data (smc, "end", (GCallback) _end_cb, NULL, NULL, 0);
	burgner_scsi_stage_start (smc, TRUE);
}


static void
_fd_writer_error_cb (BurgnerFifoFdWriter *fd_writer,
		     int errno)
{
	fprintf (stderr, _("Error while reading %s: %s\n"), data_track, strerror (errno));
}

static void
_open_device_cb (BurgnerScsiDriver *drv,
		 BurgnerScsiDriverOpId id,
		 GUri *uri,
		 BurgnerScsiDevice *dev,
		 gint requested_dma_size,
		 gint real_dma_size,
		 GError *error,
		 gpointer user_data)
{
	g_assert (drv == driver);
	g_assert (user_data == NULL);

	if (error) {
		fprintf (stderr, _("%s: can't open %s: %s\n"),
					argv_0, device_uri,
					error->message
						? error->message
						: _("[No description]"));
		exit (1);
	}

	fprintf (stderr, _("DMA size: %d\n"), real_dma_size);

	g_assert (dev);
	device = BURGNER_SCSI_DEVICE (g_object_ref (dev));

		fifo = burgner_scsi_device_get_fifo (device);
		g_object_set (fifo,
			      "size", buffer_size,
			      NULL);

	fifo_writer = burgner_fifo_fd_writer_new ();
	g_signal_connect_data (fifo_writer,
			       "error",
			       (GCallback) _fd_writer_error_cb,
			       NULL, NULL, 0);

	start_smc ();
}

static gboolean
_idle_func (gpointer user_data)
{
	if (directories_i == -1) {
		const gchar *driver_path;

		g_assert (!driver_manager);
		driver_manager = burgner_scsi_driver_manager_new ();
		g_assert (driver_manager);

		g_signal_connect_data (driver_manager,
				       "load_driver_info_list_async_done",
				       (GCallback) _load_driver_info_list_async_done_cb,
				       NULL, NULL, 0);

		if ((driver_path = g_getenv ("BURGNER_SCSI_DRIVER_PATH"))) {
			directories = g_strsplit (driver_path, ":", 0);
		}

		burgner_scsi_driver_manager_load_driver_info_list_async
				(driver_manager,
				 LIBDIR "/burgner-0.1/libburgnerscsi/drivers");
	} else if (directories && directories[directories_i]) {
		burgner_scsi_driver_manager_load_driver_info_list_async
				(driver_manager,
				 directories[directories_i]);
	} else if (!device) {
		device_guri = g_uri_new ();
		if (!g_uri_set (device_guri, device_uri, FALSE, FALSE, NULL)) {
			fprintf (stderr, _("%s: %s: Invalid uri\n"),
							argv_0, device_uri);
			exit (1);
		}

		driver = burgner_scsi_driver_manager_get_driver_from_uri
						(driver_manager, device_guri);
		if (!driver) {
			fprintf (stderr, _("%s: invalid driver\n"),
						device_guri->authority);
			exit (1);
		}

		burgner_scsi_driver_open_device (driver, device_guri,
						 65536,
						 _open_device_cb, NULL);
	} else {
		g_assert_not_reached ();
	}

	return FALSE;
}

int
main (int argc, const char **argv)
{
	poptContext ctx;
	gchar *prgname;
	gint popt_r;

	g_set_prgname (argv[0]);

	bindtextdomain (GETTEXT_PACKAGE, BURGNER_LOCALEDIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);

	g_type_init ();
	g_thread_init (NULL);
	gnome_vfs_init ();
	burgner_scsi_init ();

	burgner_mmc_set_write_page_stage_get_type ();

	argv_0 = argv[0];

	main_loop = g_main_loop_new (NULL, FALSE);

	prgname = g_path_get_basename (argv[0]);
	if (strcmp (prgname, "cdrecord") == 0) {
		cdrecord_comp = TRUE;
	}
	g_free (prgname);

	ctx = poptGetContext ("burgner-cdrecord", argc, argv,
			      cdrecord_comp
			      	? cdrecord_options_table
				: options_table, 0);

	if ((popt_r = poptGetNextOpt (ctx)) < -1) {
		poptPrintUsage (ctx, stderr, 0);
		fprintf (stderr, "%s\n", poptStrerror (popt_r));
		return 1;
	}

	if (version) {
		/* Version message */
		fprintf (stderr, _("%s: %s version %s\n"),
					argv[0], PACKAGE, VERSION);
		return 0;
	}

	if (!device_uri) {
		poptPrintUsage (ctx, stderr, 0);
		fprintf (stderr, _("You must specify a device uri\n"));
		return 1;
	}

	if (blank_type) {
		if (strcmp (blank_type, "complete") == 0) {
			blank = _BLANK_COMPLETE;
		} else if (strcmp (blank_type, "minimal") == 0) {
			blank = _BLANK_MINIMAL;
		} else if (strcmp (blank_type, "session") == 0) {
			blank = _BLANK_SESSION;
		} else if (strcmp (blank_type, "unclose_session") == 0) {
			blank = _BLANK_UNCLOSE_SESSION;
		} else {
			poptPrintUsage (ctx, stderr, 0);
			fprintf (stderr, _("%s: %s: unknown blank type\n"),
						argv[0], blank_type);
			return 1;
		}
	}

	if (session) {
		if (strcmp (session, "allow_next") == 0) {
			session_closure = _SESSION_ALLOW_NEXT;
		} else if (strcmp (session, "last") == 0) {
			session_closure = _SESSION_LAST;
		} else if (strcmp (session, "no_info") == 0) {
			session_closure = _SESSION_NO_INFO;
		} else if (strcmp (session, "do_not_close") == 0) {
			session_closure = _SESSION_DO_NOT_CLOSE;
		} else {
			poptPrintUsage (ctx, stderr, 0);
			fprintf (stderr, _("%s: %s: unknown session behavior"),
						argv[0], session);
			exit (1);
		}
	}

	if (eject && session_closure == _SESSION_DO_NOT_CLOSE) {
		fprintf (stderr, "warning: not closing the session and ejecting the disc is not recommanded\n");
	}

	track_list = g_list_reverse (track_list);
	next_track = track_list;

	poptFreeContext (ctx);

	g_idle_add (_idle_func, NULL);

	g_main_loop_run (main_loop);


	if (smc)
		g_object_unref (smc);

	if (fifo_writer)
		g_object_unref (fifo_writer);

	if (big_endian_filter)
		g_object_unref (big_endian_filter);

	if (fifo)
		g_object_unref (fifo);

	if (driver)
		g_object_unref (driver);

	if (driver_manager)
		g_object_unref (driver_manager);

	return ret_val;
}

