/*
 * 
 *
 * This file is part of the src library.
 *
 * Copyright (c) 2003 Alexandre Archambault
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 * Authors:
 *	Alexandre Archambault <aarchambault@free.fr>
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "../libburgnerisofs/burgner-isofs-i18n.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <popt.h>
#include <glib-object.h>
#include <libgnomevfs/gnome-vfs.h>
#define WITH_GNOMEVFS

#include <libburgnerisofs/burgner-isofs-tree.h>
#include <libburgnerisofs/burgner-isofs-list-maker.h>
#include <libburgnerisofs/burgner-isofs-writer.h>

#include <libburgnerisofs/burgner-isofs-primary-builder.h>
#include <libburgnerisofs/burgner-isofs-eltorito-builder.h>

#include <libburgnerisofs/burgner-isofs-eltorito-read-system-type.h>
#include <libburgnerisofs/burgner-isofs-tvd-info.h>
#include <libburgnerisofs/burgner-isofs-list-iteration.h>

#include <libburgnerisofs/burgner-isofs-file.h>
#include <libburgnerisofs/burgner-isofs-void-item.h>



static GMainLoop *main_loop = NULL;
static gint retval = EXIT_SUCCESS;

static BurgnerIsofsTvdInfo tvd_info = {
	{ 0, 0, 0, 0, 0, 0, 0, 0 },
	{ 0, 0, 0, 0, 0, 0, 0, 0 },
	{ 0, 0, 0, 0, 0, 0, 0, 0 },
	{ 0, 0, 0, 0, 0, 0, 0, 0 },

	{ NULL, BURGNER_ISOFS_TREE_VOLUME_DESCRIPTOR_IDENTIFIER_TYPE_NONE },
	{ NULL, BURGNER_ISOFS_TREE_VOLUME_DESCRIPTOR_IDENTIFIER_TYPE_NONE },
	{ "BURGNER 0.1.0" , BURGNER_ISOFS_TREE_VOLUME_DESCRIPTOR_IDENTIFIER_TYPE_STR },

	NULL,
	NULL,
	NULL,

	NULL,
	NULL,
	NULL,

	1,
	1
};


static gboolean with_rockridge = FALSE;
static gboolean rockridge_reasonable = FALSE;
static gboolean do_relocation = TRUE;

static gint reloc_dir_level = 9;



static gint volume_set_size = 1;
static gint volume_sequence_number = 1;

static gchar *reloc_dir_id = NULL;


static gchar *output_file = NULL;
static GIOChannel *output_channel = NULL;

static gchar *eltorito_manufacturer = NULL;
static gchar *eltorito_media_image = NULL;
static gboolean eltorito_bootable = FALSE;
static gchar *eltorito_platform_string = NULL;
static gchar *eltorito_media_type_string = NULL;

static BurgnerIsofsEltoritoPlatformId eltorito_platform = BURGNER_ISOFS_ELTORITO_PLATFORM_ID_80x86;
static BurgnerIsofsEltoritoMediaType eltorito_media_type = BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_NO_EMULATION;
static guchar eltorito_system_type = 0;
static gchar *eltorito_media_image_path = NULL;


static gboolean version = FALSE;

static gboolean cache_inodes = FALSE;

static gboolean relax_character_set = FALSE;
static gboolean allow_multidot = FALSE;
static gboolean allow_lower_case = FALSE;
static gboolean allow_long_names = FALSE;
static gboolean allow_long_extensions = FALSE;
static gboolean allow_no_extension = FALSE;
static gboolean non_unique_names = FALSE;

static gboolean optimize = TRUE;
static gboolean with_path_tables = TRUE;
static gboolean with_first_16_empty_sectors = TRUE;

static guint32 first_sector_lba = 0;
static const gchar *first_sector_string = NULL;



static BurgnerIsofsIdentifierOptions
_burgner_mkisofs_get_identifier_options (void)
{
	BurgnerIsofsIdentifierOptions op = BURGNER_ISOFS_IDENTIFIER_UNIQUE;

	if (relax_character_set)
		op &= BURGNER_ISOFS_IDENTIFIER_RELAX_CHARACTER_SET;

	if (allow_multidot)
		op &= BURGNER_ISOFS_IDENTIFIER_ALLOW_MULTIDOT;	

	if (allow_lower_case)
		op &= BURGNER_ISOFS_IDENTIFIER_ALLOW_LOWER_CASE;

	if (allow_long_names)
		op &= BURGNER_ISOFS_IDENTIFIER_ALLOW_LONG_NAMES;

	if (allow_long_extensions)
		op &= BURGNER_ISOFS_IDENTIFIER_ALLOW_LONG_EXTENSIONS;

	if (allow_no_extension)
		op &= BURGNER_ISOFS_IDENTIFIER_ALLOW_NO_EXTENSION;

	if (non_unique_names)
		op &= ~BURGNER_ISOFS_IDENTIFIER_UNIQUE;

	return op;
}


static struct poptOption names_options_table[] = {
	{ "relax-character-set", '\0', POPT_ARG_NONE, &relax_character_set, 0,
	  N_("Allow non alphanumeric or not '_' characters in names"), NULL },
	{ "allow-multidot", '\0', POPT_ARG_NONE, &allow_multidot, 0,
	  N_("Allow multiple dots in names"), NULL },
	{ "allow-lower-case", '\0', POPT_ARG_NONE, &allow_lower_case, 0,
	  N_("Allow lower case characters in names"), NULL },
	{ "allow-long-names", '\0', POPT_ARG_NONE, &allow_long_names, 0,
	  N_("Allow long (more than 8.3, but still less than 32 characters) names"), NULL },
	{ "allow-long-extensions", '\0', POPT_ARG_NONE, &allow_long_extensions, 0,
	  N_("Allow long (more than 3 characters) extensions in names"), NULL },
	{ "allow-no-extension", '\0', POPT_ARG_NONE, &allow_no_extension, 0,
	  N_("Allow names not to have an extension"), NULL },
	{ "non-unique-names", '\0', POPT_ARG_NONE, &non_unique_names, 0,
	  N_("Allow names not to be unique (multiple files may have the same name, "
			  "use only with RockRidge, if you know what you are doing)"), NULL },

	POPT_TABLEEND
};

static struct poptOption eltorito_options_table[] = {
	{ "eltorito-manufacturer", '\0', POPT_ARG_STRING, &eltorito_manufacturer, 0,
	  N_("Set the Eltorito manufacturer"), N_("manufacturer") },
	{ "eltorito-media-image", '\0', POPT_ARG_STRING, &eltorito_media_image, 0,
	  N_("Set the file containing the media image (can be in the tree, but can also not)"), N_("media-image") },
	{ "eltorito-media-type", '\0', POPT_ARG_STRING, &eltorito_media_type_string, 0,
	  N_("Set the media type to emulate (can be autodetected for diskettes): no-emulation (default), 1.2M-disk, 1.44M-disk, 2.88M-disk, or hard-disk"), N_("media-type") },
	{ "eltorito-bootable", '\0', POPT_ARG_NONE, &eltorito_bootable, 0,
	  N_("Whether the CD-ROM will be bootable or not (if you only want to emulate a device)"), NULL },
	{ "eltorito-platform", '\0', POPT_ARG_STRING, &eltorito_platform_string, 0,
	  N_("Set the type of platform the image is bootable"), N_("[80x86]|power-pc|mac") },

#if 0
	/* TODO (in the BurgnerIsofsEltoritoBuilder) */
	{ "eltorito-boot-catalog", '\0', POPT_ARG_STRING, &eltorito_boot_catalog, 0,
	  N_("If you want the boot catalog to appear in the image file system, set the path where it will be inserted (in the image) with this option"), NULL },
#endif

	POPT_TABLEEND
};

static struct poptOption options_table[] = {
	{ "with-rockridge", 'R', POPT_ARG_VAL, &with_rockridge, TRUE,
	  N_("Enable the RockRidge extensions"), NULL },
	{ "without-rockridge", '\0', POPT_ARG_VAL, &with_rockridge, FALSE,
	  N_("Disable the RockRidge extensions"), NULL },

	{ "rockridge-reasonable", 'r', POPT_ARG_NONE, &rockridge_reasonable, 0,
	  N_("Set POSIX attributes (UID, GID, permissions) to reasonable values"), NULL },

	{ NULL, '\0', POPT_ARG_INCLUDE_TABLE, names_options_table, 0,
	  N_("Options which concerns ISO9660 names (they all violate the ISO9660 standard, use with care)"), NULL },

	{ NULL, '\0', POPT_ARG_INCLUDE_TABLE, eltorito_options_table, 0,
	  N_("Options which concerns El torito (bootable CD)"), NULL },

	{ "cache-inodes", '\0', POPT_ARG_NONE, &cache_inodes, 0,
	  N_("Cache inodes (make several files point on the same content if they are hard links)"), NULL },
	{ "with-optimize", '\0', POPT_ARG_VAL, &optimize, TRUE,
	  N_("Enable size optimizations"), NULL },
	{ "without-optimize", '\0', POPT_ARG_VAL, &optimize, FALSE,
	  N_("Disable size optimizations"), NULL },
	{ "with-path-tables", '\0', POPT_ARG_VAL, &with_path_tables, TRUE,
	  N_("Enable path tables (default)"), NULL },
	{ "without-path-tables", '\0', POPT_ARG_VAL, &with_path_tables, FALSE,
	  N_("Disable path tables (totally violates ISO9660, but save 4k)"), NULL },
	{ "with-first-16-empty-sectors", '\0', POPT_ARG_VAL, &with_first_16_empty_sectors, TRUE,
	  N_("Let the first 16 sectors void (default)"), NULL },
	{ "without-first-16-empty-sectors", '\0', POPT_ARG_VAL, &with_first_16_empty_sectors, FALSE,
	  N_("Put some file system data in the first 16 sectors (totally violates ISO9660, but save 30k)"), NULL },

	{ "publisher", 'P', POPT_ARG_STRING, &tvd_info.publisher_id.str, 0,
	  N_("Set the publisher of the CD"), N_("publisher") },
	{ "preparer", 'p', POPT_ARG_STRING, &tvd_info.data_preparer_id.str, 0,
	  N_("Set the preparer if the CD"), N_("preparer") },
	{ "application", 'A', POPT_ARG_STRING, &tvd_info.application_id.str, 0,
	  N_("Set the application which generated the image"), N_("application") },
	{ "volume-id", 'V', POPT_ARG_STRING, &tvd_info.volume_id, 0,
	  N_("Set the volume identification"), N_("volume-id") },

	{ "system-id", '\0', POPT_ARG_STRING, &tvd_info.system_id, 0,
	  N_("Set the system identification"), N_("system-id") },
	{ "volume-set-id", '\0', POPT_ARG_STRING, &tvd_info.volume_set_id, 0,
	  N_("Set the volume set identification"), N_("volume-set-id") },

	{ "volume-set-size", '\0', POPT_ARG_INT, &volume_set_size, 0,
	  N_("Set the volume set size"), N_("volume-set-size") },
	{ "volume-number", '\0', POPT_ARG_INT, &volume_sequence_number, 0,
	  N_("Set the volume number in the volume set"), N_("volume_number") },

	{ "copyright-file", '\0', POPT_ARG_STRING, &tvd_info.copyright_file_id, 0,
	  N_("Set which file (in the image) is the copyright file"), N_("copyright-file-id") },
	{ "abstract-file", '\0', POPT_ARG_STRING, &tvd_info.abstract_file_id, 0,
	  N_("Set which file (in the image) is the abstract file"), N_("abstract-file-id") },
	{ "bibliographic-file", '\0', POPT_ARG_STRING, &tvd_info.bibliographic_file_id, 0,
	  N_("Set which file (in the image) is the bibliographic file"), N_("bibliographic-file-id") },

	{ "first-sector-lba", '\0', POPT_ARG_STRING, &first_sector_string, 0,
	  N_("Set the absolute LBA on the disc of the first data sector"), N_("lba") },

	{ "with-relocation", '\0', POPT_ARG_VAL, &do_relocation, TRUE,
	  N_("Enable deep directory relocation"), NULL },
	{ "without-relocation", '\0', POPT_ARG_VAL, &do_relocation, FALSE,
	  N_("Disable deep directory relocation"), NULL },
	{ "reloc-dir-name", '\0', POPT_ARG_STRING, &reloc_dir_id, 0,
	  N_("Set the name of the relocated directories in the main tree"), N_("reloc-dir-name") },
	{ "reloc-dir-level", '\0', POPT_ARG_INT | POPT_ARGFLAG_SHOW_DEFAULT, &reloc_dir_level, 0,
	  N_("Set the directory relocation level"), N_("reloc-level") },

	{ "output", 'o', POPT_ARG_STRING, &output_file, 0,
	  N_("Set the output file ('-' for standard output - this is the default)"), N_("file-name") },

	{ "version", '\0', POPT_ARG_NONE, &version, 0,
	  N_("Print version and exit"), NULL },

	POPT_AUTOHELP
	POPT_TABLEEND
};

static BurgnerIsofsTree *tree = NULL;
static BurgnerIsofsListMaker *list_maker = NULL;
static GList *items_list = NULL;
static BurgnerIsofsWriter *writer = NULL;


#ifdef WITH_GNOMEVFS
static BurgnerIsofsFileType
_burgner_isofs_file_type_from_gnome_vfs_file_type (GnomeVFSFileType type)
{
	BurgnerIsofsFileType r = BURGNER_ISOFS_FILE_TYPE_UNKNOWN;

	switch (type) {
		case GNOME_VFS_FILE_TYPE_UNKNOWN:
			r = BURGNER_ISOFS_FILE_TYPE_UNKNOWN;
			break;
		case GNOME_VFS_FILE_TYPE_REGULAR:
			r = BURGNER_ISOFS_FILE_TYPE_REGULAR;
			break;
		case GNOME_VFS_FILE_TYPE_DIRECTORY:
			r = BURGNER_ISOFS_FILE_TYPE_DIRECTORY;
			break;
		case GNOME_VFS_FILE_TYPE_FIFO:
			r = BURGNER_ISOFS_FILE_TYPE_FIFO;
			break;
		case GNOME_VFS_FILE_TYPE_SOCKET:
			r = BURGNER_ISOFS_FILE_TYPE_SOCKET;
			break;
		case GNOME_VFS_FILE_TYPE_CHARACTER_DEVICE:
			r = BURGNER_ISOFS_FILE_TYPE_CHARACTER_DEVICE;
			break;
		case GNOME_VFS_FILE_TYPE_BLOCK_DEVICE:
			r = BURGNER_ISOFS_FILE_TYPE_BLOCK_DEVICE;
			break;
		case GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK:
			r = BURGNER_ISOFS_FILE_TYPE_SYMBOLIC_LINK;
			break;
		default:
			g_return_val_if_reached (BURGNER_ISOFS_FILE_TYPE_UNKNOWN);
			break;
	}

	return r;
}

static inline BurgnerIsofsFilePermission
_burgner_isofs_file_permission_from_gnome_vfs_file_permissions (GnomeVFSFilePermissions perm)
{
	return (BurgnerIsofsFilePermission) perm;
}


static gboolean
_burgner_mkisofs_gnome_vfs_load_directory_in_tree (BurgnerIsofsTree *tree,
						   BurgnerIsofsFileInfo *info,
						   GnomeVFSURI *uri,
						   GError **error)
{
	GnomeVFSDirectoryHandle *handle;
	GnomeVFSFileInfo *file_info;
	GnomeVFSResult res;

	res = gnome_vfs_directory_open_from_uri (&handle, uri, GNOME_VFS_FILE_INFO_DEFAULT);
	if (res != GNOME_VFS_OK) {
		g_set_error (error,
			     g_quark_from_string ("burgner-mkisofs"),
			     1,
			     _("opening directory %s: %s"),
			     gnome_vfs_uri_get_path (uri),
			     gnome_vfs_result_to_string (res));
		return FALSE;
	}

	file_info = gnome_vfs_file_info_new ();
	while ((res = gnome_vfs_directory_read_next (handle, file_info)) == GNOME_VFS_OK) {
		BurgnerIsofsFileInfo *c_info;

		if (file_info->name[0] == '.' &&
		    (file_info->name[1] == '\0' || (file_info->name[1] == '.' && file_info->name[2] == '\0')))
			continue;

		if (file_info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) {
			GnomeVFSURI *c_uri;

			c_info = burgner_isofs_tree_add_directory (tree, info, file_info->name, FALSE);

			c_info->type = _burgner_isofs_file_type_from_gnome_vfs_file_type (file_info->type);
			if (rockridge_reasonable) {
				c_info->permissions = BURGNER_ISOFS_FILE_PERMISSION_OWNER_READ |
							BURGNER_ISOFS_FILE_PERMISSION_OWNER_EXEC |
							BURGNER_ISOFS_FILE_PERMISSION_GROUP_READ |
							BURGNER_ISOFS_FILE_PERMISSION_GROUP_EXEC |
							BURGNER_ISOFS_FILE_PERMISSION_OTHERS_READ |
							BURGNER_ISOFS_FILE_PERMISSION_OTHERS_EXEC;
				c_info->uid = c_info->gid = 0;
			} else {
				c_info->permissions = _burgner_isofs_file_permission_from_gnome_vfs_file_permissions (file_info->permissions);
				c_info->uid = file_info->uid;
				c_info->gid = file_info->gid;
			}

			c_info->link_count = file_info->link_count;
			c_info->inode_number = file_info->inode;
			c_info->symlink = g_strdup (file_info->symlink_name);

			c_info->modify_time = file_info->mtime;
			c_info->access_time = file_info->atime;
			c_info->attributes_time = file_info->ctime;

			c_uri = gnome_vfs_uri_append_file_name (uri, file_info->name);
			if (!_burgner_mkisofs_gnome_vfs_load_directory_in_tree (tree, c_info, c_uri, error)) {
				gnome_vfs_file_info_unref (file_info);
				gnome_vfs_uri_unref (c_uri);
				gnome_vfs_directory_close (handle);

				return FALSE;
			}

			gnome_vfs_uri_unref (c_uri);
		} else {
			GUri *c_uri;
			gchar *c_uri_str;

			c_uri = g_uri_new ();
			c_uri_str = g_strdup_printf ("file://primary/%s/%s", gnome_vfs_uri_get_path (uri), file_info->name);
			if (!g_uri_set (c_uri, c_uri_str, FALSE, FALSE, error)) {
				gnome_vfs_file_info_unref (file_info);
				gnome_vfs_directory_close (handle);
				g_uri_unref (c_uri);
				g_free (c_uri_str);

				return FALSE;
			}
			g_free (c_uri_str);

			c_info = burgner_isofs_tree_add_file (tree, info, file_info->name, FALSE);
			c_info->uri = c_uri;

			c_info->type = _burgner_isofs_file_type_from_gnome_vfs_file_type (file_info->type);

			if (rockridge_reasonable) {
				c_info->permissions = BURGNER_ISOFS_FILE_PERMISSION_OWNER_READ |
							BURGNER_ISOFS_FILE_PERMISSION_GROUP_READ |
							BURGNER_ISOFS_FILE_PERMISSION_OTHERS_READ;
				c_info->uid = c_info->gid = 0;
			} else {
				c_info->permissions = _burgner_isofs_file_permission_from_gnome_vfs_file_permissions (file_info->permissions);
				c_info->uid = file_info->uid;
				c_info->gid = file_info->gid;
			}

			c_info->modify_time = file_info->mtime;
			c_info->access_time = file_info->atime;
			c_info->attributes_time = file_info->ctime;

			c_info->link_count = file_info->link_count;
			c_info->inode_number = file_info->inode;
			c_info->size = file_info->size;
			c_info->device = file_info->device;
			c_info->symlink = g_strdup (file_info->symlink_name);
		}

		gnome_vfs_file_info_clear (file_info);
	}

	gnome_vfs_directory_close (handle);
	gnome_vfs_file_info_unref (file_info);

	if (res != GNOME_VFS_ERROR_EOF) {
		g_set_error (error,
			     g_quark_from_string ("burgner-mkisofs"),
			     2,
			     _("loading directory %s: %s"),
			     gnome_vfs_uri_get_path (uri),
			     gnome_vfs_result_to_string (res));
		return FALSE;
	}

	return TRUE;
}
#endif

static gboolean
_burgner_mkisofs_process_file (BurgnerIsofsFileInfo *info,
			       const gchar *file_name,
			       GError **error)
{
#ifdef WITH_GNOMEVFS
	GnomeVFSURI *base_uri = gnome_vfs_uri_new ("file:///"),
		    *uri = gnome_vfs_uri_append_path (base_uri, file_name);
	GnomeVFSResult res;
	GnomeVFSFileInfo *file_info;

	GUri *g_uri;
	gchar *g_uri_lit;

	gnome_vfs_uri_unref (base_uri);

	file_info = gnome_vfs_file_info_new ();
	res = gnome_vfs_get_file_info_uri (uri, file_info, GNOME_VFS_FILE_INFO_DEFAULT);

	if (res != GNOME_VFS_OK) {
		g_set_error (error,
			     g_quark_from_string ("burgner-mkisofs"),
			     0,
			     _("getting info about %s: %s"),
			     file_name, gnome_vfs_result_to_string (res));
		gnome_vfs_file_info_unref (file_info);
		return FALSE;
	}

	g_uri = g_uri_new ();

	g_uri_lit = g_strdup_printf ("file://primary/%s", file_name);
	if (!g_uri_set (g_uri, g_uri_lit, FALSE, FALSE, error)) {
		g_free (g_uri_lit);
		g_uri_unref (g_uri);
		gnome_vfs_file_info_unref (file_info);
		return FALSE;
	}
	g_free (g_uri_lit);

	info->uri = g_uri;
	info->type = _burgner_isofs_file_type_from_gnome_vfs_file_type (file_info->type);
	info->permissions = _burgner_isofs_file_permission_from_gnome_vfs_file_permissions (file_info->permissions);
	info->uid = file_info->uid;
	info->gid = file_info->gid;
	info->link_count = file_info->link_count;
	info->inode_number = file_info->inode;
	info->device = file_info->device;
	info->size = file_info->size;
	info->symlink = g_strdup (file_info->symlink_name);
	info->modify_time = file_info->mtime;
	info->access_time = file_info->atime;
	info->attributes_time = file_info->ctime;


	if ((info->is_directory = (info->type == BURGNER_ISOFS_FILE_TYPE_DIRECTORY))) {
		return _burgner_mkisofs_gnome_vfs_load_directory_in_tree (tree, info, uri, error);
	}

	return TRUE;
#else
	return TRUE;
#endif
}

#define BUF_SIZE	(2*1024*1024)
static guchar buf[BUF_SIZE];

static gboolean _burgner_mkisofs_write_idle (gpointer user_data);

static guint32 image_size = 0;
static guint32 image_written = 0;


static void
update_line (void)
{
	guint p = image_size ? ((image_written >> 11) * 100 / (image_size >> 11)) : 0;

	fprintf (stderr, _("\rWrote %2.2u%% (%u M / %u M) [%c%c%c%c%c%c%c%c%c%c]"),
			p,
			image_written >> 20, image_size >> 20,
			(int) (p >= 9 ? '#' : ' '),
			(int) (p >= 18 ? '#' : ' '),
			(int) (p >= 27 ? '#' : ' '),
			(int) (p >= 36 ? '#' : ' '),
			(int) (p >= 45 ? '#' : ' '),
			(int) (p >= 54 ? '#' : ' '),
			(int) (p >= 63 ? '#' : ' '),
			(int) (p >= 72 ? '#' : ' '),
			(int) (p >= 81 ? '#' : ' '),
			(int) (p >= 91 ? '#' : ' '));
}

static void
_burgner_mkisofs_write_async_cb (BurgnerIsofsWriter *writer,
				 gpointer _buf, guint32 _buf_size, guint32 written,
				 GError *error,
				 gpointer user_data)
{
	guchar *b;

	if (error) {
		fprintf (stderr, _("\nError while writing: %s\n"), error && error->message ? error->message : _("[no description]"));
		retval = EXIT_FAILURE;
		g_main_loop_quit (main_loop);
		return;
	}

	image_written += written;

	b = buf;

	while (written) {
		guint32 w = 0;
		GError *err = NULL;
		
		if (g_io_channel_write_chars (output_channel, b, written, &w, &err) != G_IO_STATUS_NORMAL) {
			fprintf (stderr, _("\nError while writing: %s\n"), err && err->message ? err->message : _("[no description]"));
			retval = EXIT_FAILURE;
			g_main_loop_quit (main_loop);
			return;
		}

		written -= w;
		b += w;
	}

	update_line ();

	g_idle_add (_burgner_mkisofs_write_idle, NULL);
}

static void
_burgner_mkisofs_write (void)
{
	GError *err = NULL;
	guint32 to_write;

	while ((to_write = burgner_isofs_writer_to_write_sync (writer))) {
		guint32 to_be_written = MIN (BUF_SIZE, to_write), written;
		guchar *b = buf;

		written = burgner_isofs_writer_write_sync (writer, buf, to_be_written, NULL);

		image_written += written;

		while (written) {
			guint32 w = 0;
			
			if (g_io_channel_write_chars (output_channel, b, written, &w, &err) != G_IO_STATUS_NORMAL) {
				fprintf (stderr, _("\nError while writing: %s\n"), err && err->message ? err->message : _("[no description]"));
				retval = EXIT_FAILURE;
				g_main_loop_quit (main_loop);
				return;
			}

			written -= w;
			b += w;
		}

		update_line ();
	}

	if ((to_write = burgner_isofs_writer_to_write_async (writer))) {
		burgner_isofs_writer_write_async (writer, buf, MIN (BUF_SIZE, to_write),
						  _burgner_mkisofs_write_async_cb, NULL);
	} else if (burgner_isofs_writer_end_reached (writer)) {
		fprintf (stderr, _("\nWriting done\n"));
		retval = EXIT_SUCCESS;
		g_main_loop_quit (main_loop);
	}
}

static gboolean
_burgner_mkisofs_write_idle (gpointer user_data)
{
	_burgner_mkisofs_write ();
	return FALSE;
}

static gboolean
_burgner_mkisofs_find_eltorito_boot_image (GNode *node,
					   gpointer user_data)
{
	BurgnerIsofsFileInfo *i = (BurgnerIsofsFileInfo *) node->data;
	const gchar *path;

	if (i->uri && (path = g_uri_get_path (i->uri)) && (strcmp (path, eltorito_media_image_path) == 0)) {
		*((BurgnerIsofsFileInfo **) user_data) = i;
		return TRUE;
	}

	return FALSE;
}

static gboolean
_burgner_mkisofs_list_iteration_func (GList *list,
				      GList *actual,
				      BurgnerIsofsItem *item,
				      BurgnerIsofsItemFlags flags,
				      guint32 padding,
				      guint32 s_c, guint32 b_o,
				      guint32 n_s, guint32 n_b,
				      guint32 a_s, guint32 a_b,
				      gpointer user_data)
{
	guint32 *ls = (guint32 *) user_data;

	g_assert (user_data);

	if (padding)
		*ls += padding;

	if (BURGNER_IS_ISOFS_VOID_ITEM (item) &&
	    GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (item), "removable")))
		*ls += (n_s << 11) + n_b;

	return FALSE;
}

int
main (int argc, const char **argv)
{
	poptContext ctx;
	const gchar *arg;
	gchar *n;
	gint popt_r;
	BurgnerIsofsTreeBuilder *builder;

	bindtextdomain (GETTEXT_PACKAGE, BURGNER_LOCALEDIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);

	g_type_init ();
	g_thread_init (NULL);
	gnome_vfs_init ();

	n = g_path_get_basename (argv[0]);
	g_set_prgname (n);
	g_free (n);

	main_loop = g_main_loop_new (NULL, FALSE);

	ctx = poptGetContext ("burgner-mkisofs", argc, argv, options_table, 0);
	if ((popt_r = poptGetNextOpt (ctx)) < -1) {
		poptPrintUsage (ctx, stderr, 0);
		fprintf (stderr, "%s\n", poptStrerror (popt_r));
		return EXIT_FAILURE;
	}

	if (version) {
		fprintf (stderr, _("%s: %s version %s\n"), argv[0], PACKAGE, VERSION);
		return EXIT_SUCCESS;
	}

	tree = burgner_isofs_tree_new ();

	if (rockridge_reasonable && !with_rockridge)
		with_rockridge = TRUE;

	if (first_sector_string) {
		gchar *ret = NULL;
		first_sector_lba = strtoul (first_sector_string, &ret, 10);
		if (*ret != '\0') {
			poptPrintUsage (ctx, stderr, 0);
			fprintf (stderr, _("Invalid first sector LBA (must be a decimal number): '%s'\n"), first_sector_string);
			return EXIT_FAILURE;
		}
	}

	if (eltorito_media_type_string) {
		if (g_ascii_strcasecmp (eltorito_media_type_string, "no-emulation") == 0) {
			eltorito_media_type = BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_NO_EMULATION;
		} else if (g_ascii_strcasecmp (eltorito_media_type_string, "1.2M-disk") == 0) {
			eltorito_media_type = BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_1_2_MEG_DISKETTE;
		} else if (g_ascii_strcasecmp (eltorito_media_type_string, "1.44M-disk") == 0) {
			eltorito_media_type = BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_1_44_MEG_DISKETTE;
		} else if (g_ascii_strcasecmp (eltorito_media_type_string, "2.88M-disk") == 0) {
			eltorito_media_type = BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_2_88_MEG_DISKETTE;
		} else if (g_ascii_strcasecmp (eltorito_media_type_string, "hard-disk") == 0) {
			eltorito_media_type = BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_HARD_DISK;
		} else {
			poptPrintUsage (ctx, stderr, 0);
			fprintf (stderr, _("Invalid Eltorito media type: %s\n"), eltorito_media_type_string);
			return EXIT_FAILURE;
		}
	}

	if (eltorito_platform_string) {
		if (g_ascii_strcasecmp (eltorito_platform_string, "80x86") == 0) {
			eltorito_platform = BURGNER_ISOFS_ELTORITO_PLATFORM_ID_80x86;
		} else if (g_ascii_strcasecmp (eltorito_platform_string, "power-pc") == 0) {
			eltorito_platform = BURGNER_ISOFS_ELTORITO_PLATFORM_ID_POWER_PC;
		} else if (g_ascii_strcasecmp (eltorito_platform_string, "mac") == 0) {
			eltorito_platform = BURGNER_ISOFS_ELTORITO_PLATFORM_ID_MAC;
		} else {
			poptPrintUsage (ctx, stderr, 0);
			fprintf (stderr, _("Invalid Eltorito platform: %s\n"), eltorito_platform_string);
			return EXIT_FAILURE;
		}
	}

	if (eltorito_bootable) {
		GIOChannel *channel;
		GError *err = NULL;
		guchar mbr[BURGNER_ISOFS_ELTORITO_MBR_SIZE];
		guint bytes_read = 0;

		if (!eltorito_media_image) {
			poptPrintUsage (ctx, stderr, 0);
			fprintf (stderr, _("You must specify a file containing the Eltorito media\n"));
			return EXIT_FAILURE;
		}

		if (eltorito_media_type != BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_NO_EMULATION) {
			if (!(channel = g_io_channel_new_file (eltorito_media_image, "r", &err))) {
				fprintf (stderr, _("Error while opening Eltorito media image (%s): %s\n"), eltorito_media_image, err && err->message ? err->message : _("[no description]"));
				return EXIT_FAILURE;
			}

			g_io_channel_set_encoding (channel, NULL, NULL);
			g_io_channel_set_buffered (channel, FALSE);
			g_io_channel_set_flags (channel, g_io_channel_get_flags (channel) & ~G_IO_FLAG_NONBLOCK, NULL);

			if (g_io_channel_read_chars (channel, mbr, BURGNER_ISOFS_ELTORITO_MBR_SIZE, &bytes_read, NULL) != G_IO_STATUS_NORMAL || bytes_read != BURGNER_ISOFS_ELTORITO_MBR_SIZE) {
				fprintf (stderr, _("Error while reading Eltorito media image (%s) first 512 bytes\n"), eltorito_media_image);
				return EXIT_FAILURE;
			}

			if (!burgner_isofs_eltorito_read_system_type (mbr, &eltorito_system_type, &err)) {
				fprintf (stderr, _("Error while parsing Eltorito media image (%s) first 512 bytes: %s\n"),
						eltorito_media_image,
						err && err->message ? err->message : _("[no description]"));
				return EXIT_FAILURE;
			}
		}
	}

	while ((arg = poptGetArg (ctx))) {
		BurgnerIsofsFileInfo *info;
		const gchar *file_name;
		gchar *abs_name, *e;
		GError *error = NULL;

		if ((e = strchr (arg, '='))) {
			gchar *img_path = g_strndup (arg, e - arg);
			info = burgner_isofs_tree_get_info_from_path (tree, img_path, TRUE);
			g_free (img_path);

#if 0
			if (g_path_is_absolute (e+1)) {
#endif
				file_name = e+1;
#if 0
			} else {
				gchar *cd = g_get_current_dir ();
				file_name = g_strdup_printf ("%s%s%s", cd, G_DIR_SEPARATOR_S, e+1);
				g_free (cd);
			}
#endif
		} else {
			gchar *name;
			if ((name = g_path_get_basename (arg))) {
				info = burgner_isofs_tree_add_file (tree,
								    burgner_isofs_tree_get_root_info (tree),
								    name, FALSE);
				g_free (name);
			} else if ((name = g_path_get_dirname (arg))) {
				info = burgner_isofs_tree_add_directory (tree,
									 burgner_isofs_tree_get_root_info (tree),
									 name, FALSE);
				g_free (name);
			} else {
				info = burgner_isofs_tree_add_file (tree,
								    burgner_isofs_tree_get_root_info (tree),
								    "none", FALSE);
			}

			file_name = arg;
		}

		if (g_path_is_absolute (file_name)) {
			abs_name = g_strdup (file_name);
		} else {
			gchar *cd = g_get_current_dir ();
			abs_name = g_strdup_printf ("%s%s%s", cd, G_DIR_SEPARATOR_S, file_name);
			g_free (cd);
		}

		if (!_burgner_mkisofs_process_file (info, abs_name, &error)) {
			fprintf (stderr, _("'%s': %s\n"), arg, error->message ? error->message : _("No error description"));
			g_free (abs_name);
			return EXIT_FAILURE;
		}

		g_free (abs_name);
	}

	burgner_isofs_tree_create_identifiers (tree, _burgner_mkisofs_get_identifier_options ());

	list_maker = burgner_isofs_list_maker_new ();

	tvd_info.volume_set_size = volume_set_size;
	tvd_info.volume_sequence_number = volume_sequence_number;

	if (tvd_info.publisher_id.str) {
		tvd_info.publisher_id.type = BURGNER_ISOFS_TREE_VOLUME_DESCRIPTOR_IDENTIFIER_TYPE_STR;
	}

	if (tvd_info.data_preparer_id.str) {
		tvd_info.data_preparer_id.type = BURGNER_ISOFS_TREE_VOLUME_DESCRIPTOR_IDENTIFIER_TYPE_STR;
	} 

	if (!tvd_info.application_id.str) {
		tvd_info.application_id.str = PACKAGE VERSION " SEE BURGNER.TUXFAMILY.ORG";
	}

	tvd_info.application_id.type = BURGNER_ISOFS_TREE_VOLUME_DESCRIPTOR_IDENTIFIER_TYPE_STR;


	builder = burgner_isofs_primary_builder_new ();
	g_object_set (builder,
		      "cache_inodes", cache_inodes,
		      "no_path_tables", (with_path_tables == FALSE),
		      "with_reloc", do_relocation,
		      "reloc_level", reloc_dir_level,
		      "reloc_dir_id", reloc_dir_id ? reloc_dir_id : "RR_MOVED",
		      "with_rrip", with_rockridge,
		      "tvd_info", &tvd_info,
		      NULL);

	burgner_isofs_list_maker_add_builder (list_maker, "primary", builder);
	g_object_unref (builder);

	if (with_rockridge) {
		fprintf (stderr, _("Rockridge enabled (%s)\n"),
				rockridge_reasonable ? _("reasonable attributes") : _("as is"));
	}

	if (cache_inodes) {
		fprintf (stderr, _("Inodes caching enabled\n"));
	}

	if (eltorito_media_image) {
		BurgnerIsofsFileInfo *info = NULL;
		gboolean is_in_tree = FALSE;

		if (g_path_is_absolute (eltorito_media_image)) {
			eltorito_media_image_path = g_strdup (eltorito_media_image);
		} else {
			gchar *cd = g_get_current_dir ();
			eltorito_media_image_path = g_strdup_printf ("/%s%s%s", cd, G_DIR_SEPARATOR_S, eltorito_media_image);
			g_free (cd);
		}

		g_node_traverse (burgner_isofs_tree_get_root_node (tree),
				 G_PRE_ORDER, G_TRAVERSE_ALL, -1,
				 _burgner_mkisofs_find_eltorito_boot_image, &info);

		if (!info) {
			GnomeVFSFileInfo *f_info;
			GnomeVFSResult res;
			GString *f_uri = g_string_new ("file://");

			g_string_append (f_uri, eltorito_media_image_path);
			f_info = gnome_vfs_file_info_new ();

			if ((res = gnome_vfs_get_file_info (f_uri->str, f_info, GNOME_VFS_FILE_INFO_DEFAULT)) != GNOME_VFS_OK) {
				fprintf (stderr, _("Can't get informations about %s: %s\n"),
							eltorito_media_image_path,
							gnome_vfs_result_to_string (res));
				return EXIT_FAILURE;
			}

			info = burgner_isofs_file_info_new ();
			info->uri = g_uri_new ();
			g_uri_set_path (info->uri, eltorito_media_image_path);
			info->size = f_info->size;
			info->type = _burgner_isofs_file_type_from_gnome_vfs_file_type (f_info->type);
		} else {
			is_in_tree = TRUE;
		}

		if (info->type != BURGNER_ISOFS_FILE_TYPE_REGULAR) {
			fprintf (stderr, _("Eltorito boot image (%s) is not a regular file.\n"), eltorito_media_image);
			return EXIT_FAILURE;
		}

		if (info->size % 512) {
			fprintf (stderr, _("Eltorito boot image (%s) size is not a multiple of 512.\n"), eltorito_media_image);
			return EXIT_FAILURE;
		}

		if (!eltorito_media_type_string) {
			switch (info->size / 512) {
				case 2880:
					eltorito_media_type = BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_1_44_MEG_DISKETTE;
					break;
				case 5760:
					eltorito_media_type = BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_2_88_MEG_DISKETTE;
					break;
				case 2400:
					eltorito_media_type = BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_1_2_MEG_DISKETTE;
					break;
				default:
					eltorito_media_type = BURGNER_ISOFS_ELTORITO_MEDIA_TYPE_HARD_DISK;
					break;
			}
		}

		fprintf (stderr, _("Eltorito enabled: %s, %s, %s\n"),
					burgner_isofs_eltorito_media_type_to_string (eltorito_media_type),
					burgner_isofs_eltorito_platform_id_to_string (eltorito_platform),
					eltorito_bootable ? _("bootable") : _("not bootable"));
		if (is_in_tree)
			fprintf (stderr, _("Found Eltorito boot image in tree\n"));

		builder = burgner_isofs_eltorito_builder_new ();
		g_object_set (builder,
				"manufacturer", eltorito_manufacturer,
				"platform_id", eltorito_platform,
				"media_type", eltorito_media_type,
				"system_type", eltorito_system_type,
				"bootable", eltorito_bootable,
				"info", info,
				"is_in_tree", is_in_tree,
				NULL);

		burgner_isofs_list_maker_add_builder (list_maker, "eltorito", builder);
		g_object_unref (builder);
	}

	items_list = burgner_isofs_list_maker_create_list (list_maker,
							   burgner_isofs_tree_get_root_node (tree),
							   optimize,
							   with_first_16_empty_sectors);

	{
		guint32 s_c = 0, b_o = 0;
		guint32 lost_space = 0;

		burgner_isofs_list_iteration_foreach (items_list, &s_c, &b_o, _burgner_mkisofs_list_iteration_func, &lost_space);

		b_o += s_c << 11;

		image_size = b_o;
		fprintf (stderr, _("Image size: %u B (%u MB), bytes lost: %u B (%u kB), %s\n"),
					b_o, b_o >> 20,
					lost_space, lost_space >> 10,
					optimize ? _("optimized") : _("not optimized"));
	}

	if (!output_file || (strcmp (output_file, "-") == 0)) {
		output_channel = g_io_channel_unix_new (STDOUT_FILENO);
	} else {
		GError *err = NULL;
		if (!(output_channel = g_io_channel_new_file (output_file, "w", &err))) {
			fprintf (stderr, _("Error while opening %s: %s\n"),
					output_file,
					err && err->message ? err->message : _("[no description]"));
			return EXIT_FAILURE;
		}
	}

	g_io_channel_set_encoding (output_channel, NULL, NULL);
	g_io_channel_set_buffered (output_channel, FALSE);
	g_io_channel_set_flags (output_channel, g_io_channel_get_flags (output_channel) & ~G_IO_FLAG_NONBLOCK, NULL);

	writer = burgner_isofs_writer_new (items_list);
	burgner_isofs_writer_prepare (writer, first_sector_lba);

	g_idle_add (_burgner_mkisofs_write_idle, NULL);

	g_main_loop_run (main_loop);

	return retval;
}

