
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>

#include <glib.h>
#include "../libburgnerutils/g-uri.h"



int
main (int argc, char **argv)
{
	GError *error = NULL;
	GUri *base_uri, *rel_uri, *new_uri = NULL;
	gchar *str;

	if (argc != 3) {
		fprintf (stderr, "Usage: %s <base_uri> <relative_uri>\n", argv[0]);
		return 1;
	}

	base_uri = g_uri_new ();
	if (!g_uri_set (base_uri, argv[1], TRUE, FALSE, &error)) {
		fprintf (stderr, "Error while creating base uri: %s\n", error && error->message ? error->message : "[No message or error]");
		return 1;
	}

	rel_uri = g_uri_new ();
	if (!g_uri_set (rel_uri, argv[2], TRUE, FALSE, &error)) {
		fprintf (stderr, "Error while creating relative uri: %s\n",
				error && error->message ? error->message : "[No message or error]");
		return 1;
	}

	if (!g_uri_relative_resolve (&new_uri, rel_uri, base_uri, &error)) {
		fprintf (stderr, "Error while resolving uri: %s\n",
			error && error->message ? error->message : "[No message or error]");
		return 1;
	}

	if (!(str = g_uri_format (new_uri, G_URI_FORMAT_OPTIONS_NONE))) {
		fprintf (stderr, "Error while formatting uri!\n");
		return 1;
	}

	g_print ("%s\n", str);

	g_free (str);
	g_uri_unref (new_uri);
	g_uri_unref (rel_uri);

	g_uri_unref (base_uri);

	return 0;
}

