
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <string.h>

#include <glib.h>
#include "../libburgnerutils/g-uri.h"


static struct {
	const gchar *rel_uri;
	const gchar *new_uri;
} rel_couple_uri[] = {
      { "g:h", "g:h" },
      { "g", "http://a/b/c/g" },
      { "./g", "http://a/b/c/g" },
      { "g/", "http://a/b/c/g/" },
      { "/g", "http://a/g" },
      { "//g", "http://g" },
      { "?y", "http://a/b/c/?y" },
      { "g?y", "http://a/b/c/g?y" },
      { "#s", "#s" },
      { "g#s", "http://a/b/c/g#s" },
      { "g?y#s", "http://a/b/c/g?y#s" },
      { ";x", "http://a/b/c/;x" },
      { "g;x", "http://a/b/c/g;x" },
      { "g;x?y#s", "http://a/b/c/g;x?y#s" },
      { ".", "http://a/b/c/" },
      { "./", "http://a/b/c/" },
      { "..", "http://a/b/" },
      { "../", "http://a/b/" },
      { "../g", "http://a/b/g" },
      { "../..", "http://a/" },
      { "../../", "http://a/" },
      { "../../g", "http://a/g" },
      { "../../../g", "http://a/../g" },
      { "../../../../g", "http://a/../../g" },
      { "/./g", "http://a/./g" },
      { "/../g", "http://a/../g" },
      { "g.", "http://a/b/c/g." },
      { ".g", "http://a/b/c/.g" },
      { "g..", "http://a/b/c/g.." },
      { "..g", "http://a/b/c/..g" },
      { "./../g", "http://a/b/g" },
      { "./g/.", "http://a/b/c/g/" },
      { "g/./h", "http://a/b/c/g/h" },
      { "g/../h", "http://a/b/c/h" },
      { "g;x=1/./y", "http://a/b/c/g;x=1/y" },
      { "g;x=1/../y", "http://a/b/c/y" },
      { "g?y/./x", "http://a/b/c/g?y/./x" },
      { "g?y/../x", "http://a/b/c/g?y/../x" },
      { "g#s/./x", "http://a/b/c/g#s/./x" },
      { "http:g", "http:g" },
	{ NULL, NULL }
};

int
main (int argc, char **argv)
{
	GError *error = NULL;
	GUri *base_uri;
	gint i;

	g_print ("Uri Resolver tests...\n");
	base_uri = g_uri_new ();
	if (!g_uri_set (base_uri, "http://a/b/c/d;p?q", TRUE, FALSE, &error)) {
		g_print ("Error while creating base uri: %s\n", error && error->message ? error->message : "[No message or error]");
		return 1;
	}

	error = NULL;
	for (i = 0; rel_couple_uri[i].rel_uri; i++) {
		GUri *new_uri = NULL, *rel_uri = g_uri_new ();
		gchar *str;

		g_print ("Testing %s -> %s ...\n",
				rel_couple_uri[i].rel_uri,
				rel_couple_uri[i].new_uri);

		if (!g_uri_set (rel_uri, (gchar*) rel_couple_uri[i].rel_uri,
					TRUE, FALSE, &error)) {
			g_print ("Error while creating relative uri: %s\n",
					error && error->message ? error->message : "[No message or error]");
			return 1;
		}

		if (!g_uri_relative_resolve (&new_uri, rel_uri, base_uri, &error)) {
			g_print ("Error while resolving uri: %s\n",
				error && error->message ? error->message : "[No message or error]");
			return 1;
		}

		if (!(str = g_uri_format (new_uri, G_URI_FORMAT_OPTIONS_NONE))) {
			g_print ("Error while formatting uri!\n");
			return 1;
		}

		if (strcmp (str, rel_couple_uri[i].new_uri) != 0) {
			g_print ("New uri doesn't match: %s\n", str);
		} else {
			g_print ("OK\n");
		}

		g_free (str);
		g_uri_unref (new_uri);
		g_uri_unref (rel_uri);
	}

	g_uri_unref (base_uri);

	return 0;
}

